/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
'use strict';

const scene_view_api_pb = require('./scene_view_api_pb-3310ca49.js');
const wrappers_pb = require('./wrappers_pb-f76eff58.js');
const grpc = require('./grpc-f1cd4ae7.js');
const mapper = require('./mapper-98c59ac6.js');
const streamAttributes = require('./streamAttributes-8c3c3d8a.js');
const browser_esm = require('./browser.esm-91aa64b8.js');

/**
 * The `MeasurementController` is responsible for performing measurements of
 * registered entities, and updating the model with their measurement results.
 */
class MeasurementController {
  constructor(model, client, jwtProvider, deviceId) {
    this.model = model;
    this.client = client;
    this.jwtProvider = jwtProvider;
    this.deviceId = deviceId;
    this.outcome = Promise.resolve(undefined);
  }
  /**
   * Registers an entity to measure and performs a measurement if this entity
   * has not been previously registered.
   *
   * @param entity The entity to measure.
   * @returns A promise that resolves with the results after registering this
   * entity.
   */
  addEntity(entity) {
    return this.performMeasurement(() => this.model.addEntity(entity));
  }
  /**
   * Clears all entities and returns a promise that resolves with an empty list
   * of measurement results.
   */
  clearEntities() {
    return this.performMeasurement(() => {
      this.model.clearEntities();
      this.model.clearOutcome();
      return true;
    });
  }
  /**
   * Deregisters an entity and performs a measurement if this entity was
   * removed.
   *
   * @param entity The entity to remove.
   * @returns A promise that resolves with the results after removing this
   * entity.
   */
  removeEntity(entity) {
    return this.performMeasurement(() => this.model.removeEntity(entity));
  }
  /**
   * Registers a set of entities and performs a measurement
   *
   * @param entities The entities to measure.
   * @returns A promise that resolves with the results after registering these
   * entities.
   */
  setEntities(entities) {
    return this.performMeasurement(() => this.model.setEntities(entities));
  }
  performMeasurement(effect) {
    const previous = this.model.getEntities();
    const changed = effect();
    const entities = this.model.getEntities();
    if (changed) {
      this.measureAndUpdateModel(entities);
      this.highlightEntities(previous, entities);
    }
    return this.outcome;
  }
  measureAndUpdateModel(entities) {
    if (entities.length > 0) {
      this.outcome = this.measureEntities(entities).then((outcome) => {
        this.model.setOutcome(outcome);
        return this.model.getOutcome();
      });
    }
    else {
      this.outcome = Promise.resolve(undefined);
    }
  }
  async measureEntities(entities) {
    if (entities.length > 0) {
      const res = await grpc.requestUnary(async (handler) => {
        const meta = await grpc.createMetadata(this.jwtProvider, this.deviceId);
        const req = new scene_view_api_pb.scene_view_api_pb.MeasureRequest();
        req.setEntitiesList(entities.map((e) => e.toProto()));
        this.client.measure(req, meta, handler);
      });
      return mapper.mapMeasureResponseOrThrow(res.toObject());
    }
    else {
      return undefined;
    }
  }
  async highlightEntities(previous, entities) {
    await grpc.requestUnary(async (handler) => {
      const meta = await grpc.createMetadata(this.jwtProvider, this.deviceId);
      const entitySet = new Set(entities);
      const newPrevious = previous.filter((e) => !entitySet.has(e));
      const clearEntities = newPrevious.map((e) => {
        const update = new scene_view_api_pb.scene_view_api_pb.ModelEntityUpdate();
        update.setModelEntity(scene_view_api_pb.model_entity_pb.ModelEntity.deserializeBinary(e.modelEntity));
        update.setHighlight(new wrappers_pb.wrappers_pb.BoolValue().setValue(false));
        return update;
      });
      const highlightEntities = entities.map((e) => {
        const update = new scene_view_api_pb.scene_view_api_pb.ModelEntityUpdate();
        update.setModelEntity(scene_view_api_pb.model_entity_pb.ModelEntity.deserializeBinary(e.modelEntity));
        update.setHighlight(new wrappers_pb.wrappers_pb.BoolValue().setValue(true));
        return update;
      });
      const req = new scene_view_api_pb.scene_view_api_pb.UpdateModelEntitiesRequest();
      req.setUpdatesList([...clearEntities, ...highlightEntities]);
      this.client.updateModelEntities(req, meta, handler);
    });
  }
}

class MeasurementEntity {
  constructor(point, modelEntity) {
    this.point = point;
    this.modelEntity = modelEntity;
  }
  static fromHit(hit) {
    if (hit.hitPoint != null && hit.modelEntity != null) {
      const hitPoint = browser_esm.mapper.ifInvalidThrow(streamAttributes.fromPbVector3f)(hit.hitPoint);
      const modelEntity = streamAttributes.vertexvis.protobuf.core.ModelEntity.encode(hit.modelEntity).finish();
      return new MeasurementEntity(hitPoint, modelEntity);
    }
    else {
      throw new Error('Cannot create MeasurementEntity from Hit. Hit is missing hit point and model entity');
    }
  }
  toProto() {
    const entity = new scene_view_api_pb.scene_view_api_pb.MeasureEntity();
    const point = new scene_view_api_pb.geometry_pb.Vector3f();
    point.setX(this.point.x);
    point.setY(this.point.y);
    point.setZ(this.point.z);
    entity.setPoint(point);
    const modelEntity = scene_view_api_pb.model_entity_pb.ModelEntity.deserializeBinary(this.modelEntity);
    entity.setModelEntity(modelEntity);
    return entity;
  }
}

exports.MeasurementController = MeasurementController;
exports.MeasurementEntity = MeasurementEntity;

//# sourceMappingURL=entities-a79fb0b0.js.map