/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const queries = require('./queries-23b45632.js');
const controller = require('./controller-6a70e6f9.js');
const row = require('./row-474ad32c.js');
const types = require('./types-bafc49e5.js');
const cursors = require('./cursors-1f5d9d93.js');
const entities$1 = require('./entities-a79fb0b0.js');
const model = require('./model-07ecc08f.js');
const overlays = require('./overlays-8906fa5e.js');
const results = require('./results-dc11ea4b.js');
const bundle_esm = require('./bundle.esm-9f2927ac.js');
const controller$1 = require('./controller-66690522.js');
const streamAttributes = require('./streamAttributes-8c3c3d8a.js');
const entities = require('./entities-ed091fe1.js');
const markup = require('./markup-17396bf6.js');
const viewport = require('./viewport-b2283e4b.js');
const model$1 = require('./model-2ca989b6.js');
require('./browser.esm-91aa64b8.js');
require('./errors-959312c1.js');
require('./_commonjsHelpers-6d73bb9b.js');
require('./wrappers_pb-f76eff58.js');
require('./scene_view_api_pb-3310ca49.js');
require('./grpc-f1cd4ae7.js');
require('./mapper-f1521d42.js');
require('./bundle.esm-a19223be.js');
require('./mapper-98c59ac6.js');

const ALMOST_ONE = 0.9999;
/**
 * For any single normal vector, there are an infinite number of potential orthogonal vectors. This function will
 * calculate a single vector evaluating the length of two candidates, and picking the one with the higher squared magnitude
 * @param normal
 * @returns
 */
function calculateOrthogonalCoordinate(norm) {
  const orthogonalCandidate0 = bundle_esm.vector3.create(0, norm.z, -norm.y);
  const orthogonalCandidate1 = bundle_esm.vector3.create(-norm.z, 0, norm.x);
  const theChosenOne = bundle_esm.vector3.magnitudeSquared(orthogonalCandidate0) >
    bundle_esm.vector3.magnitudeSquared(orthogonalCandidate1)
    ? orthogonalCandidate0
    : orthogonalCandidate1;
  return bundle_esm.vector3.normalize(bundle_esm.vector3.cross(norm, theChosenOne));
}
/**
 * Computes the rotation matrix for two normals. If both normals are neither parallel or anti-parallel,
 * this will compute the rotation matrix delta based on the angle from both normals.
 * If the normals are anti-parallel, the identity matrix will be returned, as no rotation is necessary in this case.
 * If the normals are parallel, an axis direction based on a chosen orthogonal vector will be used
 * to compute the rotation matrix to rotate the plane 180 degrees.
 * @param normal1
 * @param normal2
 * @returns an anti-parallel rotation Matrix4 betwen the given normals
 */
function computeRotationMatrix(normal1, normal2) {
  const dot = bundle_esm.vector3.dot(normal1, normal2);
  // the angle is almost 0 in this case.
  if (dot > ALMOST_ONE) {
    const axisDirection = calculateOrthogonalCoordinate(normal1);
    const quaternion = bundle_esm.quaternion.fromAxisAngle(axisDirection, Math.PI);
    return bundle_esm.matrix4.makeRotation(quaternion);
  }
  // the angle is almost 180 in this case.
  else if (dot <= -ALMOST_ONE) {
    return bundle_esm.matrix4.makeIdentity();
  }
  // the angle is between 0 & 180
  else {
    const angle = bundle_esm.vector3.angleTo(normal2, normal1);
    const axisDirection = bundle_esm.vector3.normalize(bundle_esm.vector3.cross(normal1, normal2));
    return bundle_esm.matrix4.makeRotation(bundle_esm.quaternion.fromAxisAngle(axisDirection, angle + Math.PI));
  }
}
/**
 * Computes the translation & rotation matrix delta between two world positions and two normals.
 * such that the computed translation matrix will be the delta between position 1 and position 2,
 * and the rotation will be rotated to be anti-parallel.
 *
 * @param normal1
 * @param position1
 * @param normal2
 * @param position2
 *
 * @returns Matrix4 translation matrix delta from position1 to
 * position2 & an anti-parallel rotation delta.
 */
function computeTransformationDelta(normal1, position1, normal2, position2) {
  const rotationMatrix = computeRotationMatrix(normal1, normal2);
  const translationDeltaMatrix = bundle_esm.matrix4.makeTranslation(position2);
  return bundle_esm.matrix4.multiply(bundle_esm.matrix4.multiply(translationDeltaMatrix, rotationMatrix), bundle_esm.matrix4.makeTranslation(bundle_esm.vector3.negate(position1)));
}

const transformationDelta = /*#__PURE__*/Object.freeze({
  __proto__: null,
  ALMOST_ONE: ALMOST_ONE,
  calculateOrthogonalCoordinate: calculateOrthogonalCoordinate,
  computeRotationMatrix: computeRotationMatrix,
  computeTransformationDelta: computeTransformationDelta
});

exports.Camera = queries.Camera;
exports.ColorMaterial = queries.colorMaterial;
exports.CrossSectioner = queries.CrossSectioner;
exports.ItemOperationBuilder = queries.ItemOperationBuilder;
exports.LoadableResource = queries.loadableResource;
exports.ModelViewController = queries.ModelViewController;
exports.PmiAnnotationOperationsBuilder = queries.PmiAnnotationOperationsBuilder;
exports.PmiAnnotationsQueryExecutor = queries.PmiAnnotationsQueryExecutor;
exports.PmiController = queries.PmiController;
exports.Raycaster = queries.Raycaster;
exports.RootQuery = queries.RootQuery;
exports.Scene = queries.Scene;
exports.SceneElementQueryExecutor = queries.SceneElementQueryExecutor;
exports.SceneElementsOperationExecutor = queries.SceneElementsOperationExecutor;
exports.SceneItemOperationsBuilder = queries.SceneItemOperationsBuilder;
exports.SceneItemQueryExecutor = queries.SceneItemQueryExecutor;
exports.SceneItemsOperationExecutor = queries.SceneItemsOperationExecutor;
exports.SceneViewStateIdentifier = queries.sceneViewStateIdentifier;
exports.SynchronizedClock = queries.SynchronizedClock;
exports.SceneTreeController = controller.SceneTreeController;
exports.fromNodeProto = row.fromNodeProto;
exports.isLoadedRow = row.isLoadedRow;
exports.SCENE_ITEM_NAME_METADATA_KEY = types.SCENE_ITEM_NAME_METADATA_KEY;
exports.CursorManager = cursors.CursorManager;
exports.boxQueryCursor = cursors.boxQueryCursor;
exports.labelPinCursor = cursors.labelPinCursor;
exports.measurementCursor = cursors.measurementCursor;
exports.measurementWithArrowCursor = cursors.measurementWithArrowCursor;
exports.pinCursor = cursors.pinCursor;
exports.MeasurementController = entities$1.MeasurementController;
exports.MeasurementEntity = entities$1.MeasurementEntity;
exports.MeasurementModel = model.MeasurementModel;
exports.MeasurementOverlayManager = overlays.MeasurementOverlayManager;
exports.makeMinimumDistanceResult = results.makeMinimumDistanceResult;
exports.TransformController = controller$1.TransformController;
exports.AngleUnits = streamAttributes.AngleUnits;
exports.AreaUnits = streamAttributes.AreaUnits;
exports.CrossSectioning = streamAttributes.crossSectioning;
exports.DistanceUnits = streamAttributes.DistanceUnits;
exports.FrameCameraBase = streamAttributes.FrameCameraBase;
exports.Orientation = streamAttributes.Orientation;
exports.ReceivedFrame = streamAttributes.Frame;
exports.ReceivedFrameImage = streamAttributes.FrameImage;
exports.ReceivedFrameScene = streamAttributes.FrameScene;
exports.ReceivedOrthographicCamera = streamAttributes.FrameOrthographicCamera;
exports.ReceivedPerspectiveCamera = streamAttributes.FramePerspectiveCamera;
exports.StandardView = streamAttributes.StandardView;
Object.defineProperty(exports, 'EntityType', {
  enumerable: true,
  get: function () {
    return entities.EntityType;
  }
});
exports.ArrowMarkup = markup.ArrowMarkup;
exports.CircleMarkup = markup.CircleMarkup;
exports.FreeformMarkup = markup.FreeformMarkup;
exports.Viewport = viewport.Viewport;
exports.VolumeIntersectionQueryController = model$1.VolumeIntersectionQueryController;
exports.VolumeIntersectionQueryModel = model$1.VolumeIntersectionQueryModel;
exports.TransformationDelta = transformationDelta;

//# sourceMappingURL=index.cjs.js.map