/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-4ebe6acf.js');
const bundle_esm = require('./bundle.esm-9f2927ac.js');
const browser_esm = require('./browser.esm-91aa64b8.js');

const viewerDomGroupCss = ".sc-vertex-viewer-dom-group-h{display:block;transform-style:preserve-3d}";

const ViewerDomGroup = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.propertyChange = index.createEvent(this, "propertyChange", 7);
    /**
     * The local 3D position of where this element is located.
     */
    this.position = bundle_esm.vector3.origin();
    /**
     * The local 3D position of where this element is located, as a JSON string.
     * JSON representation can either be in the format of `[x, y, z]` or `{"x": 0,
     * "y": 0, "z": 0}`.
     */
    this.positionJson = '';
    /**
     * The local rotation of this element.
     */
    this.quaternion = bundle_esm.quaternion.create();
    /**
     * The local rotation of this element, as a JSON string. JSON
     * representation can either be `[x, y, z, w]` or `{"x": 0, "y": 0, "z":
     * 0, "w": 1}`.
     */
    this.quaternionJson = '';
    /**
     * The local scale of this element.
     */
    this.scale = bundle_esm.vector3.create(1, 1, 1);
    /**
     * The local scale of this element, as a JSON string. JSON string
     * representation can either be in the format of `[x, y, z]` or `{"x": 0, "y":
     * 0, "z": 0}`.
     */
    this.scaleJson = '';
    /**
     * The local matrix of this element.
     */
    this.matrix = bundle_esm.matrix4.makeIdentity();
  }
  /**
   * @ignore
   */
  handlePositionChange() {
    this.syncMatrix();
  }
  /**
   * @ignore
   */
  handlePositionJsonChanged() {
    this.syncPosition();
  }
  /**
   * @ignore
   */
  handleRotationChanged() {
    this.syncQuaternionWithRotation();
  }
  /**
   * @ignore
   */
  handleRotationJsonChanged() {
    this.syncRotation();
  }
  /**
   * @ignore
   */
  handleQuaternionChange() {
    this.syncMatrix();
  }
  /**
   * @ignore
   */
  handleQuaternionJsonChanged() {
    this.syncQuaternion();
  }
  /**
   * @ignore
   */
  handleScaleChange() {
    this.syncMatrix();
  }
  /**
   * @ignore
   */
  handleScaleJsonChanged() {
    this.syncScale();
  }
  /**
   * @ignore
   */
  handleMatrixChanged(newMatrix, oldMatrix) {
    if (!browser_esm.objects.isEqual(newMatrix, oldMatrix)) {
      this.propertyChange.emit();
    }
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.syncProperties();
  }
  /**
   * @ignore
   */
  componentShouldUpdate() {
    return false;
  }
  syncProperties() {
    this.syncPosition();
    this.syncRotation();
    this.syncQuaternion();
    this.syncScale();
    this.syncMatrix();
  }
  syncPosition() {
    this.position =
      this.positionJson.length > 0
        ? this.parseJson('positionJson', this.positionJson, bundle_esm.vector3.fromJson)
        : this.position;
  }
  syncRotation() {
    this.rotation =
      this.rotationJson != null && this.rotationJson.length > 0
        ? this.parseJson('rotationJson', this.rotationJson, bundle_esm.euler.fromJson)
        : this.rotation;
    this.syncQuaternionWithRotation();
  }
  syncQuaternionWithRotation() {
    this.quaternion =
      this.rotation != null
        ? bundle_esm.quaternion.fromEuler(this.rotation)
        : this.quaternion;
  }
  syncQuaternion() {
    this.quaternion =
      this.quaternionJson.length > 0
        ? this.parseJson('quaternionJson', this.quaternionJson, bundle_esm.quaternion.fromJson)
        : this.quaternion;
  }
  syncScale() {
    this.scale =
      this.scaleJson.length > 0
        ? this.parseJson('scaleJson', this.scaleJson, bundle_esm.vector3.fromJson)
        : this.scale;
  }
  syncMatrix() {
    this.matrix = bundle_esm.matrix4.makeTRS(this.position, this.quaternion, this.scale);
  }
  parseJson(propName, value, parse) {
    try {
      return parse(value);
    }
    catch (e) {
      console.warn(`Could not parse \`${propName}\`. Invalid JSON.`);
      throw e;
    }
  }
  /**
   * @ignore
   */
  render() {
    return (index.h(index.Host, null, index.h("slot", null)));
  }
  static get watchers() { return {
    "position": ["handlePositionChange"],
    "positionJson": ["handlePositionJsonChanged"],
    "rotation": ["handleRotationChanged"],
    "rotationJson": ["handleRotationJsonChanged"],
    "quaternion": ["handleQuaternionChange"],
    "quaternionJson": ["handleQuaternionJsonChanged"],
    "scale": ["handleScaleChange"],
    "scaleJson": ["handleScaleJsonChanged"],
    "matrix": ["handleMatrixChanged"]
  }; }
};
ViewerDomGroup.style = viewerDomGroupCss;

exports.vertex_viewer_dom_group = ViewerDomGroup;

//# sourceMappingURL=vertex-viewer-dom-group.cjs.entry.js.map