/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { Component, Event, h, Host, Method, Prop, State, Watch, } from '@stencil/core';
import classNames from 'classnames';
import { debounceEvent } from '../../lib/stencil';
/**
 * @slot search-icon - A slot that replaces the component's default search icon.
 * @slot clear-icon - A slot that replaces the component's default clear icon.
 */
export class SceneTreeSearch {
  constructor() {
    /**
     * If `true`, disables user interaction of the component.
     */
    this.disabled = false;
    /**
     * Placeholder text if `value` is empty.
     */
    this.placeholder = undefined;
    /**
     * The current text value of the component. Value is updated on user
     * interaction.
     */
    this.value = '';
    this.focused = false;
    this.isSearching = false;
    this.handleTextInput = (event) => {
      const input = event.target;
      this.value = input.value;
      if (this.debounce != null || this.value === '') {
        this.emitCurrentValue();
      }
    };
    this.handleTextFocus = () => {
      this.focused = true;
    };
    this.handleTextBlur = () => {
      var _a;
      this.focused = false;
      (_a = this.searchDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
      if (this.value !== this.lastEmittedValue) {
        this.emitCurrentValue();
      }
    };
    this.handleKeyPress = (event) => {
      var _a;
      if (event.key === 'Enter') {
        (_a = this.searchDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
        this.emitCurrentValue();
      }
    };
    this.handleClear = (event) => {
      var _a;
      event.preventDefault();
      this.value = '';
      (_a = this.searchDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
      this.emitCurrentValue();
      this.setFocus();
    };
  }
  /**
   * Gives focus to the component's internal text input.
   */
  async setFocus() {
    var _a, _b;
    // HTMLInputElement.focus() doesn't exist in tests.
    if (typeof ((_a = this.inputEl) === null || _a === void 0 ? void 0 : _a.focus) === 'function') {
      (_b = this.inputEl) === null || _b === void 0 ? void 0 : _b.focus();
    }
  }
  /**
   * @ignore
   */
  controllerChanged(controller) {
    this.setupController();
  }
  /**
   * @ignore
   */
  componentDidLoad() {
    this.handleDebounceChanged();
    this.setupController();
  }
  /**
   * @ignore
   */
  disconnectedCallback() {
    var _a;
    (_a = this.onStateChangeDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
  }
  /**
   * Clears the current search term and clears any debounced filters.
   */
  clear() {
    var _a;
    this.value = '';
    (_a = this.searchDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    this.emitCurrentValue();
  }
  /**
   * @ignore
   */
  render() {
    return (h(Host, null,
      h("div", { class: "root" },
        h("div", { class: "overlay icon icon-search" },
          h("slot", { name: "search-icon" }, this.isSearching ? (h("vertex-viewer-spinner", { slot: "search-icon", size: "xs" })) : (h("vertex-viewer-icon", { name: "search", size: "sm" })))),
        h("input", { class: classNames('input', {
            background: this.focused || this.value.length > 0,
          }), type: "text", ref: (ref) => (this.inputEl = ref), placeholder: this.placeholder, disabled: this.disabled, value: this.value, onInput: this.handleTextInput, onFocus: this.handleTextFocus, onBlur: this.handleTextBlur, onKeyPress: this.handleKeyPress }),
        h("div", { class: classNames('overlay overlay-clear', {
            show: this.value.length > 0,
          }) },
          h("button", { class: "clear-btn icon", tabIndex: -1, onMouseDown: this.handleClear, disabled: this.disabled },
            h("slot", { name: "clear-icon" },
              h("vertex-viewer-icon", { name: "close-circle", size: "sm" })))))));
  }
  handleDebounceChanged() {
    var _a, _b;
    (_a = this.searchDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    const emitter = debounceEvent(this.search, (_b = this.debounce) !== null && _b !== void 0 ? _b : 0);
    // Track this emitter in two separate variables to maintain the `EventEmitter` typing for
    // `this.search`. This allows for correct generation of `CustomEvent` types.
    this.search = emitter;
    this.searchDisposable = emitter;
  }
  setupController() {
    var _a, _b;
    (_a = this.onStateChangeDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    this.onStateChangeDisposable = (_b = this.controller) === null || _b === void 0 ? void 0 : _b.onStateChange.on((state) => {
      this.isSearching = state.isSearching;
    });
  }
  emitCurrentValue() {
    this.lastEmittedValue = this.value;
    this.search.emit(this.value);
  }
  static get is() { return "vertex-scene-tree-search"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["scene-tree-search.css"]
  }; }
  static get styleUrls() { return {
    "$": ["scene-tree-search.css"]
  }; }
  static get properties() { return {
    "debounce": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Specifies the delay, in milliseconds, to emit `search` events after user\ninput.\n\nIf this value is specified, searches will automatically occur after a\nkeystroke has occurred and the debounce threshold has elapsed.\n\nDefaults to `undefined`, and searches only occur on an `Enter` press\nor a `blur` event."
      },
      "attribute": "debounce",
      "reflect": false
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, disables user interaction of the component."
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "placeholder": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Placeholder text if `value` is empty."
      },
      "attribute": "placeholder",
      "reflect": false,
      "defaultValue": "undefined"
    },
    "controller": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "SceneTreeController",
        "resolved": "SceneTreeController | undefined",
        "references": {
          "SceneTreeController": {
            "location": "import",
            "path": "../scene-tree/lib/controller"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The scene tree controller"
      }
    },
    "value": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The current text value of the component. Value is updated on user\ninteraction."
      },
      "attribute": "value",
      "reflect": false,
      "defaultValue": "''"
    }
  }; }
  static get states() { return {
    "focused": {},
    "isSearching": {}
  }; }
  static get events() { return [{
      "method": "search",
      "name": "search",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is emitted when a user has inputted or cleared the search\nterm. The event may be delayed according to the current `debounce` value."
      },
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "setFocus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Gives focus to the component's internal text input.",
        "tags": []
      }
    },
    "clear": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {},
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Clears the current search term and clears any debounced filters.",
        "tags": []
      }
    }
  }; }
  static get watchers() { return [{
      "propName": "controller",
      "methodName": "controllerChanged"
    }]; }
}
//# sourceMappingURL=scene-tree-search.js.map