/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { Component, Element, Event, h, Host, Prop, } from '@stencil/core';
import { Events } from '../../lib/types';
import { blurElement } from './utils';
export class SceneTreeTableCell {
  constructor() {
    /**
     * The value to display in this cell if the `value` specified is
     * undefined. Defaults to "--".
     */
    this.placeholder = '--';
    /**
     * Whether to always show the requested icons in the cell. If false,
     * the icons will only appear when hovering over the cell.
     */
    this.alwaysShowIcons = false;
    /**
     * @internal
     */
    this.hovered = false;
    this.displayValue = () => {
      const resp = this.value == null || this.value.trim() === ''
        ? this.placeholder
        : this.value;
      return resp;
    };
    this.handleCellPointerEnter = () => {
      var _a, _b, _c;
      (_a = this.hoverController) === null || _a === void 0 ? void 0 : _a.setHovered((_c = (_b = this.node) === null || _b === void 0 ? void 0 : _b.id) === null || _c === void 0 ? void 0 : _c.hex);
    };
    this.handleCellPointerLeave = () => {
      var _a;
      (_a = this.hoverController) === null || _a === void 0 ? void 0 : _a.setHovered(undefined);
    };
    this.handleCellPointerUp = (event) => {
      if (!this.isScrolling &&
        this.node != null &&
        this.tree != null &&
        this.longPressTimer != null) {
        if (this.selectionHandler != null) {
          this.selectionHandler(event, this.node, this.tree);
        }
        else {
          this.performDefaultSelectionOperation(event);
        }
        this.selectionToggled.emit({ node: this.node, originalEvent: event });
      }
      this.clearLongPressTimer();
    };
    this.handleCellPointerDown = (event) => {
      this.restartLongPressTimer();
    };
    this.createActionPointerUpHandler = (action) => {
      return (event) => {
        // Blur the `hostEl` after a `preventDefault` to clear focus that
        // is left on the element after `pointerdown` event.
        event.preventDefault();
        blurElement(this.hostEl);
        action(event);
      };
    };
    this.toggleExpansion = (event) => {
      if (this.tree != null && this.node != null) {
        if (this.expansionHandler != null) {
          this.expansionHandler(event, this.node, this.tree);
        }
        else {
          this.performDefaultExpansionOperation(this.node, this.tree);
        }
        this.expandToggled.emit({ node: this.node, originalEvent: event });
      }
    };
    this.toggleVisibility = (event) => {
      if (this.tree != null && this.node != null) {
        if (this.visibilityHandler != null) {
          this.visibilityHandler(event, this.node, this.tree);
        }
        else {
          this.performDefaultVisibilityOperation(this.node, this.tree);
        }
        this.visibilityToggled.emit({ node: this.node, originalEvent: event });
      }
    };
    this.isolate = (event) => {
      if (this.tree != null && this.node != null) {
        if (this.isolateHandler != null) {
          this.isolateHandler(event, this.node, this.tree);
        }
        else {
          this.performDefaultIsolateOperation(this.node, this.tree);
        }
        this.isolatePressed.emit({ node: this.node, originalEvent: event });
      }
    };
    this.performDefaultSelectionOperation = (event) => {
      var _a, _b, _c, _d, _e, _f;
      if (!event.defaultPrevented && event.button === 0) {
        if ((event.ctrlKey || event.metaKey) && ((_a = this.node) === null || _a === void 0 ? void 0 : _a.selected)) {
          (_b = this.tree) === null || _b === void 0 ? void 0 : _b.deselectItem(this.node);
        }
        else if ((_c = this.node) === null || _c === void 0 ? void 0 : _c.selected) {
          (_d = this.tree) === null || _d === void 0 ? void 0 : _d.selectItem(this.node, {
            recurseParent: true,
          });
        }
        else if (!((_e = this.node) === null || _e === void 0 ? void 0 : _e.selected)) {
          (_f = this.tree) === null || _f === void 0 ? void 0 : _f.selectItem(this.node, {
            append: event.ctrlKey || event.metaKey,
            range: event.shiftKey,
          });
        }
      }
    };
    this.performDefaultVisibilityOperation = (node, tree) => {
      tree.toggleItemVisibility(node);
    };
    this.performDefaultIsolateOperation = (node, tree) => {
      tree.isolateItem(node);
    };
    this.performDefaultExpansionOperation = (node, tree) => {
      tree.toggleExpandItem(node);
    };
  }
  componentDidLoad() {
    var _a;
    this.hoverListener = (_a = this.hoverController) === null || _a === void 0 ? void 0 : _a.stateChanged((id) => {
      var _a, _b;
      this.hovered = id === ((_b = (_a = this.node) === null || _a === void 0 ? void 0 : _a.id) === null || _b === void 0 ? void 0 : _b.hex);
    });
    this.cellLoaded.emit();
    this.clearLongPressTimer();
  }
  disconnectedCallback() {
    var _a;
    (_a = this.hoverListener) === null || _a === void 0 ? void 0 : _a.dispose();
    this.clearLongPressTimer();
  }
  componentWillRender() {
    var _a, _b, _c, _d, _e, _f;
    this.toggleAttribute('is-hovered', this.hovered);
    this.toggleAttribute('is-hidden', !((_a = this.node) === null || _a === void 0 ? void 0 : _a.visible));
    this.toggleAttribute('is-selected', !!((_b = this.node) === null || _b === void 0 ? void 0 : _b.selected));
    this.toggleAttribute('is-partial', !!((_c = this.node) === null || _c === void 0 ? void 0 : _c.partiallyVisible));
    this.toggleAttribute('is-leaf', !!((_d = this.node) === null || _d === void 0 ? void 0 : _d.isLeaf));
    this.toggleAttribute('is-end-item', !!((_e = this.node) === null || _e === void 0 ? void 0 : _e.endItem));
    this.toggleAttribute('is-filter-hit', !!((_f = this.node) === null || _f === void 0 ? void 0 : _f.filterHit));
  }
  render() {
    var _a, _b, _c;
    // Overrides the `.hydrated` visibility when we have nothing to display
    const hiddenStyle = this.node == null ? { visibility: 'hidden' } : undefined;
    const backgroundColorStyle = this.getBackgroundColorStyle();
    const expansionIcon = this.getExpansionIcon();
    const isolateIcon = this.getIsolateIcon();
    const visibilityIcon = this.getVisibilityIcon();
    return (h(Host, { onPointerEnter: this.handleCellPointerEnter, onPointerLeave: this.handleCellPointerLeave, onPointerUp: this.handleCellPointerUp, onPointerDown: this.handleCellPointerDown, style: Object.assign(Object.assign({}, hiddenStyle), { 'background-color': backgroundColorStyle }) },
      h("div", { class: "wrapper" },
        h("div", { class: "no-shrink" },
          h("slot", { name: "left-gutter" })),
        this.expandToggle && (h("button", { class: "expand-btn no-shrink", "data-test-id": 'expand-' + ((_a = this.node) === null || _a === void 0 ? void 0 : _a.name), onPointerUp: this.createActionPointerUpHandler(this.toggleExpansion) }, expansionIcon && (h("vertex-viewer-icon", { class: "icon", name: expansionIcon, size: "sm" })))),
        h("div", { class: "content" }, this.value != null && this.value.trim() !== '' ? (h("slot", null, this.displayValue())) : (h("slot", { name: "placeholder" }, this.placeholder))),
        this.isolateButton && (h("button", { class: "isolate-btn no-shrink", "data-test-id": 'isolate-btn-' + ((_b = this.node) === null || _b === void 0 ? void 0 : _b.name), onPointerUp: this.createActionPointerUpHandler(this.isolate) }, isolateIcon && (h("vertex-viewer-icon", { class: "icon", name: "locate", size: "sm" })))),
        this.visibilityToggle && (h("button", { class: "visibility-btn no-shrink", "data-test-id": 'visibility-btn-' + ((_c = this.node) === null || _c === void 0 ? void 0 : _c.name), onPointerUp: this.createActionPointerUpHandler(this.toggleVisibility) }, visibilityIcon && (h("vertex-viewer-icon", { class: "icon", name: visibilityIcon, size: "sm" })))),
        h("div", { class: "no-shrink" },
          h("slot", { name: "right-gutter" })),
        !this.visibilityToggle && !this.isolateButton && (h("div", { class: "column-spacer" })))));
  }
  toggleAttribute(attr, value) {
    if (value) {
      this.hostEl.setAttribute(attr, '');
    }
    else {
      this.hostEl.removeAttribute(attr);
    }
  }
  clearLongPressTimer() {
    if (this.longPressTimer != null) {
      window.clearTimeout(this.longPressTimer);
    }
    this.longPressTimer = undefined;
  }
  restartLongPressTimer() {
    this.clearLongPressTimer();
    this.longPressTimer = window.setTimeout(() => {
      this.clearLongPressTimer();
    }, Events.defaultEventConfig.longPressThreshold);
  }
  getBackgroundColorStyle() {
    var _a, _b;
    const backgroundColorStyle = this.getCssVariableWithFallbacks(`--scene-tree-row-background-color-depth-${(_a = this.node) === null || _a === void 0 ? void 0 : _a.depth}`, '--scene-tree-row-background-color');
    const selectedBackgroundColorStyle = this.getCssVariableWithFallbacks('--scene-tree-selected-row-background-color', '--scene-tree-cell-background-selected');
    const hoveredBackgroundColorStyle = this.getCssVariableWithFallbacks('--scene-tree-hovered-row-background-color', '--scene-tree-cell-background-hover');
    if (!!((_b = this.node) === null || _b === void 0 ? void 0 : _b.selected)) {
      return selectedBackgroundColorStyle;
    }
    else if (this.hovered) {
      return hoveredBackgroundColorStyle;
    }
    return backgroundColorStyle;
  }
  getCssVariableWithFallbacks(variable, ...fallbacks) {
    const sequencedFallbacks = [...fallbacks].reverse();
    return [...sequencedFallbacks, variable].reduce((res, s) => `var(${s}, ${res})`, 'unset');
  }
  getIsolateIcon() {
    if (this.hovered || this.alwaysShowIcons) {
      return 'locate';
    }
    return undefined;
  }
  getExpansionIcon() {
    var _a, _b, _c;
    if (!((_a = this.node) === null || _a === void 0 ? void 0 : _a.isLeaf) && !((_b = this.node) === null || _b === void 0 ? void 0 : _b.endItem)) {
      return ((_c = this.node) === null || _c === void 0 ? void 0 : _c.expanded) ? 'chevron-down' : 'chevron-right';
    }
    return undefined;
  }
  getVisibilityIcon() {
    var _a, _b, _c, _d, _e;
    if ((this.hovered || this.alwaysShowIcons) &&
      !((_a = this.node) === null || _a === void 0 ? void 0 : _a.partiallyVisible) &&
      ((_b = this.node) === null || _b === void 0 ? void 0 : _b.visible)) {
      return 'eye-open';
    }
    else if (!((_c = this.node) === null || _c === void 0 ? void 0 : _c.partiallyVisible) && !((_d = this.node) === null || _d === void 0 ? void 0 : _d.visible)) {
      return 'eye-half';
    }
    else if ((_e = this.node) === null || _e === void 0 ? void 0 : _e.partiallyVisible) {
      return 'eye-half-dotted';
    }
    return undefined;
  }
  static get is() { return "vertex-scene-tree-table-cell"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["scene-tree-table-cell.css"]
  }; }
  static get styleUrls() { return {
    "$": ["scene-tree-table-cell.css"]
  }; }
  static get properties() { return {
    "node": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Node.AsObject",
        "resolved": "undefined | { id?: AsObject | undefined; depth: number; name: string; visible: boolean; selected: boolean; expanded: boolean; isLeaf: boolean; suppliedId?: AsObject | undefined; partiallyVisible: boolean; columnsList: string[]; filterHit: boolean; phantom: boolean; endItem: boolean; }",
        "references": {
          "Node": {
            "location": "import",
            "path": "@vertexvis/scene-tree-protos/scenetree/protos/domain_pb"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The node data that is associated to the row that this cell belongs to.\nContains information related to if the node is expanded, visible, etc."
      }
    },
    "tree": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "HTMLVertexSceneTreeElement",
        "resolved": "HTMLVertexSceneTreeElement | undefined",
        "references": {
          "HTMLVertexSceneTreeElement": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "A reference to the scene tree to perform operations for interactions. Such\nas expansion, visibility and selection."
      }
    },
    "value": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The value to display in this cell."
      },
      "attribute": "value",
      "reflect": false
    },
    "placeholder": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The value to display in this cell if the `value` specified is\nundefined. Defaults to \"--\"."
      },
      "attribute": "placeholder",
      "reflect": false,
      "defaultValue": "'--'"
    },
    "alwaysShowIcons": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Whether to always show the requested icons in the cell. If false,\nthe icons will only appear when hovering over the cell."
      },
      "attribute": "always-show-icons",
      "reflect": false,
      "defaultValue": "false"
    },
    "hovered": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      },
      "attribute": "hovered",
      "reflect": false,
      "defaultValue": "false"
    },
    "isScrolling": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      },
      "attribute": "is-scrolling",
      "reflect": false
    },
    "expandToggle": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Indicates whether to display a button for toggling the expanded state of\nthe node associated with this cell."
      },
      "attribute": "expand-toggle",
      "reflect": false
    },
    "visibilityToggle": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Indicates whether to display a button for toggling the visibility state of\nthe node associated with this cell."
      },
      "attribute": "visibility-toggle",
      "reflect": false
    },
    "isolateButton": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Indicates whether to display a button for isolating (show only + fly to)\nthe node associated with this cell."
      },
      "attribute": "isolate-button",
      "reflect": false
    },
    "selectionHandler": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "SceneTreeOperationHandler",
        "resolved": "((event: PointerEvent, node: AsObject, tree: HTMLVertexSceneTreeElement) => void) | undefined",
        "references": {
          "SceneTreeOperationHandler": {
            "location": "import",
            "path": "../scene-tree/lib/handlers"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "An optional handler that will override this cell's default selection\nbehavior. The registered handler will receive the `pointerup` event,\nthe node data for the row this cell is associated with, and a reference\nto the parent `<vertex-scene-tree>` element for performing operations."
      }
    },
    "visibilityHandler": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "SceneTreeOperationHandler",
        "resolved": "((event: PointerEvent, node: AsObject, tree: HTMLVertexSceneTreeElement) => void) | undefined",
        "references": {
          "SceneTreeOperationHandler": {
            "location": "import",
            "path": "../scene-tree/lib/handlers"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "An optional handler that will override this cell's default visibility\nbehavior. The registered handler will receive the `pointerup` event,\nthe node data for the row this cell is associated with, and a reference\nto the parent `<vertex-scene-tree>` element for performing operations."
      }
    },
    "expansionHandler": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "SceneTreeOperationHandler",
        "resolved": "((event: PointerEvent, node: AsObject, tree: HTMLVertexSceneTreeElement) => void) | undefined",
        "references": {
          "SceneTreeOperationHandler": {
            "location": "import",
            "path": "../scene-tree/lib/handlers"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "An optional handler that will override this cell's default expansion\nbehavior. The registered handler will receive the `pointerup` event,\nthe node data for the row this cell is associated with, and a reference\nto the parent `<vertex-scene-tree>` element for performing operations."
      }
    },
    "isolateHandler": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "SceneTreeOperationHandler",
        "resolved": "((event: PointerEvent, node: AsObject, tree: HTMLVertexSceneTreeElement) => void) | undefined",
        "references": {
          "SceneTreeOperationHandler": {
            "location": "import",
            "path": "../scene-tree/lib/handlers"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "An optional handler that will override this cell's default isolate\nbehavior. The registered handler will receive the `pointerup` event,\nthe node data for the row this cell is associated with, and a reference\nto the parent `<vertex-scene-tree>` element for performing operations."
      }
    },
    "hoverController": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "SceneTreeCellHoverController",
        "resolved": "SceneTreeCellHoverController | undefined",
        "references": {
          "SceneTreeCellHoverController": {
            "location": "import",
            "path": "../scene-tree-table-layout/lib/hover-controller"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      }
    }
  }; }
  static get events() { return [{
      "method": "expandToggled",
      "name": "expandToggled",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is emitted when a user requests to expand the node. This is\nemitted even if interactions are disabled."
      },
      "complexType": {
        "original": "SceneTreeTableCellEventDetails",
        "resolved": "SceneTreeTableCellEventDetails",
        "references": {
          "SceneTreeTableCellEventDetails": {
            "location": "local"
          }
        }
      }
    }, {
      "method": "visibilityToggled",
      "name": "visibilityToggled",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is emitted when a user requests to change the node's\nvisibility. This event is emitted even if interactions are disabled."
      },
      "complexType": {
        "original": "SceneTreeTableCellEventDetails",
        "resolved": "SceneTreeTableCellEventDetails",
        "references": {
          "SceneTreeTableCellEventDetails": {
            "location": "local"
          }
        }
      }
    }, {
      "method": "selectionToggled",
      "name": "selectionToggled",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is emitted when a user requests to change the node's selection\nstate. This event is emitted even if interactions are disabled."
      },
      "complexType": {
        "original": "SceneTreeTableCellEventDetails",
        "resolved": "SceneTreeTableCellEventDetails",
        "references": {
          "SceneTreeTableCellEventDetails": {
            "location": "local"
          }
        }
      }
    }, {
      "method": "isolatePressed",
      "name": "isolatePressed",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is emitted when a user requests to isolate the node.\nThis event is emitted even if interactions are disabled."
      },
      "complexType": {
        "original": "SceneTreeTableCellEventDetails",
        "resolved": "SceneTreeTableCellEventDetails",
        "references": {
          "SceneTreeTableCellEventDetails": {
            "location": "local"
          }
        }
      }
    }, {
      "method": "cellLoaded",
      "name": "cellLoaded",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [{
            "name": "private",
            "text": undefined
          }],
        "text": "Used for internals or testing."
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }]; }
  static get elementRef() { return "hostEl"; }
}
//# sourceMappingURL=scene-tree-table-cell.js.map