/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
/* eslint-disable @typescript-eslint/member-ordering */
import { Component, Event, h, Host, Prop, Watch, } from '@stencil/core';
import { Euler, Matrix4, Quaternion, Vector3 } from '@vertexvis/geometry';
import { Objects } from '@vertexvis/utils';
/**
 * The `ViewerDomElement` is responsible for managing a
 * `<vertex-viewer-dom-element>` element. These elements are intended to be
 * added as children to a `<vertex-viewer-dom-renderer>` and represent an
 * individual DOM element within a local 3D scene.
 */
export class ViewerDomElement {
  constructor() {
    /**
     * The local 3D position of where this element is located.
     */
    this.position = Vector3.origin();
    /**
     * The local 3D position of where this element is located, as a JSON string.
     * JSON representation can either be in the format of `[x, y, z]` or `{"x": 0,
     * "y": 0, "z": 0}`.
     */
    this.positionJson = '';
    /**
     * The local rotation of this element.
     */
    this.quaternion = Quaternion.create();
    /**
     * The local quaternion rotation of this element, as a JSON string. JSON
     * representation can either be `[x, y, z, w]` or `{"x": 0, "y": 0, "z":
     * 0, "w": 1}`.
     */
    this.quaternionJson = '';
    /**
     * The local scale of this element.
     */
    this.scale = Vector3.create(1, 1, 1);
    /**
     * The local scale of this element, as a JSON string. JSON string
     * representation can either be in the format of `[x, y, z]` or `{"x": 0, "y":
     * 0, "z": 0}`.
     */
    this.scaleJson = '';
    /**
     * The local matrix of this element.
     */
    this.matrix = Matrix4.makeIdentity();
    /**
     * Disables occlusion testing for this element. Defaults to enabled. When
     * enabled, the elements position will be tested against the current depth
     * buffer. If the position is occluded, then the `occluded` attribute will be
     * set.
     */
    this.occlusionOff = false;
    /**
     * Indicates if the element is hidden by geometry. This property can be used
     * with a CSS selector to modify the appearance of the element when its
     * occluded.
     *
     * @example
     *
     * ```html
     * <style>
     *   vertex-viewer-dom-element[occluded] {
     *     opacity: 0;
     *   }
     * </style>
     * ```
     */
    this.occluded = false;
    /**
     * **EXPERIMENTAL**
     * Disables detached testing for this element. Defaults to disabled. When
     * enabled, the elements position will be tested against the current depth
     * buffer. If the position is detached, then the `detached` attribute will be
     * set.
     */
    this.detachedOff = true;
    /**
     * **EXPERIMENTAL**
     * Indicates if the element is detached from geometry. This property can be used
     * with a CSS selector to modify the appearance of the element when its
     * detached.
     *
     * @example
     *
     * ```html
     * <style>
     *   vertex-viewer-dom-element[detached] {
     *     opacity: 0;
     *   }
     * </style>
     * ```
     */
    this.detached = false;
    /**
     * Disables the billboarding behavior of the element. When billboarding is
     * enabled, the element will always be oriented towards the screen.
     */
    this.billboardOff = false;
    /**
     * Disables interaction events from children.
     */
    this.interactionsOff = false;
  }
  /**
   * @ignore
   */
  handlePositionChange() {
    this.syncMatrix();
  }
  /**
   * @ignore
   */
  handlePositionJsonChanged() {
    this.syncPosition();
  }
  /**
   * @ignore
   */
  handleRotationChanged() {
    this.syncQuaternionWithRotation();
  }
  /**
   * @ignore
   */
  handleRotationJsonChanged() {
    this.syncRotation();
  }
  /**
   * @ignore
   */
  handleQuaternionChange() {
    this.syncMatrix();
  }
  /**
   * @ignore
   */
  handleQuaternionJsonChanged() {
    this.syncQuaternion();
  }
  /**
   * @ignore
   */
  handleScaleChange() {
    this.syncMatrix();
  }
  /**
   * @ignore
   */
  handleScaleJsonChanged() {
    this.syncScale();
  }
  /**
   * @ignore
   */
  handleMatrixChanged(newMatrix, oldMatrix) {
    if (!Objects.isEqual(newMatrix, oldMatrix)) {
      this.propertyChange.emit();
    }
  }
  /**
   * @ignore
   */
  connectedCallback() {
    this.syncProperties();
  }
  /**
   * @ignore
   */
  componentShouldUpdate(newValue, oldValue, prop) {
    return prop === 'occluded';
  }
  /**
   * @ignore
   */
  handleOcclusionStateChanged() {
    this.occlusionStateChanged.emit(this.occluded);
  }
  /**
   * @ignore
   */
  handleDetachedStateChanged() {
    this.detachedStateChanged.emit(this.detached);
  }
  syncProperties() {
    this.syncPosition();
    this.syncRotation();
    this.syncQuaternion();
    this.syncScale();
    this.syncMatrix();
  }
  syncPosition() {
    this.position =
      this.positionJson.length > 0
        ? this.parseJson('positionJson', this.positionJson, Vector3.fromJson)
        : this.position;
  }
  syncRotation() {
    this.rotation =
      this.rotationJson != null && this.rotationJson.length > 0
        ? this.parseJson('rotationJson', this.rotationJson, Euler.fromJson)
        : this.rotation;
    this.syncQuaternionWithRotation();
  }
  syncQuaternionWithRotation() {
    this.quaternion =
      this.rotation != null
        ? Quaternion.fromEuler(this.rotation)
        : this.quaternion;
  }
  syncQuaternion() {
    this.quaternion =
      this.quaternionJson.length > 0
        ? this.parseJson('quaternionJson', this.quaternionJson, Quaternion.fromJson)
        : this.quaternion;
  }
  syncScale() {
    this.scale =
      this.scaleJson.length > 0
        ? this.parseJson('scaleJson', this.scaleJson, Vector3.fromJson)
        : this.scale;
  }
  syncMatrix() {
    this.matrix = Matrix4.makeTRS(this.position, this.quaternion, this.scale);
  }
  parseJson(propName, value, parse) {
    try {
      return parse(value);
    }
    catch (e) {
      console.warn(`Could not parse \`${propName}\`. Invalid JSON.`);
      throw e;
    }
  }
  /**
   * @ignore
   */
  render() {
    return (h(Host, null,
      h("slot", null)));
  }
  static get is() { return "vertex-viewer-dom-element"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["viewer-dom-element.css"]
  }; }
  static get styleUrls() { return {
    "$": ["viewer-dom-element.css"]
  }; }
  static get properties() { return {
    "position": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Vector3.Vector3",
        "resolved": "Vector3",
        "references": {
          "Vector3": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The local 3D position of where this element is located."
      },
      "defaultValue": "Vector3.origin()"
    },
    "positionJson": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The local 3D position of where this element is located, as a JSON string.\nJSON representation can either be in the format of `[x, y, z]` or `{\"x\": 0,\n\"y\": 0, \"z\": 0}`."
      },
      "attribute": "position",
      "reflect": false,
      "defaultValue": "''"
    },
    "rotation": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Euler.Euler",
        "resolved": "Euler | undefined",
        "references": {
          "Euler": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The local rotation of this element in Euler angles."
      }
    },
    "rotationJson": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The local rotation of this element in Euler angles, as a JSON string. JSON\nrepresentation can either be `[x, y, z, order]` or `{\"x\": 0, \"y\": 0, \"z\":\n0, \"order\": \"xyz\"}`."
      },
      "attribute": "rotation",
      "reflect": false
    },
    "quaternion": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Quaternion.Quaternion",
        "resolved": "Quaternion",
        "references": {
          "Quaternion": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The local rotation of this element."
      },
      "defaultValue": "Quaternion.create()"
    },
    "quaternionJson": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The local quaternion rotation of this element, as a JSON string. JSON\nrepresentation can either be `[x, y, z, w]` or `{\"x\": 0, \"y\": 0, \"z\":\n0, \"w\": 1}`."
      },
      "attribute": "quaternion",
      "reflect": false,
      "defaultValue": "''"
    },
    "scale": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Vector3.Vector3",
        "resolved": "Vector3",
        "references": {
          "Vector3": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The local scale of this element."
      },
      "defaultValue": "Vector3.create(1, 1, 1)"
    },
    "scaleJson": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The local scale of this element, as a JSON string. JSON string\nrepresentation can either be in the format of `[x, y, z]` or `{\"x\": 0, \"y\":\n0, \"z\": 0}`."
      },
      "attribute": "scale",
      "reflect": false,
      "defaultValue": "''"
    },
    "matrix": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Matrix4.Matrix4",
        "resolved": "[number, number, number, number, number, number, number, number, number, number, number, number, number, number, number, number]",
        "references": {
          "Matrix4": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The local matrix of this element."
      },
      "defaultValue": "Matrix4.makeIdentity()"
    },
    "occlusionOff": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disables occlusion testing for this element. Defaults to enabled. When\nenabled, the elements position will be tested against the current depth\nbuffer. If the position is occluded, then the `occluded` attribute will be\nset."
      },
      "attribute": "occlusion-off",
      "reflect": false,
      "defaultValue": "false"
    },
    "occluded": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "example",
            "text": "```html\n<style>\n  vertex-viewer-dom-element[occluded] {\n    opacity: 0;\n  }\n</style>\n```"
          }],
        "text": "Indicates if the element is hidden by geometry. This property can be used\nwith a CSS selector to modify the appearance of the element when its\noccluded."
      },
      "attribute": "occluded",
      "reflect": true,
      "defaultValue": "false"
    },
    "detachedOff": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "**EXPERIMENTAL**\nDisables detached testing for this element. Defaults to disabled. When\nenabled, the elements position will be tested against the current depth\nbuffer. If the position is detached, then the `detached` attribute will be\nset."
      },
      "attribute": "detached-off",
      "reflect": false,
      "defaultValue": "true"
    },
    "detached": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "example",
            "text": "```html\n<style>\n  vertex-viewer-dom-element[detached] {\n    opacity: 0;\n  }\n</style>\n```"
          }],
        "text": "**EXPERIMENTAL**\nIndicates if the element is detached from geometry. This property can be used\nwith a CSS selector to modify the appearance of the element when its\ndetached."
      },
      "attribute": "detached",
      "reflect": true,
      "defaultValue": "false"
    },
    "billboardOff": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disables the billboarding behavior of the element. When billboarding is\nenabled, the element will always be oriented towards the screen."
      },
      "attribute": "billboard-off",
      "reflect": false,
      "defaultValue": "false"
    },
    "interactionsOff": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disables interaction events from children."
      },
      "attribute": "interactions-off",
      "reflect": true,
      "defaultValue": "false"
    }
  }; }
  static get events() { return [{
      "method": "occlusionStateChanged",
      "name": "occlusionStateChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Dispatched when the occlusion state is changed."
      },
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      }
    }, {
      "method": "detachedStateChanged",
      "name": "detachedStateChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "**EXPERIMENTAL**\nDispatched when the detached state is changed."
      },
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      }
    }, {
      "method": "propertyChange",
      "name": "propertyChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is emitted when any property on the dom group changes"
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }]; }
  static get watchers() { return [{
      "propName": "position",
      "methodName": "handlePositionChange"
    }, {
      "propName": "positionJson",
      "methodName": "handlePositionJsonChanged"
    }, {
      "propName": "rotation",
      "methodName": "handleRotationChanged"
    }, {
      "propName": "rotationJson",
      "methodName": "handleRotationJsonChanged"
    }, {
      "propName": "quaternion",
      "methodName": "handleQuaternionChange"
    }, {
      "propName": "quaternionJson",
      "methodName": "handleQuaternionJsonChanged"
    }, {
      "propName": "scale",
      "methodName": "handleScaleChange"
    }, {
      "propName": "scaleJson",
      "methodName": "handleScaleJsonChanged"
    }, {
      "propName": "matrix",
      "methodName": "handleMatrixChanged"
    }, {
      "propName": "occluded",
      "methodName": "handleOcclusionStateChanged"
    }, {
      "propName": "detached",
      "methodName": "handleDetachedStateChanged"
    }]; }
}
//# sourceMappingURL=viewer-dom-element.js.map