/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { Component, Element, Event, h, Host, Method, Prop, State, Watch, } from '@stencil/core';
import { getMarkupBoundingClientRect } from '../viewer-markup/dom';
import { isValidPointData, isValidStartEvent, translatePointToScreen, } from '../viewer-markup/markup-utils';
import { SvgShadow } from '../viewer-markup/viewer-markup-components';
import { ArrowMarkupInteractionHandler } from './interactions';
import { arrowheadPointsToPathPoints, arrowheadPointsToPolygonPoints, createLineAnchorStylePoints, parsePoint, } from './utils';
import { BoundingBox1d } from './viewer-markup-arrow-components';
export class ViewerMarkupArrow {
  constructor() {
    /**
     * The style of the starting anchor. This defaults to none.
     */
    this.startLineAnchorStyle = 'none';
    /**
     * The style of the ending anchor. This defaults to 'arrow-triangle.'
     */
    this.endLineAnchorStyle = 'arrow-triangle';
    /**
     * A mode that specifies how the markup component should behave. When
     * unset, the component will not respond to interactions with the handles.
     * When `edit`, the markup anchors are interactive and the user is able
     * to reposition them. When `create`, anytime the user clicks on the canvas,
     * a new markup will be performed.
     */
    this.mode = '';
    this.interactionHandler = new ArrowMarkupInteractionHandler(this.hostEl, this.interactionBegin, this.interactionEnd);
    this.handleWindowPointerDown = (event) => {
      if (isValidStartEvent(event)) {
        this.interactionHandler.startInteraction(event);
      }
    };
    this.handleTouchStart = (event) => {
      event.preventDefault();
    };
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.updateViewport();
    this.handleViewerChanged(this.viewer);
    this.updatePointsFromProps();
  }
  componentDidLoad() {
    this.updatePointsFromProps();
    const resize = new ResizeObserver(() => this.updateViewport());
    resize.observe(this.hostEl);
    if (this.mode === 'create') {
      window.addEventListener('pointerdown', this.handleWindowPointerDown);
    }
  }
  componentDidRender() {
    if (this.mode === '') {
      this.viewRendered.emit();
    }
  }
  disconnectedCallback() {
    this.dispose();
  }
  async dispose() {
    var _a;
    (_a = this.registeredInteraction) === null || _a === void 0 ? void 0 : _a.dispose();
    this.registeredInteraction = undefined;
    window.removeEventListener('pointerdown', this.handleWindowPointerDown);
  }
  /**
   * @ignore
   */
  async handleViewerChanged(newViewer) {
    var _a;
    (_a = this.registeredInteraction) === null || _a === void 0 ? void 0 : _a.dispose();
    this.registeredInteraction = undefined;
    if (newViewer != null) {
      this.registeredInteraction = await newViewer.registerInteractionHandler(this.interactionHandler);
    }
  }
  handleStartJsonChange() {
    this.updatePointsFromProps();
  }
  handleEndJsonChange() {
    this.updatePointsFromProps();
  }
  handleModeChange() {
    if (this.mode !== 'create') {
      window.removeEventListener('pointerdown', this.handleWindowPointerDown);
    }
  }
  updateViewport() {
    const rect = getMarkupBoundingClientRect(this.hostEl);
    this.elementBounds = rect;
  }
  updatePointsFromProps() {
    this.start = this.start || parsePoint(this.startJson);
    this.end = this.end || parsePoint(this.endJson);
  }
  renderLineAnchorStyle(anchorStyle, arrowheadPoints) {
    if (anchorStyle === 'arrow-triangle') {
      return (h("polygon", { id: "line-anchor-arrow-triangle", class: "head", points: arrowheadPointsToPolygonPoints(arrowheadPoints) }));
    }
    else if (anchorStyle === 'arrow-line') {
      return (h("path", { id: "line-anchor-arrow-line", class: "head", d: arrowheadPointsToPathPoints(arrowheadPoints) }));
    }
    else if (anchorStyle === 'hash') {
      return (h("line", { id: "line-anchor-hash", class: "head", x1: arrowheadPoints.hash.leftPoint.x, y1: arrowheadPoints.hash.leftPoint.y, x2: arrowheadPoints.hash.rightPoint.x, y2: arrowheadPoints.hash.rightPoint.y }));
    }
    else if (anchorStyle === 'dot') {
      return (h("circle", { id: "line-anchor-circle", class: "head", cx: arrowheadPoints.tip.x, cy: arrowheadPoints.tip.y, r: arrowheadPoints.radius }));
    }
    else {
      return h("div", null);
    }
  }
  render() {
    if (this.start != null && this.end != null && this.elementBounds != null) {
      const screenStart = translatePointToScreen(this.start, this.elementBounds);
      const screenEnd = translatePointToScreen(this.end, this.elementBounds);
      if (isValidPointData(screenStart, screenEnd)) {
        const arrowheadStartPoints = createLineAnchorStylePoints(screenEnd, screenStart);
        const arrowheadEndPoints = createLineAnchorStylePoints(screenStart, screenEnd);
        return (h(Host, null,
          h("svg", { class: "svg", onTouchStart: this.handleTouchStart },
            h("defs", null,
              h(SvgShadow, { id: "arrow-shadow" })),
            h("g", { filter: "url(#arrow-shadow)" },
              this.renderLineAnchorStyle(this.startLineAnchorStyle, arrowheadStartPoints),
              h("line", { id: "arrow-line", class: "line", x1: arrowheadEndPoints.tip.x, y1: arrowheadEndPoints.tip.y, x2: arrowheadStartPoints.tip.x, y2: arrowheadStartPoints.tip.y }),
              this.renderLineAnchorStyle(this.endLineAnchorStyle, arrowheadEndPoints)),
            this.mode === 'edit' && (h("g", null,
              h("line", { id: "bounding-box-1d-line", class: "bounds-line", x1: screenStart.x, y1: screenStart.y, x2: screenEnd.x, y2: screenEnd.y })))),
          this.mode === 'edit' && (h(BoundingBox1d, { start: screenStart, end: screenEnd, onStartAnchorPointerDown: (event) => this.interactionHandler.editAnchor('start', event), onCenterAnchorPointerDown: (event) => this.interactionHandler.editAnchor('center', event), onEndAnchorPointerDown: (event) => this.interactionHandler.editAnchor('end', event) })),
          this.mode === 'create' && (h("div", { class: "create-overlay", onTouchStart: this.handleTouchStart }))));
      }
      else {
        return h(Host, null);
      }
    }
    else {
      return (h(Host, null,
        h("div", { class: "create-overlay", onTouchStart: this.handleTouchStart })));
    }
  }
  static get is() { return "vertex-viewer-markup-arrow"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["viewer-markup-arrow.css"]
  }; }
  static get styleUrls() { return {
    "$": ["viewer-markup-arrow.css"]
  }; }
  static get properties() { return {
    "start": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Point.Point",
        "resolved": "Point | undefined",
        "references": {
          "Point": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The position of the starting anchor. Can either be an instance of a\n`Point` or a JSON string representation in the format of `[x, y]` or\n`{\"x\": 0, \"y\": 0}`.\n\nPoints are expected to be relative coordinates from `[-0.5, 0.5]`,\ne.g. `[0, 0]` corresponds to a point in the center of the viewport."
      }
    },
    "startJson": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The position of the starting anchor, as a JSON string. Can either be an\ninstance of a `Point` or a JSON string representation in the format of\n`[x, y]` or `{\"x\": 0, \"y\": 0}`.\n\nPoints are expected to be relative coordinates from `[-0.5, 0.5]`,\ne.g. `[0, 0]` corresponds to a point in the center of the viewport."
      },
      "attribute": "start",
      "reflect": false
    },
    "end": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Point.Point",
        "resolved": "Point | undefined",
        "references": {
          "Point": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The position of the ending anchor. Can either be an instance of a `Point`\nor a JSON string representation in the format of `[x, y]` or `{\"x\": 0,\n\"y\": 0}`.\n\nPoints are expected to be relative coordinates from `[-0.5, 0.5]`,\ne.g. `[0, 0]` corresponds to a point in the center of the viewport."
      }
    },
    "endJson": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The position of the ending anchor, as a JSON string. Can either be an\ninstance of a `Point` or a JSON string representation in the format of\n`[x, y]` or `{\"x\": 0, \"y\": 0}`.\n\nPoints are expected to be relative coordinates from `[-0.5, 0.5]`,\ne.g. `[0, 0]` corresponds to a point in the center of the viewport."
      },
      "attribute": "end",
      "reflect": false
    },
    "startLineAnchorStyle": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LineAnchorStyle",
        "resolved": "\"arrow-line\" | \"arrow-triangle\" | \"dot\" | \"hash\" | \"none\"",
        "references": {
          "LineAnchorStyle": {
            "location": "import",
            "path": "./utils"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The style of the starting anchor. This defaults to none."
      },
      "attribute": "start-line-anchor-style",
      "reflect": false,
      "defaultValue": "'none'"
    },
    "endLineAnchorStyle": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LineAnchorStyle",
        "resolved": "\"arrow-line\" | \"arrow-triangle\" | \"dot\" | \"hash\" | \"none\"",
        "references": {
          "LineAnchorStyle": {
            "location": "import",
            "path": "./utils"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The style of the ending anchor. This defaults to 'arrow-triangle.'"
      },
      "attribute": "end-line-anchor-style",
      "reflect": false,
      "defaultValue": "'arrow-triangle'"
    },
    "mode": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "ViewerMarkupArrowMode",
        "resolved": "\"\" | \"create\" | \"edit\"",
        "references": {
          "ViewerMarkupArrowMode": {
            "location": "local"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "A mode that specifies how the markup component should behave. When\nunset, the component will not respond to interactions with the handles.\nWhen `edit`, the markup anchors are interactive and the user is able\nto reposition them. When `create`, anytime the user clicks on the canvas,\na new markup will be performed."
      },
      "attribute": "mode",
      "reflect": true,
      "defaultValue": "''"
    },
    "viewer": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "HTMLVertexViewerElement",
        "resolved": "HTMLVertexViewerElement | undefined",
        "references": {
          "HTMLVertexViewerElement": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The viewer to connect to markups.\n\nThis property will automatically be set when a child of a\n`<vertex-viewer-markup>` or `<vertex-viewer>` element."
      }
    }
  }; }
  static get states() { return {
    "elementBounds": {}
  }; }
  static get events() { return [{
      "method": "interactionBegin",
      "name": "interactionBegin",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is dispatched anytime the user begins interacting with the\nmarkup."
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "interactionEnd",
      "name": "interactionEnd",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is dispatched when the user has finished interacting with the\nmarkup."
      },
      "complexType": {
        "original": "MarkupInteraction",
        "resolved": "MarkupInteraction",
        "references": {
          "MarkupInteraction": {
            "location": "import",
            "path": "../../lib/types/markup"
          }
        }
      }
    }, {
      "method": "viewRendered",
      "name": "viewRendered",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is dispatched when this markup element is in view\nmode (`this.mode === \"\"`), and it completes a rerender."
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "dispose": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "hostEl"; }
  static get watchers() { return [{
      "propName": "viewer",
      "methodName": "handleViewerChanged"
    }, {
      "propName": "start",
      "methodName": "handleStartJsonChange"
    }, {
      "propName": "end",
      "methodName": "handleEndJsonChange"
    }, {
      "propName": "mode",
      "methodName": "handleModeChange"
    }]; }
}
//# sourceMappingURL=viewer-markup-arrow.js.map