/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { Component, Element, Event, h, Host, Method, Prop, State, Watch, } from '@stencil/core';
import { getMarkupBoundingClientRect } from '../viewer-markup/dom';
import { isValidStartEvent, translatePointToScreen, translateRectToScreen, } from '../viewer-markup/markup-utils';
import { SvgShadow } from '../viewer-markup/viewer-markup-components';
import { parseBounds } from '../viewer-markup-circle/utils';
import { BoundingBox2d } from '../viewer-markup-circle/viewer-markup-circle-components';
import { FreeformMarkupInteractionHandler } from './interactions';
import { parsePoints } from './utils';
export class ViewerMarkupFreeform {
  constructor() {
    /**
     * A mode that specifies how the markup component should behave. When
     * unset, the component will not respond to interactions with the handles.
     * When `edit`, the markup anchors are interactive and the user is able
     * to reposition them. When `create`, anytime the user clicks on the canvas,
     * a new markup will be performed.
     */
    this.mode = '';
    this.screenPoints = [];
    this.interactionHandler = new FreeformMarkupInteractionHandler(this.hostEl, this.interactionBegin, this.interactionEnd);
    this.handleWindowPointerDown = (event) => {
      if (isValidStartEvent(event)) {
        this.interactionHandler.startInteraction(event);
      }
    };
    this.handleTouchStart = (event) => {
      event.preventDefault();
    };
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.updateViewport();
    this.handleViewerChanged(this.viewer);
    this.updatePointsFromProps();
  }
  componentDidLoad() {
    const resize = new ResizeObserver(() => this.updateViewport());
    resize.observe(this.hostEl);
    if (this.mode === 'create') {
      window.addEventListener('pointerdown', this.handleWindowPointerDown);
    }
  }
  componentDidRender() {
    if (this.mode === '') {
      this.viewRendered.emit();
    }
  }
  disconnectedCallback() {
    this.dispose();
  }
  async dispose() {
    var _a;
    (_a = this.registeredInteraction) === null || _a === void 0 ? void 0 : _a.dispose();
    this.registeredInteraction = undefined;
    window.removeEventListener('pointerdown', this.handleWindowPointerDown);
  }
  /**
   * @ignore
   */
  async handleViewerChanged(newViewer) {
    var _a;
    (_a = this.registeredInteraction) === null || _a === void 0 ? void 0 : _a.dispose();
    this.registeredInteraction = undefined;
    if (newViewer != null) {
      this.registeredInteraction = await newViewer.registerInteractionHandler(this.interactionHandler);
    }
  }
  handlePointsChange() {
    this.updatePointsFromProps();
  }
  handleBoundsChange() {
    this.updatePointsFromProps();
  }
  handleModeChange() {
    if (this.mode !== 'create') {
      window.removeEventListener('pointerdown', this.handleWindowPointerDown);
    }
  }
  updateViewport() {
    var _a;
    const rect = getMarkupBoundingClientRect(this.hostEl);
    this.elementBounds = rect;
    this.screenPoints = (_a = this.convertPointsToScreen()) !== null && _a !== void 0 ? _a : this.screenPoints;
  }
  updatePointsFromProps() {
    var _a, _b, _c;
    this.points = (_a = this.points) !== null && _a !== void 0 ? _a : parsePoints(this.pointsJson);
    this.screenPoints = (_b = this.convertPointsToScreen()) !== null && _b !== void 0 ? _b : [];
    this.bounds = (_c = this.bounds) !== null && _c !== void 0 ? _c : parseBounds(this.boundsJson);
  }
  render() {
    if (this.screenPoints.length > 0 && this.elementBounds != null) {
      return (h(Host, null,
        h("svg", { class: "svg", onTouchStart: this.handleTouchStart },
          h("defs", null,
            h(SvgShadow, { id: "freeform-markup-shadow" })),
          h("g", { filter: "url(#freeform-markup-shadow)" },
            h("path", { class: "path", d: this.screenPoints.reduce((d, pt) => `${d}L${pt.x},${pt.y}`, `M${this.screenPoints[0].x},${this.screenPoints[0].y}`), fill: "none" }))),
        this.mode === 'edit' && this.bounds != null && (h(BoundingBox2d, { bounds: translateRectToScreen(this.bounds, this.elementBounds), onTopLeftAnchorPointerDown: (e) => this.interactionHandler.editAnchor('top-left', e), onTopRightAnchorPointerDown: (e) => this.interactionHandler.editAnchor('top-right', e), onTopAnchorPointerDown: (e) => this.interactionHandler.editAnchor('top', e), onBottomLeftAnchorPointerDown: (e) => this.interactionHandler.editAnchor('bottom-left', e), onBottomRightAnchorPointerDown: (e) => this.interactionHandler.editAnchor('bottom-right', e), onBottomAnchorPointerDown: (e) => this.interactionHandler.editAnchor('bottom', e), onLeftAnchorPointerDown: (e) => this.interactionHandler.editAnchor('left', e), onRightAnchorPointerDown: (e) => this.interactionHandler.editAnchor('right', e), onCenterAnchorPointerDown: (e) => this.interactionHandler.editAnchor('center', e) })),
        this.mode === 'create' && (h("div", { class: "create-overlay", onTouchStart: this.handleTouchStart }))));
    }
    else {
      return (h(Host, null,
        h("div", { class: "create-overlay", onTouchStart: this.handleTouchStart })));
    }
  }
  convertPointsToScreen() {
    var _a;
    const elementBounds = this.elementBounds;
    if (elementBounds != null) {
      return (_a = this.points) === null || _a === void 0 ? void 0 : _a.map((pt) => translatePointToScreen(pt, elementBounds));
    }
  }
  static get is() { return "vertex-viewer-markup-freeform"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["viewer-markup-freeform.css"]
  }; }
  static get styleUrls() { return {
    "$": ["viewer-markup-freeform.css"]
  }; }
  static get properties() { return {
    "points": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Point.Point[]",
        "resolved": "Point[] | undefined",
        "references": {
          "Point": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The positions of the various points of this freeform markup. Can either be an array of\n`Point`s or a JSON string representation in the format of `[[x1, y1], [x2, y2]]` or\n`[{\"x\": 0, \"y\": 0}, {\"x\": 0, \"y\": 0}]`.\n\nPoints are expected to be relative coordinates from `[-0.5, 0.5]`,\ne.g. `[0, 0]` corresponds to a point in the center of the viewport."
      }
    },
    "pointsJson": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The positions of the various points of this freeform markup. Can either be an array of\n`Point`s or a JSON string representation in the format of `[[x1, y1], [x2, y2]]` or\n`[{\"x\": 0, \"y\": 0}, {\"x\": 0, \"y\": 0}]`.\n\nPoints are expected to be relative coordinates from `[-0.5, 0.5]`,\ne.g. `[0, 0]` corresponds to a point in the center of the viewport."
      },
      "attribute": "points",
      "reflect": false
    },
    "bounds": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Rectangle.Rectangle",
        "resolved": "Rectangle | undefined",
        "references": {
          "Rectangle": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The bounds of the freeform. Can either be an instance of a `Rectangle` or\na JSON string representation in the format of `[x, y, width, height]` or\n`{\"x\": 0, \"y\": 0, \"width\": 10, \"height\": 10}`.\n\nBounds are expected to have relative coordinates, with `[x, y]` from `[-0.5, 0.5]`\nand `[width, height]` from `[0, 1]`, e.g. `[0, 0, 0.25, 0.25]`corresponds to a freeform\nwith a diameter of one fourth the viewport's smallest size in the center of the viewport."
      }
    },
    "boundsJson": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The bounds of the freeform. Can either be an instance of a `Rectangle` or\na JSON string representation in the format of `[x, y, width, height]` or\n`{\"x\": 0, \"y\": 0, \"width\": 0.1, \"height\": 0.1}`.\n\nBounds are expected to have relative coordinates, with `[x, y]` from `[-0.5, 0.5]`\nand `[width, height]` from `[0, 1]`, e.g. `[0, 0, 0.25, 0.25]`corresponds to a freeform\nwith a diameter of one fourth the viewport's smallest size in the center of the viewport."
      },
      "attribute": "bounds",
      "reflect": false
    },
    "mode": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "ViewerMarkupFreeformMode",
        "resolved": "\"\" | \"create\" | \"edit\"",
        "references": {
          "ViewerMarkupFreeformMode": {
            "location": "local"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "A mode that specifies how the markup component should behave. When\nunset, the component will not respond to interactions with the handles.\nWhen `edit`, the markup anchors are interactive and the user is able\nto reposition them. When `create`, anytime the user clicks on the canvas,\na new markup will be performed."
      },
      "attribute": "mode",
      "reflect": true,
      "defaultValue": "''"
    },
    "viewer": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "HTMLVertexViewerElement",
        "resolved": "HTMLVertexViewerElement | undefined",
        "references": {
          "HTMLVertexViewerElement": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The viewer to connect to markups.\n\nThis property will automatically be set when a child of a\n`<vertex-viewer-markup>` or `<vertex-viewer>` element."
      }
    }
  }; }
  static get states() { return {
    "elementBounds": {},
    "screenPoints": {}
  }; }
  static get events() { return [{
      "method": "interactionBegin",
      "name": "interactionBegin",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is dispatched anytime the user begins interacting with the\nmarkup."
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "interactionEnd",
      "name": "interactionEnd",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is dispatched when the user has finished interacting with the\nmarkup."
      },
      "complexType": {
        "original": "MarkupInteraction",
        "resolved": "MarkupInteraction",
        "references": {
          "MarkupInteraction": {
            "location": "import",
            "path": "../../lib/types/markup"
          }
        }
      }
    }, {
      "method": "viewRendered",
      "name": "viewRendered",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is dispatched when this markup element is in view\nmode (`this.mode === \"\"`), and it completes a rerender."
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "dispose": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "hostEl"; }
  static get watchers() { return [{
      "propName": "viewer",
      "methodName": "handleViewerChanged"
    }, {
      "propName": "points",
      "methodName": "handlePointsChange"
    }, {
      "propName": "bounds",
      "methodName": "handleBoundsChange"
    }, {
      "propName": "mode",
      "methodName": "handleModeChange"
    }]; }
}
//# sourceMappingURL=viewer-markup-freeform.js.map