/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { Component, Element, Event, h, Host, Method, Prop, State, Watch, } from '@stencil/core';
import { stampTemplateWithId } from '../../lib/templates';
import { ArrowMarkup, CircleMarkup, FreeformMarkup, } from '../../lib/types/markup';
import { isVertexViewerArrowMarkup, } from '../viewer-markup-arrow/utils';
import { isVertexViewerCircleMarkup } from '../viewer-markup-circle/utils';
import { isVertexViewerFreeformMarkup } from '../viewer-markup-freeform/utils';
export class ViewerMarkupTool {
  constructor() {
    /**
     * The type of markup.
     *
     * This property will automatically be set when a child of a
     * `<vertex-viewer-markup>` element.
     */
    this.tool = 'arrow';
    /**
     * Disables markups.
     *
     * This property will automatically be set when a child of a
     * `<vertex-viewer-markup>` element.
     */
    this.disabled = false;
    /**
     * The style of the starting anchor. This defaults to none.
     */
    this.startLineAnchorStyle = 'none';
    /**
     * The style of the ending anchor. This defaults to 'arrow-triangle.'
     */
    this.endLineAnchorStyle = 'arrow-triangle';
    /**
     * Captures internal state that you want to preserve across dev refreshes, but
     * don't want to trigger a render when the state changes.
     */
    this.stateMap = {};
    this.handleMarkupInteractionBegin = () => {
      this.markupBegin.emit();
    };
    this.handleMarkupInteractionEnd = () => {
      const { markupElement } = this.stateMap;
      if (isVertexViewerFreeformMarkup(markupElement)) {
        const { points, bounds } = markupElement;
        if (points != null && points.length > 0 && bounds != null) {
          this.markupEnd.emit(new FreeformMarkup({ points, bounds }));
        }
      }
      else if (isVertexViewerCircleMarkup(markupElement)) {
        const { bounds } = markupElement;
        if (bounds != null) {
          this.markupEnd.emit(new CircleMarkup({ bounds }));
        }
      }
      else if (isVertexViewerArrowMarkup(markupElement)) {
        const { start, end } = markupElement;
        if (start != null && end != null) {
          this.markupEnd.emit(new ArrowMarkup({ start, end }));
        }
      }
    };
  }
  async handleViewerChanged() {
    if (this.stateMap.markupElement != null) {
      this.stateMap.markupElement.viewer = this.viewer;
    }
  }
  /**
   * @ignore
   */
  handleToolChanged() {
    this.updateMarkupElement();
  }
  /**
   * @ignore
   */
  handleArrowTemplateIdChanged() {
    this.updateMarkupElement();
  }
  /**
   * @ignore
   */
  handleCircleTemplateIdChanged() {
    this.updateMarkupElement();
  }
  /**
   * @ignore
   */
  handleFreeformTemplateIdChanged() {
    this.updateMarkupElement();
  }
  /**
   * @ignore
   */
  handleDisabledChanged() {
    this.updateMarkupElement();
  }
  /**
   * @ignore
   */
  handleStartLineAnchorStyleChanged() {
    this.updateMarkupElement();
  }
  /**
   * @ignore
   */
  handleEndLineAnchorStyleChanged() {
    this.updateMarkupElement();
  }
  /**
   * @ignore
   */
  componentDidLoad() {
    this.updateMarkupElement();
  }
  /**
   * Resets the state of the internally managed markup element
   * to allow for creating a new markup. This state is automatically
   * managed when this element is placed as a child of a
   * `<vertex-viewer-markup>` element.
   */
  async reset() {
    const { markupElement } = this.stateMap;
    if (isVertexViewerFreeformMarkup(markupElement)) {
      markupElement.points = undefined;
      markupElement.bounds = undefined;
      markupElement.mode = 'create';
    }
    else if (isVertexViewerCircleMarkup(markupElement)) {
      markupElement.bounds = undefined;
      markupElement.mode = 'create';
    }
    else if (isVertexViewerArrowMarkup(markupElement)) {
      markupElement.start = undefined;
      markupElement.end = undefined;
      markupElement.mode = 'create';
    }
  }
  /**
   * @ignore
   */
  render() {
    if (!this.disabled) {
      if (this.tool === 'arrow' ||
        this.tool === 'circle' ||
        this.tool === 'freeform') {
        return (h(Host, null,
          h("slot", null)));
      }
      else {
        return h(Host, null, `Unsupported tool type '${this.tool}'.`);
      }
    }
    else {
      return h(Host, null);
    }
  }
  createArrowMarkupElement() {
    if (this.arrowTemplateId != null) {
      const element = stampTemplateWithId(window.document.body, this.arrowTemplateId, isVertexViewerArrowMarkup, () => console.warn(`Arrow template with ID ${this.arrowTemplateId} not found. Using default arrow element.`), () => console.warn(`Arrow template does not contain a vertex-viewer-markup-arrow. Using default arrow element.`));
      if (element != null) {
        return element;
      }
    }
    return document.createElement('vertex-viewer-markup-arrow');
  }
  createCircleMarkupElement() {
    if (this.circleTemplateId != null) {
      const element = stampTemplateWithId(window.document.body, this.circleTemplateId, isVertexViewerCircleMarkup, () => console.warn(`Circle template with ID ${this.circleTemplateId} not found. Using default circle element.`), () => console.warn(`Circle template does not contain a vertex-viewer-markup-circle. Using default circle element.`));
      if (element != null) {
        return element;
      }
    }
    return document.createElement('vertex-viewer-markup-circle');
  }
  createFreeformMarkupElement() {
    if (this.freeformTemplateId != null) {
      const element = stampTemplateWithId(window.document.body, this.freeformTemplateId, isVertexViewerFreeformMarkup, () => console.warn(`Freeform template with ID ${this.freeformTemplateId} not found. Using default freeform element.`), () => console.warn(`Freeform template does not contain a vertex-viewer-markup-freeform. Using default freeform element.`));
      if (element != null) {
        return element;
      }
    }
    return document.createElement('vertex-viewer-markup-freeform');
  }
  createNewMarkupElement() {
    switch (this.tool) {
      case 'arrow':
        return this.createArrowMarkupElement();
      case 'circle':
        return this.createCircleMarkupElement();
      case 'freeform':
        return this.createFreeformMarkupElement();
    }
  }
  updateMarkupElement() {
    const { markupElement } = this.stateMap;
    if (markupElement != null) {
      markupElement.remove();
      markupElement.dispose();
      markupElement.viewer = undefined;
      markupElement.removeEventListener('interactionBegin', this.handleMarkupInteractionBegin);
      markupElement.removeEventListener('interactionEnd', this.handleMarkupInteractionEnd);
    }
    if (!this.disabled) {
      const newMarkupElement = this.createNewMarkupElement();
      if (this.tool === 'arrow') {
        newMarkupElement.startLineAnchorStyle = this.startLineAnchorStyle;
        newMarkupElement.endLineAnchorStyle = this.endLineAnchorStyle;
      }
      newMarkupElement.mode = 'create';
      newMarkupElement.viewer = this.viewer;
      newMarkupElement.addEventListener('interactionBegin', this.handleMarkupInteractionBegin);
      newMarkupElement.addEventListener('interactionEnd', this.handleMarkupInteractionEnd);
      this.stateMap.markupElement = newMarkupElement;
      this.hostEl.append(newMarkupElement);
    }
  }
  static get is() { return "vertex-viewer-markup-tool"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["viewer-markup-tool.css"]
  }; }
  static get styleUrls() { return {
    "$": ["viewer-markup-tool.css"]
  }; }
  static get properties() { return {
    "arrowTemplateId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "An HTML template that describes the HTML to use for new arrow\nmarkup. It's expected that the template contains a\n`<vertex-viewer-markup-arrow>`."
      },
      "attribute": "arrow-template-id",
      "reflect": false
    },
    "circleTemplateId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "An HTML template that describes the HTML to use for new circle\nmarkup. It's expected that the template contains a\n`<vertex-viewer-markup-circle>`."
      },
      "attribute": "circle-template-id",
      "reflect": false
    },
    "freeformTemplateId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "An HTML template that describes the HTML to use for new freeform\nmarkup. It's expected that the template contains a\n`<vertex-viewer-markup-freeform>`."
      },
      "attribute": "freeform-template-id",
      "reflect": false
    },
    "tool": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "ViewerMarkupToolType",
        "resolved": "\"arrow\" | \"circle\" | \"freeform\"",
        "references": {
          "ViewerMarkupToolType": {
            "location": "local"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The type of markup.\n\nThis property will automatically be set when a child of a\n`<vertex-viewer-markup>` element."
      },
      "attribute": "tool",
      "reflect": false,
      "defaultValue": "'arrow'"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disables markups.\n\nThis property will automatically be set when a child of a\n`<vertex-viewer-markup>` element."
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "viewer": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "HTMLVertexViewerElement",
        "resolved": "HTMLVertexViewerElement | undefined",
        "references": {
          "HTMLVertexViewerElement": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The viewer to connect to markup.\n\nThis property will automatically be set when a child of a\n`<vertex-viewer-markup>` or `<vertex-viewer>` element."
      }
    },
    "startLineAnchorStyle": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LineAnchorStyle",
        "resolved": "\"arrow-line\" | \"arrow-triangle\" | \"dot\" | \"hash\" | \"none\"",
        "references": {
          "LineAnchorStyle": {
            "location": "import",
            "path": "../viewer-markup-arrow/utils"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The style of the starting anchor. This defaults to none."
      },
      "attribute": "start-line-anchor-style",
      "reflect": false,
      "defaultValue": "'none'"
    },
    "endLineAnchorStyle": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LineAnchorStyle",
        "resolved": "\"arrow-line\" | \"arrow-triangle\" | \"dot\" | \"hash\" | \"none\"",
        "references": {
          "LineAnchorStyle": {
            "location": "import",
            "path": "../viewer-markup-arrow/utils"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The style of the ending anchor. This defaults to 'arrow-triangle.'"
      },
      "attribute": "end-line-anchor-style",
      "reflect": false,
      "defaultValue": "'arrow-triangle'"
    }
  }; }
  static get states() { return {
    "stateMap": {}
  }; }
  static get events() { return [{
      "method": "markupBegin",
      "name": "markupBegin",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is dispatched when a user begins a new markup."
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "markupEnd",
      "name": "markupEnd",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is dispatched when a user has finished their markup."
      },
      "complexType": {
        "original": "Markup",
        "resolved": "ArrowMarkup | CircleMarkup | FreeformMarkup",
        "references": {
          "Markup": {
            "location": "import",
            "path": "../../lib/types/markup"
          }
        }
      }
    }]; }
  static get methods() { return {
    "reset": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Resets the state of the internally managed markup element\nto allow for creating a new markup. This state is automatically\nmanaged when this element is placed as a child of a\n`<vertex-viewer-markup>` element.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "hostEl"; }
  static get watchers() { return [{
      "propName": "viewer",
      "methodName": "handleViewerChanged"
    }, {
      "propName": "tool",
      "methodName": "handleToolChanged"
    }, {
      "propName": "arrowTemplateId",
      "methodName": "handleArrowTemplateIdChanged"
    }, {
      "propName": "circleTemplateId",
      "methodName": "handleCircleTemplateIdChanged"
    }, {
      "propName": "freeformTemplateId",
      "methodName": "handleFreeformTemplateIdChanged"
    }, {
      "propName": "disabled",
      "methodName": "handleDisabledChanged"
    }, {
      "propName": "startLineAnchorStyle",
      "methodName": "handleStartLineAnchorStyleChanged"
    }, {
      "propName": "endLineAnchorStyle",
      "methodName": "handleEndLineAnchorStyleChanged"
    }]; }
}
//# sourceMappingURL=viewer-markup-tool.js.map