/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { Component, Element, Event, h, Host, Listen, Method, Prop, State, Watch, } from '@stencil/core';
import { Point } from '@vertexvis/geometry';
import { stampTemplateWithId } from '../../lib/templates';
import { ArrowMarkup, CircleMarkup, FreeformMarkup, } from '../../lib/types/markup';
import { isVertexViewerArrowMarkup, } from '../viewer-markup-arrow/utils';
import { isVertexViewerCircleMarkup } from '../viewer-markup-circle/utils';
import { isVertexViewerFreeformMarkup } from '../viewer-markup-freeform/utils';
export class ViewerMarkup {
  constructor() {
    /**
     * The type of markup to perform.
     */
    this.tool = 'arrow';
    /**
     * If `true`, disables adding or editing of markup through user
     * interaction.
     */
    this.disabled = false;
    /**
     * Indicates if new markup should be automatically selected.
     */
    this.selectNew = false;
    /**
     * The style of the starting anchor. This defaults to none.
     */
    this.startLineAnchorStyle = 'none';
    /**
     * The style of the ending anchor. This defaults to 'arrow-triangle.'
     */
    this.endLineAnchorStyle = 'arrow-triangle';
    this.handlePointerMove = (event) => {
      if (this.pointerDownPosition != null &&
        Point.distance(this.pointerDownPosition, Point.create(event.clientX, event.clientY)) > 2) {
        this.toSelectMarkupId = undefined;
      }
    };
    this.handlePointerUp = async (event) => {
      if (!this.disabled) {
        const el = event.target;
        const markups = await this.getMarkupElements();
        const markup = markups.find((m) => m === el);
        if ((markup === null || markup === void 0 ? void 0 : markup.id) != null &&
          markup.id !== '' &&
          markup.id === this.toSelectMarkupId) {
          this.selectedMarkupId = el.id;
        }
        else if (this.pointerDownPosition != null &&
          Point.distance(this.pointerDownPosition, Point.create(event.clientX, event.clientY)) <= 2) {
          this.selectedMarkupId = undefined;
        }
        this.toSelectMarkupId = undefined;
        this.pointerDownPosition = undefined;
      }
      window.removeEventListener('pointerup', this.handlePointerUp);
      window.removeEventListener('pointermove', this.handlePointerMove);
    };
  }
  disconnectedCallback() {
    window.removeEventListener('pointerup', this.handlePointerUp);
    window.removeEventListener('pointermove', this.handlePointerMove);
  }
  /**
   * Adds a new markup as a child to this component. A new markup
   * component will be created from the template specified by
   * `arrow-template-id`, `circle-template-id`, or if undefined
   * a default element will be created.
   *
   * @param markup The markup to add.
   * @returns The markup element that was created.
   * @see {@link ViewerMarkups.arrowTemplateId}
   * @see {@link ViewerMarkups.circleTemplateId}
   */
  async addMarkup(markup) {
    if (markup instanceof ArrowMarkup) {
      const { start, end, id } = markup;
      const el = this.createArrowMarkupElement();
      el.id = id;
      el.start = start;
      el.end = end;
      el.startLineAnchorStyle = this.startLineAnchorStyle;
      el.endLineAnchorStyle = this.endLineAnchorStyle;
      return this.appendMarkupElement(el);
    }
    else if (markup instanceof CircleMarkup) {
      const { bounds, id } = markup;
      const el = this.createCircleMarkupElement();
      el.id = id;
      el.bounds = bounds;
      return this.appendMarkupElement(el);
    }
    else if (markup instanceof FreeformMarkup) {
      const { bounds, points, id } = markup;
      const el = this.createFreeformMarkupElement();
      el.id = id;
      el.points = points;
      el.bounds = bounds;
      return this.appendMarkupElement(el);
    }
    else {
      throw new Error(`Cannot add markup. Unknown type '${markup}'.`);
    }
  }
  /**
   * Removes a markup with the given ID, and returns the HTML element
   * associated to the markup. Returns `undefined` if no markup is
   * found.
   *
   * @param id The ID of the markup to remove.
   * @returns The markup element, or undefined.
   */
  async removeMarkup(id) {
    const markup = await this.getMarkupElement(id);
    if (markup != null) {
      markup.remove();
      this.markupRemoved.emit(markup);
    }
    return markup;
  }
  /**
   * Returns a list of markup elements that are children of this component.
   *
   * @returns A list of all markups.
   * @see {@link ViewerMarkup.getMarkupElement}
   */
  async getMarkupElements() {
    return Array.from(this.hostEl.children).filter((e) => isVertexViewerArrowMarkup(e) ||
      isVertexViewerCircleMarkup(e) ||
      isVertexViewerFreeformMarkup(e));
  }
  /**
   * Returns the markup element associated to the given ID.
   *
   * @param id The ID of the markup element to return.
   * @returns A markup element, or `undefined`.
   * @see {@link ViewerMarkup.getMarkupElements}
   */
  async getMarkupElement(id) {
    const markup = await this.getMarkupElements();
    return markup.find((el) => el.id === id);
  }
  /**
   * @ignore
   */
  async handleSelectedMarkupIdChanged() {
    const markup = await this.getMarkupElements();
    markup.forEach((m) => {
      if (m.id === this.selectedMarkupId) {
        m.mode = 'edit';
        this.markupSelectionChanged.emit(m);
      }
      else {
        m.mode = '';
      }
    });
    if (this.selectedMarkupId == null) {
      this.markupSelectionChanged.emit(undefined);
    }
  }
  /**
   * @ignore
   */
  handleToolChanged() {
    this.updatePropsOnMarkupTool();
  }
  /**
   * @ignore
   */
  handleArrowTemplateIdChanged() {
    this.updatePropsOnMarkupTool();
  }
  /**
   * @ignore
   */
  handleCircleTemplateIdChanged() {
    this.updatePropsOnMarkupTool();
  }
  /**
   * @ignore
   */
  handleFreeformTemplateIdChanged() {
    this.updatePropsOnMarkupTool();
  }
  /**
   * @ignore
   */
  async handleViewerChanged(newViewer) {
    this.updatePropsOnMarkupTool();
    this.updatePropsOnMarkups();
  }
  /**
   * @ignore
   */
  handleDisabledChanged() {
    this.updatePropsOnMarkupTool();
  }
  /**
   * @ignore
   */
  async handleStartLineAnchorStyleChanged() {
    this.updatePropsOnMarkupTool();
    // If a piece of arrow markup is currently selected, then update its anchor style
    const selectedArrow = await this.getSelectedArrowMarkup();
    if (selectedArrow != null) {
      selectedArrow.startLineAnchorStyle = this.startLineAnchorStyle;
      this.markupChanged.emit(selectedArrow);
    }
  }
  /**
   * @ignore
   */
  async handleEndLineAnchorStyleChanged() {
    this.updatePropsOnMarkupTool();
    // If a piece of arrow markup is currently selected, then update its anchor style
    const selectedArrow = await this.getSelectedArrowMarkup();
    if (selectedArrow != null) {
      selectedArrow.endLineAnchorStyle = this.endLineAnchorStyle;
      this.markupChanged.emit(selectedArrow);
    }
  }
  /**
   * @ignore
   */
  async handleMarkupEnd(event) {
    var _a;
    const e = event;
    const el = await this.addMarkup(e.detail);
    await new Promise((resolve) => {
      const handleRender = () => {
        el.removeEventListener('viewRendered', handleRender);
        resolve();
      };
      el.addEventListener('viewRendered', handleRender);
    });
    if (this.selectNew) {
      this.selectedMarkupId = e.detail.id;
    }
    (_a = this.getMarkupTool()) === null || _a === void 0 ? void 0 : _a.reset();
  }
  /**
   * @ignore
   */
  async handleMarkupUpdated(event) {
    const e = event;
    // Do not emit the markupChanged event if the markup was just created
    if (!e.detail.newlyCreatedMarkup) {
      this.markupChanged.emit(e.detail.markup);
    }
  }
  /**
   * @ignore
   */
  async handleMarkupPointerDown(event) {
    if (!this.disabled) {
      const el = event.target;
      const markups = await this.getMarkupElements();
      const markup = markups.find((m) => m === el);
      if ((markup === null || markup === void 0 ? void 0 : markup.id) != null && (markup === null || markup === void 0 ? void 0 : markup.id) !== '') {
        this.toSelectMarkupId = el.id;
      }
      this.pointerDownPosition = Point.create(event.clientX, event.clientY);
      window.addEventListener('pointermove', this.handlePointerMove);
      window.addEventListener('pointerup', this.handlePointerUp);
    }
  }
  /**
   * @ignore
   */
  componentDidLoad() {
    this.updatePropsOnMarkupTool();
  }
  /**
   * @ignore
   */
  render() {
    return (h(Host, null,
      h("slot", null)));
  }
  appendMarkupElement(el) {
    this.updatePropsOnMarkup(el);
    this.hostEl.appendChild(el);
    this.markupAdded.emit(el);
    return el;
  }
  createArrowMarkupElement() {
    if (this.arrowTemplateId != null) {
      const element = stampTemplateWithId(window.document.body, this.arrowTemplateId, isVertexViewerArrowMarkup, () => console.warn(`Arrow template with ID ${this.arrowTemplateId} not found. Using default arrow element.`), () => console.warn(`Arrow template does not contain a vertex-viewer-markup-arrow. Using default arrow element.`));
      if (element != null) {
        return element;
      }
    }
    return document.createElement('vertex-viewer-markup-arrow');
  }
  createCircleMarkupElement() {
    if (this.circleTemplateId != null) {
      const element = stampTemplateWithId(window.document.body, this.circleTemplateId, isVertexViewerCircleMarkup, () => console.warn(`Circle template with ID ${this.circleTemplateId} not found. Using default circle element.`), () => console.warn(`Circle template does not contain a vertex-viewer-markup-circle. Using default circle element.`));
      if (element != null) {
        return element;
      }
    }
    return document.createElement('vertex-viewer-markup-circle');
  }
  createFreeformMarkupElement() {
    if (this.freeformTemplateId != null) {
      const element = stampTemplateWithId(window.document.body, this.freeformTemplateId, isVertexViewerFreeformMarkup, () => console.warn(`Freeform template with ID ${this.freeformTemplateId} not found. Using default freeform element.`), () => console.warn(`Freeform template does not contain a vertex-viewer-markup-freeform. Using default freeform element.`));
      if (element != null) {
        return element;
      }
    }
    return document.createElement('vertex-viewer-markup-freeform');
  }
  async updatePropsOnMarkups() {
    const markup = await this.getMarkupElements();
    markup.forEach((m) => this.updatePropsOnMarkup(m));
  }
  updatePropsOnMarkup(element) {
    element.viewer = this.viewer;
    element.classList.add('viewer-markup__markup');
  }
  updatePropsOnMarkupTool() {
    const tool = this.getMarkupTool();
    if (tool != null) {
      tool.disabled = this.disabled;
      tool.arrowTemplateId = this.arrowTemplateId;
      tool.circleTemplateId = this.circleTemplateId;
      tool.freeformTemplateId = this.freeformTemplateId;
      tool.tool = this.tool;
      tool.viewer = this.viewer;
      tool.startLineAnchorStyle = this.startLineAnchorStyle;
      tool.endLineAnchorStyle = this.endLineAnchorStyle;
    }
  }
  getMarkupTool() {
    return this.hostEl.querySelector('vertex-viewer-markup-tool');
  }
  async getSelectedArrowMarkup() {
    if (this.selectedMarkupId != null) {
      const selected = await this.getMarkupElement(this.selectedMarkupId);
      return isVertexViewerArrowMarkup(selected) ? selected : undefined;
    }
  }
  static get is() { return "vertex-viewer-markup"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["viewer-markup.css"]
  }; }
  static get styleUrls() { return {
    "$": ["viewer-markup.css"]
  }; }
  static get properties() { return {
    "arrowTemplateId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "An HTML template that describes the HTML to use for new arrow\nmarkup. It's expected that the template contains a\n`<vertex-viewer-markup-arrow>`."
      },
      "attribute": "arrow-template-id",
      "reflect": false
    },
    "circleTemplateId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "An HTML template that describes the HTML to use for new circle\nmarkup. It's expected that the template contains a\n`<vertex-viewer-markup-circle>`."
      },
      "attribute": "circle-template-id",
      "reflect": false
    },
    "freeformTemplateId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "An HTML template that describes the HTML to use for new freeform\nmarkup. It's expected that the template contains a\n`<vertex-viewer-markup-freeform>`."
      },
      "attribute": "freeform-template-id",
      "reflect": false
    },
    "tool": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "ViewerMarkupToolType",
        "resolved": "\"arrow\" | \"circle\" | \"freeform\"",
        "references": {
          "ViewerMarkupToolType": {
            "location": "import",
            "path": "../viewer-markup-tool/viewer-markup-tool"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The type of markup to perform."
      },
      "attribute": "tool",
      "reflect": false,
      "defaultValue": "'arrow'"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, disables adding or editing of markup through user\ninteraction."
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "viewer": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "HTMLVertexViewerElement",
        "resolved": "HTMLVertexViewerElement | undefined",
        "references": {
          "HTMLVertexViewerElement": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The viewer to connect to markup. If nested within a <vertex-viewer>,\nthis property will be populated automatically."
      }
    },
    "selectedMarkupId": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The ID of the markup that is selected."
      },
      "attribute": "selected-markup-id",
      "reflect": false
    },
    "selectNew": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Indicates if new markup should be automatically selected."
      },
      "attribute": "select-new",
      "reflect": false,
      "defaultValue": "false"
    },
    "startLineAnchorStyle": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LineAnchorStyle",
        "resolved": "\"arrow-line\" | \"arrow-triangle\" | \"dot\" | \"hash\" | \"none\"",
        "references": {
          "LineAnchorStyle": {
            "location": "import",
            "path": "../viewer-markup-arrow/utils"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The style of the starting anchor. This defaults to none."
      },
      "attribute": "start-line-anchor-style",
      "reflect": false,
      "defaultValue": "'none'"
    },
    "endLineAnchorStyle": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LineAnchorStyle",
        "resolved": "\"arrow-line\" | \"arrow-triangle\" | \"dot\" | \"hash\" | \"none\"",
        "references": {
          "LineAnchorStyle": {
            "location": "import",
            "path": "../viewer-markup-arrow/utils"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The style of the ending anchor. This defaults to 'arrow-triangle.'"
      },
      "attribute": "end-line-anchor-style",
      "reflect": false,
      "defaultValue": "'arrow-triangle'"
    }
  }; }
  static get states() { return {
    "toSelectMarkupId": {},
    "pointerDownPosition": {}
  }; }
  static get events() { return [{
      "method": "markupAdded",
      "name": "markupAdded",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Dispatched when a new markup is added, either through user interaction\nor programmatically."
      },
      "complexType": {
        "original": "| HTMLVertexViewerMarkupArrowElement\n    | HTMLVertexViewerMarkupCircleElement\n    | HTMLVertexViewerMarkupFreeformElement",
        "resolved": "HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement",
        "references": {
          "HTMLVertexViewerMarkupArrowElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupCircleElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupFreeformElement": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "markupChanged",
      "name": "markupChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Dispatched when an existing piece of markup is changed, either through user interaction\nor programmatically."
      },
      "complexType": {
        "original": "| HTMLVertexViewerMarkupArrowElement\n    | HTMLVertexViewerMarkupCircleElement\n    | HTMLVertexViewerMarkupFreeformElement",
        "resolved": "HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement",
        "references": {
          "HTMLVertexViewerMarkupArrowElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupCircleElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupFreeformElement": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "markupRemoved",
      "name": "markupRemoved",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Dispatched when a markup is removed, either through user\ninteraction or programmatically."
      },
      "complexType": {
        "original": "| HTMLVertexViewerMarkupArrowElement\n    | HTMLVertexViewerMarkupCircleElement\n    | HTMLVertexViewerMarkupFreeformElement",
        "resolved": "HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement",
        "references": {
          "HTMLVertexViewerMarkupArrowElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupCircleElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupFreeformElement": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "markupSelectionChanged",
      "name": "markupSelectionChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Dispatched when markup selection changes. Will either be the\nselected element or `undefined` indicating that selection\nwas cleared."
      },
      "complexType": {
        "original": "| HTMLVertexViewerMarkupArrowElement\n    | HTMLVertexViewerMarkupCircleElement\n    | HTMLVertexViewerMarkupFreeformElement\n    | undefined",
        "resolved": "HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement | undefined",
        "references": {
          "HTMLVertexViewerMarkupArrowElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupCircleElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupFreeformElement": {
            "location": "global"
          }
        }
      }
    }]; }
  static get methods() { return {
    "addMarkup": {
      "complexType": {
        "signature": "(markup: Markup) => Promise<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "markup The markup to add."
              }],
            "text": "The markup to add."
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupArrowElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupCircleElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupFreeformElement": {
            "location": "global"
          },
          "Markup": {
            "location": "import",
            "path": "../../lib/types/markup"
          }
        },
        "return": "Promise<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement>"
      },
      "docs": {
        "text": "Adds a new markup as a child to this component. A new markup\ncomponent will be created from the template specified by\n`arrow-template-id`, `circle-template-id`, or if undefined\na default element will be created.",
        "tags": [{
            "name": "param",
            "text": "markup The markup to add."
          }, {
            "name": "returns",
            "text": "The markup element that was created."
          }, {
            "name": "see",
            "text": "{@link ViewerMarkups.arrowTemplateId }"
          }, {
            "name": "see",
            "text": "{@link ViewerMarkups.circleTemplateId }"
          }]
      }
    },
    "removeMarkup": {
      "complexType": {
        "signature": "(id: string) => Promise<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement | undefined>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "id The ID of the markup to remove."
              }],
            "text": "The ID of the markup to remove."
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupArrowElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupCircleElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupFreeformElement": {
            "location": "global"
          }
        },
        "return": "Promise<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement | undefined>"
      },
      "docs": {
        "text": "Removes a markup with the given ID, and returns the HTML element\nassociated to the markup. Returns `undefined` if no markup is\nfound.",
        "tags": [{
            "name": "param",
            "text": "id The ID of the markup to remove."
          }, {
            "name": "returns",
            "text": "The markup element, or undefined."
          }]
      }
    },
    "getMarkupElements": {
      "complexType": {
        "signature": "() => Promise<Array<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement>>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupArrowElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupCircleElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupFreeformElement": {
            "location": "global"
          },
          "Array": {
            "location": "global"
          }
        },
        "return": "Promise<(HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement)[]>"
      },
      "docs": {
        "text": "Returns a list of markup elements that are children of this component.",
        "tags": [{
            "name": "returns",
            "text": "A list of all markups."
          }, {
            "name": "see",
            "text": "{@link ViewerMarkup.getMarkupElement}"
          }]
      }
    },
    "getMarkupElement": {
      "complexType": {
        "signature": "(id: string) => Promise<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement | undefined>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "id The ID of the markup element to return."
              }],
            "text": "The ID of the markup element to return."
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupArrowElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupCircleElement": {
            "location": "global"
          },
          "HTMLVertexViewerMarkupFreeformElement": {
            "location": "global"
          }
        },
        "return": "Promise<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement | undefined>"
      },
      "docs": {
        "text": "Returns the markup element associated to the given ID.",
        "tags": [{
            "name": "param",
            "text": "id The ID of the markup element to return."
          }, {
            "name": "returns",
            "text": "A markup element, or `undefined`."
          }, {
            "name": "see",
            "text": "{@link ViewerMarkup.getMarkupElements}"
          }]
      }
    }
  }; }
  static get elementRef() { return "hostEl"; }
  static get watchers() { return [{
      "propName": "selectedMarkupId",
      "methodName": "handleSelectedMarkupIdChanged"
    }, {
      "propName": "tool",
      "methodName": "handleToolChanged"
    }, {
      "propName": "arrowTemplateId",
      "methodName": "handleArrowTemplateIdChanged"
    }, {
      "propName": "circleTemplateId",
      "methodName": "handleCircleTemplateIdChanged"
    }, {
      "propName": "freeformTemplateId",
      "methodName": "handleFreeformTemplateIdChanged"
    }, {
      "propName": "viewer",
      "methodName": "handleViewerChanged"
    }, {
      "propName": "disabled",
      "methodName": "handleDisabledChanged"
    }, {
      "propName": "startLineAnchorStyle",
      "methodName": "handleStartLineAnchorStyleChanged"
    }, {
      "propName": "endLineAnchorStyle",
      "methodName": "handleEndLineAnchorStyleChanged"
    }]; }
  static get listeners() { return [{
      "name": "markupEnd",
      "method": "handleMarkupEnd",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "interactionEnd",
      "method": "handleMarkupUpdated",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "pointerdown",
      "method": "handleMarkupPointerDown",
      "target": undefined,
      "capture": false,
      "passive": true
    }]; }
}
//# sourceMappingURL=viewer-markup.js.map