/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { Component, Element, Event, 
// eslint-disable-next-line @typescript-eslint/no-unused-vars
h, Host, Method, Prop, State, Watch, } from '@stencil/core';
import { Point } from '@vertexvis/geometry';
import classNames from 'classnames';
import { getMouseClientPosition } from '../../lib/dom';
import { PinController } from '../../lib/pins/controller';
import { getPinColors, isTextPin, PinModel, } from '../../lib/pins/model';
import { readDOM } from '../../lib/stencil';
import { translatePointToRelative, translatePointToScreen, } from '../viewer-pin-tool/utils';
import { getComputedStyle } from './utils';
export class VertexPinLabel {
  constructor() {
    this.focused = false;
    this.textareaRows = 1;
    this.computeContentSize = () => {
      readDOM(() => {
        if (this.contentEl != null) {
          this.contentElBounds = this.contentEl.getBoundingClientRect();
          const computedStyles = getComputedStyle(this.contentEl);
          this.textareaRows = Math.max(1, Math.ceil((parseFloat(computedStyles.height) -
            parseFloat(computedStyles.borderWidth) * 2) /
            parseFloat(computedStyles.lineHeight)));
        }
      });
    };
    this.handleInputKeyDown = (event) => {
      event.stopPropagation();
      if (event.key === 'Enter' &&
        (event.ctrlKey || event.metaKey || event.shiftKey)) {
        this.textareaRows += 1;
        this.value += '\n';
      }
      else if (event.key === 'Enter') {
        event.preventDefault();
        this.submit();
      }
    };
    this.handlePointerDown = (event) => {
      var _a;
      if (!this.focused) {
        if (this.elementBounds != null) {
          if (event.buttons !== 2) {
            event.preventDefault();
            event.stopPropagation();
          }
          this.relativePointerDownPosition = translatePointToRelative(getMouseClientPosition(event, this.elementBounds), this.elementBounds);
          this.pinPointerDownPosition = (_a = this.pin) === null || _a === void 0 ? void 0 : _a.label.point;
          window.addEventListener('pointermove', this.handlePointerMove);
          window.addEventListener('pointerup', this.handlePointerUp);
        }
      }
    };
    this.handlePointerMove = (event) => {
      var _a;
      if (this.elementBounds != null &&
        this.relativePointerDownPosition &&
        this.pinPointerDownPosition != null) {
        // Prevent selection of any text while interacting with the label
        event.preventDefault();
        event.stopPropagation();
        const point = getMouseClientPosition(event, this.elementBounds);
        const relative = translatePointToRelative(point, this.elementBounds);
        const relativeDelta = Point.subtract(relative, this.relativePointerDownPosition);
        const myUpdatedPin = this.pin != null
          ? Object.assign(Object.assign({}, this.pin), { label: Object.assign(Object.assign({}, this.pin.label), { point: Point.add(this.pinPointerDownPosition, relativeDelta) }) }) : undefined;
        if (myUpdatedPin) {
          (_a = this.pinController) === null || _a === void 0 ? void 0 : _a.updatePin(myUpdatedPin);
          this.computeScreenPosition();
        }
      }
    };
    this.handlePointerUp = (event) => {
      var _a, _b;
      if (((_a = this.pinController) === null || _a === void 0 ? void 0 : _a.getToolMode()) === 'edit' &&
        this.relativePointerDownPosition != null &&
        this.elementBounds != null) {
        const pointerDownScreen = translatePointToScreen(this.relativePointerDownPosition, this.elementBounds);
        const distanceBetweenStartAndEndPoint = Point.distance(pointerDownScreen, getMouseClientPosition(event, this.elementBounds));
        if (distanceBetweenStartAndEndPoint <= 2) {
          this.focused = true;
          this.labelFocused.emit((_b = this.pin) === null || _b === void 0 ? void 0 : _b.id);
        }
      }
      this.relativePointerDownPosition = undefined;
      this.pinPointerDownPosition = undefined;
      window.removeEventListener('pointermove', this.handlePointerMove);
      window.removeEventListener('pointerup', this.handlePointerUp);
    };
    this.handleTextBlur = () => {
      this.submit();
    };
    this.handleTextInput = (event) => {
      const input = event.target;
      this.value = input.value;
    };
    this.value = this.getPinText();
  }
  /**
   * Gives focus to the the component's internal text input.
   */
  async setFocus() {
    var _a, _b, _c;
    // HTMLInputElement.focus() doesn't exist in tests.
    if (typeof ((_a = this.inputEl) === null || _a === void 0 ? void 0 : _a.focus) === 'function') {
      this.focused = true;
      this.labelFocused.emit((_b = this.pin) === null || _b === void 0 ? void 0 : _b.id);
      (_c = this.inputEl) === null || _c === void 0 ? void 0 : _c.focus();
    }
  }
  /**
   * @internal
   * submits the current text, unfocuses the input and emits the blur event to consumers
   */
  async submit() {
    var _a, _b;
    this.focused = false;
    this.labelBlurred.emit((_a = this.pin) === null || _a === void 0 ? void 0 : _a.id);
    if (this.pin != null) {
      (_b = this.pinController) === null || _b === void 0 ? void 0 : _b.updatePin(Object.assign(Object.assign({}, this.pin), { label: {
          point: this.pin.label.point,
          text: this.value,
        } }));
    }
  }
  watchFocusChange() {
    var _a, _b;
    this.labelChanged.emit();
    if (this.focused) {
      (_a = this.pinController) === null || _a === void 0 ? void 0 : _a.setSelectedPinId((_b = this.pin) === null || _b === void 0 ? void 0 : _b.id);
    }
  }
  watchPinChange() {
    this.value = this.getPinText();
    this.computeScreenPosition();
  }
  watchElementBoundsChange() {
    this.computeScreenPosition();
  }
  componentWillLoad() {
    this.computeScreenPosition();
  }
  componentDidLoad() {
    this.resizeObserver = new ResizeObserver(() => {
      this.labelChanged.emit();
    });
    this.contentResizeObserver = new ResizeObserver(this.computeContentSize);
    if (this.hostEl != null) {
      this.resizeObserver.observe(this.hostEl);
    }
    if (this.contentEl != null) {
      this.contentResizeObserver.observe(this.contentEl);
    }
    if (this.pinController == null) {
      this.pinController = new PinController(new PinModel());
    }
  }
  disconnectedCallback() {
    var _a, _b;
    (_a = this.resizeObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
    (_b = this.contentResizeObserver) === null || _b === void 0 ? void 0 : _b.disconnect();
  }
  componentDidRender() {
    if (this.focused && this.inputEl != null) {
      this.setFocus();
    }
  }
  render() {
    var _a, _b, _c;
    const { primaryColor, accentColor } = getPinColors(this.pin);
    return (h(Host, null,
      h("div", { class: classNames('pin-label-input-wrapper', {
          focused: this.focused,
        }), onPointerDown: this.handlePointerDown, style: {
          top: `${((_a = this.computedScreenPosition) === null || _a === void 0 ? void 0 : _a.y.toString()) || 0}px`,
          left: `${((_b = this.computedScreenPosition) === null || _b === void 0 ? void 0 : _b.x.toString()) || 0}px`,
          minWidth: this.computeMinWidth(),
          maxWidth: this.computeMaxWidth(),
          maxHeight: this.computeMaxHeight(),
          borderColor: primaryColor,
          background: accentColor,
        } },
        h("textarea", { id: `pin-label-input-${(_c = this.pin) === null || _c === void 0 ? void 0 : _c.id}`, class: classNames('pin-label-input', 'pin-label-text', {
            ['readonly']: !this.focused,
          }), disabled: !this.focused, ref: (ref) => (this.inputEl = ref), value: this.value, rows: this.textareaRows, onKeyDown: this.handleInputKeyDown, onInput: this.handleTextInput, onBlur: this.handleTextBlur }),
        !this.focused && (h("div", { class: "pin-input-drag-target", onPointerDown: this.handlePointerDown }))),
      h("div", { ref: (el) => (this.contentEl = el), class: classNames('pin-label-text', 'pin-label-hidden'), style: {
          maxWidth: this.computeMaxWidth(),
          maxHeight: this.computeMaxHeight(),
        } }, this.hiddenContent())));
  }
  hiddenContent() {
    // This corrects some inconsistencies in how a newline character
    // is represented in a div. Leveraging `<br>`s results in a more
    // consistent representation of the newlines in a textarea.
    return this.value.includes('\n')
      ? this.value
        .split('\n')
        .reduce((res, str) => [...res, str, h("br", null)], [])
      : [this.value];
  }
  computeMinWidth() {
    if (this.contentElBounds != null) {
      const expected = `${this.contentElBounds.width + 8}px`;
      return `min(${expected}, ${this.computeMaxWidth()})`;
    }
    else {
      return `var(--viewer-annotations-pin-label-min-width)`;
    }
  }
  computeMaxWidth() {
    const expected = `var(--viewer-annotations-pin-label-max-width)`;
    return `min(${expected}, ${this.computeRemainingWidth()})`;
  }
  computeMaxHeight() {
    const expected = `var(--viewer-annotations-pin-label-max-height)`;
    return `min(${expected}, ${this.computeRemainingHeight()})`;
  }
  computeRemainingWidth() {
    var _a, _b;
    return `calc(${((_a = this.elementBounds) === null || _a === void 0 ? void 0 : _a.width.toString()) || 0}px - ${((_b = this.computedScreenPosition) === null || _b === void 0 ? void 0 : _b.x.toString()) || 0}px)`;
  }
  computeRemainingHeight() {
    var _a, _b;
    return `calc(${((_a = this.elementBounds) === null || _a === void 0 ? void 0 : _a.height.toString()) || 0}px - ${((_b = this.computedScreenPosition) === null || _b === void 0 ? void 0 : _b.y.toString()) || 0}px)`;
  }
  getPinText() {
    var _a;
    if (((_a = this.pin) === null || _a === void 0 ? void 0 : _a.label.text) != null) {
      this.value = this.pin.label.text;
    }
    else {
      this.value = '';
    }
    return this.value;
  }
  computeScreenPosition() {
    this.computedScreenPosition =
      isTextPin(this.pin) &&
        this.elementBounds != null &&
        this.pin.label.point != null
        ? translatePointToScreen(this.pin.label.point, this.elementBounds)
        : undefined;
  }
  static get is() { return "vertex-viewer-pin-label"; }
  static get originalStyleUrls() { return {
    "$": ["viewer-pin-label.css"]
  }; }
  static get styleUrls() { return {
    "$": ["viewer-pin-label.css"]
  }; }
  static get properties() { return {
    "pin": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "TextPin",
        "resolved": "TextPin | undefined",
        "references": {
          "TextPin": {
            "location": "import",
            "path": "../../lib/pins/model"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The pin to draw for the group"
      }
    },
    "elementBounds": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "DOMRect",
        "resolved": "DOMRect | undefined",
        "references": {
          "DOMRect": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The dimensions of the canvas for the pins"
      }
    },
    "value": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The current text value of the component. Value is updated on user\ninteraction."
      },
      "attribute": "value",
      "reflect": false
    },
    "pinController": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "PinController",
        "resolved": "PinController | undefined",
        "references": {
          "PinController": {
            "location": "import",
            "path": "../../lib/pins/controller"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The controller that drives behavior for pin operations"
      }
    }
  }; }
  static get states() { return {
    "focused": {},
    "computedScreenPosition": {},
    "textareaRows": {},
    "contentElBounds": {}
  }; }
  static get events() { return [{
      "method": "labelChanged",
      "name": "labelChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "labelFocused",
      "name": "labelFocused",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted whenever the label is focused, with the ID of the\nassociated pin (or undefined if no pin is provided)."
      },
      "complexType": {
        "original": "string | undefined",
        "resolved": "string | undefined",
        "references": {}
      }
    }, {
      "method": "labelBlurred",
      "name": "labelBlurred",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted whenever the label is blurred, with the ID of the\nassociated pin (or undefined if no pin is provided)."
      },
      "complexType": {
        "original": "string | undefined",
        "resolved": "string | undefined",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "setFocus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Gives focus to the the component's internal text input.",
        "tags": []
      }
    },
    "submit": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": [{
            "name": "internal",
            "text": "submits the current text, unfocuses the input and emits the blur event to consumers"
          }]
      }
    }
  }; }
  static get elementRef() { return "hostEl"; }
  static get watchers() { return [{
      "propName": "focused",
      "methodName": "watchFocusChange"
    }, {
      "propName": "pin",
      "methodName": "watchPinChange"
    }, {
      "propName": "elementBounds",
      "methodName": "watchElementBoundsChange"
    }]; }
}
//# sourceMappingURL=viewer-pin-label.js.map