/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { Component, Element, 
// eslint-disable-next-line @typescript-eslint/no-unused-vars
h, Host, Prop, State, Watch, } from '@stencil/core';
import { PinController } from '../../lib/pins/controller';
import { PinsInteractionHandler } from '../../lib/pins/interactions';
import { PinModel, } from '../../lib/pins/model';
import { getMarkupBoundingClientRect } from '../viewer-markup/dom';
export class ViewerPinTool {
  constructor() {
    /**
     * The model that contains the entities and outcomes from performing pin annotations
     */
    this.pinModel = new PinModel();
    /**
     * The type of pin.
     *
     * This property will automatically be set.
     */
    this.tool = 'pin-icon';
    /**
     * The mode of the pin tool
     */
    this.mode = 'view';
    this.pins = [];
    this.handleSetProjectionMatrix = () => {
      var _a, _b;
      this.projectionViewMatrix =
        (_b = (_a = this.viewer) === null || _a === void 0 ? void 0 : _a.frame) === null || _b === void 0 ? void 0 : _b.scene.camera.projectionViewMatrix;
    };
  }
  /**
   * @ignore
   */
  watchModeChange() {
    var _a;
    (_a = this.pinController) === null || _a === void 0 ? void 0 : _a.setToolMode(this.mode);
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  watchTypeChange() {
    var _a;
    (_a = this.pinController) === null || _a === void 0 ? void 0 : _a.setToolType(this.tool);
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  watchAccentColorChange() {
    var _a;
    (_a = this.pinController) === null || _a === void 0 ? void 0 : _a.setAccentColor(this.accentColor);
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  watchPrimaryColorChange() {
    var _a;
    (_a = this.pinController) === null || _a === void 0 ? void 0 : _a.setPrimaryColor(this.primaryColor);
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  connectedCallback() {
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.updateViewport();
    this.setupController();
    this.setupInteractionHandler();
    this.pinModel.onEntitiesChanged((entities) => {
      this.pins = entities;
    });
    this.pinModel.onSelectionChange((selectedId) => {
      this.selectedPinId = selectedId;
    });
    this.handleViewerChanged(this.viewer, undefined);
  }
  componentDidLoad() {
    this.resizeObserver = new ResizeObserver(() => this.updateViewport());
    this.resizeObserver.observe(this.hostEl);
  }
  /**
   * @ignore
   */
  disconnectedCallback() {
    this.clearInteractionHandler();
    this.clearModelListeners();
  }
  /**
   * @ignore
   */
  handleViewerChanged(newViewer, oldViewer) {
    this.setupInteractionHandler();
    if (oldViewer != null) {
      oldViewer.removeEventListener('frameDrawn', this.handleSetProjectionMatrix);
    }
    if (newViewer != null) {
      newViewer.addEventListener('frameDrawn', this.handleSetProjectionMatrix);
    }
    this.handleSetProjectionMatrix();
  }
  render() {
    return (h(Host, null,
      h("vertex-viewer-dom-renderer", { viewer: this.viewer, drawMode: "2d" }, this.pins.map((pin) => {
        return (h("vertex-viewer-pin-group", { id: `pin-group-${pin.id}`, "data-is-dom-group-element": true, pin: pin, elementBounds: this.elementBounds, pinModel: this.pinModel, pinController: this.pinController, projectionViewMatrix: this.projectionViewMatrix, selected: this.selectedPinId === pin.id }));
      }))));
  }
  setupController() {
    const attributes = this.accentColor || this.primaryColor
      ? {
        accentColor: this.accentColor,
        primaryColor: this.primaryColor,
      }
      : undefined;
    this.pinController = new PinController(this.pinModel, this.mode, this.tool, attributes);
  }
  clearInteractionHandler() {
    var _a;
    (_a = this.registeredInteractionHandler) === null || _a === void 0 ? void 0 : _a.then((handler) => handler.dispose());
    this.registeredInteractionHandler = undefined;
  }
  setupInteractionHandler() {
    var _a;
    const hostStyles = window.getComputedStyle(this.hostEl);
    const xOffset = hostStyles
      .getPropertyValue('--viewer-pin-tool-initial-label-offset-x')
      .trim();
    const yOffset = hostStyles
      .getPropertyValue('--viewer-pin-tool-initial-label-offset-y')
      .trim();
    this.clearInteractionHandler();
    if (this.pinController != null) {
      this.registeredInteractionHandler =
        (_a = this.viewer) === null || _a === void 0 ? void 0 : _a.registerInteractionHandler(new PinsInteractionHandler(this.pinController, parseInt(xOffset), parseInt(yOffset)));
    }
  }
  clearModelListeners() {
    var _a, _b;
    (_a = this.onEntitiesChangedHandler) === null || _a === void 0 ? void 0 : _a.dispose();
    this.onEntitiesChangedHandler = undefined;
    (_b = this.resizeObserver) === null || _b === void 0 ? void 0 : _b.disconnect();
  }
  updateViewport() {
    const rect = getMarkupBoundingClientRect(this.hostEl);
    this.elementBounds = rect;
  }
  static get is() { return "vertex-viewer-pin-tool"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["viewer-pin-tool.css"]
  }; }
  static get styleUrls() { return {
    "$": ["viewer-pin-tool.css"]
  }; }
  static get properties() { return {
    "pinController": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "PinController",
        "resolved": "PinController | undefined",
        "references": {
          "PinController": {
            "location": "import",
            "path": "../../lib/pins/controller"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The controller that is responsible for drawing pins and updating the model"
      }
    },
    "pinModel": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "PinModel",
        "resolved": "PinModel",
        "references": {
          "PinModel": {
            "location": "import",
            "path": "../../lib/pins/model"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The model that contains the entities and outcomes from performing pin annotations"
      },
      "defaultValue": "new PinModel()"
    },
    "viewer": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "HTMLVertexViewerElement",
        "resolved": "HTMLVertexViewerElement | undefined",
        "references": {
          "HTMLVertexViewerElement": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The viewer that this component is bound to. This is automatically assigned\nif added to the light-dom of a parent viewer element."
      }
    },
    "tool": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "ViewerPinToolType",
        "resolved": "\"pin-icon\" | \"pin-text\"",
        "references": {
          "ViewerPinToolType": {
            "location": "import",
            "path": "../../lib/pins/model"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The type of pin.\n\nThis property will automatically be set."
      },
      "attribute": "tool",
      "reflect": false,
      "defaultValue": "'pin-icon'"
    },
    "mode": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "ViewerPinToolMode",
        "resolved": "\"edit\" | \"view\"",
        "references": {
          "ViewerPinToolMode": {
            "location": "import",
            "path": "../../lib/pins/model"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The mode of the pin tool"
      },
      "attribute": "mode",
      "reflect": false,
      "defaultValue": "'view'"
    },
    "primaryColor": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "Color.Color | string | undefined",
        "resolved": "Color | string | undefined",
        "references": {
          "Color": {
            "location": "import",
            "path": "@vertexvis/utils"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The primary color for new pins. Setting this will override the primary template color, and will be used for any new\npins created with this `vertex-viewer-pin-tool`. This styling applies to pin anchors, and borders, etc."
      },
      "attribute": "primary-color",
      "reflect": false
    },
    "accentColor": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "Color.Color | string | undefined",
        "resolved": "Color | string | undefined",
        "references": {
          "Color": {
            "location": "import",
            "path": "@vertexvis/utils"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The accent color for new pins. Setting this will override the accent template color, and will be used for any new\npins created with this `vertex-viewer-pin-tool`. This styling applies to some background colors, etc"
      },
      "attribute": "accent-color",
      "reflect": false
    }
  }; }
  static get states() { return {
    "pins": {},
    "selectedPinId": {},
    "elementBounds": {},
    "projectionViewMatrix": {}
  }; }
  static get elementRef() { return "hostEl"; }
  static get watchers() { return [{
      "propName": "mode",
      "methodName": "watchModeChange"
    }, {
      "propName": "tool",
      "methodName": "watchTypeChange"
    }, {
      "propName": "accentColor",
      "methodName": "watchAccentColorChange"
    }, {
      "propName": "primaryColor",
      "methodName": "watchPrimaryColorChange"
    }, {
      "propName": "viewer",
      "methodName": "handleViewerChanged"
    }]; }
}
//# sourceMappingURL=viewer-pin-tool.js.map