/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { Component, Element, Event, h, Host, Method, Prop, State, Watch, } from '@stencil/core';
import { Angle, Euler, Matrix4, Point, Quaternion, Vector3, } from '@vertexvis/geometry';
import classNames from 'classnames';
import { readDOM, writeDOM } from '../../lib/stencil';
import { TransformController } from '../../lib/transforms/controller';
import { AngleUnits, DistanceUnits, } from '../../lib/types';
import { focusInputElement } from './dom';
import { computeHandleDeltaTransform, computeInputDeltaTransform, computeInputDisplayValue, computeInputPosition, convertCanvasPointToWorld, convertPointToCanvas, } from './util';
import { TransformWidgetInput, TransformWidgetInputWrapper, } from './viewer-transform-widget-components';
import { TransformWidget } from './widget';
export class ViewerTransformWidget {
  constructor() {
    /**
     * Determines whether or not the x-rotation is disabled on the widget
     */
    this.xRotationDisabled = false;
    /**
     * Determines whether or not the y-rotation is disabled on the widget
     */
    this.yRotationDisabled = false;
    /**
     * Determines whether or not the z-rotation is disabled on the widget
     */
    this.zRotationDisabled = false;
    /**
     * Determines whether or not the x-translation is disabled on the widget
     */
    this.xTranslationDisabled = false;
    /**
     * Determines whether or not the y-translation is disabled on the widget
     */
    this.yTranslationDisabled = false;
    /**
     * Determines whether or not the z-translation is disabled on the widget
     */
    this.zTranslationDisabled = false;
    /**
     * Whether to show inputs beside the widget handles when they are interacted with.
     * Defaults to `true`.
     */
    this.showInputs = true;
    /**
     * The unit to show for translation inputs. Defaults to `millimeters`.
     *
     * @see DistanceUnitType
     */
    this.distanceUnit = 'millimeters';
    /**
     * The unit to show for rotation inputs. Defaults to `degrees`.
     *
     * @see AngleUnitType
     */
    this.angleUnit = 'degrees';
    /**
     * The number of decimal places to show in the input. Defaults to `1`.
     */
    this.decimalPlaces = 1;
    /**
     * **EXPERIMENTAL.**
     *
     * Enables Command+Z and Control+Z keybindings to perform an undo of
     * the previous delta transformation.
     *
     * @see `EXPERIMENTAL_undo`
     */
    this.EXPERIMENTAL_undoKeybindings = false;
    this.inputValue = 0;
    this.isEndingTransform = false;
    this.inputShouldFocus = false;
    this.xArrowColor = '#ea3324';
    this.yArrowColor = '#4faf32';
    this.zArrowColor = '#0000ff';
    this.hoveredColor = '#ffff00';
    this.lastAngle = 0;
    this.handleHoveredDrawableChanged = (drawable) => {
      this.hovered = drawable;
    };
    this.handleViewerFrameDrawn = () => {
      this.updatePropsFromViewer();
    };
    this.handleViewerDimensionsChange = () => {
      writeDOM(() => {
        if (this.viewer != null && this.canvasRef != null) {
          this.canvasRef.width = this.viewer.viewport.width;
          this.canvasRef.height = this.viewer.viewport.height;
          this.updateCanvasBounds(this.canvasRef);
        }
      });
    };
    this.handleResize = () => {
      if (this.canvasRef != null) {
        this.updateCanvasBounds(this.canvasRef);
      }
    };
    this.handleInputResize = () => {
      if (this.inputWrapperRef != null) {
        const wrapperElement = this.inputWrapperRef;
        readDOM(() => {
          this.inputBounds = wrapperElement.getBoundingClientRect();
        });
      }
    };
    this.handleStyleChange = () => {
      readDOM(() => {
        const hostStyles = window.getComputedStyle(this.hostEl);
        this.xArrowColor = hostStyles
          .getPropertyValue('--viewer-transform-widget-x-axis-arrow-color')
          .trim();
        this.yArrowColor = hostStyles
          .getPropertyValue('--viewer-transform-widget-y-axis-arrow-color')
          .trim();
        this.zArrowColor = hostStyles
          .getPropertyValue('--viewer-transform-widget-z-axis-arrow-color')
          .trim();
        this.hoveredColor = hostStyles
          .getPropertyValue('--viewer-transform-widget-hovered-arrow-color')
          .trim();
        this.getTransformWidget().updateColors({
          xArrow: this.xArrowColor,
          yArrow: this.yArrowColor,
          zArrow: this.zArrowColor,
          hovered: this.hoveredColor,
        });
      });
    };
    this.handlePointerMove = (event) => {
      if (this.dragging == null) {
        const canvasPoint = convertPointToCanvas(Point.create(event.clientX, event.clientY), this.getCanvasBounds());
        const widget = this.getTransformWidget();
        if (canvasPoint != null && widget.boundsContainsPoint(canvasPoint)) {
          widget.updateCursor(canvasPoint);
        }
        else {
          widget.updateCursor(undefined);
          this.hovered = undefined;
        }
      }
    };
    this.handleBeginDrag = async (event) => {
      var _a, _b, _c;
      const canvasBounds = this.getCanvasBounds();
      if (this.hovered != null &&
        canvasBounds != null &&
        this.viewer != null &&
        this.position != null &&
        this.viewer.frame != null) {
        this.dragging = this.hovered;
        if (this.dragging.identifier !== ((_a = this.lastDragged) === null || _a === void 0 ? void 0 : _a.identifier)) {
          this.inputValue = 0;
          this.dragStartTransform = this.currentTransform;
          this.updateInputPosition();
          this.updateInputValue();
        }
        this.lastDragged = undefined;
        const currentCanvas = convertPointToCanvas(Point.create(event.clientX, event.clientY), canvasBounds);
        const widgetCenter = this.viewer.viewport.transformWorldToViewport(this.position, this.viewer.frame.scene.camera.projectionViewMatrix);
        this.lastAngle =
          currentCanvas != null
            ? Angle.fromPoints(widgetCenter, currentCanvas)
            : 0;
        this.lastWorldPosition = convertCanvasPointToWorld(currentCanvas, (_b = this.viewer) === null || _b === void 0 ? void 0 : _b.frame, (_c = this.viewer) === null || _c === void 0 ? void 0 : _c.viewport, this.currentTransform);
        this.interactionStarted.emit();
        window.removeEventListener('pointermove', this.handlePointerMove);
        window.addEventListener('pointermove', this.handleDrag);
        window.addEventListener('pointerup', this.handleEndTransform);
      }
    };
    this.handleDrag = async (event) => {
      var _a, _b, _c;
      // Prevent selection of text and interaction with view cube while dragging the widget
      event.preventDefault();
      const canvasBounds = this.getCanvasBounds();
      if (this.dragging != null &&
        this.lastWorldPosition != null &&
        canvasBounds != null &&
        this.viewer != null &&
        this.viewer.frame != null &&
        this.position != null) {
        // Begin the transform on the first `pointermove` event as opposed to the
        // `pointerdown` to prevent accidental no-op transforms (identity matrix).
        await ((_a = this.controller) === null || _a === void 0 ? void 0 : _a.beginTransform());
        const currentCanvas = convertPointToCanvas(Point.create(event.clientX, event.clientY), canvasBounds);
        const widgetCenter = this.viewer.viewport.transformWorldToViewport(this.position, this.viewer.frame.scene.camera.projectionViewMatrix);
        const currentWorld = convertCanvasPointToWorld(currentCanvas, (_b = this.viewer) === null || _b === void 0 ? void 0 : _b.frame, (_c = this.viewer) === null || _c === void 0 ? void 0 : _c.viewport, this.currentTransform);
        if (currentWorld != null &&
          currentCanvas != null &&
          widgetCenter != null) {
          const angle = Angle.fromPoints(widgetCenter, currentCanvas);
          this.transform(this.lastWorldPosition, currentWorld, angle - this.lastAngle);
          this.updateInputPosition();
          this.updateInputValue();
          this.lastWorldPosition = currentWorld;
          this.lastAngle = angle;
        }
      }
    };
    this.handleEndTransform = async (event) => {
      var _a, _b;
      const canvasPoint = convertPointToCanvas(Point.create(event.clientX, event.clientY), this.getCanvasBounds());
      const widget = this.getTransformWidget();
      this.lastDragged = this.dragging;
      this.dragging = undefined;
      this.lastWorldPosition = undefined;
      this.lastAngle = 0;
      this.updateInputPosition();
      this.updateInputValue();
      widget.updateCursor(canvasPoint);
      widget.updateTransform(this.currentTransform);
      this.beginEndTransform();
      window.removeEventListener('pointermove', this.handleDrag);
      window.removeEventListener('pointerup', this.handleEndTransform);
      try {
        const delta = (_a = this.controller) === null || _a === void 0 ? void 0 : _a.getCurrentDelta();
        await ((_b = this.controller) === null || _b === void 0 ? void 0 : _b.endTransform());
        this.interactionEnded.emit(delta);
      }
      catch (e) {
        console.error('Failed to end transform interaction', e);
      }
      window.addEventListener('pointermove', this.handlePointerMove);
      this.completeEndTransform();
    };
    this.handleInputStep = (step) => {
      if (this.inputValue != null && this.lastInputValue != null) {
        // If modifying angle units, we want to adjust the step to always
        // perform the equivalent of a 1 degree rotation.
        if (this.isModifyingAngleUnits()) {
          const angles = new AngleUnits(this.angleUnit);
          const previous = angles.convertFrom(this.lastInputValue);
          const next = Angle.normalizeRadians(previous + Angle.toRadians(step));
          this.inputValue = angles.convertTo(next);
        }
        else {
          this.inputValue = this.lastInputValue + step;
        }
        this.updateTransformFromInput(this.inputValue);
      }
    };
    this.handleInputChange = (value) => {
      this.updateTransformFromInput(value);
    };
    this.updateTransformFromInput = async (value) => {
      var _a, _b, _c;
      if (this.lastDragged != null &&
        this.currentTransform != null &&
        this.startingTransform != null &&
        this.lastInputValue != null) {
        this.transformCurrent(computeInputDeltaTransform(this.currentTransform, this.lastDragged.identifier, value, this.lastInputValue, this.distanceUnit, this.angleUnit));
        this.updateInputValue();
        this.lastInputValue = value;
        await ((_a = this.controller) === null || _a === void 0 ? void 0 : _a.beginTransform());
        this.getTransformWidget().updateTransform(this.currentTransform);
        await ((_b = this.controller) === null || _b === void 0 ? void 0 : _b.updateTransform(Matrix4.multiply(this.currentTransform, Matrix4.invert(this.startingTransform))));
        this.updateInputPosition();
        await ((_c = this.controller) === null || _c === void 0 ? void 0 : _c.endTransformDebounced(this.beginEndTransform, this.completeEndTransform));
      }
    };
    this.updatePropsFromViewer = () => {
      const { frame } = this.viewer || {};
      if (frame != null) {
        const widget = this.getTransformWidget();
        widget.updateFrame(frame, this.dragging == null);
        this.updateInputPosition();
      }
    };
    this.setupTransformWidget = (canvasRef) => {
      var _a, _b;
      console.debug(`Initializing transform widget. [initial-position=${JSON.stringify(this.position)}, has-initial-frame=${((_a = this.viewer) === null || _a === void 0 ? void 0 : _a.frame) != null}]`);
      this.widget = new TransformWidget(canvasRef, {
        xArrow: this.xArrowColor,
        yArrow: this.yArrowColor,
        zArrow: this.zArrowColor,
        hovered: this.hoveredColor,
      });
      if (this.rotation != null) {
        this.currentTransform = this.getTransformForNewRotation(this.rotation);
        this.startingTransform = this.currentTransform;
        this.widget.updateTransform(this.currentTransform);
      }
      if (this.position != null) {
        this.currentTransform = this.getTransformForNewPosition(this.position);
        this.startingTransform = this.currentTransform;
        this.widget.updateTransform(this.currentTransform);
      }
      if (((_b = this.viewer) === null || _b === void 0 ? void 0 : _b.frame) != null) {
        this.widget.updateFrame(this.viewer.frame, true);
      }
      this.handleSettingDisabledAxis();
      this.hoveredChangeDisposable = this.widget.onHoveredChanged(this.handleHoveredDrawableChanged);
      return this.widget;
    };
    this.updateCanvasBounds = (canvasElement) => {
      readDOM(() => {
        this.canvasBounds = canvasElement.getBoundingClientRect();
        this.getTransformWidget().updateDimensions(canvasElement);
      });
    };
    this.updateInputValue = () => {
      var _a, _b, _c;
      const dragging = (_a = this.dragging) !== null && _a !== void 0 ? _a : this.lastDragged;
      if (dragging != null &&
        this.currentTransform != null &&
        this.dragStartTransform != null) {
        this.lastInputValue = this.inputValue;
        this.inputValue = computeInputDisplayValue(dragging.identifier, this.currentTransform, this.dragStartTransform, this.distanceUnit, this.angleUnit);
        if (this.inputRef != null) {
          const definedValue = (_c = (_b = this.getDisplayedDistance()) !== null && _b !== void 0 ? _b : this.getDisplayedAngle()) !== null && _c !== void 0 ? _c : 0;
          const displayValue = `${parseFloat(definedValue.toFixed(this.decimalPlaces))}`;
          this.inputRef.value = displayValue;
        }
      }
    };
    this.transformCurrent = (transform) => {
      this.currentTransform =
        this.currentTransform != null
          ? Matrix4.multiply(transform, this.currentTransform)
          : transform;
    };
    this.updateInputPosition = () => {
      var _a, _b;
      const dragging = (_a = this.dragging) !== null && _a !== void 0 ? _a : this.lastDragged;
      if (this.showInputs &&
        ((_b = this.viewer) === null || _b === void 0 ? void 0 : _b.frame) != null &&
        this.position != null &&
        dragging != null) {
        const widget = this.getTransformWidget();
        const widgetBounds = widget.getFullBounds();
        this.inputPosition =
          widgetBounds != null
            ? computeInputPosition(this.viewer.viewport, widgetBounds, dragging.points.toArray())
            : undefined;
      }
    };
    this.beginEndTransform = () => {
      const widget = this.getTransformWidget();
      widget.updateDisabledAxis({
        xRotation: true,
        yRotation: true,
        zRotation: true,
        xTranslation: true,
        yTranslation: true,
        zTranslation: true,
      });
      this.isEndingTransform = true;
    };
    this.completeEndTransform = () => {
      const widget = this.getTransformWidget();
      widget.updateDisabledAxis({
        xRotation: this.xRotationDisabled,
        yRotation: this.yRotationDisabled,
        zRotation: this.zRotationDisabled,
        xTranslation: this.xTranslationDisabled,
        yTranslation: this.yTranslationDisabled,
        zTranslation: this.zTranslationDisabled,
      });
      this.isEndingTransform = false;
      this.inputShouldFocus = true;
      if (this.currentTransform != null) {
        this.position = Vector3.fromMatrixPosition(this.currentTransform);
        this.rotation = Euler.fromRotationMatrix(this.currentTransform);
      }
    };
    this.isModifyingAngleUnits = () => {
      var _a, _b, _c;
      const draggingIdentifier = (_b = (_a = this.dragging) === null || _a === void 0 ? void 0 : _a.identifier) !== null && _b !== void 0 ? _b : (_c = this.lastDragged) === null || _c === void 0 ? void 0 : _c.identifier;
      return !!(draggingIdentifier === null || draggingIdentifier === void 0 ? void 0 : draggingIdentifier.includes('rotate'));
    };
    this.getDisplayedUnits = () => {
      if (this.isModifyingAngleUnits()) {
        return new AngleUnits(this.angleUnit);
      }
      return new DistanceUnits(this.distanceUnit);
    };
    this.getDisplayedAngle = () => {
      if (this.isModifyingAngleUnits()) {
        return this.inputValue;
      }
    };
    this.getDisplayedDistance = () => {
      var _a, _b, _c;
      const draggingIdentifier = (_b = (_a = this.dragging) === null || _a === void 0 ? void 0 : _a.identifier) !== null && _b !== void 0 ? _b : (_c = this.lastDragged) === null || _c === void 0 ? void 0 : _c.identifier;
      if (draggingIdentifier === null || draggingIdentifier === void 0 ? void 0 : draggingIdentifier.includes('translate')) {
        return this.inputValue;
      }
    };
    this.getTransformForNewPosition = (newPosition) => {
      if (newPosition != null) {
        const c = this.currentTransform != null
          ? this.currentTransform
          : Matrix4.makeIdentity();
        const currentRotation = Matrix4.makeRotation(Quaternion.fromMatrixRotation(c));
        const position = Matrix4.makeTranslation(newPosition);
        return Matrix4.multiply(position, currentRotation);
      }
    };
    this.getTransformForNewRotation = (newRotationEuler) => {
      const c = this.currentTransform != null
        ? this.currentTransform
        : Matrix4.makeIdentity();
      const oldRotation = Matrix4.invert(Matrix4.makeRotation(Quaternion.fromMatrixRotation(c)));
      const newRotation = Matrix4.makeRotation(Quaternion.fromEuler(newRotationEuler));
      const oldTranslation = Matrix4.multiply(c, oldRotation);
      return Matrix4.multiply(oldTranslation, newRotation);
    };
    this.getCanvasBounds = () => {
      if (this.canvasBounds != null) {
        return this.canvasBounds;
      }
      else if (this.canvasRef != null) {
        this.updateCanvasBounds(this.canvasRef);
        return this.canvasBounds;
      }
    };
    this.getTransformWidget = () => {
      if (this.widget == null && this.canvasRef != null) {
        return this.setupTransformWidget(this.canvasRef);
      }
      else if (this.widget != null) {
        return this.widget;
      }
      else {
        throw new Error('Transform widget was not initialized. The canvas element may not have been initialized.');
      }
    };
  }
  componentDidLoad() {
    window.addEventListener('pointermove', this.handlePointerMove);
    this.canvasResizeObserver = new ResizeObserver(this.handleResize);
    this.inputResizeObserver = new ResizeObserver(this.handleInputResize);
    this.hostStyleObserver = new MutationObserver(this.handleStyleChange);
    if (this.canvasRef != null) {
      this.canvasResizeObserver.observe(this.canvasRef);
      this.setupTransformWidget(this.canvasRef);
    }
    this.hostStyleObserver.observe(this.hostEl, {
      attributes: true,
      attributeFilter: ['style'],
    });
    this.handleViewerChanged(this.viewer, undefined);
    this.handleStyleChange();
  }
  disconnectedCallback() {
    var _a, _b, _c, _d, _e;
    window.removeEventListener('pointermove', this.handlePointerMove);
    (_a = this.canvasResizeObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
    (_b = this.inputResizeObserver) === null || _b === void 0 ? void 0 : _b.disconnect();
    (_c = this.hostStyleObserver) === null || _c === void 0 ? void 0 : _c.disconnect();
    (_d = this.hoveredChangeDisposable) === null || _d === void 0 ? void 0 : _d.dispose();
    (_e = this.widget) === null || _e === void 0 ? void 0 : _e.dispose();
  }
  componentDidRender() {
    if (this.inputShouldFocus && this.inputRef != null) {
      focusInputElement(this.inputRef);
      this.inputShouldFocus = false;
    }
  }
  /**
   * **EXPERIMENTAL.**
   *
   * Performs an undo of the most recent set of transform manipulations,
   * categorized by the last pause in interactivity (pointerup on a handle,
   * Enter press on an input, keyup on an input).
   *
   * Note that this *does not* work repeatedly. I.e. only one undo can be
   * performed, and once another set of transform manipulations is performed,
   * the history is lost.
   */
  async EXPERIMENTAL_undo() {
    var _a, _b;
    const undoDelta = await ((_a = this.controller) === null || _a === void 0 ? void 0 : _a.EXPERIMENTAL_undoTransform());
    if (this.currentTransform != null && undoDelta != null) {
      this.currentTransform = Matrix4.multiply(undoDelta, this.currentTransform);
      (_b = this.widget) === null || _b === void 0 ? void 0 : _b.updateTransform(this.currentTransform);
      this.updateInputPosition();
      this.updateInputValue();
      this.lastInputValue = this.inputValue;
      this.position = Vector3.fromMatrixPosition(this.currentTransform);
      this.rotation = Euler.fromRotationMatrix(this.currentTransform);
    }
  }
  /**
   * @ignore
   */
  handleViewerChanged(newViewer, oldViewer) {
    var _a;
    oldViewer === null || oldViewer === void 0 ? void 0 : oldViewer.removeEventListener('frameDrawn', this.handleViewerFrameDrawn);
    oldViewer === null || oldViewer === void 0 ? void 0 : oldViewer.removeEventListener('dimensionschange', this.handleViewerDimensionsChange);
    newViewer === null || newViewer === void 0 ? void 0 : newViewer.addEventListener('frameDrawn', this.handleViewerFrameDrawn);
    newViewer === null || newViewer === void 0 ? void 0 : newViewer.addEventListener('dimensionschange', this.handleViewerDimensionsChange);
    if ((newViewer === null || newViewer === void 0 ? void 0 : newViewer.stream) != null) {
      (_a = this.controller) === null || _a === void 0 ? void 0 : _a.dispose();
      this.controller = new TransformController(newViewer.stream);
    }
  }
  /**
   * @ignore
   */
  handleDisabledPropertyChanged() {
    var _a;
    (_a = this.widget) === null || _a === void 0 ? void 0 : _a.updateDisabledAxis({
      xRotation: this.xRotationDisabled,
      yRotation: this.yRotationDisabled,
      zRotation: this.zRotationDisabled,
      xTranslation: this.xTranslationDisabled,
      yTranslation: this.yTranslationDisabled,
      zTranslation: this.zTranslationDisabled,
    });
  }
  /**
   * @ignore
   */
  handleRotationChanged(newRotation, oldRotation) {
    var _a;
    const rotationToApply = newRotation !== null && newRotation !== void 0 ? newRotation : Euler.create();
    this.currentTransform = this.getTransformForNewRotation(rotationToApply);
    this.startingTransform = this.currentTransform;
    // If the removal of the previous rotation above results in an identity matrix,
    // clear the transformation on the widget to prevent it from appearing at the origin.
    if (newRotation == null &&
      this.currentTransform != null &&
      Matrix4.isIdentity(this.currentTransform)) {
      this.currentTransform = undefined;
      this.startingTransform = this.currentTransform;
    }
    (_a = this.widget) === null || _a === void 0 ? void 0 : _a.updateTransform(this.currentTransform);
    console.debug(`Updating widget rotation [previous=${JSON.stringify(oldRotation)}, current=${JSON.stringify(newRotation)}]`);
    this.rotationChanged.emit(newRotation);
  }
  /**
   * @ignore
   */
  handlePositionChanged(newPosition, oldPosition) {
    var _a, _b;
    this.currentTransform = this.getTransformForNewPosition(newPosition);
    this.startingTransform = this.currentTransform;
    console.debug(`Updating widget position [previous=${JSON.stringify(oldPosition)}, current=${JSON.stringify(newPosition)}]`);
    (_a = this.widget) === null || _a === void 0 ? void 0 : _a.updateTransform(this.currentTransform);
    if (newPosition == null) {
      this.lastDragged = undefined;
      this.inputPosition = undefined;
      (_b = this.controller) === null || _b === void 0 ? void 0 : _b.clearTransform();
    }
    this.positionChanged.emit(newPosition);
  }
  /**
   * @ignore
   */
  handleInputFormattingChanged() {
    this.updateInputValue();
    this.lastInputValue = this.inputValue;
  }
  render() {
    var _a, _b, _c, _d, _e, _f;
    return (h(Host, null,
      h("canvas", { ref: (el) => {
          this.canvasRef = el;
        }, class: classNames('widget', {
          hovered: this.hovered != null,
        }), width: (_a = this.viewer) === null || _a === void 0 ? void 0 : _a.viewport.width, height: (_b = this.viewer) === null || _b === void 0 ? void 0 : _b.viewport.height, onPointerDown: this.handleBeginDrag }),
      this.showInputs && this.inputPosition && ((_c = this.viewer) === null || _c === void 0 ? void 0 : _c.viewport) && (h(TransformWidgetInputWrapper, { ref: (el) => {
          var _a, _b;
          if (el != null) {
            (_a = this.inputResizeObserver) === null || _a === void 0 ? void 0 : _a.observe(el);
          }
          else if (this.inputRef != null) {
            (_b = this.inputResizeObserver) === null || _b === void 0 ? void 0 : _b.unobserve(this.inputRef);
          }
          this.inputWrapperRef = el;
        }, bounds: this.inputBounds, viewport: this.viewer.viewport, point: this.inputPosition.point, placement: this.inputPosition.placement, displayUnit: this.getDisplayedUnits() },
        h(TransformWidgetInput, { ref: (el) => {
            this.inputRef = el;
          }, identifier: (_e = (_d = this.dragging) === null || _d === void 0 ? void 0 : _d.identifier) !== null && _e !== void 0 ? _e : (_f = this.lastDragged) === null || _f === void 0 ? void 0 : _f.identifier, disabled: this.isEndingTransform, onChange: this.handleInputChange, onIncrement: () => this.handleInputStep(1), onDecrement: () => this.handleInputStep(-1), onBlur: () => {
            this.inputShouldFocus = false;
          }, onUndo: this.EXPERIMENTAL_undoKeybindings
            ? () => this.EXPERIMENTAL_undo()
            : undefined })))));
  }
  transform(previous, next, angle) {
    var _a, _b;
    if (this.position != null &&
      this.startingTransform != null &&
      this.currentTransform != null &&
      this.dragging != null &&
      this.viewer != null &&
      this.viewer.frame != null) {
      this.transformCurrent(computeHandleDeltaTransform(this.currentTransform, previous, next, (_a = this.viewer) === null || _a === void 0 ? void 0 : _a.frame.scene.camera.viewVector, angle, this.dragging.identifier));
      this.getTransformWidget().updateTransform(this.currentTransform);
      (_b = this.controller) === null || _b === void 0 ? void 0 : _b.updateTransform(Matrix4.multiply(this.currentTransform, Matrix4.invert(this.startingTransform)));
    }
  }
  handleSettingDisabledAxis() {
    if (this.widget) {
      this.widget.updateDisabledAxis({
        xRotation: this.xRotationDisabled,
        yRotation: this.yRotationDisabled,
        zRotation: this.zRotationDisabled,
        xTranslation: this.xTranslationDisabled,
        yTranslation: this.yTranslationDisabled,
        zTranslation: this.zTranslationDisabled,
      });
    }
    else {
      console.warn('Cannot set disabled values - no widget defined');
    }
  }
  static get is() { return "vertex-viewer-transform-widget"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["viewer-transform-widget.css"]
  }; }
  static get styleUrls() { return {
    "$": ["viewer-transform-widget.css"]
  }; }
  static get properties() { return {
    "viewer": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "HTMLVertexViewerElement",
        "resolved": "HTMLVertexViewerElement | undefined",
        "references": {
          "HTMLVertexViewerElement": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The viewer to connect to transforms. If nested within a <vertex-viewer>,\nthis property will be populated automatically."
      }
    },
    "position": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Vector3.Vector3",
        "resolved": "Vector3 | undefined",
        "references": {
          "Vector3": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The starting position of this transform widget. This position will be updated\nas transforms occur. Setting this value to `undefined` will remove the widget."
      }
    },
    "rotation": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Euler.Euler",
        "resolved": "Euler | undefined",
        "references": {
          "Euler": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The starting angle for the transform widget. This rotation will be updated\nas the rotations occur."
      }
    },
    "controller": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "TransformController",
        "resolved": "TransformController | undefined",
        "references": {
          "TransformController": {
            "location": "import",
            "path": "../../lib/transforms/controller"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The controller that is responsible for performing transforms."
      }
    },
    "xRotationDisabled": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines whether or not the x-rotation is disabled on the widget"
      },
      "attribute": "x-rotation-disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "yRotationDisabled": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines whether or not the y-rotation is disabled on the widget"
      },
      "attribute": "y-rotation-disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "zRotationDisabled": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines whether or not the z-rotation is disabled on the widget"
      },
      "attribute": "z-rotation-disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "xTranslationDisabled": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines whether or not the x-translation is disabled on the widget"
      },
      "attribute": "x-translation-disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "yTranslationDisabled": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines whether or not the y-translation is disabled on the widget"
      },
      "attribute": "y-translation-disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "zTranslationDisabled": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines whether or not the z-translation is disabled on the widget"
      },
      "attribute": "z-translation-disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "showInputs": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Whether to show inputs beside the widget handles when they are interacted with.\nDefaults to `true`."
      },
      "attribute": "show-inputs",
      "reflect": false,
      "defaultValue": "true"
    },
    "distanceUnit": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "DistanceUnitType",
        "resolved": "\"centimeters\" | \"feet\" | \"inches\" | \"meters\" | \"millimeters\" | \"yards\"",
        "references": {
          "DistanceUnitType": {
            "location": "import",
            "path": "../../lib/types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "see",
            "text": "DistanceUnitType"
          }],
        "text": "The unit to show for translation inputs. Defaults to `millimeters`."
      },
      "attribute": "distance-unit",
      "reflect": false,
      "defaultValue": "'millimeters'"
    },
    "angleUnit": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "AngleUnitType",
        "resolved": "\"degrees\" | \"radians\"",
        "references": {
          "AngleUnitType": {
            "location": "import",
            "path": "../../lib/types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "see",
            "text": "AngleUnitType"
          }],
        "text": "The unit to show for rotation inputs. Defaults to `degrees`."
      },
      "attribute": "angle-unit",
      "reflect": false,
      "defaultValue": "'degrees'"
    },
    "decimalPlaces": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The number of decimal places to show in the input. Defaults to `1`."
      },
      "attribute": "decimal-places",
      "reflect": false,
      "defaultValue": "1"
    },
    "EXPERIMENTAL_undoKeybindings": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "see",
            "text": " `EXPERIMENTAL_undo`"
          }],
        "text": "**EXPERIMENTAL.**\n\nEnables Command+Z and Control+Z keybindings to perform an undo of\nthe previous delta transformation."
      },
      "attribute": "e-x-p-e-r-i-m-e-n-t-a-l_undo-keybindings",
      "reflect": false,
      "defaultValue": "false"
    },
    "hovered": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Drawable",
        "resolved": "Drawable<DrawablePoints> | undefined",
        "references": {
          "Drawable": {
            "location": "import",
            "path": "../../lib/transforms/drawable"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }, {
            "name": "ignore",
            "text": "Visible for testing."
          }],
        "text": ""
      }
    }
  }; }
  static get states() { return {
    "inputPosition": {},
    "inputValue": {},
    "isEndingTransform": {},
    "inputShouldFocus": {}
  }; }
  static get events() { return [{
      "method": "positionChanged",
      "name": "positionChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is emitted when the position of the widget changes."
      },
      "complexType": {
        "original": "Vector3.Vector3 | undefined",
        "resolved": "Vector3 | undefined",
        "references": {
          "Vector3": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      }
    }, {
      "method": "rotationChanged",
      "name": "rotationChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is emitted when the rotation of the widget changes."
      },
      "complexType": {
        "original": "Euler.Euler | undefined",
        "resolved": "Euler | undefined",
        "references": {
          "Euler": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      }
    }, {
      "method": "interactionEnded",
      "name": "interactionEnded",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is emitted when the interaction has ended"
      },
      "complexType": {
        "original": "Matrix4.Matrix4 | undefined",
        "resolved": "[number, number, number, number, number, number, number, number, number, number, number, number, number, number, number, number] | undefined",
        "references": {
          "Matrix4": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      }
    }, {
      "method": "interactionStarted",
      "name": "interactionStarted",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "An event that is emitted an interaction with the widget has started"
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "EXPERIMENTAL_undo": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "**EXPERIMENTAL.**\n\nPerforms an undo of the most recent set of transform manipulations,\ncategorized by the last pause in interactivity (pointerup on a handle,\nEnter press on an input, keyup on an input).\n\nNote that this *does not* work repeatedly. I.e. only one undo can be\nperformed, and once another set of transform manipulations is performed,\nthe history is lost.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "hostEl"; }
  static get watchers() { return [{
      "propName": "viewer",
      "methodName": "handleViewerChanged"
    }, {
      "propName": "xTranslationDisabled",
      "methodName": "handleDisabledPropertyChanged"
    }, {
      "propName": "yTranslationDisabled",
      "methodName": "handleDisabledPropertyChanged"
    }, {
      "propName": "zTranslationDisabled",
      "methodName": "handleDisabledPropertyChanged"
    }, {
      "propName": "xRotationDisabled",
      "methodName": "handleDisabledPropertyChanged"
    }, {
      "propName": "yRotationDisabled",
      "methodName": "handleDisabledPropertyChanged"
    }, {
      "propName": "zRotationDisabled",
      "methodName": "handleDisabledPropertyChanged"
    }, {
      "propName": "rotation",
      "methodName": "handleRotationChanged"
    }, {
      "propName": "position",
      "methodName": "handlePositionChanged"
    }, {
      "propName": "distanceUnit",
      "methodName": "handleInputFormattingChanged"
    }, {
      "propName": "angleUnit",
      "methodName": "handleInputFormattingChanged"
    }, {
      "propName": "decimalPlaces",
      "methodName": "handleInputFormattingChanged"
    }]; }
}
//# sourceMappingURL=viewer-transform-widget.js.map