/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
/* eslint-disable @typescript-eslint/member-ordering */
import { Component, h, Host, Prop, State, Watch } from '@stencil/core';
import { Vector3 } from '@vertexvis/geometry';
import { readDOM } from '../../lib/stencil';
import { FramePerspectiveCamera, Orientation, StandardView, } from '../../lib/types';
import { TriadAxis, ViewCubeCorner, ViewCubeEdge, ViewCubeShadow, ViewCubeSide, } from './viewer-view-cube-components';
export class ViewerViewCube {
  constructor() {
    this.boxLength = 80;
    this.triadPosition = Vector3.origin();
    /**
     * The label for the side of the cube on the positive x-axis.
     */
    this.xPositiveLabel = 'Right';
    /**
     * The label for the side of the cube on the negative x-axis.
     */
    this.xNegativeLabel = 'Left';
    /**
     * The label for the side of the cube on the positive y-axis.
     */
    this.yPositiveLabel = 'Top';
    /**
     * The label for the side of the cube on the negative y-axis.
     */
    this.yNegativeLabel = 'Bottom';
    /**
     * The label for the side of the cube on the positive z-axis.
     */
    this.zPositiveLabel = 'Front';
    /**
     * The label for the side of the cube on the negative z-axis.
     */
    this.zNegativeLabel = 'Back';
    /**
     * Disables interactions for standard views.
     */
    this.standardViewsOff = false;
    /**
     * Whether to perform a `viewAll` when clicking on the view cube. If this
     * is set to `false`, the current `lookAt` point will be maintained, and the
     * camera's `position` and `up` vectors will be aligned to the standard view.
     * Defaults to `true`.
     *
     * **Note** Setting this value to `false` can result in the camera being placed
     * underneath geometry depending on the current `viewVector` length, resulting
     * in a view that may be unexpected.
     */
    this.viewAll = true;
    /**
     * The duration of the animation, in milliseconds, when a user performs a
     * standard view interaction. Set to 0 to disable animations.
     */
    this.animationDuration = 500;
    /**
     * Disables the display of the triad.
     */
    this.triadOff = false;
    /**
     * @internal
     */
    this.worldOrientation = Orientation.DEFAULT;
    this.updateMatrices = () => {
      var _a;
      if (((_a = this.viewer) === null || _a === void 0 ? void 0 : _a.frame) != null) {
        const { camera } = this.viewer.frame.scene;
        const halfLength = this.boxLength / 2;
        this.triadPosition = Vector3.create(-halfLength - 5, -halfLength - 5, -halfLength - 5);
        // Used to scale the camera position so the cube is the same size as the
        // container. This isn't exact, but pretty close.
        const lengthScalar = 3.125;
        const scale = this.boxLength * lengthScalar;
        const fovY = 21.5;
        this.camera = new FramePerspectiveCamera(Vector3.scale(scale, Vector3.negate(camera.direction)), Vector3.origin(), camera.up, 0.1, 100, 1, fovY);
        this.worldOrientation = this.viewer.frame.scene.worldOrientation;
      }
    };
  }
  handleViewerChanged(newViewer, oldViewer) {
    oldViewer === null || oldViewer === void 0 ? void 0 : oldViewer.removeEventListener('frameDrawn', this.updateMatrices);
    newViewer === null || newViewer === void 0 ? void 0 : newViewer.addEventListener('frameDrawn', this.updateMatrices);
    this.updateMatrices();
  }
  handleStandardView(standardView) {
    if (this.standardViewsOff) {
      return async () => undefined;
    }
    else {
      return async () => {
        if (this.viewer != null) {
          const scene = await this.viewer.scene();
          const animation = this.animationDuration > 0
            ? { animation: { milliseconds: this.animationDuration } }
            : {};
          const worldStandardView = standardView.transformMatrix(this.worldOrientation.matrix);
          // Check to see if any geometry is visible. If not, don't perform viewAll
          const currentBoundingBox = scene.boundingBox();
          if (!this.viewAll) {
            scene
              .camera()
              .standardViewFixedLookAt(worldStandardView)
              .render(animation);
          }
          else if ((currentBoundingBox === null || currentBoundingBox === void 0 ? void 0 : currentBoundingBox.max) != null &&
            Vector3.isEqual(currentBoundingBox.max, Vector3.origin()) &&
            Vector3.isEqual(currentBoundingBox.min, Vector3.origin())) {
            scene.camera().standardView(worldStandardView).render(animation);
          }
          else {
            scene
              .camera()
              .standardView(worldStandardView)
              .viewAll()
              .render(animation);
          }
        }
      };
    }
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.handleViewerChanged(this.viewer);
  }
  /**
   * @ignore
   */
  componentDidLoad() {
    if (this.rendererEl != null) {
      const observer = new ResizeObserver(() => this.handleRendererResized());
      observer.observe(this.rendererEl);
      this.handleRendererResized();
    }
  }
  handleRendererResized() {
    readDOM(() => {
      if (this.rendererEl != null) {
        const rect = this.rendererEl.getBoundingClientRect();
        this.boxLength = Math.min(rect.width, rect.height);
      }
    });
  }
  /**
   * @ignore
   */
  render() {
    return (h(Host, null,
      h("vertex-viewer-dom-renderer", { ref: (ref) => (this.rendererEl = ref), class: "renderer", camera: this.camera },
        !this.triadOff && (h("vertex-viewer-dom-group", { class: "triad", position: this.triadPosition },
          h(TriadAxis, { label: "X", length: this.boxLength, rotationAxis: Vector3.origin() }),
          h(TriadAxis, { label: "Y", length: this.boxLength, rotationAxis: Vector3.back() }),
          h(TriadAxis, { label: "Z", length: this.boxLength, rotationAxis: Vector3.down() }))),
        h("vertex-viewer-dom-group", { class: "cube", matrix: this.worldOrientation.matrix },
          h(ViewCubeShadow, { length: this.boxLength + 10 }),
          h(ViewCubeSide, { label: this.zPositiveLabel, length: this.boxLength, side: "front", onPointerDown: this.handleStandardView(StandardView.FRONT), disabled: this.standardViewsOff }),
          h(ViewCubeSide, { label: this.zNegativeLabel, length: this.boxLength, side: "back", onPointerDown: this.handleStandardView(StandardView.BACK), disabled: this.standardViewsOff }),
          h(ViewCubeSide, { label: this.xNegativeLabel, length: this.boxLength, side: "left", onPointerDown: this.handleStandardView(StandardView.RIGHT), disabled: this.standardViewsOff }),
          h(ViewCubeSide, { label: this.xPositiveLabel, length: this.boxLength, side: "right", onPointerDown: this.handleStandardView(StandardView.LEFT), disabled: this.standardViewsOff }),
          h(ViewCubeSide, { label: this.yPositiveLabel, length: this.boxLength, side: "top", onPointerDown: this.handleStandardView(StandardView.TOP), disabled: this.standardViewsOff }),
          h(ViewCubeSide, { label: this.yNegativeLabel, length: this.boxLength, side: "bottom", onPointerDown: this.handleStandardView(StandardView.BOTTOM), disabled: this.standardViewsOff }),
          h(ViewCubeEdge, { length: this.boxLength, face1Side: "top", face1Edge: "bottom", face2Side: "front", face2Edge: "top", onPointerDown: this.handleStandardView(StandardView.TOP_FRONT), disabled: this.standardViewsOff }),
          h(ViewCubeEdge, { length: this.boxLength, face1Side: "front", face1Edge: "right", face2Side: "right", face2Edge: "left", onPointerDown: this.handleStandardView(StandardView.FRONT_LEFT), disabled: this.standardViewsOff }),
          h(ViewCubeEdge, { length: this.boxLength, face1Side: "bottom", face1Edge: "top", face2Side: "front", face2Edge: "bottom", onPointerDown: this.handleStandardView(StandardView.BOTTOM_FRONT), disabled: this.standardViewsOff }),
          h(ViewCubeEdge, { length: this.boxLength, face1Side: "front", face1Edge: "left", face2Side: "left", face2Edge: "right", onPointerDown: this.handleStandardView(StandardView.FRONT_RIGHT), disabled: this.standardViewsOff }),
          h(ViewCubeEdge, { length: this.boxLength, face1Side: "top", face1Edge: "right", face2Side: "right", face2Edge: "top", onPointerDown: this.handleStandardView(StandardView.TOP_LEFT), disabled: this.standardViewsOff }),
          h(ViewCubeEdge, { length: this.boxLength, face1Side: "back", face1Edge: "left", face2Side: "right", face2Edge: "right", onPointerDown: this.handleStandardView(StandardView.BACK_LEFT), disabled: this.standardViewsOff }),
          h(ViewCubeEdge, { length: this.boxLength, face1Side: "bottom", face1Edge: "right", face2Side: "right", face2Edge: "bottom", onPointerDown: this.handleStandardView(StandardView.BOTTOM_LEFT), disabled: this.standardViewsOff }),
          h(ViewCubeEdge, { length: this.boxLength, face1Side: "top", face1Edge: "top", face2Side: "back", face2Edge: "top", onPointerDown: this.handleStandardView(StandardView.TOP_BACK), disabled: this.standardViewsOff }),
          h(ViewCubeEdge, { length: this.boxLength, face1Side: "back", face1Edge: "right", face2Side: "left", face2Edge: "left", onPointerDown: this.handleStandardView(StandardView.BACK_RIGHT), disabled: this.standardViewsOff }),
          h(ViewCubeEdge, { length: this.boxLength, face1Side: "bottom", face1Edge: "bottom", face2Side: "back", face2Edge: "bottom", onPointerDown: this.handleStandardView(StandardView.BOTTOM_BACK), disabled: this.standardViewsOff }),
          h(ViewCubeEdge, { length: this.boxLength, face1Side: "top", face1Edge: "left", face2Side: "left", face2Edge: "top", onPointerDown: this.handleStandardView(StandardView.TOP_RIGHT), disabled: this.standardViewsOff }),
          h(ViewCubeEdge, { length: this.boxLength, face1Side: "bottom", face1Edge: "left", face2Side: "left", face2Edge: "bottom", onPointerDown: this.handleStandardView(StandardView.BOTTOM_RIGHT), disabled: this.standardViewsOff }),
          h(ViewCubeCorner, { length: this.boxLength, face1Side: "top", face1Corner: "bottom-left", face2Side: "front", face2Corner: "top-left", face3Side: "left", face3Corner: "top-right", onPointerDown: this.handleStandardView(StandardView.TOP_FRONT_RIGHT), disabled: this.standardViewsOff }),
          h(ViewCubeCorner, { length: this.boxLength, face1Side: "top", face1Corner: "bottom-right", face2Side: "front", face2Corner: "top-right", face3Side: "right", face3Corner: "top-left", onPointerDown: this.handleStandardView(StandardView.TOP_FRONT_LEFT), disabled: this.standardViewsOff }),
          h(ViewCubeCorner, { length: this.boxLength, face1Side: "bottom", face1Corner: "top-right", face2Side: "front", face2Corner: "bottom-right", face3Side: "right", face3Corner: "bottom-left", onPointerDown: this.handleStandardView(StandardView.BOTTOM_FRONT_LEFT), disabled: this.standardViewsOff }),
          h(ViewCubeCorner, { length: this.boxLength, face1Side: "bottom", face1Corner: "top-left", face2Side: "front", face2Corner: "bottom-left", face3Side: "left", face3Corner: "bottom-right", onPointerDown: this.handleStandardView(StandardView.BOTTOM_FRONT_RIGHT), disabled: this.standardViewsOff }),
          h(ViewCubeCorner, { length: this.boxLength, face1Side: "top", face1Corner: "top-right", face2Side: "back", face2Corner: "top-left", face3Side: "right", face3Corner: "top-right", onPointerDown: this.handleStandardView(StandardView.TOP_BACK_LEFT), disabled: this.standardViewsOff }),
          h(ViewCubeCorner, { length: this.boxLength, face1Side: "top", face1Corner: "top-left", face2Side: "back", face2Corner: "top-right", face3Side: "left", face3Corner: "top-left", onPointerDown: this.handleStandardView(StandardView.TOP_BACK_RIGHT), disabled: this.standardViewsOff }),
          h(ViewCubeCorner, { length: this.boxLength, face1Side: "bottom", face1Corner: "bottom-left", face2Side: "back", face2Corner: "bottom-right", face3Side: "left", face3Corner: "bottom-left", onPointerDown: this.handleStandardView(StandardView.BOTTOM_BACK_RIGHT), disabled: this.standardViewsOff }),
          h(ViewCubeCorner, { length: this.boxLength, face1Side: "bottom", face1Corner: "bottom-right", face2Side: "back", face2Corner: "bottom-left", face3Side: "right", face3Corner: "bottom-right", onPointerDown: this.handleStandardView(StandardView.BOTTOM_BACK_LEFT), disabled: this.standardViewsOff })))));
  }
  static get is() { return "vertex-viewer-view-cube"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["viewer-view-cube.css"]
  }; }
  static get styleUrls() { return {
    "$": ["viewer-view-cube.css"]
  }; }
  static get properties() { return {
    "xPositiveLabel": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The label for the side of the cube on the positive x-axis."
      },
      "attribute": "x-positive-label",
      "reflect": false,
      "defaultValue": "'Right'"
    },
    "xNegativeLabel": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The label for the side of the cube on the negative x-axis."
      },
      "attribute": "x-negative-label",
      "reflect": false,
      "defaultValue": "'Left'"
    },
    "yPositiveLabel": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The label for the side of the cube on the positive y-axis."
      },
      "attribute": "y-positive-label",
      "reflect": false,
      "defaultValue": "'Top'"
    },
    "yNegativeLabel": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The label for the side of the cube on the negative y-axis."
      },
      "attribute": "y-negative-label",
      "reflect": false,
      "defaultValue": "'Bottom'"
    },
    "zPositiveLabel": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The label for the side of the cube on the positive z-axis."
      },
      "attribute": "z-positive-label",
      "reflect": false,
      "defaultValue": "'Front'"
    },
    "zNegativeLabel": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The label for the side of the cube on the negative z-axis."
      },
      "attribute": "z-negative-label",
      "reflect": false,
      "defaultValue": "'Back'"
    },
    "standardViewsOff": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disables interactions for standard views."
      },
      "attribute": "standard-views-off",
      "reflect": false,
      "defaultValue": "false"
    },
    "viewAll": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Whether to perform a `viewAll` when clicking on the view cube. If this\nis set to `false`, the current `lookAt` point will be maintained, and the\ncamera's `position` and `up` vectors will be aligned to the standard view.\nDefaults to `true`.\n\n**Note** Setting this value to `false` can result in the camera being placed\nunderneath geometry depending on the current `viewVector` length, resulting\nin a view that may be unexpected."
      },
      "attribute": "view-all",
      "reflect": false,
      "defaultValue": "true"
    },
    "animationDuration": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The duration of the animation, in milliseconds, when a user performs a\nstandard view interaction. Set to 0 to disable animations."
      },
      "attribute": "animation-duration",
      "reflect": false,
      "defaultValue": "500"
    },
    "triadOff": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disables the display of the triad."
      },
      "attribute": "triad-off",
      "reflect": false,
      "defaultValue": "false"
    },
    "worldOrientation": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Orientation",
        "resolved": "Orientation",
        "references": {
          "Orientation": {
            "location": "import",
            "path": "../../lib/types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      },
      "defaultValue": "Orientation.DEFAULT"
    },
    "camera": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "FramePerspectiveCamera",
        "resolved": "FramePerspectiveCamera | undefined",
        "references": {
          "FramePerspectiveCamera": {
            "location": "import",
            "path": "../../lib/types"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      }
    },
    "viewer": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "HTMLVertexViewerElement",
        "resolved": "HTMLVertexViewerElement | undefined",
        "references": {
          "HTMLVertexViewerElement": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The viewer element that is connected to the view cube."
      }
    }
  }; }
  static get states() { return {
    "boxLength": {},
    "triadPosition": {}
  }; }
  static get watchers() { return [{
      "propName": "viewer",
      "methodName": "handleViewerChanged"
    }]; }
}
//# sourceMappingURL=viewer-view-cube.js.map