/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { Component, Element, Event, h, Host, Listen, Method, Prop, State, Watch, } from '@stencil/core';
import { Dimensions, Point } from '@vertexvis/geometry';
import { SceneViewAPIClient } from '@vertexvis/scene-view-protos/sceneview/protos/scene_view_api_pb_service';
import { toProtoDuration, WebSocketClientImpl } from '@vertexvis/stream-api';
import { Color, EventDispatcher, UUID } from '@vertexvis/utils';
import classnames from 'classnames';
import { AnnotationController, } from '../../lib/annotations/controller';
import { parseAndValidateConfig, } from '../../lib/config';
import { CursorManager } from '../../lib/cursors';
import { cssCursor } from '../../lib/dom';
import { ComponentInitializationError, InteractionHandlerError, ViewerInitializationError, } from '../../lib/errors';
import { InteractionApiOrthographic, InteractionApiPerspective, } from '../../lib/interactions';
import { FlyToPartKeyInteraction } from '../../lib/interactions/flyToPartKeyInteraction';
import { FlyToPositionKeyInteraction } from '../../lib/interactions/flyToPositionKeyInteraction';
import { MouseInteractionHandler } from '../../lib/interactions/mouseInteractionHandler';
import { MultiPointerInteractionHandler } from '../../lib/interactions/multiPointerInteractionHandler';
import { PointerInteractionHandler } from '../../lib/interactions/pointerInteractionHandler';
import { TapInteractionHandler } from '../../lib/interactions/tapInteractionHandler';
import { TouchInteractionHandler } from '../../lib/interactions/touchInteractionHandler';
import { fromPbFrameOrThrow, toPbCameraTypeOrThrow } from '../../lib/mappers';
import { paintTime } from '../../lib/meters';
import { ModelViewController } from '../../lib/model-views/controller';
import { PmiController } from '../../lib/pmi';
import { createCanvasRenderer, measureCanvasRenderer, } from '../../lib/rendering';
import { SceneItemController } from '../../lib/scene-items/controller';
import { Scene } from '../../lib/scenes/scene';
import { writeDOM } from '../../lib/stencil';
import { getStorageEntry, StorageKeys, upsertStorageEntry, } from '../../lib/storage';
import { ViewerStream } from '../../lib/stream/stream';
import { FrameCamera, SceneViewSummary, StencilBufferManager, Viewport, } from '../../lib/types';
import { DEFAULT_VIEWER_SCENE_WAIT_MS, getElementBoundingClientRect, getElementPropertyValue, } from './utils';
export class Viewer {
  constructor() {
    /**
     * Sets the default environment for the viewer. This setting is used for
     * auto-configuring network hosts.
     *
     * Use the `config` property for manually setting hosts.
     *
     * @see Viewer.config
     */
    this.configEnv = 'platprod';
    /**
     * Enables or disables the default mouse and touch interactions provided by
     * the viewer. Enabled by default.
     */
    this.cameraControls = true;
    /**
     * The type of camera model to represent the scene with. Can be either
     * `perspective` or `orthographic`, and defaults to `perspective`.
     */
    this.cameraType = 'perspective';
    /**
     * Enables or disables the default keyboard shortcut interactions provided by
     * the viewer. Enabled by default, requires `cameraControls` being enabled.
     *
     */
    this.keyboardControls = true;
    /**
     * Enables or disables the default rotation interaction being changed to
     * rotate around the pointer down location.
     */
    this.rotateAroundTapPoint = true;
    /**
     * Specifies how phantom parts should appear.
     * The opacity must be between 0 and 1, where 0 is completely hidden and 1 is completely visible.
     */
    this.phantom = { opacity: 0.1 };
    /**
     * Specifies whether to use the default lights for the scene. When false, default
     * lights are used. When true, no default lights are used, and the lights must
     * be specified separately.
     */
    this.noDefaultLights = false;
    /**
     * Specifies whether to enable temporal refinement of still images.
     */
    this.enableTemporalRefinement = true;
    /**
     * @private
     * @internal
     * Specifies experimental rendering options. For Vertex use only.
     */
    this.experimentalRenderingOptions = '';
    /**
     * An optional value that will debounce frame updates when resizing
     * this viewer element.
     */
    this.resizeDebounce = 100;
    /**
     * @internal
     */
    this.stencilBuffer = new StencilBufferManager(this.hostElement);
    /**
     * Represents the current viewport of the viewer. The viewport represents the
     * dimensions of the canvas where a frame is rendered. It contains methods for
     * translating between viewport coordinates, frame coordinates and world
     * coordinates.
     */
    this.viewport = Viewport.fromDimensions(Dimensions.create(0, 0));
    /**
     * This stores internal state that you want to preserve across live-reloads,
     * but shouldn't trigger a refresh if the data changes. Marking this with
     * @State to allow to preserve state across live-reloads.
     */
    this.stateMap = {
      cursorManager: new CursorManager(),
      streamState: { type: 'disconnected' },
    };
    this.interactionHandlers = [];
    this.defaultInteractionHandlerDisposables = [];
    this.tapKeyInteractions = [];
    this.defaultTapKeyInteractions = [];
    this.internalFrameDrawnDispatcher = new EventDispatcher();
    this.handleElementResize = this.handleElementResize.bind(this);
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    var _a;
    this.updateResolvedConfig();
    this.calculateComponentDimensions();
    this.resizeObserver = new ResizeObserver(this.handleElementResize);
    this.registerSlotChangeListeners();
    const config = this.getResolvedConfig();
    const client = new SceneViewAPIClient(config.network.sceneViewHost);
    this.annotations = new AnnotationController(client, () => this.token, () => this.deviceId);
    this.annotations.onStateChange.on((state) => this.annotationStateChanged.emit(state));
    this.stream =
      (_a = this.stream) !== null && _a !== void 0 ? _a : new ViewerStream(new WebSocketClientImpl(), {
        loggingEnabled: config.flags.logWsMessages,
        enableTemporalRefinement: this.enableTemporalRefinement,
      });
    this.addStreamListeners();
    this.modelViews = new ModelViewController(client, this.stream, () => this.token, () => this.deviceId);
    this.pmi = new PmiController(client, () => this.token, () => this.deviceId);
    this.sceneItems = new SceneItemController(client, () => this.token, () => this.deviceId);
    this.updateStreamAttributes();
    this.stateMap.cursorManager.onChanged.on(() => this.handleCursorChanged());
  }
  /**
   * @ignore
   */
  async componentDidLoad() {
    var _a;
    this.interactionApi = this.createInteractionApi();
    if (this.canvasContainerElement != null) {
      (_a = this.resizeObserver) === null || _a === void 0 ? void 0 : _a.observe(this.canvasContainerElement);
    }
    if (this.src != null) {
      this.load(this.src, { cameraType: this.cameraType }).catch((e) => {
        console.error('Error loading scene', e);
      });
    }
    await this.initializeDefaultInteractionHandlers();
    this.injectViewerApi();
  }
  /**
   * @ignore
   */
  render() {
    var _a;
    return (h(Host, null,
      h("div", { ref: (ref) => (this.viewerContainerElement = ref), class: "viewer-container", style: { cursor: cssCursor((_a = this.cursor) !== null && _a !== void 0 ? _a : '') }, onContextMenu: (event) => event.preventDefault() },
        h("div", { ref: (ref) => (this.canvasContainerElement = ref), class: classnames('canvas-container', {
            'enable-pointer-events ': window.PointerEvent != null,
          }) },
          h("canvas", { ref: (ref) => {
              this.canvasElement = ref;
              this.stateMap.interactionTarget = ref;
            }, class: "canvas" }),
          this.errorMessage != null ? (h("div", { class: "error-message" }, this.errorMessage)) : null),
        h("slot", null))));
  }
  /**
   * @internal
   */
  async dispatchFrameDrawn(frame) {
    this.frame = frame;
    this.internalFrameDrawnDispatcher.emit(frame);
    this.frameDrawn.emit(frame);
  }
  /**
   * Registers and initializes an interaction handler with the viewer. Returns a
   * `Disposable` that should be used to deregister the interaction handler.
   *
   * `InteractionHandler`s are used to build custom mouse and touch interactions
   * for the viewer. Use `<vertex-viewer camera-controls="false" />` to disable
   * the default camera controls provided by the viewer.
   *
   * @example
   * ```
   * class CustomInteractionHandler extends InteractionHandler {
   *   private element: HTMLElement;
   *   private api: InteractionApi;
   *
   *   public dispose(): void {
   *     this.element.removeEventListener('click', this.handleElementClick);
   *   }
   *
   *   public initialize(element: HTMLElement, api: InteractionApi): void {
   *     this.api = api;
   *     this.element = element;
   *     this.element.addEventListener('click', this.handleElementClick);
   *   }
   *
   *   private handleElementClick = (event: MouseEvent) => {
   *     api.tap({ x: event.clientX, y: event.clientY });
   *   }
   * }
   *
   * const viewer = document.querySelector("vertex-viewer");
   * viewer.registerInteractionHandler(new CustomInteractionHandler);
   * ```
   *
   * @param interactionHandler The interaction handler to register.
   * @returns {Promise<void>} A promise containing the disposable to use to
   *  deregister the handler.
   */
  async registerInteractionHandler(interactionHandler) {
    this.interactionHandlers.push(interactionHandler);
    this.initializeInteractionHandler(interactionHandler);
    return {
      dispose: () => {
        const index = this.interactionHandlers.indexOf(interactionHandler);
        if (index !== -1) {
          this.interactionHandlers[index].dispose();
          this.interactionHandlers.splice(index, 1);
        }
      },
    };
  }
  /**
   * Registers a key interaction to be invoked when a specific set of
   * keys are pressed during a `tap` event.
   *
   * `KeyInteraction`s are used to build custom keyboard shortcuts for the
   * viewer using the current state of they keyboard to determine whether
   * the `fn` should be invoked. Use `<vertex-viewer keyboard-controls="false" />`
   * to disable the default keyboard shortcuts provided by the viewer.
   *
   * @example
   * ```
   * class CustomKeyboardInteraction extends KeyInteraction<TapEventDetails> {
   *   constructor(private viewer: HTMLVertexViewerElement) {}
   *
   *   public predicate(keyState: KeyState): boolean {
   *     return keyState['Alt'];
   *   }
   *
   *   public async fn(event: TapEventDetails) {
   *     const scene = await this.viewer.scene();
   *     const result = await scene.raycaster().hitItems(event.position);
   *
   *     if (result.hits.length > 0) {
   *       await scene
   *         .camera()
   *         .fitTo(q => q.withItemId(result.hits[0].itemId))
   *         .render();
   *     }
   *   }
   * }
   * ```
   *
   * @param keyInteraction - The `KeyInteraction` to register.
   */
  async registerTapKeyInteraction(keyInteraction) {
    this.tapKeyInteractions = [...this.tapKeyInteractions, keyInteraction];
  }
  /**
   * The HTML element that will handle interaction events from the user. Used by
   * components to listen for interaction events from the same element as the
   * viewer. Note, this property maybe removed in the future when refactoring
   * our interaction handling.
   *
   * @internal
   * @deprecated Use `InteractionHandler`.
   */
  async getInteractionTarget_DEPRECATED() {
    if (this.stateMap.interactionTarget != null) {
      return this.stateMap.interactionTarget;
    }
    else
      throw new Error('Interaction target is undefined.');
  }
  /**
   * Adds a cursor to the viewer, and displays it if the cursor has the highest
   * priority.
   *
   * Cursors are managed as a prioritized list. A cursor is displayed if it has
   * the highest priority or if the cursor is the most recently added cursor in
   * the set of cursors with the same priority.
   *
   * To remove a cursor, call `dispose()` on the returned disposable.
   *
   * @param cursor The cursor to add.
   * @param priority The priority of the cursor.
   * @returns A disposable that can be used to remove the cursor.
   * @see See {@link CursorManager} for constants to pass to `priority`.
   */
  async addCursor(cursor, priority) {
    return this.stateMap.cursorManager.add(cursor, priority);
  }
  async getInteractionHandlers() {
    return this.interactionHandlers;
  }
  /**
   * @internal
   * @ignore
   */
  async getKeyInteractions() {
    return this.tapKeyInteractions;
  }
  async getBaseInteractionHandler() {
    return this.baseInteractionHandler;
  }
  /**
   * @deprecated Use `token`.
   */
  async getJwt() {
    return this.token;
  }
  handleSrcChanged(src) {
    if (src != null) {
      this.load(src, { cameraType: this.cameraType });
    }
    else {
      this.unload();
    }
  }
  /**
   * @ignore
   */
  handleCameraControlsChanged() {
    this.initializeDefaultCameraInteractionHandlers();
  }
  /**
   * @ignore
   */
  handleKeyboardControlsChanged() {
    this.initializeDefaultKeyboardInteractionHandlers();
  }
  /**
   * @ignore
   */
  handleRotateAboutTapPointChanged() {
    var _a, _b;
    this.updateStreamAttributes();
    if (this.rotateAroundTapPoint) {
      (_a = this.baseInteractionHandler) === null || _a === void 0 ? void 0 : _a.setPrimaryInteractionType('rotate-point');
    }
    else {
      (_b = this.baseInteractionHandler) === null || _b === void 0 ? void 0 : _b.setPrimaryInteractionType('rotate');
    }
  }
  handleCameraTypeChanged(updatedCameraType, previousCameraType) {
    if (updatedCameraType !== previousCameraType) {
      this.updateCameraType();
    }
  }
  /**
   * @ignore
   */
  handleStreamAttributesChanged() {
    this.updateStreamAttributes();
  }
  /**
   * @ignore
   */
  handleEnableTemporalRefinementChanged() {
    this.updateEnableTemporalRefinement();
  }
  /**
   * @ignore
   */
  handleConfigChanged() {
    this.updateResolvedConfig();
  }
  /**
   * @ignore
   */
  handleConfigEnvChanged() {
    this.updateResolvedConfig();
  }
  /**
   * Loads the given scene into the viewer and return a `Promise` that
   * resolves when the scene has been loaded. The specified scene is
   * provided as a URN in the following format:
   *
   *  * `urn:vertex:scene:<sceneid>`
   *
   * @param urn The URN of the resource to load.
   * @param options Optional configurations when loading the scene.
   *    cameraType (Optional) The camera type to load. If not included, the default camera type for the resource will be used.
   */
  async load(urn, options) {
    var _a, _b;
    if (this.stream != null && this.dimensions != null) {
      const { EXPERIMENTAL_annotationPollingIntervalInMs } = this.getResolvedConfig();
      this.calculateComponentDimensions();
      this.stream.update({
        streamAttributes: this.getStreamAttributes(),
        config: parseAndValidateConfig(this.configEnv, this.config),
        dimensions: this.dimensions,
        frameBgColor: this.getBackgroundColor(),
      });
      await ((_a = this.stream) === null || _a === void 0 ? void 0 : _a.load(urn, this.clientId, this.getDeviceId(), this.getResolvedConfig(), options === null || options === void 0 ? void 0 : options.cameraType));
      this.sceneReady.emit();
      if (EXPERIMENTAL_annotationPollingIntervalInMs !== undefined) {
        (_b = this.annotations) === null || _b === void 0 ? void 0 : _b.connect(EXPERIMENTAL_annotationPollingIntervalInMs);
      }
    }
    else {
      throw new ViewerInitializationError('Cannot load scene. Viewer has not been initialized.');
    }
  }
  /**
   * Disconnects the websocket and removes any internal state associated with
   * the scene.
   */
  async unload() {
    var _a;
    if (this.stream != null) {
      (_a = this.annotations) === null || _a === void 0 ? void 0 : _a.disconnect();
      this.stream.disconnect();
      this.frame = undefined;
      this.errorMessage = undefined;
    }
    if (this.canvasElement != null) {
      const context = this.canvasElement.getContext('2d');
      if (context != null) {
        context.clearRect(0, 0, this.canvasElement.width, this.canvasElement.height);
      }
    }
  }
  /**
   * Disconnects the websocket and clears the internal state associated with
   * the scene before reconnecting to the same scene.
   */
  async reload() {
    if (this.src != null) {
      await this.unload();
      await this.load(this.src, { cameraType: this.cameraType });
    }
  }
  /**
   * Returns an object that is used to perform operations on the `Scene` that's
   * currently being viewed. These operations include updating items,
   * positioning the camera and performing hit tests.
   */
  async scene() {
    return this.createScene();
  }
  /**
   * Returns `true` indicating that the scene is ready to be interacted with.
   */
  async isSceneReady() {
    return this.stateMap.streamState.type === 'connected';
  }
  async handleTapEvent(event) {
    this.tapKeyInteractions
      .filter((i) => i.predicate(event.detail))
      .forEach((i) => i.fn(event.detail));
  }
  emitConnectionChange(status) {
    if (status.status === 'connected') {
      // NOTE: Uncomment once FSS is deployed.
      // if (status.jwt.length === 0) {
      //   throw new MissingJWTError('JWT is empty');
      // }
    }
    this.connectionChange.emit(status);
  }
  handleElementResize(entries) {
    const dimensionsHaveChanged = entries.length > 0 &&
      this.dimensions != null &&
      !Dimensions.isEqual(entries[0].contentRect, this.viewport);
    if (dimensionsHaveChanged) {
      if (this.resizeTimer != null) {
        clearTimeout(this.resizeTimer);
        this.resizeTimer = undefined;
      }
      if (!this.isResizing) {
        this.resizeTimer = setTimeout(() => {
          this.isResizing = true;
          this.isResizeUpdate = true;
          this.recalculateComponentDimensions();
        }, this.resizeDebounce);
      }
    }
  }
  registerSlotChangeListeners() {
    this.mutationObserver = new MutationObserver((_) => this.injectViewerApi());
    this.mutationObserver.observe(this.hostElement, {
      childList: true,
      subtree: true,
    });
    this.styleObserver = new MutationObserver((_) => this.syncViewerStyles());
    this.styleObserver.observe(this.hostElement, {
      attributes: true,
      attributeFilter: ['style'],
    });
  }
  injectViewerApi() {
    function queryChildren(el) {
      return Array.from(el.querySelectorAll('*'));
    }
    const children = queryChildren(this.hostElement);
    children
      .filter((node) => node.nodeName.startsWith('VERTEX-'))
      .reduce((elements, element) => [
      ...elements,
      element,
      ...queryChildren(element),
    ], [])
      .forEach((node) => {
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      node.viewer = this.hostElement;
    });
  }
  syncViewerStyles() {
    var _a;
    const backgroundColor = this.getBackgroundColor();
    (_a = this.stream) === null || _a === void 0 ? void 0 : _a.update({
      frameBgColor: backgroundColor,
      streamAttributes: this.getStreamAttributes(),
    });
  }
  calculateComponentDimensions() {
    var _a;
    const maxPixelCount = 2073600;
    const bounds = this.getBounds();
    if ((bounds === null || bounds === void 0 ? void 0 : bounds.width) != null && (bounds === null || bounds === void 0 ? void 0 : bounds.height) != null) {
      const measuredViewport = Dimensions.create(bounds.width, bounds.height);
      const trimmedViewport = Dimensions.scaleFit(maxPixelCount, measuredViewport);
      this.hostDimensions = measuredViewport;
      this.dimensions =
        trimmedViewport != null
          ? Dimensions.create(trimmedViewport.width, trimmedViewport.height)
          : undefined;
      this.viewport = Viewport.fromDimensions((_a = this.getCanvasDimensions()) !== null && _a !== void 0 ? _a : Dimensions.create(0, 0));
    }
  }
  recalculateComponentDimensions() {
    var _a;
    if (this.isResizing) {
      this.calculateComponentDimensions();
      this.isResizing = false;
      if (((_a = this.stream) === null || _a === void 0 ? void 0 : _a.getState().type) === 'connected') {
        this.updateDimensions(this.dimensions);
      }
    }
  }
  reportPerformance(timings) {
    if (this.stateMap.streamState.type === 'connected') {
      const payload = {
        timings: timings.map((t) => ({
          receiveToPaintDuration: toProtoDuration(t.duration),
        })),
      };
      this.getStream().recordPerformance(payload, false);
    }
  }
  addStreamListeners() {
    this.stateMap.streamListeners = this.getStream().stateChanged.on((s) => {
      this.handleStreamStateChanged(this.stateMap.streamState, s);
    });
  }
  handleStreamStateChanged(previous, state) {
    this.stateMap.streamState = state;
    if (state.type === 'connecting') {
      this.handleConnecting(previous, state);
    }
    else if (state.type === 'connected') {
      this.handleConnected(previous, state);
    }
    else if (state.type === 'connection-failed') {
      this.handleConnectionFailed(previous, state);
    }
    else if (state.type === 'disconnected') {
      this.handleDisconnected(previous, state);
    }
  }
  handleConnecting(previous, state) {
    if (previous.type !== 'connecting') {
      this.token = undefined;
      this.errorMessage = undefined;
      this.emitConnectionChange({ status: 'connecting' });
    }
  }
  handleConnected(previous, state) {
    var _a;
    this.token = state.token.token;
    if (previous.type !== 'connected') {
      this.errorMessage = undefined;
      this.canvasRenderer = measureCanvasRenderer(paintTime, createCanvasRenderer(), this.getResolvedConfig().flags.logFrameRate, (timings) => this.reportPerformance(timings));
      this.emitConnectionChange({
        status: 'connected',
        jwt: state.token.token,
      });
      this.deviceIdChange.emit(state.deviceId);
    }
    if (((_a = this.frame) === null || _a === void 0 ? void 0 : _a.getId()) !== state.frame.getId()) {
      this.updateFrame(state.frame);
    }
    this.updateDimensions(this.dimensions);
  }
  handleConnectionFailed(previous, state) {
    if (previous.type !== 'connection-failed') {
      this.token = undefined;
      this.errorMessage = state.message;
      this.emitConnectionChange({
        status: 'connection-failed',
        errorMessage: state.message,
      });
    }
  }
  handleDisconnected(previous, state) {
    if (previous.type !== 'disconnected') {
      this.token = undefined;
      this.errorMessage = undefined;
      this.emitConnectionChange({ status: 'disconnected' });
    }
  }
  updateDimensions(dimensions) {
    var _a;
    (_a = this.stream) === null || _a === void 0 ? void 0 : _a.update({ dimensions });
    this.dimensionschange.emit(dimensions);
  }
  async updateFrame(frame) {
    var _a;
    const canvasDimensions = this.getCanvasDimensions();
    if (this.canvasElement != null &&
      canvasDimensions != null &&
      ((_a = this.frame) === null || _a === void 0 ? void 0 : _a.getId()) !== frame.getId()) {
      const canvas = this.canvasElement.getContext('2d');
      if (canvas != null) {
        const previousFrame = this.frame;
        this.frame = SceneViewSummary.copySummaryIfInvalid(frame, previousFrame);
        this.updateInteractionApi(previousFrame);
        const data = {
          canvas,
          canvasDimensions,
          dimensions: this.dimensions,
          frame: this.frame,
          viewport: this.viewport,
          beforeDraw: () => {
            this.updateCanvasDimensions(canvasDimensions);
            this.isResizeUpdate = false;
          },
          predicate: () => {
            if (this.isResizeUpdate) {
              return (this.dimensions == null ||
                Dimensions.isEqual(this.dimensions, data.frame.image.imageAttr.frameDimensions));
            }
            return true;
          },
        };
        this.frameReceived.emit(this.frame);
        if (this.frame.scene.hasChanged) {
          this.sceneChanged.emit();
        }
        const drawnFrame = await this.canvasRenderer(data);
        if (drawnFrame != null) {
          this.updateViewerBackground();
          this.dispatchFrameDrawn(drawnFrame);
        }
      }
    }
  }
  updateViewerBackground() {
    const backgroundColor = this.getBackgroundColor();
    writeDOM(() => {
      var _a, _b;
      (_a = this.viewerContainerElement) === null || _a === void 0 ? void 0 : _a.style.setProperty('background', backgroundColor != null ? Color.toHexString(backgroundColor) : '#ffffff');
      (_b = this.canvasContainerElement) === null || _b === void 0 ? void 0 : _b.style.setProperty('background', backgroundColor != null ? Color.toHexString(backgroundColor) : '#ffffff');
    });
  }
  async initializeDefaultInteractionHandlers() {
    var _a;
    await this.initializeDefaultCameraInteractionHandlers();
    await this.initializeDefaultTapInteractionHandler();
    this.initializeDefaultKeyboardInteractionHandlers();
    if (this.rotateAroundTapPoint) {
      (_a = this.baseInteractionHandler) === null || _a === void 0 ? void 0 : _a.setPrimaryInteractionType('rotate-point');
    }
  }
  clearDefaultCameraInteractionHandlers() {
    this.defaultInteractionHandlerDisposables.forEach((disposable) => disposable.dispose());
    this.defaultInteractionHandlerDisposables = [];
  }
  clearDefaultKeyboardInteractions() {
    this.defaultTapKeyInteractions.forEach((interaction) => {
      const index = this.tapKeyInteractions.indexOf(interaction);
      if (index !== -1) {
        this.tapKeyInteractions.splice(index, 1);
      }
    });
    this.tapKeyInteractions = [];
  }
  async initializeDefaultCameraInteractionHandlers() {
    var _a, _b;
    this.clearDefaultCameraInteractionHandlers();
    if (this.cameraControls) {
      if (window.PointerEvent != null) {
        this.baseInteractionHandler =
          (_a = this.baseInteractionHandler) !== null && _a !== void 0 ? _a : new PointerInteractionHandler(() => this.getResolvedConfig());
        const baseDisposable = await this.registerInteractionHandler(this.baseInteractionHandler);
        const multiPointerDisposable = await this.registerInteractionHandler(new MultiPointerInteractionHandler());
        this.defaultInteractionHandlerDisposables = [
          baseDisposable,
          multiPointerDisposable,
        ];
      }
      else {
        // fallback to touch events and mouse events as a default
        this.baseInteractionHandler =
          (_b = this.baseInteractionHandler) !== null && _b !== void 0 ? _b : new MouseInteractionHandler(() => this.getResolvedConfig());
        const baseDisposable = await this.registerInteractionHandler(this.baseInteractionHandler);
        const touchDisposable = await this.registerInteractionHandler(new TouchInteractionHandler());
        this.defaultInteractionHandlerDisposables = [
          baseDisposable,
          touchDisposable,
        ];
      }
    }
  }
  initializeDefaultKeyboardInteractionHandlers() {
    var _a;
    this.clearDefaultKeyboardInteractions();
    if (this.keyboardControls && this.stream != null) {
      (_a = this.baseInteractionHandler) === null || _a === void 0 ? void 0 : _a.setDefaultKeyboardControls(this.keyboardControls);
      const flyToPart = new FlyToPartKeyInteraction(this.stream, () => this.getResolvedConfig(), () => this.getImageScale());
      const flyToPosition = new FlyToPositionKeyInteraction(this.stream, () => this.getResolvedConfig(), () => this.getImageScale(), () => this.createScene());
      this.registerTapKeyInteraction(flyToPart);
      this.registerTapKeyInteraction(flyToPosition);
      this.defaultTapKeyInteractions = [flyToPart, flyToPosition];
    }
  }
  async initializeDefaultTapInteractionHandler() {
    if (this.tapHandlerDisposable == null) {
      if (window.PointerEvent != null) {
        const tapInteractionHandler = new TapInteractionHandler('pointerdown', 'pointerup', 'pointermove', () => this.getResolvedConfig());
        this.tapHandlerDisposable = await this.registerInteractionHandler(tapInteractionHandler);
      }
      else {
        const tapInteractionHandler = new TapInteractionHandler('mousedown', 'mouseup', 'mousemove', () => this.getResolvedConfig());
        this.tapHandlerDisposable = await this.registerInteractionHandler(tapInteractionHandler);
      }
    }
  }
  initializeInteractionHandler(handler) {
    if (this.stateMap.interactionTarget == null) {
      throw new InteractionHandlerError('Cannot initialize interaction handler. Interaction target is undefined.');
    }
    if (this.interactionApi == null) {
      throw new InteractionHandlerError('Cannot initialize interaction handler. Interaction APi is undefined.');
    }
    handler.initialize(this.stateMap.interactionTarget, this.interactionApi);
  }
  createInteractionApi() {
    if (this.stream == null) {
      throw new ComponentInitializationError('Cannot create interaction API. Component has not been initialized.');
    }
    return this.frame == null || this.frame.scene.camera.isPerspective()
      ? new InteractionApiPerspective(this.stream, this.stateMap.cursorManager, () => this.getResolvedConfig().interactions, () => this.createScene(), () => this.frame, () => this.viewport, this.tap, this.doubletap, this.longpress, this.interactionStarted, this.interactionFinished)
      : new InteractionApiOrthographic(this.stream, this.stateMap.cursorManager, () => this.getResolvedConfig().interactions, () => this.createScene(), () => this.frame, () => this.viewport, this.tap, this.doubletap, this.longpress, this.interactionStarted, this.interactionFinished);
  }
  handleCursorChanged() {
    window.requestAnimationFrame(() => {
      this.cursor = this.stateMap.cursorManager.getActiveCursor();
    });
  }
  async createScene() {
    const state = await this.waitForConnectedState();
    const cameraTypeMapper = toPbCameraTypeOrThrow();
    const { frame, sceneId, sceneViewId, worldOrientation } = state;
    return new Scene(this.getStream(), frame, fromPbFrameOrThrow(worldOrientation), cameraTypeMapper, () => this.getImageScale(), this.viewport, sceneId, sceneViewId);
  }
  /**
   * This function is currently not in use, but will be required
   * when we want to automatically configure the background color of
   * JPEG images.
   */
  getBackgroundColor() {
    if (this.canvasContainerElement != null) {
      const imageBackground = getElementPropertyValue(this.hostElement, '--image-background');
      const viewerBackground = getElementPropertyValue(this.hostElement, '--viewer-background');
      const propertyColor = imageBackground !== null && imageBackground !== void 0 ? imageBackground : viewerBackground;
      return propertyColor != null
        ? Color.fromCss(propertyColor)
        : Color.create(255, 255, 255);
    }
  }
  getBounds() {
    return getElementBoundingClientRect(this.hostElement);
  }
  getCanvasDimensions() {
    return this.getResolvedConfig().flags.letterboxFrames
      ? this.dimensions
      : this.hostDimensions;
  }
  getImageScale() {
    const canvasDimensions = this.getCanvasDimensions();
    if (this.dimensions != null && canvasDimensions != null) {
      return Point.create(this.dimensions.width / canvasDimensions.width, this.dimensions.height / canvasDimensions.height);
    }
  }
  getStreamAttributes() {
    return {
      depthBuffers: this.getDepthBufferStreamAttributesValue(),
      experimentalRenderingOptions: this.experimentalRenderingOptions,
      featureHighlighting: this.featureHighlighting,
      featureLines: this.featureLines,
      featureMaps: this.featureMaps,
      frames: {
        frameBackgroundColor: this.getBackgroundColor(),
      },
      noDefaultLights: this.noDefaultLights,
      phantom: this.phantom,
      sceneComparison: this.sceneComparison,
      selectionHighlighting: this.selectionHighlighting,
    };
  }
  updateCanvasDimensions(dimensions) {
    if (this.canvasElement != null) {
      this.canvasElement.width = dimensions.width;
      this.canvasElement.height = dimensions.height;
    }
  }
  updateStreamAttributes() {
    var _a;
    (_a = this.stream) === null || _a === void 0 ? void 0 : _a.update({ streamAttributes: this.getStreamAttributes() });
  }
  updateEnableTemporalRefinement() {
    var _a;
    (_a = this.stream) === null || _a === void 0 ? void 0 : _a.update({
      enableTemporalRefinement: this.enableTemporalRefinement,
    });
  }
  updateInteractionApi(previousFrame) {
    if (this.frame != null) {
      const hasChangedFromPerspective = (previousFrame == null || previousFrame.scene.camera.isPerspective()) &&
        this.frame.scene.camera.isOrthographic();
      const hasChangedFromOrthographic = (previousFrame == null ||
        previousFrame.scene.camera.isOrthographic()) &&
        this.frame.scene.camera.isPerspective();
      if (hasChangedFromPerspective || hasChangedFromOrthographic) {
        this.interactionApi = this.createInteractionApi();
        this.cameraType = this.frame.scene.camera.isPerspective()
          ? 'perspective'
          : 'orthographic';
        this.cameraTypeChanged.emit(this.cameraType);
        this.interactionHandlers.forEach((handler) => this.initializeInteractionHandler(handler));
      }
    }
  }
  updateCameraType() {
    var _a, _b;
    if (this.frame != null) {
      if (this.cameraType === 'orthographic' &&
        this.frame.scene.camera.isPerspective()) {
        (_a = this.stream) === null || _a === void 0 ? void 0 : _a.replaceCamera({
          camera: FrameCamera.toProtobuf(FrameCamera.toOrthographic(this.frame.scene.camera, this.frame.scene.boundingBox)),
        });
      }
      else if (this.cameraType === 'perspective' &&
        this.frame.scene.camera.isOrthographic()) {
        (_b = this.stream) === null || _b === void 0 ? void 0 : _b.replaceCamera({
          camera: FrameCamera.toProtobuf(FrameCamera.toPerspective(this.frame.scene.camera)),
        });
      }
    }
  }
  getDepthBufferStreamAttributesValue() {
    var _a;
    const depthBuffer = (_a = this.depthBuffers) !== null && _a !== void 0 ? _a : (this.rotateAroundTapPoint ? 'final' : undefined);
    return depthBuffer;
  }
  updateResolvedConfig() {
    this.resolvedConfig = parseAndValidateConfig(this.configEnv, this.config);
  }
  getResolvedConfig() {
    return getRequiredProp('Resolved config is undefined', () => this.resolvedConfig);
  }
  getStream() {
    return getRequiredProp('Stream is undefined', () => this.stream);
  }
  getDeviceId() {
    if (this.deviceId == null) {
      try {
        this.deviceId = getStorageEntry(StorageKeys.DEVICE_ID, (entry) => entry['device-id']);
      }
      catch (e) {
        console.warn('Cannot read device ID. Local storage is not supported.');
      }
      if (this.deviceId == null) {
        this.deviceId = UUID.create();
        try {
          upsertStorageEntry(StorageKeys.DEVICE_ID, {
            ['device-id']: this.deviceId,
          });
        }
        catch (e) {
          console.warn('Cannot write device ID. Local storage is not supported.');
        }
      }
    }
    return this.deviceId;
  }
  async waitForConnectedState() {
    if (this.stateMap.streamState.type !== 'connected') {
      console.debug('Stream was not in a connected state. Waiting for successful connection.');
      return new Promise((resolve, reject) => {
        const disposable = this.getStream().onStateChanged((state) => {
          if (state.type === 'connected') {
            resolve(state);
          }
        });
        setTimeout(() => {
          disposable.dispose();
          reject(new Error('Timed out waiting for connected state.'));
        }, DEFAULT_VIEWER_SCENE_WAIT_MS);
      });
    }
    return this.stateMap.streamState;
  }
  static get is() { return "vertex-viewer"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["viewer.css"]
  }; }
  static get styleUrls() { return {
    "$": ["viewer.css"]
  }; }
  static get properties() { return {
    "src": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "A URN of the scene resource to load when the component is mounted in the\nDOM tree. The specified resource is a URN in the following format:\n\n * `urn:vertex:scene:<sceneid>`"
      },
      "attribute": "src",
      "reflect": false
    },
    "clientId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The Client ID associated with your Vertex Application."
      },
      "attribute": "client-id",
      "reflect": false
    },
    "deviceId": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [{
            "name": "private",
            "text": undefined
          }, {
            "name": "internal",
            "text": undefined
          }],
        "text": "Property used for internals or testing."
      },
      "attribute": "device-id",
      "reflect": false
    },
    "config": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "PartialConfig | string",
        "resolved": "Config | string | undefined",
        "references": {
          "PartialConfig": {
            "location": "import",
            "path": "../../lib/config"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "An object or JSON encoded string that defines configuration settings for\nthe viewer."
      },
      "attribute": "config",
      "reflect": false
    },
    "configEnv": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Environment",
        "resolved": "\"platdev\" | \"platprod\" | \"platstaging\"",
        "references": {
          "Environment": {
            "location": "import",
            "path": "../../lib/environment"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "see",
            "text": "Viewer.config"
          }],
        "text": "Sets the default environment for the viewer. This setting is used for\nauto-configuring network hosts.\n\nUse the `config` property for manually setting hosts."
      },
      "attribute": "config-env",
      "reflect": false,
      "defaultValue": "'platprod'"
    },
    "resolvedConfig": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Config",
        "resolved": "Config | undefined",
        "references": {
          "Config": {
            "location": "import",
            "path": "../../lib/config"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      }
    },
    "cameraControls": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Enables or disables the default mouse and touch interactions provided by\nthe viewer. Enabled by default."
      },
      "attribute": "camera-controls",
      "reflect": false,
      "defaultValue": "true"
    },
    "cameraType": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "FrameCameraType",
        "resolved": "\"orthographic\" | \"perspective\"",
        "references": {
          "FrameCameraType": {
            "location": "import",
            "path": "../../lib/types/frameCamera"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The type of camera model to represent the scene with. Can be either\n`perspective` or `orthographic`, and defaults to `perspective`."
      },
      "attribute": "camera-type",
      "reflect": true,
      "defaultValue": "'perspective'"
    },
    "keyboardControls": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Enables or disables the default keyboard shortcut interactions provided by\nthe viewer. Enabled by default, requires `cameraControls` being enabled."
      },
      "attribute": "keyboard-controls",
      "reflect": false,
      "defaultValue": "true"
    },
    "rotateAroundTapPoint": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Enables or disables the default rotation interaction being changed to\nrotate around the pointer down location."
      },
      "attribute": "rotate-around-tap-point",
      "reflect": false,
      "defaultValue": "true"
    },
    "token": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": "A token that can be used to make API calls to other Vertex services."
      },
      "attribute": "token",
      "reflect": false
    },
    "depthBuffers": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "FrameType",
        "resolved": "\"all\" | \"final\" | undefined",
        "references": {
          "FrameType": {
            "location": "import",
            "path": "../../interfaces"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Specifies when a depth buffer is requested from rendering. Possible values\nare:\n\n * `undefined`: A depth buffer is never requested.\n * `final`: A depth buffer is only requested on the final frame.\n * `all`: A depth buffer is requested for every frame.\n\nDepth buffers can increase the amount of data that's sent to a client and\ncan impact rendering performance. Values of `undefined` or `final` should\nbe used when needing the highest rendering performance."
      },
      "attribute": "depth-buffers",
      "reflect": false
    },
    "phantom": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "PhantomOptions",
        "resolved": "PhantomOptions | undefined",
        "references": {
          "PhantomOptions": {
            "location": "import",
            "path": "../../interfaces"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Specifies how phantom parts should appear.\nThe opacity must be between 0 and 1, where 0 is completely hidden and 1 is completely visible."
      },
      "defaultValue": "{ opacity: 0.1 }"
    },
    "noDefaultLights": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specifies whether to use the default lights for the scene. When false, default\nlights are used. When true, no default lights are used, and the lights must\nbe specified separately."
      },
      "attribute": "no-default-lights",
      "reflect": false,
      "defaultValue": "false"
    },
    "enableTemporalRefinement": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specifies whether to enable temporal refinement of still images."
      },
      "attribute": "enable-temporal-refinement",
      "reflect": false,
      "defaultValue": "true"
    },
    "experimentalRenderingOptions": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "private",
            "text": undefined
          }, {
            "name": "internal",
            "text": "Specifies experimental rendering options. For Vertex use only."
          }],
        "text": ""
      },
      "attribute": "experimental-rendering-options",
      "reflect": false,
      "defaultValue": "''"
    },
    "featureLines": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "FeatureLineOptions",
        "resolved": "FeatureLineOptions | undefined",
        "references": {
          "FeatureLineOptions": {
            "location": "import",
            "path": "../../interfaces"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Specifies if and how to render feature lines."
      }
    },
    "selectionHighlighting": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "SelectionHighlightingOptions",
        "resolved": "SelectionHighlightingOptions | undefined",
        "references": {
          "SelectionHighlightingOptions": {
            "location": "import",
            "path": "../../interfaces"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Specifies the halo selection properties.\nParameter notes:\n * lineWidth values supported currently are 0-5. This width is currently the value x2. For example, 1 will have a pixel width of 2.\n * color is optional. This will be the color of the selected items in the viewer.\n * opacity is also optional. The opacity will be applied to everything selected besides the highlighted outer line."
      }
    },
    "featureHighlighting": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "FeatureHighlightOptions",
        "resolved": "FeatureHighlightOptions | undefined",
        "references": {
          "FeatureHighlightOptions": {
            "location": "import",
            "path": "../../interfaces"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Specifies how selected features should be highlighted."
      }
    },
    "sceneComparison": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "SceneComparisonOptions",
        "resolved": "SceneComparisonOptions | undefined",
        "references": {
          "SceneComparisonOptions": {
            "location": "import",
            "path": "../../interfaces"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Specifies if and how to compare to another scene"
      }
    },
    "featureMaps": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "FrameType",
        "resolved": "\"all\" | \"final\" | undefined",
        "references": {
          "FrameType": {
            "location": "import",
            "path": "../../interfaces"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Specifies when a feature map is returned from rendering. Feature maps\ninclude information about the surfaces, edges and cross sections that are\nin a frame.\n\nPossible values are:\n\n * `undefined`: A feature map is never requested.\n * `final`: A feature map is only requested on the final frame.\n * `all`: A feature map is requested for every frame.\n\nFeature maps can increase the amount of data that's sent to a client and\ncan impact rendering performance. Values of `undefined` or `final` should\nbe used when needing the highest rendering performance."
      },
      "attribute": "feature-maps",
      "reflect": false
    },
    "resizeDebounce": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "An optional value that will debounce frame updates when resizing\nthis viewer element."
      },
      "attribute": "resize-debounce",
      "reflect": false,
      "defaultValue": "100"
    },
    "frame": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Frame | undefined",
        "resolved": "Frame | undefined",
        "references": {
          "Frame": {
            "location": "import",
            "path": "../../lib/types/frame"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "readonly",
            "text": undefined
          }],
        "text": "The last frame that was received, which can be used to inspect the scene\nand camera information."
      }
    },
    "stream": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "ViewerStream",
        "resolved": "ViewerStream | undefined",
        "references": {
          "ViewerStream": {
            "location": "import",
            "path": "../../lib/stream/stream"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      }
    },
    "stencilBuffer": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "StencilBufferManager",
        "resolved": "StencilBufferManager",
        "references": {
          "StencilBufferManager": {
            "location": "import",
            "path": "../../lib/types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      },
      "defaultValue": "new StencilBufferManager(\n    this.hostElement\n  )"
    },
    "viewport": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Viewport",
        "resolved": "Viewport",
        "references": {
          "Viewport": {
            "location": "import",
            "path": "../../lib/types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Represents the current viewport of the viewer. The viewport represents the\ndimensions of the canvas where a frame is rendered. It contains methods for\ntranslating between viewport coordinates, frame coordinates and world\ncoordinates."
      },
      "defaultValue": "Viewport.fromDimensions(Dimensions.create(0, 0))"
    },
    "annotations": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "AnnotationController | undefined",
        "resolved": "AnnotationController | undefined",
        "references": {
          "AnnotationController": {
            "location": "import",
            "path": "../../lib/annotations/controller"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "readonly",
            "text": undefined
          }],
        "text": "The annotation controller for accessing annotations associated with the\nscene view."
      }
    },
    "modelViews": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "ModelViewController | undefined",
        "resolved": "ModelViewController | undefined",
        "references": {
          "ModelViewController": {
            "location": "import",
            "path": "../../lib/model-views/controller"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "readonly",
            "text": undefined
          }],
        "text": "The controller for accessing model views associated with the scene view."
      }
    },
    "pmi": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "PmiController | undefined",
        "resolved": "PmiController | undefined",
        "references": {
          "PmiController": {
            "location": "import",
            "path": "../../lib/pmi"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "readonly",
            "text": undefined
          }],
        "text": "The controller for accessing and viewing PMI."
      }
    },
    "sceneItems": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "SceneItemController | undefined",
        "resolved": "SceneItemController | undefined",
        "references": {
          "SceneItemController": {
            "location": "import",
            "path": "../../lib/scene-items/controller"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "readonly",
            "text": undefined
          }],
        "text": "The controller for accessing and viewing SceneItems."
      }
    }
  }; }
  static get states() { return {
    "dimensions": {},
    "hostDimensions": {},
    "errorMessage": {},
    "cursor": {},
    "stateMap": {}
  }; }
  static get events() { return [{
      "method": "tap",
      "name": "tap",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [{
            "name": "see",
            "text": "{@link Scene.raycaster Scene.raycaster} for more information."
          }],
        "text": "Emits an event whenever the user taps or clicks a location in the viewer.\nThe event includes the location of the tap or click.\n\nThis event can be used in combination with the {@link VertexViewer.scene scene} method\nto query for items at the point of the tap."
      },
      "complexType": {
        "original": "TapEventDetails",
        "resolved": "TapEventDetails",
        "references": {
          "TapEventDetails": {
            "location": "import",
            "path": "../../lib/interactions/tapEventDetails"
          }
        }
      }
    }, {
      "method": "doubletap",
      "name": "doubletap",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits an event whenever the user double taps or clicks a location in the viewer.\nThe event includes the location of the first tap or click."
      },
      "complexType": {
        "original": "TapEventDetails",
        "resolved": "TapEventDetails",
        "references": {
          "TapEventDetails": {
            "location": "import",
            "path": "../../lib/interactions/tapEventDetails"
          }
        }
      }
    }, {
      "method": "longpress",
      "name": "longpress",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits an event whenever the user taps or clicks a location in the viewer and the\nconfigured amount of time passes without receiving a mouseup or touchend.\nThe event includes the location of the tap or click."
      },
      "complexType": {
        "original": "TapEventDetails",
        "resolved": "TapEventDetails",
        "references": {
          "TapEventDetails": {
            "location": "import",
            "path": "../../lib/interactions/tapEventDetails"
          }
        }
      }
    }, {
      "method": "frameReceived",
      "name": "frameReceived",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits an event when a frame has been received by the viewer. The event\nwill include details about the drawn frame, such as the `Scene` information\nrelated to the scene."
      },
      "complexType": {
        "original": "Frame",
        "resolved": "Frame",
        "references": {
          "Frame": {
            "location": "import",
            "path": "../../lib/types/frame"
          }
        }
      }
    }, {
      "method": "frameDrawn",
      "name": "frameDrawn",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits an event when a frame has been drawn to the viewer's canvas. The event\nwill include details about the drawn frame, such as the `Scene` information\nrelated to the scene."
      },
      "complexType": {
        "original": "Frame",
        "resolved": "Frame",
        "references": {
          "Frame": {
            "location": "import",
            "path": "../../lib/types/frame"
          }
        }
      }
    }, {
      "method": "tokenExpired",
      "name": "tokenExpired",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits an event when a provided oauth2 token is about to expire, or is about to expire,\ncausing issues with establishing a websocket connection, or performing API calls."
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "connectionChange",
      "name": "connectionChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits an event when the connection status changes for the viewer"
      },
      "complexType": {
        "original": "ConnectionStatus",
        "resolved": "ConnectedStatus | ConnectingStatus | ConnectionFailedStatus | DisconnectedStatus",
        "references": {
          "ConnectionStatus": {
            "location": "local"
          }
        }
      }
    }, {
      "method": "sceneReady",
      "name": "sceneReady",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits an event when the scene is ready to be interacted with."
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "sceneChanged",
      "name": "sceneChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits an event when a frame is received with a different scene attribute."
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "interactionStarted",
      "name": "interactionStarted",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits an event when the user has started an interaction."
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "interactionFinished",
      "name": "interactionFinished",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits an event when the user hs finished an interaction."
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "cameraTypeChanged",
      "name": "cameraTypeChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits an event when the camera type changes."
      },
      "complexType": {
        "original": "FrameCameraType",
        "resolved": "\"orthographic\" | \"perspective\"",
        "references": {
          "FrameCameraType": {
            "location": "import",
            "path": "../../lib/types/frameCamera"
          }
        }
      }
    }, {
      "method": "annotationStateChanged",
      "name": "annotationStateChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits an event when the state for annotation changes."
      },
      "complexType": {
        "original": "AnnotationState",
        "resolved": "AnnotationState",
        "references": {
          "AnnotationState": {
            "location": "import",
            "path": "../../lib/annotations/controller"
          }
        }
      }
    }, {
      "method": "deviceIdChange",
      "name": "deviceIdChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [{
            "name": "private",
            "text": undefined
          }],
        "text": "Used for internals or testing."
      },
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      }
    }, {
      "method": "dimensionschange",
      "name": "dimensionschange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "Dimensions.Dimensions",
        "resolved": "Dimensions",
        "references": {
          "Dimensions": {
            "location": "import",
            "path": "@vertexvis/geometry"
          }
        }
      }
    }]; }
  static get methods() { return {
    "dispatchFrameDrawn": {
      "complexType": {
        "signature": "(frame: Frame) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "Frame": {
            "location": "import",
            "path": "../../lib/types/frame"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": [{
            "name": "internal",
            "text": undefined
          }]
      }
    },
    "registerInteractionHandler": {
      "complexType": {
        "signature": "(interactionHandler: InteractionHandler) => Promise<Disposable>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "interactionHandler The interaction handler to register."
              }],
            "text": "The interaction handler to register."
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "Disposable": {
            "location": "import",
            "path": "@vertexvis/utils"
          },
          "InteractionHandler": {
            "location": "import",
            "path": "../../lib/interactions/interactionHandler"
          }
        },
        "return": "Promise<Disposable>"
      },
      "docs": {
        "text": "Registers and initializes an interaction handler with the viewer. Returns a\n`Disposable` that should be used to deregister the interaction handler.\n\n`InteractionHandler`s are used to build custom mouse and touch interactions\nfor the viewer. Use `<vertex-viewer camera-controls=\"false\" />` to disable\nthe default camera controls provided by the viewer.",
        "tags": [{
            "name": "example",
            "text": "```\nclass CustomInteractionHandler extends InteractionHandler {\n  private element: HTMLElement;\n  private api: InteractionApi;\n\n  public dispose(): void {\n    this.element.removeEventListener('click', this.handleElementClick);\n  }\n\n  public initialize(element: HTMLElement, api: InteractionApi): void {\n    this.api = api;\n    this.element = element;\n    this.element.addEventListener('click', this.handleElementClick);\n  }\n\n  private handleElementClick = (event: MouseEvent) => {\n    api.tap({ x: event.clientX, y: event.clientY });\n  }\n}\n\nconst viewer = document.querySelector(\"vertex-viewer\");\nviewer.registerInteractionHandler(new CustomInteractionHandler);\n```"
          }, {
            "name": "param",
            "text": "interactionHandler The interaction handler to register."
          }, {
            "name": "returns",
            "text": "A promise containing the disposable to use to\nderegister the handler."
          }]
      }
    },
    "registerTapKeyInteraction": {
      "complexType": {
        "signature": "(keyInteraction: KeyInteraction<TapEventDetails>) => Promise<void>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "keyInteraction - The `KeyInteraction` to register."
              }],
            "text": "- The `KeyInteraction` to register."
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "KeyInteraction": {
            "location": "import",
            "path": "../../lib/interactions/keyInteraction"
          },
          "TapEventDetails": {
            "location": "import",
            "path": "../../lib/interactions/tapEventDetails"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Registers a key interaction to be invoked when a specific set of\nkeys are pressed during a `tap` event.\n\n`KeyInteraction`s are used to build custom keyboard shortcuts for the\nviewer using the current state of they keyboard to determine whether\nthe `fn` should be invoked. Use `<vertex-viewer keyboard-controls=\"false\" />`\nto disable the default keyboard shortcuts provided by the viewer.",
        "tags": [{
            "name": "example",
            "text": "```\nclass CustomKeyboardInteraction extends KeyInteraction<TapEventDetails> {\n  constructor(private viewer: HTMLVertexViewerElement) {}\n\n  public predicate(keyState: KeyState): boolean {\n    return keyState['Alt'];\n  }\n\n  public async fn(event: TapEventDetails) {\n    const scene = await this.viewer.scene();\n    const result = await scene.raycaster().hitItems(event.position);\n\n    if (result.hits.length > 0) {\n      await scene\n        .camera()\n        .fitTo(q => q.withItemId(result.hits[0].itemId))\n        .render();\n    }\n  }\n}\n```"
          }, {
            "name": "param",
            "text": "keyInteraction - The `KeyInteraction` to register."
          }]
      }
    },
    "getInteractionTarget_DEPRECATED": {
      "complexType": {
        "signature": "() => Promise<HTMLElement>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLElement": {
            "location": "global"
          }
        },
        "return": "Promise<HTMLElement>"
      },
      "docs": {
        "text": "The HTML element that will handle interaction events from the user. Used by\ncomponents to listen for interaction events from the same element as the\nviewer. Note, this property maybe removed in the future when refactoring\nour interaction handling.",
        "tags": [{
            "name": "internal",
            "text": undefined
          }, {
            "name": "deprecated",
            "text": "Use `InteractionHandler`."
          }]
      }
    },
    "addCursor": {
      "complexType": {
        "signature": "(cursor: Cursor, priority?: number | undefined) => Promise<Disposable>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "cursor The cursor to add."
              }],
            "text": "The cursor to add."
          }, {
            "tags": [{
                "name": "param",
                "text": "priority The priority of the cursor."
              }],
            "text": "The priority of the cursor."
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "Disposable": {
            "location": "import",
            "path": "@vertexvis/utils"
          },
          "Cursor": {
            "location": "import",
            "path": "../../lib/cursors"
          }
        },
        "return": "Promise<Disposable>"
      },
      "docs": {
        "text": "Adds a cursor to the viewer, and displays it if the cursor has the highest\npriority.\n\nCursors are managed as a prioritized list. A cursor is displayed if it has\nthe highest priority or if the cursor is the most recently added cursor in\nthe set of cursors with the same priority.\n\nTo remove a cursor, call `dispose()` on the returned disposable.",
        "tags": [{
            "name": "param",
            "text": "cursor The cursor to add."
          }, {
            "name": "param",
            "text": "priority The priority of the cursor."
          }, {
            "name": "returns",
            "text": "A disposable that can be used to remove the cursor."
          }, {
            "name": "see",
            "text": "See {@link CursorManager} for constants to pass to `priority`."
          }]
      }
    },
    "getInteractionHandlers": {
      "complexType": {
        "signature": "() => Promise<InteractionHandler[]>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "InteractionHandler": {
            "location": "import",
            "path": "../../lib/interactions/interactionHandler"
          }
        },
        "return": "Promise<InteractionHandler[]>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "getKeyInteractions": {
      "complexType": {
        "signature": "() => Promise<KeyInteraction<TapEventDetails>[]>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "KeyInteraction": {
            "location": "import",
            "path": "../../lib/interactions/keyInteraction"
          },
          "TapEventDetails": {
            "location": "import",
            "path": "../../lib/interactions/tapEventDetails"
          }
        },
        "return": "Promise<KeyInteraction<TapEventDetails>[]>"
      },
      "docs": {
        "text": "",
        "tags": [{
            "name": "internal",
            "text": undefined
          }, {
            "name": "ignore",
            "text": undefined
          }]
      }
    },
    "getBaseInteractionHandler": {
      "complexType": {
        "signature": "() => Promise<BaseInteractionHandler | undefined>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "BaseInteractionHandler": {
            "location": "import",
            "path": "../../lib/interactions/baseInteractionHandler"
          }
        },
        "return": "Promise<BaseInteractionHandler | undefined>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "getJwt": {
      "complexType": {
        "signature": "() => Promise<string | undefined>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<string | undefined>"
      },
      "docs": {
        "text": "",
        "tags": [{
            "name": "deprecated",
            "text": "Use `token`."
          }]
      }
    },
    "load": {
      "complexType": {
        "signature": "(urn: string, options?: LoadOptions | undefined) => Promise<void>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "urn The URN of the resource to load."
              }],
            "text": "The URN of the resource to load."
          }, {
            "tags": [{
                "name": "param",
                "text": "options Optional configurations when loading the scene.\ncameraType (Optional) The camera type to load. If not included, the default camera type for the resource will be used."
              }],
            "text": "Optional configurations when loading the scene.\ncameraType (Optional) The camera type to load. If not included, the default camera type for the resource will be used."
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "LoadOptions": {
            "location": "local"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Loads the given scene into the viewer and return a `Promise` that\nresolves when the scene has been loaded. The specified scene is\nprovided as a URN in the following format:\n\n * `urn:vertex:scene:<sceneid>`",
        "tags": [{
            "name": "param",
            "text": "urn The URN of the resource to load."
          }, {
            "name": "param",
            "text": "options Optional configurations when loading the scene.\ncameraType (Optional) The camera type to load. If not included, the default camera type for the resource will be used."
          }]
      }
    },
    "unload": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Disconnects the websocket and removes any internal state associated with\nthe scene.",
        "tags": []
      }
    },
    "reload": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Disconnects the websocket and clears the internal state associated with\nthe scene before reconnecting to the same scene.",
        "tags": []
      }
    },
    "scene": {
      "complexType": {
        "signature": "() => Promise<Scene>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "Scene": {
            "location": "import",
            "path": "../../lib/scenes/scene"
          }
        },
        "return": "Promise<Scene>"
      },
      "docs": {
        "text": "Returns an object that is used to perform operations on the `Scene` that's\ncurrently being viewed. These operations include updating items,\npositioning the camera and performing hit tests.",
        "tags": []
      }
    },
    "isSceneReady": {
      "complexType": {
        "signature": "() => Promise<boolean>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<boolean>"
      },
      "docs": {
        "text": "Returns `true` indicating that the scene is ready to be interacted with.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "hostElement"; }
  static get watchers() { return [{
      "propName": "src",
      "methodName": "handleSrcChanged"
    }, {
      "propName": "cameraControls",
      "methodName": "handleCameraControlsChanged"
    }, {
      "propName": "keyboardControls",
      "methodName": "handleKeyboardControlsChanged"
    }, {
      "propName": "rotateAroundTapPoint",
      "methodName": "handleRotateAboutTapPointChanged"
    }, {
      "propName": "cameraType",
      "methodName": "handleCameraTypeChanged"
    }, {
      "propName": "experimentalRenderingOptions",
      "methodName": "handleStreamAttributesChanged"
    }, {
      "propName": "depthBuffers",
      "methodName": "handleStreamAttributesChanged"
    }, {
      "propName": "featureHighlighting",
      "methodName": "handleStreamAttributesChanged"
    }, {
      "propName": "featureLines",
      "methodName": "handleStreamAttributesChanged"
    }, {
      "propName": "featureMaps",
      "methodName": "handleStreamAttributesChanged"
    }, {
      "propName": "noDefaultLights",
      "methodName": "handleStreamAttributesChanged"
    }, {
      "propName": "phantom",
      "methodName": "handleStreamAttributesChanged"
    }, {
      "propName": "sceneComparison",
      "methodName": "handleStreamAttributesChanged"
    }, {
      "propName": "selectionHighlighting",
      "methodName": "handleStreamAttributesChanged"
    }, {
      "propName": "enableTemporalRefinement",
      "methodName": "handleEnableTemporalRefinementChanged"
    }, {
      "propName": "config",
      "methodName": "handleConfigChanged"
    }, {
      "propName": "configEnv",
      "methodName": "handleConfigEnvChanged"
    }]; }
  static get listeners() { return [{
      "name": "tap",
      "method": "handleTapEvent",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
function getRequiredProp(errorMsg, getter) {
  const value = getter();
  if (value != null) {
    return value;
  }
  else
    throw new Error(errorMsg);
}
//# sourceMappingURL=viewer.js.map