/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { Vector3 } from '@vertexvis/geometry';
import { Viewport } from '../types';
// Scalar that is used in combination with a perspective camera's
// components to determine the relative size of the meshes.
// This attempts to keep the widget approximately the same
// size as zooming occurs.
export const DEFAULT_PERSPECTIVE_MESH_SCALAR = 0.005;
// Scalar that is used in combination with an orthographic camera's
// components to determine the relative size of the meshes.
// This attempts to keep the widget approximately the same
// size as zooming occurs.
export const DEFAULT_ORTHOGRAPHIC_MESH_SCALAR = 0.00625;
export class ReglComponent {
  constructor(canvasElement) {
    this.canvasElement = canvasElement;
    this.availableElements = [];
    this.drawableElements = [];
    this.viewport = new Viewport(canvasElement.width, canvasElement.height);
  }
  dispose() {
    var _a;
    (_a = this.reglFrameDisposable) === null || _a === void 0 ? void 0 : _a.cancel();
  }
  updateFrame(frame, redraw = true) {
    this.frame = frame;
    if (redraw) {
      this.updateAndDraw();
    }
  }
  updateDimensions(canvasElement) {
    this.viewport = new Viewport(canvasElement.width, canvasElement.height);
    this.createOrUpdateElements();
  }
  updateAndDraw() {
    if (this.frame != null && this.hasData()) {
      this.createOrUpdateElements();
      this.sortMeshes(this.frame, ...this.availableElements);
      this.draw();
    }
  }
  /**
   * @internal - visible for testing
   */
  getDrawableElements() {
    return this.drawableElements;
  }
  draw() {
    var _a;
    if (this.reglFrameDisposable == null) {
      this.reglFrameDisposable = (_a = this.reglCommand) === null || _a === void 0 ? void 0 : _a.frame(() => {
        this.drawableElements.forEach((el) => el === null || el === void 0 ? void 0 : el.draw({
          fill: el.fillColor,
          opacity: !!el.disabled ? 0.2 : 1,
        }));
      });
    }
  }
  clear() {
    var _a;
    (_a = this.reglCommand) === null || _a === void 0 ? void 0 : _a.clear({
      color: [0, 0, 0, 0],
    });
  }
  sortMeshes(frame, ...drawableElements) {
    const compare = (d1, d2) => d1.points.shortestDistanceFrom(frame.scene.camera.position) -
      d2.points.shortestDistanceFrom(frame.scene.camera.position);
    // Reverse sorted elements to draw the closest element last.
    // This causes it to appear above any other element.
    this.drawableElements = drawableElements.filter((el) => el != null)
      .filter((el) => el.points.valid)
      .sort(compare)
      .reverse();
  }
  computeTriangleSize(position, frame) {
    return frame.scene.camera.isOrthographic()
      ? frame.scene.camera.fovHeight * DEFAULT_ORTHOGRAPHIC_MESH_SCALAR
      : Vector3.magnitude(Vector3.subtract(position, frame.scene.camera.position)) * DEFAULT_PERSPECTIVE_MESH_SCALAR;
  }
}
//# sourceMappingURL=regl-component.js.map