/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { c as angle, e as rectangle, p as point, d as dimensions } from './bundle.esm.js';
import { h } from '@stencil/core/internal/client';

/**
 * Returns a new `Point` with the given horizontal and vertical position.
 */
function create(x = 0, y = 0) {
  return { x, y };
}
/**
 * Returns the distance between two points.
 */
function distance(a, b) {
  const delta = subtract(a, b);
  return Math.sqrt(delta.x * delta.x + delta.y * delta.y);
}
/**
 * Returns a new `Point` where `b` is subtracted from `a`.
 */
function subtract(a, b) {
  return { x: a.x - b.x, y: a.y - b.y };
}
/**
 * Returns a new `Point` where `b` is added to `a`.
 */
function add(a, b) {
  return { x: a.x + b.x, y: a.y + b.y };
}
/**
 * Returns a new `Point` where `x` and `y` are multiplied by the given scale
 * factor.
 */
function scaleProportional(pt, scale) {
  return {
    x: pt.x * scale,
    y: pt.y * scale,
  };
}
/**
 * Returns the magnitude of a point.
 */
function magnitude(pt) {
  return Math.sqrt(pt.x * pt.x + pt.y * pt.y);
}
/**
 * Transforms a vector into the corresponding normal (unit) vector.
 */
function normalizeVector(pt) {
  const magnitudeOfPoint = magnitude(pt);
  if (magnitudeOfPoint === 0) {
    return create(0, 0);
  }
  else {
    return scaleProportional(pt, 1 / magnitudeOfPoint);
  }
}
/**
 * Returns a new normal (unit) vector pointing between the two given points.
 */
function normalDirectionVector(ptA, ptB) {
  return normalizeVector(subtract(ptB, ptA));
}
/**
 * Returns a vector orthogonal to the vector between the two given points.
 */
function orthogonalVector(ptA, ptB) {
  const unitVectorBetweenPoints = normalDirectionVector(ptA, ptB);
  // Handle vectors that are parallel to the x or y axis
  if (unitVectorBetweenPoints.x === 0 || unitVectorBetweenPoints.y === 0) {
    return create(-1 * unitVectorBetweenPoints.y, unitVectorBetweenPoints.x);
  }
  if (Math.abs(unitVectorBetweenPoints.x) > Math.abs(unitVectorBetweenPoints.y)) {
    const vectorXValue = 1 - Math.pow(unitVectorBetweenPoints.x, 2);
    const vectorYValue = -1 * unitVectorBetweenPoints.x * unitVectorBetweenPoints.y;
    return normalizeVector(create(vectorXValue, vectorYValue));
  }
  else {
    const vectorXValue = -1 * unitVectorBetweenPoints.x * unitVectorBetweenPoints.y;
    const vectorYValue = 1 - Math.pow(unitVectorBetweenPoints.y, 2);
    return normalizeVector(create(vectorXValue, vectorYValue));
  }
}
/**
 * Parses a JSON string representation of a Point and returns an object.
 *
 * @param json A JSON string, either in the form `[x,y]` or `{"x": 0, "y": 0}`
 * @returns A parsed Point.
 */
function fromJson(json) {
  const obj = JSON.parse(json);
  if (Array.isArray(obj)) {
    const [x, y] = obj;
    return create(x, y);
  }
  else {
    const { x, y } = obj;
    return create(x, y);
  }
}

function createArrowheadPoints(start, end, arrowSideLength, arrowHeadTheta) {
  const arrowOrthogonalVector = orthogonalVector(start, end);
  const normalizedDirection = normalDirectionVector(start, end);
  const arrowLeft = subtract(end, add(scaleProportional(normalizedDirection, arrowSideLength * Math.cos(arrowHeadTheta)), scaleProportional(arrowOrthogonalVector, arrowSideLength * Math.sin(arrowHeadTheta))));
  const arrowRight = subtract(end, subtract(scaleProportional(normalizedDirection, arrowSideLength * Math.cos(arrowHeadTheta)), scaleProportional(arrowOrthogonalVector, arrowSideLength * Math.sin(arrowHeadTheta))));
  return {
    leftPoint: arrowLeft,
    rightPoint: arrowRight,
  };
}
function createLineAnchorStylePoints(start, end, triangleArrowAngle = 65, lineArrowAngle = 85) {
  // Adjust the size of the end style to the distance between the start and end points
  const distance$1 = distance(start, end);
  const arrowHeadHeight = Math.max(4, Math.min(16, distance$1 * 0.25));
  const hashHeight = Math.max(4, Math.min(12, distance$1 * 0.16));
  const radius = Math.min(5, distance$1 * 0.1);
  // Triangle arrow position
  const triangleArrowRelativeHeight = arrowHeadHeight / distance$1;
  const triangleArrowBasePosition = add(scaleProportional(end, 1 - triangleArrowRelativeHeight), scaleProportional(start, triangleArrowRelativeHeight));
  const triangleArrowTheta = angle.toRadians(triangleArrowAngle / 2);
  const triangleArrowSideLength = arrowHeadHeight / Math.cos(triangleArrowTheta);
  const triangleArrow = createArrowheadPoints(start, end, triangleArrowSideLength, triangleArrowTheta);
  // Line arrow position
  const lineArrowTheta = angle.toRadians(lineArrowAngle / 2);
  const lineArrowSideLength = arrowHeadHeight / Math.cos(lineArrowTheta);
  const lineArrow = createArrowheadPoints(start, end, lineArrowSideLength, lineArrowTheta);
  // Hash position
  // Setting the arrowHeadAngle to 90 degrees results in a
  // straight hash mark perpendicular to the direction of the line
  const hashLineTheta = angle.toRadians(90);
  const hashLine = createArrowheadPoints(start, end, hashHeight, hashLineTheta);
  return {
    tip: end,
    base: triangleArrowBasePosition,
    arrowTriangle: triangleArrow,
    arrowLine: lineArrow,
    hash: hashLine,
    radius: radius,
  };
}
function arrowheadPointsToPolygonPoints(points) {
  return [
    points.tip,
    points.arrowTriangle.rightPoint,
    points.base,
    points.arrowTriangle.leftPoint,
  ]
    .map((pt) => `${pt.x},${pt.y}`)
    .join(' ');
}
function arrowheadPointsToPathPoints(points) {
  return `M${points.arrowLine.rightPoint.x} ${points.arrowLine.rightPoint.y} L${points.tip.x} ${points.tip.y} L${points.arrowLine.leftPoint.x} ${points.arrowLine.leftPoint.y} L${points.tip.x} ${points.tip.y} Z`;
}
function isVertexViewerArrowMarkup(el) {
  return (el instanceof HTMLElement && el.nodeName === 'VERTEX-VIEWER-MARKUP-ARROW');
}
function parsePoint(value) {
  return typeof value === 'string' ? fromJson(value) : value;
}

function isVertexViewerCircleMarkup(el) {
  return (el instanceof HTMLElement && el.nodeName === 'VERTEX-VIEWER-MARKUP-CIRCLE');
}
function parseBounds(value) {
  return typeof value === 'string' ? rectangle.fromJson(value) : value;
}

function parsePoints(value) {
  return typeof value === 'string'
    ? JSON.parse(value).map((values) => point.fromJson(JSON.stringify(values)))
    : value;
}
function isVertexViewerFreeformMarkup(el) {
  return (el instanceof HTMLElement && el.nodeName === 'VERTEX-VIEWER-MARKUP-FREEFORM');
}

function getBoundingBox2dAnchorPosition(rect, position) {
  switch (position) {
    case 'top-left':
      return point.create(rect.x, rect.y);
    case 'top':
      return point.create(rect.x + rect.width / 2, rect.y);
    case 'top-right':
      return point.create(rect.x + rect.width, rect.y);
    case 'right':
      return point.create(rect.x + rect.width, rect.y + rect.height / 2);
    case 'bottom-right':
      return point.create(rect.x + rect.width, rect.y + rect.height);
    case 'bottom':
      return point.create(rect.x + rect.width / 2, rect.y + rect.height);
    case 'bottom-left':
      return point.create(rect.x, rect.y + rect.height);
    case 'left':
      return point.create(rect.x, rect.y + rect.height / 2);
    case 'center':
      return rectangle.center(rect);
  }
}
function translatePointToScreen(pt, canvasDimensions) {
  const scaleFactor = canvasDimensions.height;
  return point.add(point.scale(pt, scaleFactor, scaleFactor), dimensions.center(canvasDimensions));
}
function translateDimensionsToScreen(dimensions$1, canvasDimensions) {
  const scaleFactor = canvasDimensions.height;
  return dimensions.scale(scaleFactor, scaleFactor, dimensions$1);
}
/**
 * Translates a rectangle in relative units, to a rectangle in screen units.
 *
 * Shape coordinates are persisted in relative units, so they can be presented
 * at any size.
 */
function translateRectToScreen(rect, canvasDimensions) {
  const position = translatePointToScreen(rect, canvasDimensions);
  const dimensions = translateDimensionsToScreen(rect, canvasDimensions);
  return rectangle.fromPointAndDimensions(position, dimensions);
}
/**
 * Translates a point in screen units, to a point in relative units.
 */
function translatePointToRelative(pt, canvasDimensions) {
  const scaleFactor = 1 / canvasDimensions.height;
  const point$1 = point.scale(point.subtract(pt, dimensions.center(canvasDimensions)), scaleFactor, scaleFactor);
  return point$1;
}
/**
 * Translates a set of points in relative `original` units to
 * points in relative `bounds` units.
 */
function translatePointsToBounds(points, original, bounds) {
  return points.map((pt) => point.add(point.scale(point.subtract(pt, original), bounds.width / (original.width || 1), bounds.height / (original.height || 1)), bounds));
}
function createRectangle(initialPoint, currentPoint, maintainAspectRatio) {
  const bounds = rectangle.fromPoints(initialPoint, currentPoint);
  if (maintainAspectRatio) {
    const fitBoundsSize = Math.max(bounds.width, bounds.height);
    const isPortrait = bounds.height > bounds.width;
    const currentIsLeftOfInitial = currentPoint.x <= initialPoint.x;
    const currentIsAboveInitial = currentPoint.y <= initialPoint.y;
    const fitBoundsX = currentIsLeftOfInitial
      ? isPortrait
        ? initialPoint.x - fitBoundsSize
        : currentPoint.x
      : initialPoint.x;
    const fitBoundsY = currentIsAboveInitial
      ? isPortrait
        ? currentPoint.y
        : initialPoint.y - fitBoundsSize
      : initialPoint.y;
    return rectangle.create(fitBoundsX, fitBoundsY, fitBoundsSize, fitBoundsSize);
  }
  else {
    return bounds;
  }
}
function transformRectangle(bounds, start, current, anchor, maintainAspectRatio) {
  const delta = point.subtract(current, start);
  const { x: left, y: top, width: w, height: h } = bounds;
  const right = left + w;
  const bottom = top + h;
  const topLeft = point.create(left, top);
  const bottomLeft = point.create(left, bottom);
  const topRight = point.create(right, top);
  const bottomRight = point.create(right, bottom);
  switch (anchor) {
    case 'top-left':
      return createRectangle(bottomRight, current, !!maintainAspectRatio);
    case 'top':
      return createRectangle(bottomRight, point.create(left, current.y), !!maintainAspectRatio);
    case 'top-right':
      return createRectangle(bottomLeft, current, !!maintainAspectRatio);
    case 'right':
      return createRectangle(bottomLeft, point.create(current.x, top), !!maintainAspectRatio);
    case 'bottom-right':
      return createRectangle(topLeft, current, !!maintainAspectRatio);
    case 'bottom':
      return createRectangle(topLeft, point.create(right, current.y), !!maintainAspectRatio);
    case 'bottom-left':
      return createRectangle(topRight, current, !!maintainAspectRatio);
    case 'left':
      return createRectangle(bottomRight, point.create(current.x, top), !!maintainAspectRatio);
    case 'center':
      return rectangle.create(bounds.x + delta.x, bounds.y + delta.y, bounds.width, bounds.height);
  }
}
function isVertexViewerMarkupElement(el) {
  return (isVertexViewerArrowMarkup(el) ||
    isVertexViewerCircleMarkup(el) ||
    isVertexViewerFreeformMarkup(el));
}
function isValidPointData(...points) {
  return points.every((pt) => !isNaN(pt.x) && !isNaN(pt.y));
}
function isValidStartEvent(event) {
  const el = event.target;
  return isVertexViewerMarkupElement(el) && el.mode !== 'edit';
}

// eslint-disable-next-line @typescript-eslint/no-unused-vars
const RelativeAnchor = ({ id, rotation, name, point, onPointerDown }, children) => {
  return (h("div", { id: id, class: "bounds-anchor-position", style: {
      top: `${point.y}px`,
      left: `${point.x}px`,
    }, onTouchStart: (event) => event.preventDefault(), onPointerDown: onPointerDown },
    h("div", { class: "bounds-anchor", style: {
        transform: `rotateZ(${rotation !== null && rotation !== void 0 ? rotation : 0}deg)`,
      } },
      h("slot", { name: name }, children))));
};
const SvgShadow = ({ id }) => {
  return (h("filter", { id: id, filterUnits: "userSpaceOnUse" },
    h("feGaussianBlur", { in: "SourceAlpha", stdDeviation: "2" }),
    h("feOffset", { dx: "0", dy: "1", result: "offsetblur" }),
    h("feFlood", { "flood-color": "#000000", "flood-opacity": "0.25" }),
    h("feComposite", { in2: "offsetblur", operator: "in" }),
    h("feMerge", null,
      h("feMergeNode", null),
      h("feMergeNode", { in: "SourceGraphic" }))));
};

class MarkupInteractionHandler {
  constructor() {
    this.handlePointerDown = (event) => {
      this.handleInteractionAttempt(event);
    };
    this.handlePointerMove = (event) => {
      this.handleInteractionMove(event);
    };
    this.handlePointerUp = (event) => {
      window.removeEventListener('pointermove', this.handlePointerMove);
      window.removeEventListener('pointerup', this.handlePointerUp);
      this.handleInteractionEnd(event);
    };
    this.resizeObserver = new ResizeObserver(([entry]) => {
      this.elementBounds = this.computeBoundingRect();
    });
  }
  initialize(element, api) {
    this.element = element;
    this.api = api;
    this.elementBounds = this.computeBoundingRect();
    this.resizeObserver.observe(this.element);
    this.element.addEventListener('pointerdown', this.handlePointerDown);
  }
  dispose() {
    var _a;
    this.resizeObserver.disconnect();
    (_a = this.element) === null || _a === void 0 ? void 0 : _a.removeEventListener('pointerdown', this.handlePointerDown);
    this.element = undefined;
    this.api = undefined;
  }
  acceptInteraction() {
    window.addEventListener('pointermove', this.handlePointerMove);
    window.addEventListener('pointerup', this.handlePointerUp);
  }
}

export { MarkupInteractionHandler as M, RelativeAnchor as R, SvgShadow as S, isVertexViewerCircleMarkup as a, isVertexViewerFreeformMarkup as b, translatePointToScreen as c, isValidStartEvent as d, arrowheadPointsToPolygonPoints as e, arrowheadPointsToPathPoints as f, isValidPointData as g, createLineAnchorStylePoints as h, isVertexViewerArrowMarkup as i, transformRectangle as j, parseBounds as k, translateRectToScreen as l, translatePointsToBounds as m, parsePoints as n, getBoundingBox2dAnchorPosition as o, parsePoint as p, translatePointToRelative as t };

//# sourceMappingURL=interactions.js.map