/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { c as color, E as EventDispatcher } from './browser.esm.js';

function isTextPin(pin) {
  return (pin === null || pin === void 0 ? void 0 : pin.type) === 'text';
}
function isIconPin(pin) {
  return (pin === null || pin === void 0 ? void 0 : pin.type) === 'icon';
}
function convertColorToString(color$1) {
  return typeof color$1 === 'string' ? color$1 : color.toHexString(color$1);
}
function getPinColors(pin) {
  var _a, _b, _c, _d, _e, _f, _g, _h;
  return {
    primaryColor: ((_b = (_a = pin === null || pin === void 0 ? void 0 : pin.attributes) === null || _a === void 0 ? void 0 : _a.style) === null || _b === void 0 ? void 0 : _b.primaryColor) == null
      ? undefined
      : convertColorToString((_d = (_c = pin === null || pin === void 0 ? void 0 : pin.attributes) === null || _c === void 0 ? void 0 : _c.style) === null || _d === void 0 ? void 0 : _d.primaryColor),
    accentColor: ((_f = (_e = pin === null || pin === void 0 ? void 0 : pin.attributes) === null || _e === void 0 ? void 0 : _e.style) === null || _f === void 0 ? void 0 : _f.accentColor) == null
      ? undefined
      : convertColorToString((_h = (_g = pin === null || pin === void 0 ? void 0 : pin.attributes) === null || _g === void 0 ? void 0 : _g.style) === null || _h === void 0 ? void 0 : _h.accentColor),
  };
}
class PinModel {
  constructor() {
    this.entities = {};
    this.entitiesChanged = new EventDispatcher();
    this.selectionChanged = new EventDispatcher();
  }
  /**
   * Registers an entity to be drawn in the canvas
   *
   * @param pin A pin entity to draw.
   * @returns `true` if the entity has been added.
   */
  addPin(pin, suppressEvent = false) {
    if (this.entities[pin.id] == null) {
      this.entities = Object.assign(Object.assign({}, this.entities), { [pin.id]: pin });
      if (!suppressEvent) {
        this.entitiesChanged.emit(this.getPins());
      }
      return true;
    }
    else {
      return false;
    }
  }
  /**
   * Clears all registered entities from the model.
   */
  clearPins() {
    this.getPins().forEach((e) => this.removePin(e));
  }
  /**
   * Returns all the entities registered with the model.
   */
  getPins() {
    return Object.keys(this.entities).map((key) => this.entities[key]);
  }
  /**
   * Returns single entity by id if present in the model.
   */
  getPinById(id) {
    return this.entities[id];
  }
  getSelectedPinId() {
    return this.selectedPinId;
  }
  /**
   * Unregisters an entity from the model.
   *
   * @param entity The entity to remove.
   * @returns `true` if the entity was removed.
   */
  removePin(entity) {
    if (this.entities[entity.id] != null) {
      delete this.entities[entity.id];
      this.entitiesChanged.emit(this.getPins());
      return true;
    }
    else {
      return false;
    }
  }
  /**
   * Sets the set of entities to be placed with the model.
   *
   * @param entities A set of entities to draw.
   * @returns `true` if the entity has been added.
   */
  setPins(pins) {
    this.clearPins();
    pins.forEach((e) => this.addPin(e, true));
    this.entitiesChanged.emit(this.getPins());
    return true;
  }
  /**
   * Sets the set of entities to be placed with the model.
   *
   * @param pin A pin to set
   * @returns `true` if the entity has been set
   */
  setPin(pin) {
    this.entities = Object.assign(Object.assign({}, this.entities), { [pin.id]: pin });
    this.entitiesChanged.emit(this.getPins());
    return true;
  }
  /**
   * Sets the set of entities to be placed with the model.
   *
   * @param pin A pin to set
   * @returns `true` if the entity has been set
   */
  updatePin(pin) {
    const pinById = this.getPinById(pin.id);
    if (pinById != null) {
      this.entities = Object.assign(Object.assign({}, this.entities), { [pin.id]: pin });
      this.entitiesChanged.emit(this.getPins());
      return true;
    }
    return false;
  }
  /**
   * Registers an event listener that will be invoked when the model's
   * pin entities change.
   *
   * @param listener The listener to add.
   * @returns A disposable that can be used to remove the listener.
   */
  onEntitiesChanged(listener) {
    return this.entitiesChanged.on(listener);
  }
  /**
   * Registers an event listener that will be invoked when the model's
   * pin selection changes
   *
   * @param listener The listener to add.
   * @returns A disposable that can be used to remove the listener.
   */
  onSelectionChange(listener) {
    return this.selectionChanged.on(listener);
  }
  /**
   * Sets the selected pin Id
   * @param pinId
   */
  setSelectedPin(pinId) {
    this.selectedPinId = pinId;
    this.selectionChanged.emit(this.selectedPinId);
  }
}

export { PinModel as P, isIconPin as a, getPinColors as g, isTextPin as i };

//# sourceMappingURL=model3.js.map