/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { c as classnames } from './index2.js';
import { d as debounceEvent } from './stencil.js';
import { d as defineCustomElement$2 } from './viewer-icon.js';
import { d as defineCustomElement$1 } from './viewer-spinner.js';

const sceneTreeSearchCss = "button{border:none;background:transparent;cursor:pointer;margin:0;padding:0}:host{--scene-tree-search-search-icon-offset:8px;--scene-tree-search-clear-button-offset:8px;--scene-tree-search-input-padding:0.5rem 32px;--scene-tree-search-input-border:none;--scene-tree-search-input-border-radius:0.25rem;--scene-tree-search-input-background:none;--scene-tree-search-focused-input-background:var(--neutral-200);--scene-tree-search-focused-input-outline:none;display:block;width:100%;font-size:0.875rem}.root{width:100%;position:relative}.input{background:var(--scene-tree-search-input-background);border:var(--scene-tree-search-input-border);border-radius:var(--scene-tree-search-input-border-radius);box-sizing:border-box;font-size:inherit;padding:var(--scene-tree-search-input-padding);outline:none;width:100%}.input.background{background:var(--scene-tree-search-focused-input-background);outline:var(--scene-tree-search-focused-input-outline)}.overlay{position:absolute;top:0;bottom:0;display:flex;align-items:center;justify-content:center;pointer-events:none}.overlay-clear{display:none;right:var(--scene-tree-search-clear-button-offset)}.overlay-clear.show{display:flex}.icon{color:var(--neutral-700)}.icon-search{position:absolute;left:var(--scene-tree-search-search-icon-offset);pointer-events:none}.clear-btn{display:flex;align-items:center;justify-content:center;pointer-events:initial;cursor:pointer}.clear-btn:hover{color:var(--neutral-800)}.clear-btn:disabled{pointer-events:none;opacity:0.5}";

const SceneTreeSearch = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.search = createEvent(this, "search", 7);
    /**
     * If `true`, disables user interaction of the component.
     */
    this.disabled = false;
    /**
     * Placeholder text if `value` is empty.
     */
    this.placeholder = undefined;
    /**
     * The current text value of the component. Value is updated on user
     * interaction.
     */
    this.value = '';
    this.focused = false;
    this.isSearching = false;
    this.handleTextInput = (event) => {
      const input = event.target;
      this.value = input.value;
      if (this.debounce != null || this.value === '') {
        this.emitCurrentValue();
      }
    };
    this.handleTextFocus = () => {
      this.focused = true;
    };
    this.handleTextBlur = () => {
      var _a;
      this.focused = false;
      (_a = this.searchDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
      if (this.value !== this.lastEmittedValue) {
        this.emitCurrentValue();
      }
    };
    this.handleKeyPress = (event) => {
      var _a;
      if (event.key === 'Enter') {
        (_a = this.searchDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
        this.emitCurrentValue();
      }
    };
    this.handleClear = (event) => {
      var _a;
      event.preventDefault();
      this.value = '';
      (_a = this.searchDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
      this.emitCurrentValue();
      this.setFocus();
    };
  }
  /**
   * Gives focus to the component's internal text input.
   */
  async setFocus() {
    var _a, _b;
    // HTMLInputElement.focus() doesn't exist in tests.
    if (typeof ((_a = this.inputEl) === null || _a === void 0 ? void 0 : _a.focus) === 'function') {
      (_b = this.inputEl) === null || _b === void 0 ? void 0 : _b.focus();
    }
  }
  /**
   * @ignore
   */
  controllerChanged(controller) {
    this.setupController();
  }
  /**
   * @ignore
   */
  componentDidLoad() {
    this.handleDebounceChanged();
    this.setupController();
  }
  /**
   * @ignore
   */
  disconnectedCallback() {
    var _a;
    (_a = this.onStateChangeDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
  }
  /**
   * Clears the current search term and clears any debounced filters.
   */
  clear() {
    var _a;
    this.value = '';
    (_a = this.searchDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    this.emitCurrentValue();
  }
  /**
   * @ignore
   */
  render() {
    return (h(Host, null, h("div", { class: "root" }, h("div", { class: "overlay icon icon-search" }, h("slot", { name: "search-icon" }, this.isSearching ? (h("vertex-viewer-spinner", { slot: "search-icon", size: "xs" })) : (h("vertex-viewer-icon", { name: "search", size: "sm" })))), h("input", { class: classnames('input', {
        background: this.focused || this.value.length > 0,
      }), type: "text", ref: (ref) => (this.inputEl = ref), placeholder: this.placeholder, disabled: this.disabled, value: this.value, onInput: this.handleTextInput, onFocus: this.handleTextFocus, onBlur: this.handleTextBlur, onKeyPress: this.handleKeyPress }), h("div", { class: classnames('overlay overlay-clear', {
        show: this.value.length > 0,
      }) }, h("button", { class: "clear-btn icon", tabIndex: -1, onMouseDown: this.handleClear, disabled: this.disabled }, h("slot", { name: "clear-icon" }, h("vertex-viewer-icon", { name: "close-circle", size: "sm" })))))));
  }
  handleDebounceChanged() {
    var _a, _b;
    (_a = this.searchDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    const emitter = debounceEvent(this.search, (_b = this.debounce) !== null && _b !== void 0 ? _b : 0);
    // Track this emitter in two separate variables to maintain the `EventEmitter` typing for
    // `this.search`. This allows for correct generation of `CustomEvent` types.
    this.search = emitter;
    this.searchDisposable = emitter;
  }
  setupController() {
    var _a, _b;
    (_a = this.onStateChangeDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    this.onStateChangeDisposable = (_b = this.controller) === null || _b === void 0 ? void 0 : _b.onStateChange.on((state) => {
      this.isSearching = state.isSearching;
    });
  }
  emitCurrentValue() {
    this.lastEmittedValue = this.value;
    this.search.emit(this.value);
  }
  static get watchers() { return {
    "controller": ["controllerChanged"]
  }; }
  static get style() { return sceneTreeSearchCss; }
}, [1, "vertex-scene-tree-search", {
    "debounce": [2],
    "disabled": [4],
    "placeholder": [1],
    "controller": [16],
    "value": [1025],
    "focused": [32],
    "isSearching": [32],
    "setFocus": [64],
    "clear": [64]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-scene-tree-search", "vertex-viewer-icon", "vertex-viewer-spinner"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-scene-tree-search":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, SceneTreeSearch);
      }
      break;
    case "vertex-viewer-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
    case "vertex-viewer-spinner":
      if (!customElements.get(tagName)) {
        defineCustomElement$1();
      }
      break;
  } });
}
defineCustomElement();

export { SceneTreeSearch as S, defineCustomElement as d };

//# sourceMappingURL=scene-tree-search.js.map