/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { a as VolumeIntersectionQueryModel, V as VolumeIntersectionQueryController } from './model2.js';
import { b as boxQueryCursor } from './cursors.js';
import { g as getMouseClientPosition } from './dom.js';
import { d as defineCustomElement$2 } from './viewer-layer.js';

class VolumeIntersectionQueryInteractionHandler {
  constructor(controller) {
    this.controller = controller;
    this.handleDragBegin = this.handleDragBegin.bind(this);
    this.handleDrag = this.handleDrag.bind(this);
    this.handleDragEnd = this.handleDragEnd.bind(this);
    this.enabled = true;
  }
  initialize(element, api) {
    this.element = element;
    this.api = api;
    this.element.addEventListener('pointerdown', this.handleDragBegin);
    this.addCrosshairCursor();
  }
  dispose() {
    var _a, _b, _c;
    (_a = this.element) === null || _a === void 0 ? void 0 : _a.removeEventListener('pointerdown', this.handleDragBegin);
    window.removeEventListener('pointermove', this.handleDrag);
    window.removeEventListener('pointerup', this.handleDragEnd);
    (_b = this.crosshairCursorDisposable) === null || _b === void 0 ? void 0 : _b.dispose();
    (_c = this.waitCursorDisposable) === null || _c === void 0 ? void 0 : _c.dispose();
    this.element = undefined;
    this.api = undefined;
    this.enabled = true;
  }
  disable() {
    this.enabled = false;
  }
  enable() {
    this.enabled = true;
  }
  handleDragBegin(event) {
    var _a;
    if (this.enabled &&
      this.isPrimaryMouseButtonClick(event) &&
      !this.isInteracting) {
      this.elementBounds = (_a = this.element) === null || _a === void 0 ? void 0 : _a.getBoundingClientRect();
      this.isInteracting = true;
      this.controller.setStartPoint(getMouseClientPosition(event, this.elementBounds));
      window.addEventListener('pointermove', this.handleDrag);
      window.addEventListener('pointerup', this.handleDragEnd);
    }
  }
  handleDrag(event) {
    if (this.enabled) {
      this.controller.setEndPoint(getMouseClientPosition(event, this.elementBounds));
    }
  }
  async handleDragEnd() {
    var _a, _b;
    this.isInteracting = false;
    window.removeEventListener('pointermove', this.handleDrag);
    window.removeEventListener('pointerup', this.handleDragEnd);
    (_a = this.crosshairCursorDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    if (this.enabled) {
      this.addWaitCursor();
      try {
        await this.controller.execute();
      }
      finally {
        (_b = this.waitCursorDisposable) === null || _b === void 0 ? void 0 : _b.dispose();
        this.addCrosshairCursor();
      }
    }
  }
  isPrimaryMouseButtonClick(event) {
    return event.buttons === 1;
  }
  addCrosshairCursor() {
    var _a, _b;
    (_a = this.crosshairCursorDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    this.crosshairCursorDisposable = (_b = this.api) === null || _b === void 0 ? void 0 : _b.addCursor(boxQueryCursor);
  }
  addWaitCursor() {
    var _a, _b;
    (_a = this.waitCursorDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    this.waitCursorDisposable = (_b = this.api) === null || _b === void 0 ? void 0 : _b.addCursor('wait');
  }
}

const viewerBoxQueryToolCss = ":host{--viewer-box-query-outline-exclusive-color:#0099cc;--viewer-box-query-outline-exclusive-border-style:solid;--viewer-box-query-outline-inclusive-color:#00cc00;--viewer-box-query-outline-inclusive-border-style:dashed;--viewer-box-query-outline-border-radius:0.25rem;--viewer-box-query-outline-fill-opacity:0.25}.bounds{position:absolute}.outline{position:absolute;top:0;left:0;width:100%;height:100%;box-sizing:border-box;border-radius:var(--viewer-box-query-outline-border-radius)}:host([exclusive=\"true\"]) .outline{border:3px var(--viewer-box-query-outline-exclusive-border-style) var(--viewer-box-query-outline-exclusive-color)}:host([inclusive=\"true\"]) .outline{border:3px var(--viewer-box-query-outline-inclusive-border-style) var(--viewer-box-query-outline-inclusive-color)}.fill{position:absolute;top:0;left:0;width:100%;height:100%;opacity:var(--viewer-box-query-outline-fill-opacity);border-radius:var(--viewer-box-query-outline-border-radius)}:host([exclusive=\"true\"]) .fill{background-color:var(--viewer-box-query-outline-exclusive-color)}:host([inclusive=\"true\"]) .fill{background-color:var(--viewer-box-query-outline-inclusive-color)}";

const ViewerBoxQueryTool = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.controllerChanged = createEvent(this, "controllerChanged", 7);
    /**
     * The default operation to perform when a drag has completed and the intersection
     * query will be run. Defaults to `clearAndSelect`, and can be changed to `select` or `deselect`.
     *
     * `clearAndSelect` will clear all existing selection, and select the results of the query.
     * `select` will maintain existing selection, and select the results of the query.
     * `deselect` will maintain existing selection, and deselect the results of the query.
     *
     * The operation behavior for this intersection query tool can also be changed by
     * providing a custom implementation of the `VolumeIntersectionQueryController`, or
     * by using the `setOperationTransform` method of the default controller.
     */
    this.operationType = 'clearAndSelect';
    this.handleScreenBoundsChanged = this.handleScreenBoundsChanged.bind(this);
    this.handleExecuteStarted = this.handleExecuteStarted.bind(this);
    this.handleExecuteComplete = this.handleExecuteComplete.bind(this);
  }
  componentWillLoad() {
    var _a;
    this.model = (_a = this.model) !== null && _a !== void 0 ? _a : new VolumeIntersectionQueryModel(this.mode);
    this.screenBoundsChangedDisposable = this.model.onScreenBoundsChanged(this.handleScreenBoundsChanged);
    this.handleViewerChanged(this.viewer);
    this.handleControllerChange(this.controller);
    this.handleDefaultOperationChange(this.operationType);
  }
  disconnectedCallback() {
    var _a, _b, _c, _d;
    (_a = this.model) === null || _a === void 0 ? void 0 : _a.reset();
    (_b = this.screenBoundsChangedDisposable) === null || _b === void 0 ? void 0 : _b.dispose();
    (_c = this.operationStartedDisposable) === null || _c === void 0 ? void 0 : _c.dispose();
    (_d = this.operationCompleteDisposable) === null || _d === void 0 ? void 0 : _d.dispose();
    this.deregisterInteractionHandler();
  }
  /**
   * @ignore
   */
  handleViewerChanged(newViewer) {
    this.deregisterInteractionHandler();
    if (this.model != null && newViewer != null) {
      this.controller = new VolumeIntersectionQueryController(this.model, newViewer);
      this.handleDefaultOperationChange(this.operationType);
      this.registerInteractionHandler(this.controller, newViewer);
    }
  }
  /**
   * @ignore
   */
  handleDefaultOperationChange(updatedOperationType) {
    switch (updatedOperationType) {
      case 'clearAndSelect':
        this.setDefaultClearAndSelectOperation();
        break;
      case 'select':
        this.setDefaultSelectOperation();
        break;
      case 'deselect':
        this.setDefaultDeselectOperation();
        break;
    }
  }
  /**
   * @ignore
   */
  handleModeChange(updatedMode) {
    var _a;
    (_a = this.model) === null || _a === void 0 ? void 0 : _a.setMode(updatedMode);
  }
  /**
   * @ignore
   */
  handleControllerChange(controller) {
    var _a, _b;
    (_a = this.operationStartedDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    (_b = this.operationCompleteDisposable) === null || _b === void 0 ? void 0 : _b.dispose();
    this.operationStartedDisposable = controller === null || controller === void 0 ? void 0 : controller.onExecuteStarted(this.handleExecuteStarted);
    this.operationStartedDisposable = controller === null || controller === void 0 ? void 0 : controller.onExecuteComplete(this.handleExecuteComplete);
    this.controllerChanged.emit(this.controller);
  }
  /**
   * @ignore
   */
  render() {
    return (h(Host, null, h("vertex-viewer-layer", null, this.details != null && (h("div", { class: "bounds", style: {
        left: `${this.details.screenBounds.x}px`,
        top: `${this.details.screenBounds.y}px`,
        width: `${this.details.screenBounds.width}px`,
        height: `${this.details.screenBounds.height}px`,
      } }, h("slot", { name: "bounds" }, h("div", { class: "outline" }), h("div", { class: "fill" })))))));
  }
  handleScreenBoundsChanged(details) {
    this.details = details;
    this.updateTypeAttribute(details === null || details === void 0 ? void 0 : details.type);
  }
  handleExecuteStarted() {
    var _a;
    (_a = this.interactionHandler) === null || _a === void 0 ? void 0 : _a.disable();
  }
  handleExecuteComplete() {
    var _a;
    (_a = this.interactionHandler) === null || _a === void 0 ? void 0 : _a.enable();
  }
  async registerInteractionHandler(controller, viewer) {
    this.interactionHandler = new VolumeIntersectionQueryInteractionHandler(controller);
    this.interactionHandlerDisposable = await viewer.registerInteractionHandler(this.interactionHandler);
  }
  deregisterInteractionHandler() {
    var _a, _b;
    (_a = this.interactionHandlerDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    (_b = this.interactionHandler) === null || _b === void 0 ? void 0 : _b.dispose();
    this.interactionHandler = undefined;
  }
  updateTypeAttribute(type) {
    this.hostEl.setAttribute('inclusive', `${type === 'inclusive'}`);
    this.hostEl.setAttribute('exclusive', `${type === 'exclusive'}`);
  }
  setDefaultClearAndSelectOperation() {
    var _a, _b;
    (_a = this.controller) === null || _a === void 0 ? void 0 : _a.setOperationTransform((builder) => builder.select());
    (_b = this.controller) === null || _b === void 0 ? void 0 : _b.setAdditionalTransforms([
      (op) => op.items.where((q) => q.withSelected()).deselect(),
    ]);
  }
  setDefaultSelectOperation() {
    var _a, _b;
    (_a = this.controller) === null || _a === void 0 ? void 0 : _a.setOperationTransform((builder) => builder.select());
    (_b = this.controller) === null || _b === void 0 ? void 0 : _b.setAdditionalTransforms([]);
  }
  setDefaultDeselectOperation() {
    var _a, _b;
    (_a = this.controller) === null || _a === void 0 ? void 0 : _a.setOperationTransform((builder) => builder.deselect());
    (_b = this.controller) === null || _b === void 0 ? void 0 : _b.setAdditionalTransforms([]);
  }
  get hostEl() { return this; }
  static get watchers() { return {
    "viewer": ["handleViewerChanged"],
    "operationType": ["handleDefaultOperationChange"],
    "mode": ["handleModeChange"],
    "controller": ["handleControllerChange"]
  }; }
  static get style() { return viewerBoxQueryToolCss; }
}, [1, "vertex-viewer-box-query-tool", {
    "viewer": [16],
    "controller": [1040],
    "model": [1040],
    "operationType": [1, "operation-type"],
    "mode": [1],
    "details": [32]
  }]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-viewer-box-query-tool", "vertex-viewer-layer"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-viewer-box-query-tool":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, ViewerBoxQueryTool);
      }
      break;
    case "vertex-viewer-layer":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}
defineCustomElement$1();

const VertexViewerBoxQueryTool = ViewerBoxQueryTool;
const defineCustomElement = defineCustomElement$1;

export { VertexViewerBoxQueryTool, defineCustomElement };

//# sourceMappingURL=vertex-viewer-box-query-tool.js.map