/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { p as point } from './bundle.esm.js';
import { s as stampTemplateWithId } from './templates.js';
import { A as ArrowMarkup, C as CircleMarkup, F as FreeformMarkup } from './markup.js';
import { i as isVertexViewerArrowMarkup, a as isVertexViewerCircleMarkup, b as isVertexViewerFreeformMarkup } from './interactions.js';
import { d as defineCustomElement$4 } from './viewer-markup-arrow.js';
import { d as defineCustomElement$3 } from './viewer-markup-circle.js';
import { d as defineCustomElement$2 } from './viewer-markup-freeform.js';

const viewerMarkupCss = ":host{display:block;position:absolute;top:0;bottom:0;left:0;right:0;pointer-events:none}";

const ViewerMarkup = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.markupAdded = createEvent(this, "markupAdded", 7);
    this.markupChanged = createEvent(this, "markupChanged", 7);
    this.markupRemoved = createEvent(this, "markupRemoved", 7);
    this.markupSelectionChanged = createEvent(this, "markupSelectionChanged", 7);
    /**
     * The type of markup to perform.
     */
    this.tool = 'arrow';
    /**
     * If `true`, disables adding or editing of markup through user
     * interaction.
     */
    this.disabled = false;
    /**
     * Indicates if new markup should be automatically selected.
     */
    this.selectNew = false;
    /**
     * The style of the starting anchor. This defaults to none.
     */
    this.startLineAnchorStyle = 'none';
    /**
     * The style of the ending anchor. This defaults to 'arrow-triangle.'
     */
    this.endLineAnchorStyle = 'arrow-triangle';
    this.handlePointerMove = (event) => {
      if (this.pointerDownPosition != null &&
        point.distance(this.pointerDownPosition, point.create(event.clientX, event.clientY)) > 2) {
        this.toSelectMarkupId = undefined;
      }
    };
    this.handlePointerUp = async (event) => {
      if (!this.disabled) {
        const el = event.target;
        const markups = await this.getMarkupElements();
        const markup = markups.find((m) => m === el);
        if ((markup === null || markup === void 0 ? void 0 : markup.id) != null &&
          markup.id !== '' &&
          markup.id === this.toSelectMarkupId) {
          this.selectedMarkupId = el.id;
        }
        else if (this.pointerDownPosition != null &&
          point.distance(this.pointerDownPosition, point.create(event.clientX, event.clientY)) <= 2) {
          this.selectedMarkupId = undefined;
        }
        this.toSelectMarkupId = undefined;
        this.pointerDownPosition = undefined;
      }
      window.removeEventListener('pointerup', this.handlePointerUp);
      window.removeEventListener('pointermove', this.handlePointerMove);
    };
  }
  disconnectedCallback() {
    window.removeEventListener('pointerup', this.handlePointerUp);
    window.removeEventListener('pointermove', this.handlePointerMove);
  }
  /**
   * Adds a new markup as a child to this component. A new markup
   * component will be created from the template specified by
   * `arrow-template-id`, `circle-template-id`, or if undefined
   * a default element will be created.
   *
   * @param markup The markup to add.
   * @returns The markup element that was created.
   * @see {@link ViewerMarkups.arrowTemplateId}
   * @see {@link ViewerMarkups.circleTemplateId}
   */
  async addMarkup(markup) {
    if (markup instanceof ArrowMarkup) {
      const { start, end, id } = markup;
      const el = this.createArrowMarkupElement();
      el.id = id;
      el.start = start;
      el.end = end;
      el.startLineAnchorStyle = this.startLineAnchorStyle;
      el.endLineAnchorStyle = this.endLineAnchorStyle;
      return this.appendMarkupElement(el);
    }
    else if (markup instanceof CircleMarkup) {
      const { bounds, id } = markup;
      const el = this.createCircleMarkupElement();
      el.id = id;
      el.bounds = bounds;
      return this.appendMarkupElement(el);
    }
    else if (markup instanceof FreeformMarkup) {
      const { bounds, points, id } = markup;
      const el = this.createFreeformMarkupElement();
      el.id = id;
      el.points = points;
      el.bounds = bounds;
      return this.appendMarkupElement(el);
    }
    else {
      throw new Error(`Cannot add markup. Unknown type '${markup}'.`);
    }
  }
  /**
   * Removes a markup with the given ID, and returns the HTML element
   * associated to the markup. Returns `undefined` if no markup is
   * found.
   *
   * @param id The ID of the markup to remove.
   * @returns The markup element, or undefined.
   */
  async removeMarkup(id) {
    const markup = await this.getMarkupElement(id);
    if (markup != null) {
      markup.remove();
      this.markupRemoved.emit(markup);
    }
    return markup;
  }
  /**
   * Returns a list of markup elements that are children of this component.
   *
   * @returns A list of all markups.
   * @see {@link ViewerMarkup.getMarkupElement}
   */
  async getMarkupElements() {
    return Array.from(this.hostEl.children).filter((e) => isVertexViewerArrowMarkup(e) ||
      isVertexViewerCircleMarkup(e) ||
      isVertexViewerFreeformMarkup(e));
  }
  /**
   * Returns the markup element associated to the given ID.
   *
   * @param id The ID of the markup element to return.
   * @returns A markup element, or `undefined`.
   * @see {@link ViewerMarkup.getMarkupElements}
   */
  async getMarkupElement(id) {
    const markup = await this.getMarkupElements();
    return markup.find((el) => el.id === id);
  }
  /**
   * @ignore
   */
  async handleSelectedMarkupIdChanged() {
    const markup = await this.getMarkupElements();
    markup.forEach((m) => {
      if (m.id === this.selectedMarkupId) {
        m.mode = 'edit';
        this.markupSelectionChanged.emit(m);
      }
      else {
        m.mode = '';
      }
    });
    if (this.selectedMarkupId == null) {
      this.markupSelectionChanged.emit(undefined);
    }
  }
  /**
   * @ignore
   */
  handleToolChanged() {
    this.updatePropsOnMarkupTool();
  }
  /**
   * @ignore
   */
  handleArrowTemplateIdChanged() {
    this.updatePropsOnMarkupTool();
  }
  /**
   * @ignore
   */
  handleCircleTemplateIdChanged() {
    this.updatePropsOnMarkupTool();
  }
  /**
   * @ignore
   */
  handleFreeformTemplateIdChanged() {
    this.updatePropsOnMarkupTool();
  }
  /**
   * @ignore
   */
  async handleViewerChanged(newViewer) {
    this.updatePropsOnMarkupTool();
    this.updatePropsOnMarkups();
  }
  /**
   * @ignore
   */
  handleDisabledChanged() {
    this.updatePropsOnMarkupTool();
  }
  /**
   * @ignore
   */
  async handleStartLineAnchorStyleChanged() {
    this.updatePropsOnMarkupTool();
    // If a piece of arrow markup is currently selected, then update its anchor style
    const selectedArrow = await this.getSelectedArrowMarkup();
    if (selectedArrow != null) {
      selectedArrow.startLineAnchorStyle = this.startLineAnchorStyle;
      this.markupChanged.emit(selectedArrow);
    }
  }
  /**
   * @ignore
   */
  async handleEndLineAnchorStyleChanged() {
    this.updatePropsOnMarkupTool();
    // If a piece of arrow markup is currently selected, then update its anchor style
    const selectedArrow = await this.getSelectedArrowMarkup();
    if (selectedArrow != null) {
      selectedArrow.endLineAnchorStyle = this.endLineAnchorStyle;
      this.markupChanged.emit(selectedArrow);
    }
  }
  /**
   * @ignore
   */
  async handleMarkupEnd(event) {
    var _a;
    const e = event;
    const el = await this.addMarkup(e.detail);
    await new Promise((resolve) => {
      const handleRender = () => {
        el.removeEventListener('viewRendered', handleRender);
        resolve();
      };
      el.addEventListener('viewRendered', handleRender);
    });
    if (this.selectNew) {
      this.selectedMarkupId = e.detail.id;
    }
    (_a = this.getMarkupTool()) === null || _a === void 0 ? void 0 : _a.reset();
  }
  /**
   * @ignore
   */
  async handleMarkupUpdated(event) {
    const e = event;
    // Do not emit the markupChanged event if the markup was just created
    if (!e.detail.newlyCreatedMarkup) {
      this.markupChanged.emit(e.detail.markup);
    }
  }
  /**
   * @ignore
   */
  async handleMarkupPointerDown(event) {
    if (!this.disabled) {
      const el = event.target;
      const markups = await this.getMarkupElements();
      const markup = markups.find((m) => m === el);
      if ((markup === null || markup === void 0 ? void 0 : markup.id) != null && (markup === null || markup === void 0 ? void 0 : markup.id) !== '') {
        this.toSelectMarkupId = el.id;
      }
      this.pointerDownPosition = point.create(event.clientX, event.clientY);
      window.addEventListener('pointermove', this.handlePointerMove);
      window.addEventListener('pointerup', this.handlePointerUp);
    }
  }
  /**
   * @ignore
   */
  componentDidLoad() {
    this.updatePropsOnMarkupTool();
  }
  /**
   * @ignore
   */
  render() {
    return (h(Host, null, h("slot", null)));
  }
  appendMarkupElement(el) {
    this.updatePropsOnMarkup(el);
    this.hostEl.appendChild(el);
    this.markupAdded.emit(el);
    return el;
  }
  createArrowMarkupElement() {
    if (this.arrowTemplateId != null) {
      const element = stampTemplateWithId(window.document.body, this.arrowTemplateId, isVertexViewerArrowMarkup, () => console.warn(`Arrow template with ID ${this.arrowTemplateId} not found. Using default arrow element.`), () => console.warn(`Arrow template does not contain a vertex-viewer-markup-arrow. Using default arrow element.`));
      if (element != null) {
        return element;
      }
    }
    return document.createElement('vertex-viewer-markup-arrow');
  }
  createCircleMarkupElement() {
    if (this.circleTemplateId != null) {
      const element = stampTemplateWithId(window.document.body, this.circleTemplateId, isVertexViewerCircleMarkup, () => console.warn(`Circle template with ID ${this.circleTemplateId} not found. Using default circle element.`), () => console.warn(`Circle template does not contain a vertex-viewer-markup-circle. Using default circle element.`));
      if (element != null) {
        return element;
      }
    }
    return document.createElement('vertex-viewer-markup-circle');
  }
  createFreeformMarkupElement() {
    if (this.freeformTemplateId != null) {
      const element = stampTemplateWithId(window.document.body, this.freeformTemplateId, isVertexViewerFreeformMarkup, () => console.warn(`Freeform template with ID ${this.freeformTemplateId} not found. Using default freeform element.`), () => console.warn(`Freeform template does not contain a vertex-viewer-markup-freeform. Using default freeform element.`));
      if (element != null) {
        return element;
      }
    }
    return document.createElement('vertex-viewer-markup-freeform');
  }
  async updatePropsOnMarkups() {
    const markup = await this.getMarkupElements();
    markup.forEach((m) => this.updatePropsOnMarkup(m));
  }
  updatePropsOnMarkup(element) {
    element.viewer = this.viewer;
    element.classList.add('viewer-markup__markup');
  }
  updatePropsOnMarkupTool() {
    const tool = this.getMarkupTool();
    if (tool != null) {
      tool.disabled = this.disabled;
      tool.arrowTemplateId = this.arrowTemplateId;
      tool.circleTemplateId = this.circleTemplateId;
      tool.freeformTemplateId = this.freeformTemplateId;
      tool.tool = this.tool;
      tool.viewer = this.viewer;
      tool.startLineAnchorStyle = this.startLineAnchorStyle;
      tool.endLineAnchorStyle = this.endLineAnchorStyle;
    }
  }
  getMarkupTool() {
    return this.hostEl.querySelector('vertex-viewer-markup-tool');
  }
  async getSelectedArrowMarkup() {
    if (this.selectedMarkupId != null) {
      const selected = await this.getMarkupElement(this.selectedMarkupId);
      return isVertexViewerArrowMarkup(selected) ? selected : undefined;
    }
  }
  get hostEl() { return this; }
  static get watchers() { return {
    "selectedMarkupId": ["handleSelectedMarkupIdChanged"],
    "tool": ["handleToolChanged"],
    "arrowTemplateId": ["handleArrowTemplateIdChanged"],
    "circleTemplateId": ["handleCircleTemplateIdChanged"],
    "freeformTemplateId": ["handleFreeformTemplateIdChanged"],
    "viewer": ["handleViewerChanged"],
    "disabled": ["handleDisabledChanged"],
    "startLineAnchorStyle": ["handleStartLineAnchorStyleChanged"],
    "endLineAnchorStyle": ["handleEndLineAnchorStyleChanged"]
  }; }
  static get style() { return viewerMarkupCss; }
}, [1, "vertex-viewer-markup", {
    "arrowTemplateId": [1, "arrow-template-id"],
    "circleTemplateId": [1, "circle-template-id"],
    "freeformTemplateId": [1, "freeform-template-id"],
    "tool": [1],
    "disabled": [4],
    "viewer": [16],
    "selectedMarkupId": [1025, "selected-markup-id"],
    "selectNew": [4, "select-new"],
    "startLineAnchorStyle": [1025, "start-line-anchor-style"],
    "endLineAnchorStyle": [1025, "end-line-anchor-style"],
    "toSelectMarkupId": [32],
    "pointerDownPosition": [32],
    "addMarkup": [64],
    "removeMarkup": [64],
    "getMarkupElements": [64],
    "getMarkupElement": [64]
  }, [[0, "markupEnd", "handleMarkupEnd"], [0, "interactionEnd", "handleMarkupUpdated"], [1, "pointerdown", "handleMarkupPointerDown"]]]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-viewer-markup", "vertex-viewer-markup-arrow", "vertex-viewer-markup-circle", "vertex-viewer-markup-freeform"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-viewer-markup":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, ViewerMarkup);
      }
      break;
    case "vertex-viewer-markup-arrow":
      if (!customElements.get(tagName)) {
        defineCustomElement$4();
      }
      break;
    case "vertex-viewer-markup-circle":
      if (!customElements.get(tagName)) {
        defineCustomElement$3();
      }
      break;
    case "vertex-viewer-markup-freeform":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}
defineCustomElement$1();

const VertexViewerMarkup = ViewerMarkup;
const defineCustomElement = defineCustomElement$1;

export { VertexViewerMarkup, defineCustomElement };

//# sourceMappingURL=vertex-viewer-markup.js.map