/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { proxyCustomElement, HTMLElement, h, Host } from '@stencil/core/internal/client';
import { S as SceneViewAPIClient_1 } from './scene_view_api_pb_service.js';
import { p as parseAndValidateConfig } from './config.js';
import { a as MeasurementEntity, M as MeasurementController } from './entities.js';
import { E as EntityType } from './streamAttributes.js';
import './scene_view_api_pb.js';
import { M as MeasurementModel } from './model.js';
import { M as MeasurementOverlayManager } from './overlays.js';
import { p as point } from './bundle.esm.js';
import { a as measurementWithArrowCursor } from './cursors.js';
import { g as getMouseClientPosition } from './dom.js';
import { E as ElementRectObserver } from './elementRectObserver.js';
import { d as defineCustomElement$3 } from './viewer-measurement-line.js';
import { d as defineCustomElement$2 } from './viewer-measurement-overlays.js';

class MeasurementInteractionHandler {
  constructor(controller, measurableEntityTypes) {
    this.rectObserver = new ElementRectObserver();
    this.handlePointerMove = async (event) => {
      if (await this.isMeasurableEntityUnderPointer(event)) {
        this.clearCursor();
        this.addCursor(measurementWithArrowCursor);
      }
      else {
        this.clearCursor();
      }
    };
    this.handlePointerDown = (event) => {
      this.ifNoInteraction(event, async () => {
        if (await this.isMeasurableEntityUnderPointer(event)) {
          this.measureEntityUnderPointer(event);
        }
        else {
          await this.controller.clearEntities();
        }
      });
    };
    this.controller = controller;
    this.measurableEntityTypes = measurableEntityTypes;
  }
  get elementRect() {
    return this.rectObserver.rect;
  }
  initialize(element, api) {
    this.element = element;
    this.api = api;
    this.rectObserver.observe(element);
    element.addEventListener('pointermove', this.handlePointerMove);
    element.addEventListener('pointerdown', this.handlePointerDown);
  }
  dispose() {
    var _a, _b, _c;
    this.rectObserver.disconnect();
    (_a = this.measurementInteraction) === null || _a === void 0 ? void 0 : _a.dispose();
    this.clearCursor();
    (_b = this.element) === null || _b === void 0 ? void 0 : _b.removeEventListener('pointermove', this.handlePointerMove);
    (_c = this.element) === null || _c === void 0 ? void 0 : _c.removeEventListener('pointerdown', this.handlePointerDown);
    this.element = undefined;
    this.api = undefined;
  }
  addCursor(cursor) {
    this.cursor = this.ifInitialized(({ api }) => api.addCursor(cursor));
  }
  clearCursor() {
    var _a;
    (_a = this.cursor) === null || _a === void 0 ? void 0 : _a.dispose();
    this.cursor = undefined;
  }
  async isMeasurableEntityUnderPointer(event) {
    var _a;
    const pt = getMouseClientPosition(event, this.elementRect);
    const type = await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.getEntityTypeAtPoint(pt));
    return type != null && this.measurableEntityTypes.includes(type);
  }
  measureEntityUnderPointer(event) {
    this.ifInitialized(async ({ api }) => {
      const pt = getMouseClientPosition(event, this.elementRect);
      const [hit] = await api.hitItems(pt);
      const entity = hit != null ? MeasurementEntity.fromHit(hit) : undefined;
      if (entity != null) {
        await this.controller.addEntity(entity);
      }
      else {
        await this.controller.clearEntities();
      }
    });
  }
  ifInitialized(f) {
    if (this.element != null && this.api != null) {
      return f({ element: this.element, api: this.api });
    }
    else {
      throw new Error('Measurement interaction handler not initialized.');
    }
  }
  ifNoInteraction(event, f) {
    const startPos = point.create(event.clientX, event.clientY);
    let didInteract = false;
    const handleMouseMove = (event) => {
      const pos = point.create(event.clientX, event.clientY);
      const dis = point.distance(startPos, pos);
      didInteract = dis > 2;
    };
    const handleMouseUp = () => {
      window.removeEventListener('pointermove', handleMouseMove);
      window.removeEventListener('pointerup', handleMouseUp);
      if (!didInteract) {
        f();
      }
    };
    window.addEventListener('pointermove', handleMouseMove);
    window.addEventListener('pointerup', handleMouseUp);
  }
}

const viewerMeasurementPreciseCss = ":host{display:block;position:absolute;top:0;bottom:0;left:0;right:0;pointer-events:none}";

const ViewerMeasurementPrecise = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    /**
     * The model that contains the entities and outcomes from performing precise
     * measurements.
     */
    this.measurementModel = new MeasurementModel();
    /**
     * The manager that is responsible for measurement overlays to present by this
     * component.
     */
    this.measurementOverlays = new MeasurementOverlayManager();
    /**
     * @internal
     *
     * An internal property that can be used to opt-in to performing measurements
     * on other types of entities.
     */
    this.measurableEntityTypes = [
      EntityType.PRECISE_SURFACE,
      EntityType.IMPRECISE_SURFACE,
    ];
    /**
     * The environment that will be used to request measurement results.
     */
    this.configEnv = 'platprod';
  }
  /**
   * @ignore
   */
  connectedCallback() {
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.setupController();
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  disconnectedCallback() {
    this.clearInteractionHandler();
  }
  /**
   * @ignore
   */
  handleMeasurableEntityTypesChanged() {
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  handleMeasurementControllerChanged() {
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  handleMeasurementModelChanged() {
    this.setupController();
  }
  /**
   * @ignore
   */
  handleViewerChanged() {
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  render() {
    return (h(Host, null, h("vertex-viewer-measurement-overlays", { viewer: this.viewer, measurementOverlays: this.measurementOverlays })));
  }
  setupController() {
    var _a;
    const config = parseAndValidateConfig(this.configEnv, this.config);
    const client = new SceneViewAPIClient_1(config.network.sceneViewHost);
    this.measurementController = new MeasurementController(this.measurementModel, client, () => { var _a; return (_a = this.viewer) === null || _a === void 0 ? void 0 : _a.token; }, (_a = this.viewer) === null || _a === void 0 ? void 0 : _a.deviceId);
  }
  clearInteractionHandler() {
    var _a;
    (_a = this.registeredInteractionHandler) === null || _a === void 0 ? void 0 : _a.then((handler) => handler.dispose());
    this.registeredInteractionHandler = undefined;
  }
  setupInteractionHandler() {
    var _a;
    this.clearInteractionHandler();
    if (this.measurementController != null) {
      this.registeredInteractionHandler =
        (_a = this.viewer) === null || _a === void 0 ? void 0 : _a.registerInteractionHandler(new MeasurementInteractionHandler(this.measurementController, this.measurableEntityTypes));
    }
  }
  static get watchers() { return {
    "measurableEntityTypes": ["handleMeasurableEntityTypesChanged"],
    "measurementController": ["handleMeasurementControllerChanged"],
    "measurementModel": ["handleMeasurementModelChanged"],
    "viewer": ["handleViewerChanged"]
  }; }
  static get style() { return viewerMeasurementPreciseCss; }
}, [1, "vertex-viewer-measurement-precise", {
    "measurementModel": [16],
    "measurementOverlays": [16],
    "measurementController": [1040],
    "measurableEntityTypes": [16],
    "viewer": [16],
    "configEnv": [1, "config-env"],
    "config": [1]
  }]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-viewer-measurement-precise", "vertex-viewer-measurement-line", "vertex-viewer-measurement-overlays"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-viewer-measurement-precise":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, ViewerMeasurementPrecise);
      }
      break;
    case "vertex-viewer-measurement-line":
      if (!customElements.get(tagName)) {
        defineCustomElement$3();
      }
      break;
    case "vertex-viewer-measurement-overlays":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}
defineCustomElement$1();

const VertexViewerMeasurementPrecise = ViewerMeasurementPrecise;
const defineCustomElement = defineCustomElement$1;

export { VertexViewerMeasurementPrecise, defineCustomElement };

//# sourceMappingURL=vertex-viewer-measurement-precise.js.map