/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { proxyCustomElement, HTMLElement, h, Host } from '@stencil/core/internal/client';
import { a as translatePointToRelative, P as PinController, d as defineCustomElement$3 } from './viewer-pin-label.js';
import { p as point } from './bundle.esm.js';
import { a as uuid } from './browser.esm.js';
import { l as labelPinCursor, p as pinCursor } from './cursors.js';
import { g as getMouseClientPosition } from './dom.js';
import { E as ElementRectObserver } from './elementRectObserver.js';
import { E as EntityType } from './streamAttributes.js';
import { P as PinModel } from './model3.js';
import { g as getMarkupBoundingClientRect } from './dom2.js';
import { d as defineCustomElement$7 } from './viewer-dom-element.js';
import { d as defineCustomElement$6 } from './viewer-dom-renderer.js';
import { d as defineCustomElement$5 } from './viewer-icon.js';
import { d as defineCustomElement$4 } from './viewer-pin-group.js';
import { d as defineCustomElement$2 } from './viewer-pin-label-line.js';

class PinsInteractionHandler {
  constructor(controller, xOffset, yOffset) {
    this.rectObserver = new ElementRectObserver();
    this.droppableSurfaces = [
      EntityType.PRECISE_SURFACE,
      EntityType.IMPRECISE_SURFACE,
      EntityType.GENERIC_GEOMETRY,
    ];
    this.handlePointerMove = async (event) => {
      const isDroppableSurface = await this.isDroppableSurface(event);
      const draggable = this.controller.getDraggable();
      if (draggable != null && isDroppableSurface) {
        this.handleDrag(draggable, event);
      }
      if (this.controller.getDraggable() == null &&
        this.controller.getToolMode() === 'edit' &&
        isDroppableSurface) {
        this.addCursor(this.getCursorType());
      }
      else {
        this.clearCursor();
      }
    };
    this.handleDrag = async (draggable, event) => {
      const pt = getMouseClientPosition(event, this.elementRect);
      const worldPosition = await this.getWorldPositionForPoint(pt);
      if (worldPosition != null) {
        this.controller.updateDraggable(Object.assign(Object.assign({}, draggable), { lastPoint: pt }), worldPosition);
      }
    };
    this.handlePointerDown = async (pointerDown) => {
      const pointerUp = (pointerUp) => {
        const distanceBetweenStartAndEndPoint = point.distance(point.create(pointerDown.clientX, pointerUp.clientY), point.create(pointerUp.clientX, pointerUp.clientY));
        if (distanceBetweenStartAndEndPoint <= 2 && pointerDown.buttons !== 2) {
          if (this.controller.getToolMode() === 'edit') {
            const pt = getMouseClientPosition(pointerDown, this.elementRect);
            this.handlePlacePin(pt);
          }
        }
        dispose();
      };
      const dispose = () => {
        window.removeEventListener('pointerup', pointerUp);
      };
      window.addEventListener('pointerup', pointerUp);
      return {
        dispose,
      };
    };
    this.handlePointerUp = async () => {
      const draggable = this.controller.getDraggable();
      const lastPoint = draggable === null || draggable === void 0 ? void 0 : draggable.lastPoint;
      this.controller.setDraggable(undefined);
      if (lastPoint != null && draggable != null) {
        this.ifInitialized(async ({ api }) => {
          var _a, _b, _c, _d, _e, _f;
          const [hit] = await api.hitItems(lastPoint);
          if ((hit === null || hit === void 0 ? void 0 : hit.hitPoint) != null &&
            ((_a = hit === null || hit === void 0 ? void 0 : hit.hitPoint) === null || _a === void 0 ? void 0 : _a.x) != null &&
            ((_b = hit === null || hit === void 0 ? void 0 : hit.hitPoint) === null || _b === void 0 ? void 0 : _b.y) != null &&
            ((_c = hit === null || hit === void 0 ? void 0 : hit.hitPoint) === null || _c === void 0 ? void 0 : _c.z) != null) {
            this.controller.updateDraggable(Object.assign(Object.assign({}, draggable), { lastPoint }), {
              x: (_d = hit === null || hit === void 0 ? void 0 : hit.hitPoint) === null || _d === void 0 ? void 0 : _d.x,
              y: hit === null || hit === void 0 ? void 0 : hit.hitPoint.y,
              z: hit === null || hit === void 0 ? void 0 : hit.hitPoint.z,
            }, (_f = (_e = hit === null || hit === void 0 ? void 0 : hit.partId) === null || _e === void 0 ? void 0 : _e.hex) !== null && _f !== void 0 ? _f : undefined);
          }
        });
      }
    };
    this.controller = controller;
    this.xOffset = xOffset;
    this.yOffset = yOffset;
  }
  get elementRect() {
    return this.rectObserver.rect;
  }
  initialize(element, api) {
    this.element = element;
    this.api = api;
    this.rectObserver.observe(element);
    element.addEventListener('pointermove', this.handlePointerMove);
    element.addEventListener('pointerdown', this.handlePointerDown);
    window.addEventListener('pointerup', this.handlePointerUp);
  }
  dispose() {
    var _a, _b;
    this.rectObserver.disconnect();
    this.clearCursor();
    (_a = this.element) === null || _a === void 0 ? void 0 : _a.removeEventListener('pointermove', this.handlePointerMove);
    (_b = this.element) === null || _b === void 0 ? void 0 : _b.removeEventListener('pointerdown', this.handlePointerDown);
    window.removeEventListener('pointerup', this.handlePointerUp);
    this.element = undefined;
    this.api = undefined;
  }
  async getWorldPositionForPoint(pt) {
    return this.ifInitialized(async ({ api }) => {
      const vector3 = await api.getWorldPointFromViewport(pt);
      return vector3;
    });
  }
  async handlePlacePin(pt, existingPin) {
    this.ifInitialized(async ({ api }) => {
      var _a, _b, _c, _d, _e, _f;
      const [hit] = await api.hitItems(pt);
      if ((hit === null || hit === void 0 ? void 0 : hit.hitPoint) != null && this.elementRect != null) {
        if ((hit === null || hit === void 0 ? void 0 : hit.hitPoint) != null &&
          (hit === null || hit === void 0 ? void 0 : hit.hitPoint.x) != null &&
          (hit === null || hit === void 0 ? void 0 : hit.hitPoint.y) != null &&
          (hit === null || hit === void 0 ? void 0 : hit.hitPoint.z) != null) {
          const pinId = existingPin != null ? existingPin.id : uuid.create();
          const attributes = this.controller.getStyleAttributes() != null
            ? {
              style: this.controller.getStyleAttributes(),
            }
            : undefined;
          switch (this.controller.getToolType()) {
            case 'pin-icon':
              this.controller.setPin({
                type: 'icon',
                id: pinId,
                worldPosition: {
                  x: (_a = hit === null || hit === void 0 ? void 0 : hit.hitPoint) === null || _a === void 0 ? void 0 : _a.x,
                  y: hit === null || hit === void 0 ? void 0 : hit.hitPoint.y,
                  z: hit === null || hit === void 0 ? void 0 : hit.hitPoint.z,
                },
                partId: (_c = (_b = hit === null || hit === void 0 ? void 0 : hit.itemId) === null || _b === void 0 ? void 0 : _b.hex) !== null && _c !== void 0 ? _c : undefined,
                attributes,
              });
              break;
            case 'pin-text':
              const isNewPin = existingPin == null;
              const relativePoint = translatePointToRelative(pt, this.elementRect, isNewPin, this.xOffset, this.yOffset);
              this.controller.setPin({
                type: 'text',
                id: pinId,
                worldPosition: {
                  x: (_d = hit === null || hit === void 0 ? void 0 : hit.hitPoint) === null || _d === void 0 ? void 0 : _d.x,
                  y: hit === null || hit === void 0 ? void 0 : hit.hitPoint.y,
                  z: hit === null || hit === void 0 ? void 0 : hit.hitPoint.z,
                },
                partId: (_f = (_e = hit === null || hit === void 0 ? void 0 : hit.itemId) === null || _e === void 0 ? void 0 : _e.hex) !== null && _f !== void 0 ? _f : undefined,
                label: {
                  point: relativePoint,
                },
                attributes,
              });
              this.controller.setSelectedPinId(pinId);
              break;
          }
        }
      }
      else {
        this.controller.setSelectedPinId();
      }
    });
  }
  getCursorType() {
    switch (this.controller.getToolType()) {
      case 'pin-icon':
        return pinCursor;
      case 'pin-text':
        return labelPinCursor;
    }
  }
  addCursor(cursor) {
    this.clearCursor();
    this.cursor = this.ifInitialized(({ api }) => api.addCursor(cursor));
  }
  async isDroppableSurface(event) {
    var _a;
    const pt = getMouseClientPosition(event, this.elementRect);
    const type = await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.getEntityTypeAtPoint(pt));
    return type != null && this.droppableSurfaces.includes(type);
  }
  ifInitialized(f) {
    if (this.element != null && this.api != null) {
      return f({ element: this.element, api: this.api });
    }
    else {
      throw new Error('Pins interaction handler not initialized.');
    }
  }
  clearCursor() {
    var _a;
    (_a = this.cursor) === null || _a === void 0 ? void 0 : _a.dispose();
    this.cursor = undefined;
  }
}

const viewerPinToolCss = ":host{position:absolute;top:0;bottom:0;left:0;right:0;overflow:hidden;pointer-events:none;--viewer-pin-tool-initial-label-offset-x:20;--viewer-pin-tool-initial-label-offset-y:20}";

const ViewerPinTool = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    /**
     * The model that contains the entities and outcomes from performing pin annotations
     */
    this.pinModel = new PinModel();
    /**
     * The type of pin.
     *
     * This property will automatically be set.
     */
    this.tool = 'pin-icon';
    /**
     * The mode of the pin tool
     */
    this.mode = 'view';
    this.pins = [];
    this.handleSetProjectionMatrix = () => {
      var _a, _b;
      this.projectionViewMatrix =
        (_b = (_a = this.viewer) === null || _a === void 0 ? void 0 : _a.frame) === null || _b === void 0 ? void 0 : _b.scene.camera.projectionViewMatrix;
    };
  }
  /**
   * @ignore
   */
  watchModeChange() {
    var _a;
    (_a = this.pinController) === null || _a === void 0 ? void 0 : _a.setToolMode(this.mode);
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  watchTypeChange() {
    var _a;
    (_a = this.pinController) === null || _a === void 0 ? void 0 : _a.setToolType(this.tool);
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  watchAccentColorChange() {
    var _a;
    (_a = this.pinController) === null || _a === void 0 ? void 0 : _a.setAccentColor(this.accentColor);
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  watchPrimaryColorChange() {
    var _a;
    (_a = this.pinController) === null || _a === void 0 ? void 0 : _a.setPrimaryColor(this.primaryColor);
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  connectedCallback() {
    this.setupInteractionHandler();
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.updateViewport();
    this.setupController();
    this.setupInteractionHandler();
    this.pinModel.onEntitiesChanged((entities) => {
      this.pins = entities;
    });
    this.pinModel.onSelectionChange((selectedId) => {
      this.selectedPinId = selectedId;
    });
    this.handleViewerChanged(this.viewer, undefined);
  }
  componentDidLoad() {
    this.resizeObserver = new ResizeObserver(() => this.updateViewport());
    this.resizeObserver.observe(this.hostEl);
  }
  /**
   * @ignore
   */
  disconnectedCallback() {
    this.clearInteractionHandler();
    this.clearModelListeners();
  }
  /**
   * @ignore
   */
  handleViewerChanged(newViewer, oldViewer) {
    this.setupInteractionHandler();
    if (oldViewer != null) {
      oldViewer.removeEventListener('frameDrawn', this.handleSetProjectionMatrix);
    }
    if (newViewer != null) {
      newViewer.addEventListener('frameDrawn', this.handleSetProjectionMatrix);
    }
    this.handleSetProjectionMatrix();
  }
  render() {
    return (h(Host, null, h("vertex-viewer-dom-renderer", { viewer: this.viewer, drawMode: "2d" }, this.pins.map((pin) => {
      return (h("vertex-viewer-pin-group", { id: `pin-group-${pin.id}`, "data-is-dom-group-element": true, pin: pin, elementBounds: this.elementBounds, pinModel: this.pinModel, pinController: this.pinController, projectionViewMatrix: this.projectionViewMatrix, selected: this.selectedPinId === pin.id }));
    }))));
  }
  setupController() {
    const attributes = this.accentColor || this.primaryColor
      ? {
        accentColor: this.accentColor,
        primaryColor: this.primaryColor,
      }
      : undefined;
    this.pinController = new PinController(this.pinModel, this.mode, this.tool, attributes);
  }
  clearInteractionHandler() {
    var _a;
    (_a = this.registeredInteractionHandler) === null || _a === void 0 ? void 0 : _a.then((handler) => handler.dispose());
    this.registeredInteractionHandler = undefined;
  }
  setupInteractionHandler() {
    var _a;
    const hostStyles = window.getComputedStyle(this.hostEl);
    const xOffset = hostStyles
      .getPropertyValue('--viewer-pin-tool-initial-label-offset-x')
      .trim();
    const yOffset = hostStyles
      .getPropertyValue('--viewer-pin-tool-initial-label-offset-y')
      .trim();
    this.clearInteractionHandler();
    if (this.pinController != null) {
      this.registeredInteractionHandler =
        (_a = this.viewer) === null || _a === void 0 ? void 0 : _a.registerInteractionHandler(new PinsInteractionHandler(this.pinController, parseInt(xOffset), parseInt(yOffset)));
    }
  }
  clearModelListeners() {
    var _a, _b;
    (_a = this.onEntitiesChangedHandler) === null || _a === void 0 ? void 0 : _a.dispose();
    this.onEntitiesChangedHandler = undefined;
    (_b = this.resizeObserver) === null || _b === void 0 ? void 0 : _b.disconnect();
  }
  updateViewport() {
    const rect = getMarkupBoundingClientRect(this.hostEl);
    this.elementBounds = rect;
  }
  get hostEl() { return this; }
  static get watchers() { return {
    "mode": ["watchModeChange"],
    "tool": ["watchTypeChange"],
    "accentColor": ["watchAccentColorChange"],
    "primaryColor": ["watchPrimaryColorChange"],
    "viewer": ["handleViewerChanged"]
  }; }
  static get style() { return viewerPinToolCss; }
}, [1, "vertex-viewer-pin-tool", {
    "pinController": [1040],
    "pinModel": [16],
    "viewer": [16],
    "tool": [1025],
    "mode": [1025],
    "primaryColor": [1025, "primary-color"],
    "accentColor": [1025, "accent-color"],
    "pins": [32],
    "selectedPinId": [32],
    "elementBounds": [32],
    "projectionViewMatrix": [32]
  }]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-viewer-pin-tool", "vertex-viewer-dom-element", "vertex-viewer-dom-renderer", "vertex-viewer-icon", "vertex-viewer-pin-group", "vertex-viewer-pin-label", "vertex-viewer-pin-label-line"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-viewer-pin-tool":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, ViewerPinTool);
      }
      break;
    case "vertex-viewer-dom-element":
      if (!customElements.get(tagName)) {
        defineCustomElement$7();
      }
      break;
    case "vertex-viewer-dom-renderer":
      if (!customElements.get(tagName)) {
        defineCustomElement$6();
      }
      break;
    case "vertex-viewer-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$5();
      }
      break;
    case "vertex-viewer-pin-group":
      if (!customElements.get(tagName)) {
        defineCustomElement$4();
      }
      break;
    case "vertex-viewer-pin-label":
      if (!customElements.get(tagName)) {
        defineCustomElement$3();
      }
      break;
    case "vertex-viewer-pin-label-line":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}
defineCustomElement$1();

const VertexViewerPinTool = ViewerPinTool;
const defineCustomElement = defineCustomElement$1;

export { VertexViewerPinTool, defineCustomElement };

//# sourceMappingURL=vertex-viewer-pin-tool.js.map