/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { h, Fragment, proxyCustomElement, HTMLElement, Host } from '@stencil/core/internal/client';
import { v as vector3, q as quaternion, i as euler } from './bundle.esm.js';
import { r as readDOM } from './stencil.js';
import { O as Orientation, f as FramePerspectiveCamera, S as StandardView } from './streamAttributes.js';
import { c as classnames } from './index2.js';
import { d as defineCustomElement$4 } from './viewer-dom-element.js';
import { d as defineCustomElement$3 } from './viewer-dom-group.js';
import { d as defineCustomElement$2 } from './viewer-dom-renderer.js';

// eslint-disable-next-line @typescript-eslint/no-unused-vars
const viewCubeSides = {
  front: {
    direction: vector3.back(),
    quaternion: quaternion.create(),
  },
  back: {
    direction: vector3.forward(),
    quaternion: quaternion.fromAxisAngle(vector3.up(), Math.PI),
  },
  left: {
    direction: vector3.left(),
    quaternion: quaternion.fromAxisAngle(vector3.down(), Math.PI / 2),
  },
  right: {
    direction: vector3.right(),
    quaternion: quaternion.fromAxisAngle(vector3.up(), Math.PI / 2),
  },
  top: {
    direction: vector3.up(),
    quaternion: quaternion.fromAxisAngle(vector3.right(), -Math.PI / 2),
  },
  bottom: {
    direction: vector3.down(),
    quaternion: quaternion.fromAxisAngle(vector3.left(), -Math.PI / 2),
  },
};
const TriadAxis = ({ label, length, rotationAxis, }) => {
  const axisStyles = { width: `${length + 5}px` };
  const quaternion$1 = quaternion.fromAxisAngle(rotationAxis, Math.PI / 2);
  const labelPos = vector3.scale(length + 15, vector3.right());
  return (h(Fragment, null,
    h("vertex-viewer-dom-group", { quaternion: quaternion$1 },
      h("vertex-viewer-dom-element", { style: axisStyles, billboardOff: true, interactionsOff: true },
        h("div", { class: `triad-axis triad-axis-${label.toLowerCase()}` })),
      h("vertex-viewer-dom-element", { rotation: euler.create({ x: Math.PI / 2, y: 0, z: 0 }), style: axisStyles, billboardOff: true, interactionsOff: true },
        h("div", { class: `triad-axis triad-axis-${label.toLowerCase()}` })),
      h("vertex-viewer-dom-element", { position: labelPos, interactionsOff: true },
        h("div", { class: `triad-label triad-label-${label.toLowerCase()}` }, label)))));
};
const ViewCubeSide = ({ label, length, side, disabled, onPointerDown, }) => {
  const { direction, quaternion } = viewCubeSides[side];
  const position = vector3.scale(length / 2 - 0.1, direction);
  return (h("vertex-viewer-dom-element", { class: classnames('cube-side', { disabled }), position: position, quaternion: quaternion, style: { width: `${length}px`, height: `${length}px` }, onPointerDown: onPointerDown, billboardOff: true },
    h("div", { class: `cube-side-face cube-side-face-${label.toLowerCase()}` }, label)));
};
const ViewCubeCorner = ({ length, face1Side, face1Corner, face2Side, face2Corner, face3Side, face3Corner, disabled, onPointerDown, }) => {
  const { direction: dir1, quaternion: rot1 } = viewCubeSides[face1Side];
  const { direction: dir2, quaternion: rot2 } = viewCubeSides[face2Side];
  const { direction: dir3, quaternion: rot3 } = viewCubeSides[face3Side];
  const position1 = vector3.scale(length / 2, dir1);
  const position2 = vector3.scale(length / 2, dir2);
  const position3 = vector3.scale(length / 2, dir3);
  const styles = { width: `${length}px`, height: `${length}px` };
  return (h("vertex-viewer-dom-group", { class: classnames('cube-corner', { disabled }), onPointerDown: onPointerDown },
    h("vertex-viewer-dom-element", { position: position1, quaternion: rot1, style: styles, billboardOff: true },
      h("div", { class: `cube-corner-face ${face1Corner}` })),
    h("vertex-viewer-dom-element", { position: position2, quaternion: rot2, style: styles, billboardOff: true },
      h("div", { class: `cube-corner-face ${face2Corner}` })),
    h("vertex-viewer-dom-element", { position: position3, quaternion: rot3, style: styles, billboardOff: true },
      h("div", { class: `cube-corner-face ${face3Corner}` }))));
};
const ViewCubeEdge = ({ length, face1Side, face2Side, face1Edge, face2Edge, disabled, onPointerDown, }) => {
  const { direction: dir1, quaternion: rot1 } = viewCubeSides[face1Side];
  const { direction: dir2, quaternion: rot2 } = viewCubeSides[face2Side];
  const position1 = vector3.scale(length / 2, dir1);
  const position2 = vector3.scale(length / 2, dir2);
  const styles = { width: `${length}px`, height: `${length}px` };
  return (h("vertex-viewer-dom-group", { class: classnames('cube-edge', { disabled }), onPointerDown: onPointerDown },
    h("vertex-viewer-dom-element", { position: position1, quaternion: rot1, style: styles, billboardOff: true },
      h("div", { class: `cube-edge-face ${face1Edge}` })),
    h("vertex-viewer-dom-element", { position: position2, quaternion: rot2, style: styles, billboardOff: true },
      h("div", { class: `cube-edge-face ${face2Edge}` }))));
};
const ViewCubeShadow = ({ length, }) => {
  const { direction, quaternion } = viewCubeSides.top;
  const position = vector3.scale(-length / 2, direction);
  return (h("vertex-viewer-dom-element", { class: "cube-shadow", position: position, quaternion: quaternion, style: { width: `${length}px`, height: `${length}px` }, billboardOff: true, interactionsOff: true },
    h("div", { class: "cube-shadow-face" })));
};

const viewerViewCubeCss = ":host{--viewer-view-cube-side-background:var(--white);--viewer-view-cube-side-background-hover:var(--blue-500);--viewer-view-cube-side-background-hover-opacity:0.25;--viewer-view-cube-side-border:1.5px solid var(--neutral-500);--viewer-view-cube-edge-length:12px;--viewer-view-cube-opacity:0.8;--viewer-view-cube-triad-x-axis-color:var(--x-axis-color);--viewer-view-cube-triad-y-axis-color:var(--y-axis-color);--viewer-view-cube-triad-z-axis-color:var(--z-axis-color);color:var(--neutral-700);width:80px;height:80px;margin:20px;position:relative;display:flex;align-items:center;justify-content:center;user-select:none}.renderer{position:unset;width:100%;height:100%;overflow:visible !important}.reference-point{width:5px;height:5px;border-radius:50%;background-color:black}.triad{pointer-events:none}.triad-axis{width:100%;height:1.5px;outline:1px solid transparent;transform:translateX(50%)}.triad-axis-x{background-color:#ea3324}.triad-axis-y{background-color:#4faf32}.triad-axis-z{background-color:#0000ff}.triad-label{font-family:Arial, Helvetica, sans-serif;font-size:0.6875rem}.triad-label-x{color:var(--viewer-view-cube-triad-x-axis-color)}.triad-label-y{color:var(--viewer-view-cube-triad-y-axis-color)}.triad-label-z{color:var(--viewer-view-cube-triad-z-axis-color)}.cube{pointer-events:initial}.cube:hover .cube-side-face{opacity:1}.cube-side{backface-visibility:hidden;-webkit-backface-visibility:hidden}.cube-side-face{box-sizing:border-box;font-size:0.875rem;text-transform:uppercase;width:100%;height:100%;background-color:var(--viewer-view-cube-side-background);border:var(--viewer-view-cube-side-border);display:flex;align-items:center;justify-content:center;outline:1px solid transparent;opacity:var(--viewer-view-cube-opacity);transition:opacity 0.2s ease-in-out}.cube-side-face::before{content:'';position:absolute;left:calc(var(--viewer-view-cube-edge-length) - 4px);right:calc(var(--viewer-view-cube-edge-length) - 4px);top:calc(var(--viewer-view-cube-edge-length) - 4px);bottom:calc(var(--viewer-view-cube-edge-length) - 4px)}.cube-corner,.cube-edge{position:relative}.cube-corner-face,.cube-edge-face{position:absolute;backface-visibility:hidden;-webkit-backface-visibility:hidden}.cube-edge-face.top{top:0;left:var(--viewer-view-cube-edge-length);right:var(--viewer-view-cube-edge-length);height:var(--viewer-view-cube-edge-length)}.cube-edge-face.bottom{bottom:0;left:var(--viewer-view-cube-edge-length);right:var(--viewer-view-cube-edge-length);height:var(--viewer-view-cube-edge-length)}.cube-edge-face.left{left:0;top:var(--viewer-view-cube-edge-length);bottom:var(--viewer-view-cube-edge-length);width:var(--viewer-view-cube-edge-length)}.cube-edge-face.right{right:0;top:var(--viewer-view-cube-edge-length);bottom:var(--viewer-view-cube-edge-length);width:var(--viewer-view-cube-edge-length)}.cube-corner-face{width:var(--viewer-view-cube-edge-length);height:var(--viewer-view-cube-edge-length)}.cube-corner-face.top-left{top:0;left:0}.cube-corner-face.top-right{top:0;right:0}.cube-corner-face.bottom-right{bottom:0;right:0}.cube-corner-face.bottom-left{bottom:0;left:0}.cube-shadow{backface-visibility:hidden;-webkit-backface-visibility:hidden}.cube-shadow-face{position:absolute;left:4px;right:4px;top:4px;bottom:4px;background-color:black;opacity:0.12;filter:blur(4px)}.cube-side-face,.cube-corner-face,.cube-edge-face{pointer-events:initial}.cube-corner .cube-corner-face,.cube-edge .cube-edge-face,.cube-side .cube-side-face::before{background:var(--viewer-view-cube-side-background-hover);opacity:0}.cube-corner:hover .cube-corner-face,.cube-edge:hover .cube-edge-face,.cube-side:hover .cube-side-face::before{opacity:var(--viewer-view-cube-side-background-hover-opacity)}.cube-corner.disabled,.cube-edge.disabled,.cube-side.disabled{pointer-events:none}";

const ViewerViewCube = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.boxLength = 80;
    this.triadPosition = vector3.origin();
    /**
     * The label for the side of the cube on the positive x-axis.
     */
    this.xPositiveLabel = 'Right';
    /**
     * The label for the side of the cube on the negative x-axis.
     */
    this.xNegativeLabel = 'Left';
    /**
     * The label for the side of the cube on the positive y-axis.
     */
    this.yPositiveLabel = 'Top';
    /**
     * The label for the side of the cube on the negative y-axis.
     */
    this.yNegativeLabel = 'Bottom';
    /**
     * The label for the side of the cube on the positive z-axis.
     */
    this.zPositiveLabel = 'Front';
    /**
     * The label for the side of the cube on the negative z-axis.
     */
    this.zNegativeLabel = 'Back';
    /**
     * Disables interactions for standard views.
     */
    this.standardViewsOff = false;
    /**
     * Whether to perform a `viewAll` when clicking on the view cube. If this
     * is set to `false`, the current `lookAt` point will be maintained, and the
     * camera's `position` and `up` vectors will be aligned to the standard view.
     * Defaults to `true`.
     *
     * **Note** Setting this value to `false` can result in the camera being placed
     * underneath geometry depending on the current `viewVector` length, resulting
     * in a view that may be unexpected.
     */
    this.viewAll = true;
    /**
     * The duration of the animation, in milliseconds, when a user performs a
     * standard view interaction. Set to 0 to disable animations.
     */
    this.animationDuration = 500;
    /**
     * Disables the display of the triad.
     */
    this.triadOff = false;
    /**
     * @internal
     */
    this.worldOrientation = Orientation.DEFAULT;
    this.updateMatrices = () => {
      var _a;
      if (((_a = this.viewer) === null || _a === void 0 ? void 0 : _a.frame) != null) {
        const { camera } = this.viewer.frame.scene;
        const halfLength = this.boxLength / 2;
        this.triadPosition = vector3.create(-halfLength - 5, -halfLength - 5, -halfLength - 5);
        // Used to scale the camera position so the cube is the same size as the
        // container. This isn't exact, but pretty close.
        const lengthScalar = 3.125;
        const scale = this.boxLength * lengthScalar;
        const fovY = 21.5;
        this.camera = new FramePerspectiveCamera(vector3.scale(scale, vector3.negate(camera.direction)), vector3.origin(), camera.up, 0.1, 100, 1, fovY);
        this.worldOrientation = this.viewer.frame.scene.worldOrientation;
      }
    };
  }
  handleViewerChanged(newViewer, oldViewer) {
    oldViewer === null || oldViewer === void 0 ? void 0 : oldViewer.removeEventListener('frameDrawn', this.updateMatrices);
    newViewer === null || newViewer === void 0 ? void 0 : newViewer.addEventListener('frameDrawn', this.updateMatrices);
    this.updateMatrices();
  }
  handleStandardView(standardView) {
    if (this.standardViewsOff) {
      return async () => undefined;
    }
    else {
      return async () => {
        if (this.viewer != null) {
          const scene = await this.viewer.scene();
          const animation = this.animationDuration > 0
            ? { animation: { milliseconds: this.animationDuration } }
            : {};
          const worldStandardView = standardView.transformMatrix(this.worldOrientation.matrix);
          // Check to see if any geometry is visible. If not, don't perform viewAll
          const currentBoundingBox = scene.boundingBox();
          if (!this.viewAll) {
            scene
              .camera()
              .standardViewFixedLookAt(worldStandardView)
              .render(animation);
          }
          else if ((currentBoundingBox === null || currentBoundingBox === void 0 ? void 0 : currentBoundingBox.max) != null &&
            vector3.isEqual(currentBoundingBox.max, vector3.origin()) &&
            vector3.isEqual(currentBoundingBox.min, vector3.origin())) {
            scene.camera().standardView(worldStandardView).render(animation);
          }
          else {
            scene
              .camera()
              .standardView(worldStandardView)
              .viewAll()
              .render(animation);
          }
        }
      };
    }
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.handleViewerChanged(this.viewer);
  }
  /**
   * @ignore
   */
  componentDidLoad() {
    if (this.rendererEl != null) {
      const observer = new ResizeObserver(() => this.handleRendererResized());
      observer.observe(this.rendererEl);
      this.handleRendererResized();
    }
  }
  handleRendererResized() {
    readDOM(() => {
      if (this.rendererEl != null) {
        const rect = this.rendererEl.getBoundingClientRect();
        this.boxLength = Math.min(rect.width, rect.height);
      }
    });
  }
  /**
   * @ignore
   */
  render() {
    return (h(Host, null, h("vertex-viewer-dom-renderer", { ref: (ref) => (this.rendererEl = ref), class: "renderer", camera: this.camera }, !this.triadOff && (h("vertex-viewer-dom-group", { class: "triad", position: this.triadPosition }, h(TriadAxis, { label: "X", length: this.boxLength, rotationAxis: vector3.origin() }), h(TriadAxis, { label: "Y", length: this.boxLength, rotationAxis: vector3.back() }), h(TriadAxis, { label: "Z", length: this.boxLength, rotationAxis: vector3.down() }))), h("vertex-viewer-dom-group", { class: "cube", matrix: this.worldOrientation.matrix }, h(ViewCubeShadow, { length: this.boxLength + 10 }), h(ViewCubeSide, { label: this.zPositiveLabel, length: this.boxLength, side: "front", onPointerDown: this.handleStandardView(StandardView.FRONT), disabled: this.standardViewsOff }), h(ViewCubeSide, { label: this.zNegativeLabel, length: this.boxLength, side: "back", onPointerDown: this.handleStandardView(StandardView.BACK), disabled: this.standardViewsOff }), h(ViewCubeSide, { label: this.xNegativeLabel, length: this.boxLength, side: "left", onPointerDown: this.handleStandardView(StandardView.RIGHT), disabled: this.standardViewsOff }), h(ViewCubeSide, { label: this.xPositiveLabel, length: this.boxLength, side: "right", onPointerDown: this.handleStandardView(StandardView.LEFT), disabled: this.standardViewsOff }), h(ViewCubeSide, { label: this.yPositiveLabel, length: this.boxLength, side: "top", onPointerDown: this.handleStandardView(StandardView.TOP), disabled: this.standardViewsOff }), h(ViewCubeSide, { label: this.yNegativeLabel, length: this.boxLength, side: "bottom", onPointerDown: this.handleStandardView(StandardView.BOTTOM), disabled: this.standardViewsOff }), h(ViewCubeEdge, { length: this.boxLength, face1Side: "top", face1Edge: "bottom", face2Side: "front", face2Edge: "top", onPointerDown: this.handleStandardView(StandardView.TOP_FRONT), disabled: this.standardViewsOff }), h(ViewCubeEdge, { length: this.boxLength, face1Side: "front", face1Edge: "right", face2Side: "right", face2Edge: "left", onPointerDown: this.handleStandardView(StandardView.FRONT_LEFT), disabled: this.standardViewsOff }), h(ViewCubeEdge, { length: this.boxLength, face1Side: "bottom", face1Edge: "top", face2Side: "front", face2Edge: "bottom", onPointerDown: this.handleStandardView(StandardView.BOTTOM_FRONT), disabled: this.standardViewsOff }), h(ViewCubeEdge, { length: this.boxLength, face1Side: "front", face1Edge: "left", face2Side: "left", face2Edge: "right", onPointerDown: this.handleStandardView(StandardView.FRONT_RIGHT), disabled: this.standardViewsOff }), h(ViewCubeEdge, { length: this.boxLength, face1Side: "top", face1Edge: "right", face2Side: "right", face2Edge: "top", onPointerDown: this.handleStandardView(StandardView.TOP_LEFT), disabled: this.standardViewsOff }), h(ViewCubeEdge, { length: this.boxLength, face1Side: "back", face1Edge: "left", face2Side: "right", face2Edge: "right", onPointerDown: this.handleStandardView(StandardView.BACK_LEFT), disabled: this.standardViewsOff }), h(ViewCubeEdge, { length: this.boxLength, face1Side: "bottom", face1Edge: "right", face2Side: "right", face2Edge: "bottom", onPointerDown: this.handleStandardView(StandardView.BOTTOM_LEFT), disabled: this.standardViewsOff }), h(ViewCubeEdge, { length: this.boxLength, face1Side: "top", face1Edge: "top", face2Side: "back", face2Edge: "top", onPointerDown: this.handleStandardView(StandardView.TOP_BACK), disabled: this.standardViewsOff }), h(ViewCubeEdge, { length: this.boxLength, face1Side: "back", face1Edge: "right", face2Side: "left", face2Edge: "left", onPointerDown: this.handleStandardView(StandardView.BACK_RIGHT), disabled: this.standardViewsOff }), h(ViewCubeEdge, { length: this.boxLength, face1Side: "bottom", face1Edge: "bottom", face2Side: "back", face2Edge: "bottom", onPointerDown: this.handleStandardView(StandardView.BOTTOM_BACK), disabled: this.standardViewsOff }), h(ViewCubeEdge, { length: this.boxLength, face1Side: "top", face1Edge: "left", face2Side: "left", face2Edge: "top", onPointerDown: this.handleStandardView(StandardView.TOP_RIGHT), disabled: this.standardViewsOff }), h(ViewCubeEdge, { length: this.boxLength, face1Side: "bottom", face1Edge: "left", face2Side: "left", face2Edge: "bottom", onPointerDown: this.handleStandardView(StandardView.BOTTOM_RIGHT), disabled: this.standardViewsOff }), h(ViewCubeCorner, { length: this.boxLength, face1Side: "top", face1Corner: "bottom-left", face2Side: "front", face2Corner: "top-left", face3Side: "left", face3Corner: "top-right", onPointerDown: this.handleStandardView(StandardView.TOP_FRONT_RIGHT), disabled: this.standardViewsOff }), h(ViewCubeCorner, { length: this.boxLength, face1Side: "top", face1Corner: "bottom-right", face2Side: "front", face2Corner: "top-right", face3Side: "right", face3Corner: "top-left", onPointerDown: this.handleStandardView(StandardView.TOP_FRONT_LEFT), disabled: this.standardViewsOff }), h(ViewCubeCorner, { length: this.boxLength, face1Side: "bottom", face1Corner: "top-right", face2Side: "front", face2Corner: "bottom-right", face3Side: "right", face3Corner: "bottom-left", onPointerDown: this.handleStandardView(StandardView.BOTTOM_FRONT_LEFT), disabled: this.standardViewsOff }), h(ViewCubeCorner, { length: this.boxLength, face1Side: "bottom", face1Corner: "top-left", face2Side: "front", face2Corner: "bottom-left", face3Side: "left", face3Corner: "bottom-right", onPointerDown: this.handleStandardView(StandardView.BOTTOM_FRONT_RIGHT), disabled: this.standardViewsOff }), h(ViewCubeCorner, { length: this.boxLength, face1Side: "top", face1Corner: "top-right", face2Side: "back", face2Corner: "top-left", face3Side: "right", face3Corner: "top-right", onPointerDown: this.handleStandardView(StandardView.TOP_BACK_LEFT), disabled: this.standardViewsOff }), h(ViewCubeCorner, { length: this.boxLength, face1Side: "top", face1Corner: "top-left", face2Side: "back", face2Corner: "top-right", face3Side: "left", face3Corner: "top-left", onPointerDown: this.handleStandardView(StandardView.TOP_BACK_RIGHT), disabled: this.standardViewsOff }), h(ViewCubeCorner, { length: this.boxLength, face1Side: "bottom", face1Corner: "bottom-left", face2Side: "back", face2Corner: "bottom-right", face3Side: "left", face3Corner: "bottom-left", onPointerDown: this.handleStandardView(StandardView.BOTTOM_BACK_RIGHT), disabled: this.standardViewsOff }), h(ViewCubeCorner, { length: this.boxLength, face1Side: "bottom", face1Corner: "bottom-right", face2Side: "back", face2Corner: "bottom-left", face3Side: "right", face3Corner: "bottom-right", onPointerDown: this.handleStandardView(StandardView.BOTTOM_BACK_LEFT), disabled: this.standardViewsOff })))));
  }
  static get watchers() { return {
    "viewer": ["handleViewerChanged"]
  }; }
  static get style() { return viewerViewCubeCss; }
}, [1, "vertex-viewer-view-cube", {
    "xPositiveLabel": [1, "x-positive-label"],
    "xNegativeLabel": [1, "x-negative-label"],
    "yPositiveLabel": [1, "y-positive-label"],
    "yNegativeLabel": [1, "y-negative-label"],
    "zPositiveLabel": [1, "z-positive-label"],
    "zNegativeLabel": [1, "z-negative-label"],
    "standardViewsOff": [4, "standard-views-off"],
    "viewAll": [4, "view-all"],
    "animationDuration": [2, "animation-duration"],
    "triadOff": [4, "triad-off"],
    "worldOrientation": [1040],
    "camera": [1040],
    "viewer": [16],
    "boxLength": [32],
    "triadPosition": [32]
  }]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-viewer-view-cube", "vertex-viewer-dom-element", "vertex-viewer-dom-group", "vertex-viewer-dom-renderer"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-viewer-view-cube":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, ViewerViewCube);
      }
      break;
    case "vertex-viewer-dom-element":
      if (!customElements.get(tagName)) {
        defineCustomElement$4();
      }
      break;
    case "vertex-viewer-dom-group":
      if (!customElements.get(tagName)) {
        defineCustomElement$3();
      }
      break;
    case "vertex-viewer-dom-renderer":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}
defineCustomElement$1();

const VertexViewerViewCube = ViewerViewCube;
const defineCustomElement = defineCustomElement$1;

export { VertexViewerViewCube, defineCustomElement };

//# sourceMappingURL=vertex-viewer-view-cube.js.map