/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { v as vector3, q as quaternion, m as matrix4, i as euler } from './bundle.esm.js';
import { o as objects } from './browser.esm.js';

const viewerDomGroupCss = ".sc-vertex-viewer-dom-group-h{display:block;transform-style:preserve-3d}";

const ViewerDomGroup = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.propertyChange = createEvent(this, "propertyChange", 7);
    /**
     * The local 3D position of where this element is located.
     */
    this.position = vector3.origin();
    /**
     * The local 3D position of where this element is located, as a JSON string.
     * JSON representation can either be in the format of `[x, y, z]` or `{"x": 0,
     * "y": 0, "z": 0}`.
     */
    this.positionJson = '';
    /**
     * The local rotation of this element.
     */
    this.quaternion = quaternion.create();
    /**
     * The local rotation of this element, as a JSON string. JSON
     * representation can either be `[x, y, z, w]` or `{"x": 0, "y": 0, "z":
     * 0, "w": 1}`.
     */
    this.quaternionJson = '';
    /**
     * The local scale of this element.
     */
    this.scale = vector3.create(1, 1, 1);
    /**
     * The local scale of this element, as a JSON string. JSON string
     * representation can either be in the format of `[x, y, z]` or `{"x": 0, "y":
     * 0, "z": 0}`.
     */
    this.scaleJson = '';
    /**
     * The local matrix of this element.
     */
    this.matrix = matrix4.makeIdentity();
  }
  /**
   * @ignore
   */
  handlePositionChange() {
    this.syncMatrix();
  }
  /**
   * @ignore
   */
  handlePositionJsonChanged() {
    this.syncPosition();
  }
  /**
   * @ignore
   */
  handleRotationChanged() {
    this.syncQuaternionWithRotation();
  }
  /**
   * @ignore
   */
  handleRotationJsonChanged() {
    this.syncRotation();
  }
  /**
   * @ignore
   */
  handleQuaternionChange() {
    this.syncMatrix();
  }
  /**
   * @ignore
   */
  handleQuaternionJsonChanged() {
    this.syncQuaternion();
  }
  /**
   * @ignore
   */
  handleScaleChange() {
    this.syncMatrix();
  }
  /**
   * @ignore
   */
  handleScaleJsonChanged() {
    this.syncScale();
  }
  /**
   * @ignore
   */
  handleMatrixChanged(newMatrix, oldMatrix) {
    if (!objects.isEqual(newMatrix, oldMatrix)) {
      this.propertyChange.emit();
    }
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.syncProperties();
  }
  /**
   * @ignore
   */
  componentShouldUpdate() {
    return false;
  }
  syncProperties() {
    this.syncPosition();
    this.syncRotation();
    this.syncQuaternion();
    this.syncScale();
    this.syncMatrix();
  }
  syncPosition() {
    this.position =
      this.positionJson.length > 0
        ? this.parseJson('positionJson', this.positionJson, vector3.fromJson)
        : this.position;
  }
  syncRotation() {
    this.rotation =
      this.rotationJson != null && this.rotationJson.length > 0
        ? this.parseJson('rotationJson', this.rotationJson, euler.fromJson)
        : this.rotation;
    this.syncQuaternionWithRotation();
  }
  syncQuaternionWithRotation() {
    this.quaternion =
      this.rotation != null
        ? quaternion.fromEuler(this.rotation)
        : this.quaternion;
  }
  syncQuaternion() {
    this.quaternion =
      this.quaternionJson.length > 0
        ? this.parseJson('quaternionJson', this.quaternionJson, quaternion.fromJson)
        : this.quaternion;
  }
  syncScale() {
    this.scale =
      this.scaleJson.length > 0
        ? this.parseJson('scaleJson', this.scaleJson, vector3.fromJson)
        : this.scale;
  }
  syncMatrix() {
    this.matrix = matrix4.makeTRS(this.position, this.quaternion, this.scale);
  }
  parseJson(propName, value, parse) {
    try {
      return parse(value);
    }
    catch (e) {
      console.warn(`Could not parse \`${propName}\`. Invalid JSON.`);
      throw e;
    }
  }
  /**
   * @ignore
   */
  render() {
    return (h(Host, null, h("slot", null)));
  }
  static get watchers() { return {
    "position": ["handlePositionChange"],
    "positionJson": ["handlePositionJsonChanged"],
    "rotation": ["handleRotationChanged"],
    "rotationJson": ["handleRotationJsonChanged"],
    "quaternion": ["handleQuaternionChange"],
    "quaternionJson": ["handleQuaternionJsonChanged"],
    "scale": ["handleScaleChange"],
    "scaleJson": ["handleScaleJsonChanged"],
    "matrix": ["handleMatrixChanged"]
  }; }
  static get style() { return viewerDomGroupCss; }
}, [6, "vertex-viewer-dom-group", {
    "position": [1040],
    "positionJson": [1, "position"],
    "rotation": [1040],
    "rotationJson": [1, "rotation"],
    "quaternion": [1040],
    "quaternionJson": [1, "quaternion"],
    "scale": [1040],
    "scaleJson": [1, "scale"],
    "matrix": [1040]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-viewer-dom-group"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-viewer-dom-group":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, ViewerDomGroup);
      }
      break;
  } });
}
defineCustomElement();

export { ViewerDomGroup as V, defineCustomElement as d };

//# sourceMappingURL=viewer-dom-group.js.map