/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { h, proxyCustomElement, HTMLElement as HTMLElement$1, Host } from '@stencil/core/internal/client';
import { m as matrix4, v as vector3 } from './bundle.esm.js';
import { M as MultiElementInteractionHandler } from './multiElementInteractionHandler.js';
import './streamAttributes.js';
import { V as Viewport } from './viewport.js';

function isVertexViewerDomElement(el) {
  return (el instanceof HTMLElement &&
    el.nodeName === 'VERTEX-VIEWER-DOM-ELEMENT' &&
    el.matrix != null);
}

function isVertexViewerDomGroup(el) {
  return (((el instanceof HTMLElement && el.nodeName === 'VERTEX-VIEWER-DOM-GROUP') ||
    (el instanceof HTMLElement && el.dataset.isDomGroupElement != null)) &&
    el.matrix != null);
}

// eslint-disable-next-line @typescript-eslint/no-unused-vars
const Renderer2d = (_, children) => {
  return h("div", { class: "root-2d" }, children);
};
function update2d(hostEl, parentWorldMatrix, viewport, camera, depthBuffer) {
  const elements = getElementDepths(hostEl, parentWorldMatrix, camera).sort((a, b) => a.distanceToCamera - b.distanceToCamera);
  for (let i = 0; i < elements.length; i++) {
    const { element, worldMatrix, worldPosition } = elements[i];
    const depthBufferIsNull = depthBuffer == null;
    const occluded = depthBufferIsNull ||
      (!element.occlusionOff &&
        (depthBuffer === null || depthBuffer === void 0 ? void 0 : depthBuffer.isOccluded(worldPosition, viewport)));
    const detached = depthBufferIsNull ||
      (!element.detachedOff &&
        (depthBuffer === null || depthBuffer === void 0 ? void 0 : depthBuffer.isDetached(worldPosition, viewport)));
    const screenPt = getScreenPosition(worldPosition, camera.projectionViewMatrix, viewport);
    updateTransform(element, screenPt);
    updateDepth(element, i, elements.length);
    update2d(element, worldMatrix, viewport, camera, depthBuffer);
    element.occluded = occluded !== null && occluded !== void 0 ? occluded : false;
    element.detached = detached !== null && detached !== void 0 ? detached : false;
    element.classList.add('ready');
  }
}
function getElementDepths(element, parentWorldMatrix, camera) {
  const results = [];
  for (let i = 0; i < element.children.length; i++) {
    const child = element.children[i];
    if (isVertexViewerDomGroup(child)) {
      const worldMatrix = matrix4.multiply(parentWorldMatrix, child.matrix);
      results.push(...getElementDepths(child, worldMatrix, camera));
    }
    else if (isVertexViewerDomElement(child)) {
      const worldMatrix = matrix4.multiply(parentWorldMatrix, child.matrix);
      const worldPosition = vector3.fromMatrixPosition(worldMatrix);
      const distanceToCamera = vector3.distanceSquared(camera.position, worldPosition);
      results.push({
        element: child,
        worldMatrix,
        worldPosition,
        distanceToCamera,
      });
    }
    else {
      results.push(...getElementDepths(child, parentWorldMatrix, camera));
    }
  }
  return results;
}
function updateTransform(element, relativePt) {
  element.style.transform = [
    `translate(-50%, -50%)`,
    `translate(${relativePt.x}px, ${relativePt.y}px)`,
  ].join(' ');
}
function updateDepth(element, index, elementCount) {
  element.style.zIndex = `${elementCount - index}`;
}
function getScreenPosition(pt, projectionViewMatrix, viewport) {
  const ndcPt = vector3.transformMatrix(pt, projectionViewMatrix);
  return viewport.transformVectorToViewport(ndcPt);
}

// eslint-disable-next-line @typescript-eslint/no-unused-vars
const Renderer3d = ({ camera, viewport }, children) => {
  const pMatrix = matrix4.toObject(camera.projectionMatrix);
  const fovY = camera.isOrthographic()
    ? ((camera.far * 2) / (camera.fovHeight * 2)) * (viewport.height / 2)
    : pMatrix.m22 * (viewport.height / 2);
  const cameraTransform = [
    `translateZ(${fovY}px)`,
    getCameraCssMatrix(camera.viewMatrix),
    `translate(${viewport.width / 2}px, ${viewport.height / 2}px)`,
  ].join(' ');
  return (h("div", { class: "root-3d", style: { perspective: `${fovY}px` } },
    h("div", { class: "camera", style: { transform: cameraTransform } }, children)));
};
function update3d(element, parentWorldMatrix, viewport, camera, depthBuffer) {
  for (let i = 0; i < element.children.length; i++) {
    const el = element.children[i];
    if (isVertexViewerDomElement(el)) {
      updateElement(el, parentWorldMatrix, viewport, camera, depthBuffer);
    }
    else if (isVertexViewerDomGroup(el)) {
      updateGroup(el, parentWorldMatrix, viewport, camera, depthBuffer);
    }
    else {
      update3d(el, parentWorldMatrix, viewport, camera, depthBuffer);
    }
  }
}
function updateElement(element, parentWorldMatrix, viewport, camera, depthBuffer) {
  const worldMatrix = matrix4.multiply(parentWorldMatrix, element.matrix);
  const positionWorld = vector3.fromMatrixPosition(worldMatrix);
  const depthBufferIsNull = depthBuffer == null;
  const occluded = depthBufferIsNull ||
    (!element.occlusionOff && (depthBuffer === null || depthBuffer === void 0 ? void 0 : depthBuffer.isOccluded(positionWorld, viewport)));
  element.occluded = occluded !== null && occluded !== void 0 ? occluded : false;
  const detached = depthBufferIsNull ||
    (!element.detachedOff && (depthBuffer === null || depthBuffer === void 0 ? void 0 : depthBuffer.isDetached(positionWorld, viewport)));
  element.detached = detached !== null && detached !== void 0 ? detached : false;
  element.classList.add('ready');
  if (element.billboardOff) {
    element.style.transform = getElementCssMatrix(worldMatrix);
  }
  else {
    let m = camera.viewMatrix;
    m = matrix4.transpose(m);
    m = matrix4.position(m, worldMatrix);
    m = matrix4.scale(m, element.scale);
    m[3] = 0;
    m[7] = 0;
    m[11] = 0;
    m[15] = 1;
    element.style.transform = getElementCssMatrix(m);
  }
  update3d(element, worldMatrix, viewport, camera, depthBuffer);
}
function updateGroup(element, parentWorldMatrix, viewport, camera, depthBuffer) {
  const worldMatrix = matrix4.multiply(parentWorldMatrix, element.matrix);
  update3d(element, worldMatrix, viewport, camera, depthBuffer);
}
function getCameraCssMatrix(viewMatrix) {
  const elements = [
    epsilon(viewMatrix[0]),
    epsilon(-viewMatrix[1]),
    epsilon(viewMatrix[2]),
    epsilon(viewMatrix[3]),
    epsilon(viewMatrix[4]),
    epsilon(-viewMatrix[5]),
    epsilon(viewMatrix[6]),
    epsilon(viewMatrix[7]),
    epsilon(viewMatrix[8]),
    epsilon(-viewMatrix[9]),
    epsilon(viewMatrix[10]),
    epsilon(viewMatrix[11]),
    epsilon(viewMatrix[12]),
    epsilon(-viewMatrix[13]),
    epsilon(viewMatrix[14]),
    epsilon(viewMatrix[15]),
  ].join(', ');
  return `matrix3d(${elements})`;
}
function getElementCssMatrix(matrix) {
  const values = [
    epsilon(matrix[0]),
    epsilon(matrix[1]),
    epsilon(matrix[2]),
    epsilon(matrix[3]),
    epsilon(-matrix[4]),
    epsilon(-matrix[5]),
    epsilon(-matrix[6]),
    epsilon(-matrix[7]),
    epsilon(matrix[8]),
    epsilon(matrix[9]),
    epsilon(matrix[10]),
    epsilon(matrix[11]),
    epsilon(matrix[12]),
    epsilon(matrix[13]),
    epsilon(matrix[14]),
    epsilon(matrix[15]),
  ].join(', ');
  return [`translate(-50%, -50%)`, `matrix3d(${values})`].join(' ');
}
function epsilon(value) {
  return Math.abs(value) < 1e-10 ? 0 : value;
}

const viewerDomRendererCss = ".sc-vertex-viewer-dom-renderer-h{position:absolute;top:0;bottom:0;left:0;right:0;overflow:hidden;pointer-events:none}.root-3d.sc-vertex-viewer-dom-renderer{width:100%;height:100%;pointer-events:inherit}.camera.sc-vertex-viewer-dom-renderer{transform-style:preserve-3d;pointer-events:inherit;width:100%;height:100%}";

const ViewerDomRenderer = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    /**
     * Specifies the drawing mode for the renderer.
     *
     * When in `3d` mode, elements are positioned using CSS 3D transforms and will
     * scale and rotate with the camera. In `2d` mode, a simpler 2D transform is
     * used, and elements will not scale or rotate with camera changes.
     */
    this.drawMode = '3d';
    /**
     * Specifies whether to propagate events to the viewer's interaction handlers
     *
     * When `true` this <vertex-viewer-dom-renderer> will be registered as a valid event target
     * for the viewer. This enables camera interactions to be initiated from elements within this renderer.
     *
     * When `false` this <vertex-viewer-dom-renderer> will *not* be registered as a target
     * for the viewer.
     */
    this.propagateEventsToViewer = true;
    this.viewport = new Viewport(0, 0);
    this.invalidateFrameCounter = 0;
    this.interactionDisposables = [];
    this.handleViewerFrameDrawn = async () => {
      this.updatePropsFromViewer();
    };
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    var _a;
    const resized = new ResizeObserver(() => this.handleResize());
    resized.observe(this.hostEl);
    const mutation = new MutationObserver(() => this.handleChildrenChange());
    mutation.observe(this.hostEl, { childList: true });
    this.handleViewerChange(this.viewer, undefined);
    if (((_a = this.viewer) === null || _a === void 0 ? void 0 : _a.frame) != null) {
      this.handleViewerFrameDrawn();
    }
  }
  /**
   * @ignore
   */
  render() {
    if (this.camera != null) {
      if (this.drawMode === '2d') {
        return (h(Host, null, h(Renderer2d, null, h("slot", null))));
      }
      else {
        return (h(Host, null, h(Renderer3d, { camera: this.camera, viewport: this.viewport }, h("slot", null))));
      }
    }
    else {
      return h(Host, null);
    }
  }
  /**
   * @ignore
   */
  componentWillRender() {
    this.updateElements();
  }
  /**
   * @ignore
   */
  handleViewerChange(newViewer, oldViewer) {
    oldViewer === null || oldViewer === void 0 ? void 0 : oldViewer.removeEventListener('frameDrawn', this.handleViewerFrameDrawn);
    newViewer === null || newViewer === void 0 ? void 0 : newViewer.addEventListener('frameDrawn', this.handleViewerFrameDrawn);
    if (this.propagateEventsToViewer && newViewer != null) {
      this.handleEventPropagationToViewer(newViewer);
    }
  }
  /**
   * @ignore
   */
  handlePropertyChange() {
    this.invalidateFrame();
  }
  invalidateFrame() {
    this.invalidateFrameCounter = this.invalidateFrameCounter + 1;
  }
  /**
   * disposes any existing disposables, and registers new handlers on the newly provided viewer.
   * @param newViewer
   */
  handleEventPropagationToViewer(newViewer) {
    this.interactionDisposables.forEach((disposable) => {
      disposable.dispose();
    });
    this.interactionDisposables = [];
    newViewer.getInteractionHandlers().then((handlers) => {
      handlers.forEach((handler) => {
        if (handler instanceof MultiElementInteractionHandler) {
          const disposable = handler.registerAdditionalElement(this.hostEl);
          this.interactionDisposables = [
            ...this.interactionDisposables,
            disposable,
          ];
        }
      });
    });
  }
  async updateElements() {
    const { viewport, camera } = this;
    const worldMatrix = matrix4.makeIdentity();
    if (camera != null) {
      if (this.drawMode === '3d') {
        update3d(this.hostEl, worldMatrix, viewport, camera, this.depthBuffer);
      }
      else {
        update2d(this.hostEl, worldMatrix, viewport, camera, this.depthBuffer);
      }
    }
  }
  handleResize() {
    const bounds = this.hostEl.getBoundingClientRect();
    this.viewport = new Viewport(bounds.width, bounds.height);
  }
  handleChildrenChange() {
    this.invalidateFrame();
  }
  async updatePropsFromViewer() {
    const { frame } = this.viewer || {};
    this.depthBuffer = await (frame === null || frame === void 0 ? void 0 : frame.depthBuffer());
    this.camera = frame === null || frame === void 0 ? void 0 : frame.scene.camera;
  }
  get hostEl() { return this; }
  static get watchers() { return {
    "viewer": ["handleViewerChange"]
  }; }
  static get style() { return viewerDomRendererCss; }
}, [6, "vertex-viewer-dom-renderer", {
    "drawMode": [1, "draw-mode"],
    "propagateEventsToViewer": [4, "propagate-events-to-viewer"],
    "viewer": [16],
    "camera": [1040],
    "depthBuffer": [1040],
    "viewport": [32],
    "invalidateFrameCounter": [32],
    "interactionDisposables": [32]
  }, [[0, "propertyChange", "handlePropertyChange"]]]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-viewer-dom-renderer"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-viewer-dom-renderer":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, ViewerDomRenderer);
      }
      break;
  } });
}
defineCustomElement();

export { ViewerDomRenderer as V, defineCustomElement as d };

//# sourceMappingURL=viewer-dom-renderer.js.map