/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { h, proxyCustomElement, HTMLElement, createEvent, Host } from '@stencil/core/internal/client';
import { g as getMarkupBoundingClientRect } from './dom2.js';
import { M as MarkupInteractionHandler, t as translatePointToRelative, c as translatePointToScreen, R as RelativeAnchor, d as isValidStartEvent, p as parsePoint, e as arrowheadPointsToPolygonPoints, f as arrowheadPointsToPathPoints, g as isValidPointData, h as createLineAnchorStylePoints, S as SvgShadow } from './interactions.js';
import { p as point, c as angle } from './bundle.esm.js';
import { g as getMouseClientPosition } from './dom.js';
import { c as classnames } from './index2.js';

class ArrowMarkupInteractionHandler extends MarkupInteractionHandler {
  constructor(markupEl, interactionBegin, interactionEnd) {
    super();
    this.markupEl = markupEl;
    this.interactionBegin = interactionBegin;
    this.interactionEnd = interactionEnd;
    this.anchor = 'end';
  }
  editAnchor(anchor, event) {
    if (this.markupEl.mode === 'edit') {
      this.anchor = anchor;
      this.startInteraction(event);
    }
  }
  startInteraction(event) {
    this.handleInteractionAttempt(event);
  }
  computeBoundingRect() {
    return getMarkupBoundingClientRect(this.markupEl);
  }
  handleInteractionAttempt(event) {
    var _a;
    if (this.markupEl.mode !== '' &&
      this.pointerId == null &&
      this.elementBounds != null) {
      this.pointerId = event.pointerId;
      this.markupEl.start =
        (_a = this.markupEl.start) !== null && _a !== void 0 ? _a : translatePointToRelative(getMouseClientPosition(event, this.elementBounds), this.elementBounds);
      this.interactionBegin.emit();
      this.acceptInteraction();
    }
  }
  handleInteractionMove(event) {
    if (this.elementBounds != null && this.pointerId === event.pointerId) {
      const position = translatePointToRelative(getMouseClientPosition(event, this.elementBounds), this.elementBounds);
      if (this.anchor === 'start') {
        this.markupEl.start = position;
      }
      else if (this.anchor === 'end') {
        this.markupEl.end = position;
      }
      else if (this.markupEl.start != null && this.markupEl.end != null) {
        const center = point.create((this.markupEl.start.x + this.markupEl.end.x) / 2, (this.markupEl.start.y + this.markupEl.end.y) / 2);
        const xDifference = center.x - position.x;
        const yDifference = center.y - position.y;
        this.markupEl.start = point.create(this.markupEl.start.x - xDifference, this.markupEl.start.y - yDifference);
        this.markupEl.end = point.create(this.markupEl.end.x - xDifference, this.markupEl.end.y - yDifference);
      }
    }
  }
  handleInteractionEnd(event) {
    if (this.pointerId === event.pointerId) {
      const screenStart = this.markupEl.start != null && this.elementBounds != null
        ? translatePointToScreen(this.markupEl.start, this.elementBounds)
        : undefined;
      const screenEnd = this.markupEl.end != null && this.elementBounds != null
        ? translatePointToScreen(this.markupEl.end, this.elementBounds)
        : undefined;
      if (this.markupEl.mode !== '' &&
        screenStart != null &&
        screenEnd != null &&
        point.distance(screenStart, screenEnd) >= 2) {
        const newlyCreatedMarkup = this.markupEl.mode !== 'edit';
        this.interactionEnd.emit({ markup: this.markupEl, newlyCreatedMarkup });
      }
      else {
        this.markupEl.start = undefined;
        this.markupEl.end = undefined;
      }
      this.pointerId = undefined;
    }
  }
}

// eslint-disable-next-line @typescript-eslint/no-unused-vars
const BoundingBox1d = ({ start, end, onStartAnchorPointerDown, onCenterAnchorPointerDown, onEndAnchorPointerDown, }) => {
  const angle$1 = angle.normalize(angle.toDegrees(angle.fromPoints(start, end)) - 270);
  const center = point.create((start.x + end.x) / 2, (start.y + end.y) / 2);
  return (h("div", { class: "bounds-container" },
    h(RelativeAnchor, { id: "bounding-box-1d-start-anchor", name: "start-anchor", rotation: angle$1, point: start, onPointerDown: onStartAnchorPointerDown },
      h("div", { class: classnames('bounds-default-anchor', 'bounds-cap-anchor') })),
    h(RelativeAnchor, { id: "bounding-box-1d-end-anchor", name: "end-anchor", rotation: angle$1, point: end, onPointerDown: onEndAnchorPointerDown },
      h("div", { class: classnames('bounds-default-anchor', 'bounds-cap-anchor') })),
    h(RelativeAnchor, { id: "bounding-box-1d-center-anchor", name: "center-anchor", point: center, onPointerDown: onCenterAnchorPointerDown },
      h("div", { class: classnames('bounds-default-anchor', 'bounds-center-anchor') }))));
};

const viewerMarkupArrowCss = ":host{position:absolute;top:0;left:0;width:100%;height:100%;pointer-events:none;--viewer-markup-arrow-line-stroke-color:var(--red-800);--viewer-markup-arrow-line-stroke-width:4;--viewer-markup-arrow-head-stroke-color:var(--red-800);--viewer-markup-arrow-head-stroke-width:4;--viewer-markup-arrow-head-fill-color:var(--red-800);--viewer-markup-arrow-head-fill-opacity:1;--viewer-markup-arrow-bounds-cap-border-color:var(--blue-400);--viewer-markup-arrow-bounds-cap-border-width:1px;--viewer-markup-arrow-bounds-cap-background-color:white;--viewer-markup-arrow-bounds-center-anchor-border-color:white;--viewer-markup-arrow-bounds-center-anchor-border-width:1px;--viewer-markup-arrow-bounds-center-anchor-background-color:var(--blue-400);--viewer-markup-arrow-bounds-outline-width:1px;--viewer-markup-arrow-bounds-outline-color:var(--blue-400);--viewer-markup-arrow-bounds-anchor-width:9px;--viewer-markup-arrow-bounds-anchor-height:9px}.svg{pointer-events:none;width:100%;height:100%}.create-overlay{pointer-events:auto;position:absolute;left:0;top:0;width:100%;height:100%;cursor:crosshair}.line{pointer-events:auto;stroke:var(--viewer-markup-arrow-line-stroke-color);stroke-width:var(--viewer-markup-arrow-line-stroke-width);cursor:default}.head{pointer-events:auto;stroke:var(--viewer-markup-arrow-head-stroke-color);stroke-width:var(--viewer-markup-arrow-head-stroke-width);fill:var(--viewer-markup-arrow-head-fill-color);fill-opacity:var(--viewer-markup-arrow-head-fill-opacity);cursor:default}.bounds-line{pointer-events:auto;stroke:var(--viewer-markup-arrow-bounds-outline-color);stroke-width:var(--viewer-markup-arrow-bounds-outline-width)}.bounds-container{pointer-events:none;position:absolute;width:100%;height:100%;top:0;left:0;overflow:hidden}.bounds-anchor-position{position:absolute;transform:translate(-50%, -50%)}.bounds-anchor{pointer-events:auto;cursor:default;user-select:none}.bounds-default-anchor{width:var(--viewer-markup-arrow-bounds-anchor-width);height:var(--viewer-markup-arrow-bounds-anchor-height);box-sizing:border-box;box-shadow:0 0 2px rgba(0, 0, 0, 0.3)}.bounds-center-anchor{border-radius:100%;border-width:var(--viewer-markup-arrow-bounds-center-anchor-border-width);border-color:var(--viewer-markup-arrow-bounds-center-anchor-border-color);border-style:solid;background-color:var(--viewer-markup-arrow-bounds-center-anchor-background-color)}.bounds-cap-anchor{border-width:var(--viewer-markup-arrow-bounds-cap-border-width);border-color:var(--viewer-markup-arrow-bounds-cap-border-color);border-style:solid;background-color:var(--viewer-markup-arrow-bounds-cap-background-color)}";

const ViewerMarkupArrow = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.interactionBegin = createEvent(this, "interactionBegin", 7);
    this.interactionEnd = createEvent(this, "interactionEnd", 7);
    this.viewRendered = createEvent(this, "viewRendered", 7);
    /**
     * The style of the starting anchor. This defaults to none.
     */
    this.startLineAnchorStyle = 'none';
    /**
     * The style of the ending anchor. This defaults to 'arrow-triangle.'
     */
    this.endLineAnchorStyle = 'arrow-triangle';
    /**
     * A mode that specifies how the markup component should behave. When
     * unset, the component will not respond to interactions with the handles.
     * When `edit`, the markup anchors are interactive and the user is able
     * to reposition them. When `create`, anytime the user clicks on the canvas,
     * a new markup will be performed.
     */
    this.mode = '';
    this.interactionHandler = new ArrowMarkupInteractionHandler(this.hostEl, this.interactionBegin, this.interactionEnd);
    this.handleWindowPointerDown = (event) => {
      if (isValidStartEvent(event)) {
        this.interactionHandler.startInteraction(event);
      }
    };
    this.handleTouchStart = (event) => {
      event.preventDefault();
    };
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.updateViewport();
    this.handleViewerChanged(this.viewer);
    this.updatePointsFromProps();
  }
  componentDidLoad() {
    this.updatePointsFromProps();
    const resize = new ResizeObserver(() => this.updateViewport());
    resize.observe(this.hostEl);
    if (this.mode === 'create') {
      window.addEventListener('pointerdown', this.handleWindowPointerDown);
    }
  }
  componentDidRender() {
    if (this.mode === '') {
      this.viewRendered.emit();
    }
  }
  disconnectedCallback() {
    this.dispose();
  }
  async dispose() {
    var _a;
    (_a = this.registeredInteraction) === null || _a === void 0 ? void 0 : _a.dispose();
    this.registeredInteraction = undefined;
    window.removeEventListener('pointerdown', this.handleWindowPointerDown);
  }
  /**
   * @ignore
   */
  async handleViewerChanged(newViewer) {
    var _a;
    (_a = this.registeredInteraction) === null || _a === void 0 ? void 0 : _a.dispose();
    this.registeredInteraction = undefined;
    if (newViewer != null) {
      this.registeredInteraction = await newViewer.registerInteractionHandler(this.interactionHandler);
    }
  }
  handleStartJsonChange() {
    this.updatePointsFromProps();
  }
  handleEndJsonChange() {
    this.updatePointsFromProps();
  }
  handleModeChange() {
    if (this.mode !== 'create') {
      window.removeEventListener('pointerdown', this.handleWindowPointerDown);
    }
  }
  updateViewport() {
    const rect = getMarkupBoundingClientRect(this.hostEl);
    this.elementBounds = rect;
  }
  updatePointsFromProps() {
    this.start = this.start || parsePoint(this.startJson);
    this.end = this.end || parsePoint(this.endJson);
  }
  renderLineAnchorStyle(anchorStyle, arrowheadPoints) {
    if (anchorStyle === 'arrow-triangle') {
      return (h("polygon", { id: "line-anchor-arrow-triangle", class: "head", points: arrowheadPointsToPolygonPoints(arrowheadPoints) }));
    }
    else if (anchorStyle === 'arrow-line') {
      return (h("path", { id: "line-anchor-arrow-line", class: "head", d: arrowheadPointsToPathPoints(arrowheadPoints) }));
    }
    else if (anchorStyle === 'hash') {
      return (h("line", { id: "line-anchor-hash", class: "head", x1: arrowheadPoints.hash.leftPoint.x, y1: arrowheadPoints.hash.leftPoint.y, x2: arrowheadPoints.hash.rightPoint.x, y2: arrowheadPoints.hash.rightPoint.y }));
    }
    else if (anchorStyle === 'dot') {
      return (h("circle", { id: "line-anchor-circle", class: "head", cx: arrowheadPoints.tip.x, cy: arrowheadPoints.tip.y, r: arrowheadPoints.radius }));
    }
    else {
      return h("div", null);
    }
  }
  render() {
    if (this.start != null && this.end != null && this.elementBounds != null) {
      const screenStart = translatePointToScreen(this.start, this.elementBounds);
      const screenEnd = translatePointToScreen(this.end, this.elementBounds);
      if (isValidPointData(screenStart, screenEnd)) {
        const arrowheadStartPoints = createLineAnchorStylePoints(screenEnd, screenStart);
        const arrowheadEndPoints = createLineAnchorStylePoints(screenStart, screenEnd);
        return (h(Host, null, h("svg", { class: "svg", onTouchStart: this.handleTouchStart }, h("defs", null, h(SvgShadow, { id: "arrow-shadow" })), h("g", { filter: "url(#arrow-shadow)" }, this.renderLineAnchorStyle(this.startLineAnchorStyle, arrowheadStartPoints), h("line", { id: "arrow-line", class: "line", x1: arrowheadEndPoints.tip.x, y1: arrowheadEndPoints.tip.y, x2: arrowheadStartPoints.tip.x, y2: arrowheadStartPoints.tip.y }), this.renderLineAnchorStyle(this.endLineAnchorStyle, arrowheadEndPoints)), this.mode === 'edit' && (h("g", null, h("line", { id: "bounding-box-1d-line", class: "bounds-line", x1: screenStart.x, y1: screenStart.y, x2: screenEnd.x, y2: screenEnd.y })))), this.mode === 'edit' && (h(BoundingBox1d, { start: screenStart, end: screenEnd, onStartAnchorPointerDown: (event) => this.interactionHandler.editAnchor('start', event), onCenterAnchorPointerDown: (event) => this.interactionHandler.editAnchor('center', event), onEndAnchorPointerDown: (event) => this.interactionHandler.editAnchor('end', event) })), this.mode === 'create' && (h("div", { class: "create-overlay", onTouchStart: this.handleTouchStart }))));
      }
      else {
        return h(Host, null);
      }
    }
    else {
      return (h(Host, null, h("div", { class: "create-overlay", onTouchStart: this.handleTouchStart })));
    }
  }
  get hostEl() { return this; }
  static get watchers() { return {
    "viewer": ["handleViewerChanged"],
    "start": ["handleStartJsonChange"],
    "end": ["handleEndJsonChange"],
    "mode": ["handleModeChange"]
  }; }
  static get style() { return viewerMarkupArrowCss; }
}, [1, "vertex-viewer-markup-arrow", {
    "start": [1040],
    "startJson": [1, "start"],
    "end": [1040],
    "endJson": [1, "end"],
    "startLineAnchorStyle": [1025, "start-line-anchor-style"],
    "endLineAnchorStyle": [1025, "end-line-anchor-style"],
    "mode": [513],
    "viewer": [16],
    "elementBounds": [32],
    "dispose": [64]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-viewer-markup-arrow"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-viewer-markup-arrow":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, ViewerMarkupArrow);
      }
      break;
  } });
}
defineCustomElement();

export { ViewerMarkupArrow as V, defineCustomElement as d };

//# sourceMappingURL=viewer-markup-arrow.js.map