/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { e as rectangle } from './bundle.esm.js';
import { g as getMarkupBoundingClientRect } from './dom2.js';
import { M as MarkupInteractionHandler, t as translatePointToRelative, j as transformRectangle, d as isValidStartEvent, k as parseBounds, l as translateRectToScreen, S as SvgShadow } from './interactions.js';
import { g as getMouseClientPosition } from './dom.js';
import { B as BoundingBox2d } from './viewer-markup-circle-components.js';

class CircleMarkupInteractionHandler extends MarkupInteractionHandler {
  constructor(markupEl, interactionBegin, interactionEnd) {
    super();
    this.markupEl = markupEl;
    this.interactionBegin = interactionBegin;
    this.interactionEnd = interactionEnd;
    this.anchor = 'bottom-right';
  }
  editAnchor(anchor, event) {
    if (this.markupEl.mode === 'edit') {
      this.anchor = anchor;
      this.resizeBounds = this.markupEl.bounds;
      this.startInteraction(event);
    }
  }
  startInteraction(event) {
    this.handleInteractionAttempt(event);
  }
  computeBoundingRect() {
    return getMarkupBoundingClientRect(this.markupEl);
  }
  handleInteractionAttempt(event) {
    var _a;
    if (this.markupEl.mode !== '' &&
      this.pointerId == null &&
      this.elementBounds != null) {
      const position = translatePointToRelative(getMouseClientPosition(event, this.elementBounds), this.elementBounds);
      this.pointerId = event.pointerId;
      this.startPosition = position;
      this.markupEl.bounds =
        (_a = this.markupEl.bounds) !== null && _a !== void 0 ? _a : rectangle.create(position.x, position.y, 0, 0);
      this.resizeBounds = this.markupEl.bounds;
      this.interactionBegin.emit();
      this.acceptInteraction();
    }
  }
  handleInteractionMove(event) {
    var _a;
    if (this.markupEl.bounds != null &&
      this.startPosition != null &&
      this.elementBounds != null &&
      this.pointerId === event.pointerId) {
      const position = translatePointToRelative(getMouseClientPosition(event, this.elementBounds), this.elementBounds);
      this.markupEl.bounds = transformRectangle((_a = this.resizeBounds) !== null && _a !== void 0 ? _a : this.markupEl.bounds, this.startPosition, position, this.anchor, event.shiftKey);
    }
  }
  handleInteractionEnd(event) {
    var _a, _b;
    if (this.pointerId === event.pointerId) {
      if (this.markupEl.mode !== '' &&
        this.markupEl.bounds != null &&
        ((_a = this.markupEl.bounds) === null || _a === void 0 ? void 0 : _a.width) > 0 &&
        ((_b = this.markupEl.bounds) === null || _b === void 0 ? void 0 : _b.height) > 0) {
        this.anchor = 'bottom-right';
        const newlyCreatedMarkup = this.markupEl.mode !== 'edit';
        this.interactionEnd.emit({ markup: this.markupEl, newlyCreatedMarkup });
      }
      else {
        this.markupEl.bounds = undefined;
      }
      this.pointerId = undefined;
    }
  }
}

const viewerMarkupCircleCss = ":host{position:absolute;top:0;left:0;width:100%;height:100%;pointer-events:none;--viewer-markup-circle-ellipse-stroke-color:var(--red-700);--viewer-markup-circle-ellipse-stroke-width:4;--viewer-markup-circle-ellipse-fill-color:none;--viewer-markup-circle-ellipse-fill-opacity:0;--viewer-markup-circle-bounds-outline-border-color:var(--blue-400);--viewer-markup-circle-bounds-outline-border-width:1px;--viewer-markup-circle-bounds-edge-anchor-border-color:var(--blue-400);--viewer-markup-circle-bounds-edge-anchor-border-width:1px;--viewer-markup-circle-bounds-edge-anchor-background-color:white;--viewer-markup-circle-bounds-center-anchor-border-color:white;--viewer-markup-circle-bounds-center-anchor-border-width:1px;--viewer-markup-circle-bounds-center-anchor-background-color:var(--blue-400);--viewer-markup-circle-bounds-anchor-width:9px;--viewer-markup-circle-bounds-anchor-height:9px}.svg{pointer-events:none;width:100%;height:100%}.create-overlay{pointer-events:auto;position:absolute;left:0;top:0;width:100%;height:100%;cursor:crosshair}.ellipse{pointer-events:auto;stroke:var(--viewer-markup-circle-ellipse-stroke-color);stroke-width:var(--viewer-markup-circle-ellipse-stroke-width);fill:var(--viewer-markup-circle-ellipse-fill-color);fill-opacity:var(--viewer-markup-circle-ellipse-fill-opacity);cursor:default}.bounds-container{pointer-events:none;position:absolute;width:100%;height:100%;top:0;left:0;overflow:hidden}.bounds-outline{position:absolute;border-width:var(--viewer-markup-circle-bounds-outline-border-width);border-color:var(--viewer-markup-circle-bounds-outline-border-color);border-style:solid;box-shadow:0 0 3px rgba(0, 0, 0, 0.3)}.bounds-anchor-position{position:absolute;transform:translate(-50%, -50%)}.bounds-anchor{pointer-events:auto;cursor:default;user-select:none}.bounds-default-anchor{width:var(--viewer-markup-circle-bounds-anchor-width);height:var(--viewer-markup-circle-bounds-anchor-height);box-sizing:border-box;box-shadow:0 0 2px rgba(0, 0, 0, 0.3)}.bounds-center-anchor{border-radius:100%;border-width:var(--viewer-markup-circle-bounds-center-anchor-border-width);border-color:var(--viewer-markup-circle-bounds-center-anchor-border-color);border-style:solid;background-color:var(--viewer-markup-circle-bounds-center-anchor-background-color)}.bounds-edge-anchor{border-width:var(--viewer-markup-circle-bounds-edge-anchor-border-width);border-color:var(--viewer-markup-circle-bounds-edge-anchor-border-color);border-style:solid;background-color:var(--viewer-markup-circle-bounds-edge-anchor-background-color)}";

const ViewerMarkupCircle = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.interactionBegin = createEvent(this, "interactionBegin", 7);
    this.interactionEnd = createEvent(this, "interactionEnd", 7);
    this.viewRendered = createEvent(this, "viewRendered", 7);
    /**
     * A mode that specifies how the markup component should behave. When
     * unset, the component will not respond to interactions with the handles.
     * When `edit`, the markup anchors are interactive and the user is able
     * to reposition them. When `create`, anytime the user clicks on the canvas,
     * a new markup will be performed.
     */
    this.mode = '';
    this.interactionHandler = new CircleMarkupInteractionHandler(this.hostEl, this.interactionBegin, this.interactionEnd);
    this.handleWindowPointerDown = (event) => {
      if (isValidStartEvent(event)) {
        this.interactionHandler.startInteraction(event);
      }
    };
    this.handleTouchStart = (event) => {
      event.preventDefault();
    };
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.updateViewport();
    this.handleViewerChanged(this.viewer);
    this.updateBoundsFromProps();
  }
  componentDidLoad() {
    this.updateBoundsFromProps();
    const resize = new ResizeObserver(() => this.updateViewport());
    resize.observe(this.hostEl);
    if (this.mode === 'create') {
      window.addEventListener('pointerdown', this.handleWindowPointerDown);
    }
  }
  componentDidRender() {
    if (this.mode === '') {
      this.viewRendered.emit();
    }
  }
  disconnectedCallback() {
    this.dispose();
  }
  async dispose() {
    var _a;
    (_a = this.registeredHandler) === null || _a === void 0 ? void 0 : _a.dispose();
    this.registeredHandler = undefined;
    window.removeEventListener('pointerdown', this.handleWindowPointerDown);
  }
  /**
   * @ignore
   */
  async handleViewerChanged(newViewer) {
    var _a;
    (_a = this.registeredHandler) === null || _a === void 0 ? void 0 : _a.dispose();
    this.registeredHandler = undefined;
    if (newViewer != null) {
      this.registeredHandler = await newViewer.registerInteractionHandler(this.interactionHandler);
    }
  }
  handleBoundsJsonChange() {
    this.updateBoundsFromProps();
  }
  handleModeChange() {
    if (this.mode !== 'create') {
      window.removeEventListener('pointerdown', this.handleWindowPointerDown);
    }
  }
  updateViewport() {
    const rect = getMarkupBoundingClientRect(this.hostEl);
    this.elementBounds = rect;
  }
  updateBoundsFromProps() {
    var _a;
    this.bounds = (_a = this.bounds) !== null && _a !== void 0 ? _a : parseBounds(this.boundsJson);
  }
  render() {
    if (this.bounds != null && this.elementBounds != null) {
      const relativeBounds = translateRectToScreen(this.bounds, this.elementBounds);
      const center = rectangle.center(relativeBounds);
      return (h(Host, null, h("svg", { class: "svg", onTouchStart: this.handleTouchStart }, h("defs", null, h(SvgShadow, { id: "circle-shadow" })), h("g", { filter: "url(#circle-shadow)" }, h("ellipse", { class: "ellipse", cx: center.x, cy: center.y, rx: relativeBounds.width / 2, ry: relativeBounds.height / 2, stroke: '#000ff0', "stroke-width": 4, fill: 'none' }))), this.mode === 'edit' && (h(BoundingBox2d, { bounds: relativeBounds, onTopLeftAnchorPointerDown: (e) => this.interactionHandler.editAnchor('top-left', e), onTopRightAnchorPointerDown: (e) => this.interactionHandler.editAnchor('top-right', e), onTopAnchorPointerDown: (e) => this.interactionHandler.editAnchor('top', e), onBottomLeftAnchorPointerDown: (e) => this.interactionHandler.editAnchor('bottom-left', e), onBottomRightAnchorPointerDown: (e) => this.interactionHandler.editAnchor('bottom-right', e), onBottomAnchorPointerDown: (e) => this.interactionHandler.editAnchor('bottom', e), onLeftAnchorPointerDown: (e) => this.interactionHandler.editAnchor('left', e), onRightAnchorPointerDown: (e) => this.interactionHandler.editAnchor('right', e), onCenterAnchorPointerDown: (e) => this.interactionHandler.editAnchor('center', e) })), this.mode === 'create' && (h("div", { class: "create-overlay", onTouchStart: this.handleTouchStart }))));
    }
    else {
      return (h(Host, null, h("div", { class: "create-overlay", onTouchStart: this.handleTouchStart })));
    }
  }
  get hostEl() { return this; }
  static get watchers() { return {
    "viewer": ["handleViewerChanged"],
    "bounds": ["handleBoundsJsonChange"],
    "mode": ["handleModeChange"]
  }; }
  static get style() { return viewerMarkupCircleCss; }
}, [1, "vertex-viewer-markup-circle", {
    "bounds": [1040],
    "boundsJson": [1, "bounds"],
    "mode": [513],
    "viewer": [16],
    "elementBounds": [32],
    "dispose": [64]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-viewer-markup-circle"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-viewer-markup-circle":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, ViewerMarkupCircle);
      }
      break;
  } });
}
defineCustomElement();

export { ViewerMarkupCircle as V, defineCustomElement as d };

//# sourceMappingURL=viewer-markup-circle.js.map