/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { g as getMarkupBoundingClientRect } from './dom2.js';
import { M as MarkupInteractionHandler, t as translatePointToRelative, j as transformRectangle, m as translatePointsToBounds, d as isValidStartEvent, n as parsePoints, k as parseBounds, S as SvgShadow, l as translateRectToScreen, c as translatePointToScreen } from './interactions.js';
import { B as BoundingBox2d } from './viewer-markup-circle-components.js';
import { p as point, e as rectangle } from './bundle.esm.js';
import { g as getMouseClientPosition } from './dom.js';

class FreeformMarkupInteractionHandler extends MarkupInteractionHandler {
  constructor(markupEl, interactionBegin, interactionEnd) {
    super();
    this.markupEl = markupEl;
    this.interactionBegin = interactionBegin;
    this.interactionEnd = interactionEnd;
    this.anchor = 'bottom-right';
    this.handleResizeInteractionMove = (event) => {
      if (this.resizeStartPosition != null &&
        this.elementBounds != null &&
        this.resizeBounds != null &&
        this.resizePoints != null) {
        const position = translatePointToRelative(getMouseClientPosition(event, this.elementBounds), this.elementBounds);
        const updatedBounds = transformRectangle(this.resizeBounds, this.resizeStartPosition, position, this.anchor, event.shiftKey);
        this.markupEl.points = translatePointsToBounds(this.resizePoints, this.resizeBounds, updatedBounds);
        this.markupEl.bounds = updatedBounds;
      }
    };
    this.handleResizeInteractionEnd = (event) => {
      window.removeEventListener('pointermove', this.handleResizeInteractionMove);
      window.removeEventListener('pointerup', this.handleResizeInteractionEnd);
      this.resizeBounds = undefined;
      const newlyCreatedMarkup = this.markupEl.mode !== 'edit';
      this.interactionEnd.emit({ markup: this.markupEl, newlyCreatedMarkup });
    };
  }
  editAnchor(anchor, event) {
    if (this.markupEl.mode === 'edit' && this.elementBounds != null) {
      this.resizeBounds = this.markupEl.bounds;
      this.resizePoints = this.markupEl.points;
      this.anchor = anchor;
      this.resizeStartPosition = translatePointToRelative(getMouseClientPosition(event, this.elementBounds), this.elementBounds);
      window.addEventListener('pointermove', this.handleResizeInteractionMove);
      window.addEventListener('pointerup', this.handleResizeInteractionEnd);
    }
  }
  startInteraction(event) {
    this.handleInteractionAttempt(event);
  }
  computeBoundingRect() {
    return getMarkupBoundingClientRect(this.markupEl);
  }
  handleInteractionAttempt(event) {
    var _a;
    if (this.markupEl.mode !== '' &&
      this.pointerId == null &&
      this.elementBounds != null) {
      this.pointerId = event.pointerId;
      const screenPosition = getMouseClientPosition(event, this.elementBounds);
      const position = translatePointToRelative(screenPosition, this.elementBounds);
      this.updateMinAndMax(position);
      this.markupEl.points = (_a = this.markupEl.points) !== null && _a !== void 0 ? _a : [position];
      this.interactionBegin.emit();
      this.acceptInteraction();
    }
  }
  handleInteractionMove(event) {
    if (this.pointerId === event.pointerId &&
      this.markupEl.points != null &&
      this.elementBounds != null) {
      const screenPosition = getMouseClientPosition(event, this.elementBounds);
      const position = translatePointToRelative(screenPosition, this.elementBounds);
      this.updateMinAndMax(position);
      this.markupEl.points = [...this.markupEl.points, position];
    }
  }
  handleInteractionEnd(event) {
    if (this.pointerId === event.pointerId) {
      if (this.markupEl.mode !== '' &&
        this.markupEl.points != null &&
        this.markupEl.points.length > 2 &&
        this.elementBounds != null) {
        const screenPosition = getMouseClientPosition(event, this.elementBounds);
        const position = translatePointToRelative(screenPosition, this.elementBounds);
        this.updateMinAndMax(position);
        this.markupEl.points = [...this.markupEl.points, position];
        const newlyCreatedMarkup = this.markupEl.mode !== 'edit';
        this.interactionEnd.emit({ markup: this.markupEl, newlyCreatedMarkup });
      }
      else {
        this.markupEl.points = undefined;
      }
      this.min = undefined;
      this.max = undefined;
      this.pointerId = undefined;
    }
  }
  updateMinAndMax(position) {
    this.min =
      this.min != null
        ? point.create(Math.min(this.min.x, position.x), Math.min(this.min.y, position.y))
        : position;
    this.max =
      this.max != null
        ? point.create(Math.max(this.max.x, position.x), Math.max(this.max.y, position.y))
        : position;
    this.markupEl.bounds = rectangle.create(this.min.x, this.min.y, this.max.x - this.min.x, this.max.y - this.min.y);
  }
}

const viewerMarkupFreeformCss = ":host{position:absolute;top:0;left:0;width:100%;height:100%;pointer-events:none;--viewer-markup-freeform-stroke-color:var(--red-800);--viewer-markup-freeform-stroke-width:4;--viewer-markup-freeform-line-join:round;--viewer-markup-freeform-line-cap:round;--viewer-markup-freeform-bounds-outline-border-color:var(--blue-400);--viewer-markup-freeform-bounds-outline-border-width:1px;--viewer-markup-freeform-bounds-edge-anchor-border-color:var(--blue-400);--viewer-markup-freeform-bounds-edge-anchor-border-width:1px;--viewer-markup-freeform-bounds-edge-anchor-background-color:white;--viewer-markup-freeform-bounds-center-anchor-border-color:white;--viewer-markup-freeform-bounds-center-anchor-border-width:1px;--viewer-markup-freeform-bounds-center-anchor-background-color:var(--blue-400);--viewer-markup-freeform-bounds-anchor-width:9px;--viewer-markup-freeform-bounds-anchor-height:9px}.svg{pointer-events:none;width:100%;height:100%;position:absolute;top:0;left:0}.path{pointer-events:auto;cursor:default;stroke:var(--viewer-markup-freeform-stroke-color);stroke-width:var(--viewer-markup-freeform-stroke-width);stroke-linejoin:var(--viewer-markup-freeform-line-join);stroke-linecap:var(--viewer-markup-freeform-line-cap)}.create-overlay{pointer-events:auto;position:absolute;left:0;top:0;width:100%;height:100%;cursor:crosshair}.bounds-container{pointer-events:none;position:absolute;width:100%;height:100%;top:0;left:0;overflow:hidden}.bounds-outline{position:absolute;border-width:var(--viewer-markup-freeform-bounds-outline-border-width);border-color:var(--viewer-markup-freeform-bounds-outline-border-color);border-style:solid;box-shadow:0 0 3px rgba(0, 0, 0, 0.3)}.bounds-anchor-position{position:absolute;transform:translate(-50%, -50%)}.bounds-anchor{pointer-events:auto;cursor:default;user-select:none}.bounds-default-anchor{width:var(--viewer-markup-freeform-bounds-anchor-width);height:var(--viewer-markup-freeform-bounds-anchor-height);box-sizing:border-box;box-shadow:0 0 2px rgba(0, 0, 0, 0.3)}.bounds-center-anchor{border-radius:100%;border-width:var(--viewer-markup-freeform-bounds-center-anchor-border-width);border-color:var(--viewer-markup-freeform-bounds-center-anchor-border-color);border-style:solid;background-color:var(--viewer-markup-freeform-bounds-center-anchor-background-color)}.bounds-edge-anchor{border-width:var(--viewer-markup-freeform-bounds-edge-anchor-border-width);border-color:var(--viewer-markup-freeform-bounds-edge-anchor-border-color);border-style:solid;background-color:var(--viewer-markup-freeform-bounds-edge-anchor-background-color)}";

const ViewerMarkupFreeform = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.interactionBegin = createEvent(this, "interactionBegin", 7);
    this.interactionEnd = createEvent(this, "interactionEnd", 7);
    this.viewRendered = createEvent(this, "viewRendered", 7);
    /**
     * A mode that specifies how the markup component should behave. When
     * unset, the component will not respond to interactions with the handles.
     * When `edit`, the markup anchors are interactive and the user is able
     * to reposition them. When `create`, anytime the user clicks on the canvas,
     * a new markup will be performed.
     */
    this.mode = '';
    this.screenPoints = [];
    this.interactionHandler = new FreeformMarkupInteractionHandler(this.hostEl, this.interactionBegin, this.interactionEnd);
    this.handleWindowPointerDown = (event) => {
      if (isValidStartEvent(event)) {
        this.interactionHandler.startInteraction(event);
      }
    };
    this.handleTouchStart = (event) => {
      event.preventDefault();
    };
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.updateViewport();
    this.handleViewerChanged(this.viewer);
    this.updatePointsFromProps();
  }
  componentDidLoad() {
    const resize = new ResizeObserver(() => this.updateViewport());
    resize.observe(this.hostEl);
    if (this.mode === 'create') {
      window.addEventListener('pointerdown', this.handleWindowPointerDown);
    }
  }
  componentDidRender() {
    if (this.mode === '') {
      this.viewRendered.emit();
    }
  }
  disconnectedCallback() {
    this.dispose();
  }
  async dispose() {
    var _a;
    (_a = this.registeredInteraction) === null || _a === void 0 ? void 0 : _a.dispose();
    this.registeredInteraction = undefined;
    window.removeEventListener('pointerdown', this.handleWindowPointerDown);
  }
  /**
   * @ignore
   */
  async handleViewerChanged(newViewer) {
    var _a;
    (_a = this.registeredInteraction) === null || _a === void 0 ? void 0 : _a.dispose();
    this.registeredInteraction = undefined;
    if (newViewer != null) {
      this.registeredInteraction = await newViewer.registerInteractionHandler(this.interactionHandler);
    }
  }
  handlePointsChange() {
    this.updatePointsFromProps();
  }
  handleBoundsChange() {
    this.updatePointsFromProps();
  }
  handleModeChange() {
    if (this.mode !== 'create') {
      window.removeEventListener('pointerdown', this.handleWindowPointerDown);
    }
  }
  updateViewport() {
    var _a;
    const rect = getMarkupBoundingClientRect(this.hostEl);
    this.elementBounds = rect;
    this.screenPoints = (_a = this.convertPointsToScreen()) !== null && _a !== void 0 ? _a : this.screenPoints;
  }
  updatePointsFromProps() {
    var _a, _b, _c;
    this.points = (_a = this.points) !== null && _a !== void 0 ? _a : parsePoints(this.pointsJson);
    this.screenPoints = (_b = this.convertPointsToScreen()) !== null && _b !== void 0 ? _b : [];
    this.bounds = (_c = this.bounds) !== null && _c !== void 0 ? _c : parseBounds(this.boundsJson);
  }
  render() {
    if (this.screenPoints.length > 0 && this.elementBounds != null) {
      return (h(Host, null, h("svg", { class: "svg", onTouchStart: this.handleTouchStart }, h("defs", null, h(SvgShadow, { id: "freeform-markup-shadow" })), h("g", { filter: "url(#freeform-markup-shadow)" }, h("path", { class: "path", d: this.screenPoints.reduce((d, pt) => `${d}L${pt.x},${pt.y}`, `M${this.screenPoints[0].x},${this.screenPoints[0].y}`), fill: "none" }))), this.mode === 'edit' && this.bounds != null && (h(BoundingBox2d, { bounds: translateRectToScreen(this.bounds, this.elementBounds), onTopLeftAnchorPointerDown: (e) => this.interactionHandler.editAnchor('top-left', e), onTopRightAnchorPointerDown: (e) => this.interactionHandler.editAnchor('top-right', e), onTopAnchorPointerDown: (e) => this.interactionHandler.editAnchor('top', e), onBottomLeftAnchorPointerDown: (e) => this.interactionHandler.editAnchor('bottom-left', e), onBottomRightAnchorPointerDown: (e) => this.interactionHandler.editAnchor('bottom-right', e), onBottomAnchorPointerDown: (e) => this.interactionHandler.editAnchor('bottom', e), onLeftAnchorPointerDown: (e) => this.interactionHandler.editAnchor('left', e), onRightAnchorPointerDown: (e) => this.interactionHandler.editAnchor('right', e), onCenterAnchorPointerDown: (e) => this.interactionHandler.editAnchor('center', e) })), this.mode === 'create' && (h("div", { class: "create-overlay", onTouchStart: this.handleTouchStart }))));
    }
    else {
      return (h(Host, null, h("div", { class: "create-overlay", onTouchStart: this.handleTouchStart })));
    }
  }
  convertPointsToScreen() {
    var _a;
    const elementBounds = this.elementBounds;
    if (elementBounds != null) {
      return (_a = this.points) === null || _a === void 0 ? void 0 : _a.map((pt) => translatePointToScreen(pt, elementBounds));
    }
  }
  get hostEl() { return this; }
  static get watchers() { return {
    "viewer": ["handleViewerChanged"],
    "points": ["handlePointsChange"],
    "bounds": ["handleBoundsChange"],
    "mode": ["handleModeChange"]
  }; }
  static get style() { return viewerMarkupFreeformCss; }
}, [1, "vertex-viewer-markup-freeform", {
    "points": [1040],
    "pointsJson": [1, "points"],
    "bounds": [1040],
    "boundsJson": [1, "bounds"],
    "mode": [513],
    "viewer": [16],
    "elementBounds": [32],
    "screenPoints": [32],
    "dispose": [64]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-viewer-markup-freeform"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-viewer-markup-freeform":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, ViewerMarkupFreeform);
      }
      break;
  } });
}
defineCustomElement();

export { ViewerMarkupFreeform as V, defineCustomElement as d };

//# sourceMappingURL=viewer-markup-freeform.js.map