/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { proxyCustomElement, HTMLElement, h } from '@stencil/core/internal/client';
import { p as point, c as angle } from './bundle.esm.js';

const viewerMeasurementLineCss = ":host{display:block;--viewer-measurement-line-stroke:var(--white);--viewer-measurement-line-stroke-opacity:0.75;--viewer-measurement-line-stroke-width:1.25px;--viewer-measurement-line-fill:var(--neutral-900);--viewer-measurement-line-fill-width:2px;--viewer-measurement-line-start-cap-visibility:initial;--viewer-measurement-line-end-cap-visibility:initial}svg{width:100%;height:100%}.line-stroke{stroke-width:calc(\n    var(--viewer-measurement-line-fill-width) +\n      var(--viewer-measurement-line-stroke-width)\n  );stroke:var(--viewer-measurement-line-stroke);opacity:var(--viewer-measurement-line-stroke-opacity)}.line-fill{stroke-width:var(--viewer-measurement-line-fill-width);stroke:var(--viewer-measurement-line-fill)}.start-cap{visibility:var(--viewer-measurement-line-start-cap-visibility)}.end-cap{visibility:var(--viewer-measurement-line-end-cap-visibility)}";

const ViewerMeasurementLine = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    /**
     * A point that specifies the starting point of the line.
     */
    this.start = point.create();
    /**
     * A point that specifies the ending point of the line.
     */
    this.end = point.create();
    /**
     * A length of the line cap. The line cap is a line at each end of a line.
     */
    this.capLength = 0;
    /**
     * The type of [SVG pointer
     * events](https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/pointer-events)
     * that the line should respond to.
     */
    this.pointerEvents = 'none';
  }
  /**
   * @ignore
   */
  render() {
    const angle$1 = angle.fromPoints(this.start, this.end);
    const lineFillEndCaps = getEndCapPoints(this.start, this.end, angle$1, this.capLength);
    const lineStrokeEndCaps = getEndCapPoints(this.start, this.end, angle$1, this.capLength + 1);
    return (h("svg", null, h("g", { class: "line-stroke", "pointer-events": this.pointerEvents }, h("line", { class: "line", x1: this.start.x, y1: this.start.y, x2: this.end.x, y2: this.end.y }), h("line", { class: "start-cap", x1: lineStrokeEndCaps.startEndCap.start.x, y1: lineStrokeEndCaps.startEndCap.start.y, x2: lineStrokeEndCaps.startEndCap.end.x, y2: lineStrokeEndCaps.startEndCap.end.y }), h("line", { class: "end-cap", x1: lineStrokeEndCaps.endEndCap.start.x, y1: lineStrokeEndCaps.endEndCap.start.y, x2: lineStrokeEndCaps.endEndCap.end.x, y2: lineStrokeEndCaps.endEndCap.end.y })), h("g", { class: "line-fill", "pointer-events": this.pointerEvents }, h("line", { class: "line", x1: this.start.x, y1: this.start.y, x2: this.end.x, y2: this.end.y }), h("line", { class: "start-cap", x1: lineFillEndCaps.startEndCap.start.x, y1: lineFillEndCaps.startEndCap.start.y, x2: lineFillEndCaps.startEndCap.end.x, y2: lineFillEndCaps.startEndCap.end.y }), h("line", { class: "end-cap", x1: lineFillEndCaps.endEndCap.start.x, y1: lineFillEndCaps.endEndCap.start.y, x2: lineFillEndCaps.endEndCap.end.x, y2: lineFillEndCaps.endEndCap.end.y }))));
  }
  static get style() { return viewerMeasurementLineCss; }
}, [1, "vertex-viewer-measurement-line", {
    "start": [16],
    "end": [16],
    "capLength": [2, "cap-length"],
    "pointerEvents": [1, "pointer-events"]
  }]);
function getEndCapPoints(start, end, angle, length) {
  return {
    startEndCap: getPerpendicularLine(start, angle, length),
    endEndCap: getPerpendicularLine(end, angle, length),
  };
}
function getPerpendicularLine(pt, angle, length) {
  const perpAngle = angle + Math.PI / 2;
  const start = point.add(pt, point.polar(length / 2, perpAngle));
  const end = point.add(pt, point.polar(length / 2, perpAngle + Math.PI));
  return { start, end };
}
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-viewer-measurement-line"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-viewer-measurement-line":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, ViewerMeasurementLine);
      }
      break;
  } });
}
defineCustomElement();

export { ViewerMeasurementLine as V, defineCustomElement as d };

//# sourceMappingURL=viewer-measurement-line.js.map