/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { h, Fragment, proxyCustomElement, HTMLElement as HTMLElement$1, Host } from '@stencil/core/internal/client';
import './scene_view_api_pb.js';
import './wrappers_pb.js';
import './mapper.js';
import './streamAttributes.js';
import { M as MeasurementOverlayManager } from './overlays.js';
import { p as point, l as line3 } from './bundle.esm.js';
import { d as defineCustomElement$1 } from './viewer-measurement-line.js';

function translateWorldLineToViewport(line, params) {
  const { camera, viewport } = params;
  const isStartBehindCamera = camera.isPointBehindNear(line.start);
  const isEndBehindCamera = camera.isPointBehindNear(line.end);
  // If either the start or end of the line is behind the camera, then we need
  // to truncate the line so it can be presented correctly. You cannot use a
  // projection matrix to compute a point behind the near plane.
  if (isStartBehindCamera || isEndBehindCamera) {
    const pt = camera.intersectLineWithNear(line);
    const newLine = line3.create({
      start: isStartBehindCamera && pt != null ? pt : line.start,
      end: isEndBehindCamera && pt != null ? pt : line.end,
    });
    const ndc = line3.transformMatrix(newLine, camera.projectionViewMatrix);
    return {
      start: viewport.transformVectorToViewport(ndc.start),
      end: viewport.transformVectorToViewport(ndc.end),
      hideStart: isStartBehindCamera,
      hideEnd: isEndBehindCamera,
    };
  }
  else {
    const ndc = line3.transformMatrix(line, camera.projectionViewMatrix);
    return {
      start: viewport.transformVectorToViewport(ndc.start),
      end: viewport.transformVectorToViewport(ndc.end),
      hideStart: false,
      hideEnd: false,
    };
  }
}
function getViewingElementPositions(line, interactingAnchor, params) {
  const { start: startPt, end: endPt, hideStart, hideEnd, } = translateWorldLineToViewport(line, params);
  const labelPt = point.lerp(startPt, endPt, 0.5);
  const indicatorPt = interactingAnchor !== 'none'
    ? getIndicatorPtForAnchor(line, interactingAnchor, params)
    : undefined;
  return { startPt, endPt, labelPt, indicatorPt, hideStart, hideEnd };
}
function getIndicatorPtForAnchor(line, anchor, params) {
  return params.viewport.transformWorldToViewport(anchor === 'start' ? line.start : line.end, params.camera.projectionViewMatrix);
}

// eslint-disable-next-line @typescript-eslint/no-unused-vars
const MeasurementOverlayView = ({ overlay, viewport, camera }) => {
  if (overlay.type === 'line') {
    return (h(LineOverlayView, { overlay: overlay, viewport: viewport, camera: camera }));
  }
  else {
    return (h(DistanceVectorOverlayView, { overlay: overlay, viewport: viewport, camera: camera }));
  }
};
const LineOverlayView = ({ overlay, camera, viewport }) => {
  const { start, end } = translateWorldLineToViewport(overlay, {
    camera,
    viewport,
  });
  return (h("vertex-viewer-measurement-line", { class: "measurement-line", start: start, end: end }));
};
const DistanceVectorOverlayView = ({ overlay: { x, y, z }, camera, viewport }) => {
  const params = { camera, viewport };
  const { start: xs, end: xe } = translateWorldLineToViewport(x, params);
  const { start: ys, end: ye } = translateWorldLineToViewport(y, params);
  const { start: zs, end: ze } = translateWorldLineToViewport(z, params);
  return (h(Fragment, null,
    h("vertex-viewer-measurement-line", { class: "measurement-line distance-vector-x", start: xs, end: xe }),
    h("vertex-viewer-measurement-line", { class: "measurement-line distance-vector-y", start: ys, end: ye }),
    h("vertex-viewer-measurement-line", { class: "measurement-line distance-vector-z", start: zs, end: ze })));
};

const viewerMeasurementOverlaysCss = ":host{display:block;position:absolute;top:0;bottom:0;left:0;right:0;pointer-events:none;--viewer-measurement-overlays-x-axis-color:var(--x-axis-color);--viewer-measurement-overlays-y-axis-color:var(--y-axis-color);--viewer-measurement-overlays-z-axis-color:var(--z-axis-color)}.measurement-line{position:absolute;width:100%;height:100%}.measurement-line.distance-vector-x{--viewer-measurement-line-fill:var(\n    --viewer-measurement-overlays-x-axis-color\n  )}.measurement-line.distance-vector-y{--viewer-measurement-line-fill:var(\n    --viewer-measurement-overlays-y-axis-color\n  )}.measurement-line.distance-vector-z{--viewer-measurement-line-fill:var(\n    --viewer-measurement-overlays-z-axis-color\n  )}";

const ViewerMeasurementOverlays = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    /**
     * The model that contains the overlays to present.
     */
    this.measurementOverlays = new MeasurementOverlayManager();
    this.overlays = [];
    this.updateCamera = () => {
      var _a, _b;
      this.camera = (_b = (_a = this.viewer) === null || _a === void 0 ? void 0 : _a.frame) === null || _b === void 0 ? void 0 : _b.scene.camera;
    };
  }
  /**
   * @ignore
   */
  handleMeasurementOverlaysChanged() {
    this.removeModelListeners();
    this.addModelListeners();
    this.updateOverlays();
  }
  /**
   * @ignore
   */
  handleViewerChanged(newViewer, oldViewer) {
    this.removeViewerListeners(oldViewer);
    this.addViewerListeners();
    this.updateCamera();
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.addModelListeners();
    this.addViewerListeners();
    this.updateOverlays();
    this.updateCamera();
  }
  addModelListeners() {
    this.onOverlaysChangedListener = this.measurementOverlays.onOverlaysChanged(() => this.updateOverlays());
  }
  removeModelListeners() {
    var _a;
    (_a = this.onOverlaysChangedListener) === null || _a === void 0 ? void 0 : _a.dispose();
    this.onOverlaysChangedListener = undefined;
  }
  addViewerListeners() {
    var _a;
    (_a = this.viewer) === null || _a === void 0 ? void 0 : _a.addEventListener('frameReceived', this.updateCamera);
  }
  removeViewerListeners(viewer) {
    viewer === null || viewer === void 0 ? void 0 : viewer.removeEventListener('frameReceived', this.updateCamera);
  }
  updateOverlays() {
    this.overlays = this.measurementOverlays.getOverlays();
  }
  /**
   * @ignore
   */
  render() {
    return (h(Host, null, this.overlays.map((overlay) => {
      if (this.camera != null && this.viewer != null) {
        return (h(MeasurementOverlayView, { overlay: overlay, viewport: this.viewer.viewport, camera: this.camera }));
      }
    })));
  }
  static get watchers() { return {
    "measurementOverlays": ["handleMeasurementOverlaysChanged"],
    "viewer": ["handleViewerChanged"]
  }; }
  static get style() { return viewerMeasurementOverlaysCss; }
}, [1, "vertex-viewer-measurement-overlays", {
    "measurementOverlays": [16],
    "camera": [1040],
    "viewer": [16],
    "overlays": [32]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-viewer-measurement-overlays", "vertex-viewer-measurement-line"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-viewer-measurement-overlays":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, ViewerMeasurementOverlays);
      }
      break;
    case "vertex-viewer-measurement-line":
      if (!customElements.get(tagName)) {
        defineCustomElement$1();
      }
      break;
  } });
}
defineCustomElement();

export { ViewerMeasurementOverlays as V, defineCustomElement as d, getViewingElementPositions as g };

//# sourceMappingURL=viewer-measurement-overlays.js.map