/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { p as point, d as dimensions } from './bundle.esm.js';
import { c as classnames } from './index2.js';
import { g as getMouseClientPosition } from './dom.js';
import { P as PinModel, g as getPinColors, i as isTextPin } from './model3.js';
import { r as readDOM } from './stencil.js';
import { r as range } from './browser.esm.js';

/**
 * The `PinController` is responsible for adding pin entities to the viewer canvas
 */
class PinController {
  constructor(model, mode = 'view', type = 'pin-icon', styleAttributes = undefined) {
    this.model = model;
    this.mode = mode;
    this.type = type;
    this.styleAttributes = styleAttributes;
  }
  /**
   * Registers an entity to place on the canvas and places the pin on the associated part.
   *
   * @param pin The pin entity
   * @returns A void promise
   */
  addPin(pin) {
    this.model.addPin(pin);
  }
  /**
   * Clears all entities and returns a promise that resolves with an empty list
   * of pin results.
   */
  clearPins() {
    this.model.clearPins();
  }
  /**
   * Deregisters an entity and performs a pin if this entity was
   * removed.
   *
   * @param entity The entity to remove.
   * @returns A promise that resolves with the results after removing this
   * entity.
   */
  removePin(pin) {
    this.model.removePin(pin);
  }
  /**
   * Registers a set of entities and adds a pin
   *
   * @param entities The pin entities to draw.
   * @returns A promise that resolves with the results after registering these
   * entities.
   */
  setPins(pins) {
    this.model.setPins(pins);
  }
  /**
   * Sets the set of entities to be placed with the model.
   *
   * @param pin A pin to set
   * @returns `true` if the entity has been added.
   */
  setPin(pin) {
    this.model.setPin(pin);
  }
  /**
   * Updates an existing pin
   * @param pin A pin to set
   * @returns `true` if the entity has been updated, false otherwise
   */
  updatePin(pin) {
    this.model.updatePin(pin);
  }
  /**
   * Tells the controller which pin is selected in the model
   * @param pinId
   */
  setSelectedPinId(pinId) {
    this.model.setSelectedPin(pinId);
  }
  getToolMode() {
    return this.mode;
  }
  getToolType() {
    return this.type;
  }
  setToolMode(mode) {
    this.mode = mode;
  }
  setToolType(type) {
    this.type = type;
  }
  getDraggable() {
    return this.draggable;
  }
  setDraggable(draggable) {
    this.draggable = draggable;
  }
  setPrimaryColor(color) {
    this.styleAttributes = Object.assign(Object.assign({}, this.styleAttributes), { primaryColor: color });
  }
  setAccentColor(color) {
    this.styleAttributes = Object.assign(Object.assign({}, this.styleAttributes), { accentColor: color });
  }
  getStyleAttributes() {
    return this.styleAttributes;
  }
  updateDraggable(draggable, worldPosition, partId) {
    if (this.draggable != null) {
      this.draggable = draggable;
    }
    const pin = this.model.getPinById(draggable.id);
    if (pin != null) {
      this.updatePin(Object.assign(Object.assign({}, pin), { worldPosition,
        partId }));
    }
  }
}

function constrainRelativePoint(pt, range) {
  const constrainedX = Math.max(range.start, Math.min(range.end, pt.x));
  const constrainedY = Math.max(range.start, Math.min(range.end, pt.y));
  return point.create(constrainedX, constrainedY);
}
/**
 * Translates a point in relative units to a point in screen units.
 */
function translatePointToScreen(pt, canvasDimensions) {
  return point.add(point.scale(pt, canvasDimensions.width, canvasDimensions.height), dimensions.center(canvasDimensions));
}
/**
 * Translates a point in screen units to a point in relative units.
 */
function translatePointToRelative(pt, canvasDimensions, includeOffset, xOffset, yOffset) {
  const verticalScaleFactor = 1 / canvasDimensions.height;
  const horizontalScaleFactor = 1 / canvasDimensions.width;
  const canvasCenterPoint = dimensions.center(canvasDimensions);
  const point$1 = point.scale(point.subtract(pt, canvasCenterPoint), horizontalScaleFactor, verticalScaleFactor);
  if (includeOffset) {
    const offset = point.scale(point.create(xOffset, yOffset), horizontalScaleFactor, verticalScaleFactor);
    // We want to place the label towards the center of the screen.
    // The given point corresponds to the upper left corner of the label, so increase the offset
    // when placing it to the left or above to ensure that part of the label line is visible
    const centerIsToTheRightInXDirection = canvasCenterPoint.x > pt.x;
    const xOffsetCalculated = centerIsToTheRightInXDirection
      ? offset.x
      : -3 * offset.x;
    const centerIsBelowInYDirection = canvasCenterPoint.y > pt.y;
    const yOffsetCalculated = centerIsBelowInYDirection
      ? offset.y
      : -3 * offset.y;
    const offsetPoint = point.add(point$1, point.create(xOffsetCalculated, yOffsetCalculated));
    return constrainRelativePoint(offsetPoint, range.create(-0.5, 0.5));
  }
  else {
    return constrainRelativePoint(point$1, range.create(-0.5, 0.5));
  }
}

function getComputedStyle(el) {
  return window.getComputedStyle(el);
}

const viewerPinLabelCss = ":host{--viewer-annotations-pin-label-background-color:var(--viewer-annotations-pin-accent-color);--viewer-annotations-pin-label-border-color:var(--viewer-annotations-pin-primary-color);--viewer-annotations-pin-label-focused-border-color:var(--viewer-annotations-pin-primary-color);--viewer-annotations-pin-label-color:var(--white);--viewer-annotations-pin-label-padding-y:0.375em;--viewer-annotations-pin-label-padding-x:0.5em;--viewer-annotations-pin-label-border-width:2px;--viewer-annotations-pin-label-border-style:solid;--viewer-annotations-pin-label-border-radius:0.25em;--viewer-annotations-pin-label-min-width:2rem;--viewer-annotations-pin-label-max-width:25rem;--viewer-annotations-pin-label-max-height:50rem;overflow:hidden;pointer-events:none}:host([mode='']) .pin-label{color:var(--viewer-annotations-pin-label-color);background:var(--viewer-annotations-pin-label-background-color)}.pin-label-text{font-family:Arial, Helvetica, sans-serif;font-size:0.875rem;line-height:1rem;word-break:break-word;white-space:pre-wrap}.pin-label-input-wrapper{display:flex;position:absolute;pointer-events:auto;box-sizing:border-box;min-width:var(--viewer-annotations-pin-label-min-width);max-width:var(--viewer-annotations-pin-label-max-width);max-height:var(--viewer-annotations-pin-label-max-height);background:var(--viewer-annotations-pin-label-background-color);border-width:var(--viewer-annotations-pin-label-border-width);border-style:var(--viewer-annotations-pin-label-border-style);border-color:var(--viewer-annotations-pin-label-border-color);border-radius:var(--viewer-annotations-pin-label-border-radius)}.pin-label-input-wrapper.focused{min-width:var(--viewer-annotations-pin-label-max-width);border-color:var(--viewer-annotations-pin-label-focused-border-color)}.pin-input-drag-target{position:absolute;top:0;left:0;width:100%;height:100%;cursor:pointer}.pin-label-input{resize:none;position:relative;outline:none;border:none;word-break:break-word;padding:0 0 0 0;background-color:transparent;margin:var(--viewer-annotations-pin-label-padding-y) var(--viewer-annotations-pin-label-padding-x);width:var(--viewer-annotations-pin-label-min-width);flex-grow:1;overflow:hidden;max-width:calc(var(--viewer-annotations-pin-label-max-width) - calc(2 * var(--viewer-annotations-pin-label-padding-x)))}.pin-label-input.readonly{pointer-events:auto;color:black;cursor:pointer}.pin-label-hidden{position:absolute;visibility:hidden;pointer-events:none;width:auto;height:auto;box-sizing:border-box;border-width:var(--viewer-annotations-pin-label-border-width);border-style:var(--viewer-annotations-pin-label-border-style);border-color:var(--viewer-annotations-pin-label-border-color);margin:var(--viewer-annotations-pin-label-padding-y) 0;padding:0 var(--viewer-annotations-pin-label-padding-x);max-width:calc(var(--viewer-annotations-pin-label-max-width) - calc(2 * var(--viewer-annotations-pin-label-padding-x)))}";

const VertexPinLabel = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.labelChanged = createEvent(this, "labelChanged", 7);
    this.labelFocused = createEvent(this, "labelFocused", 7);
    this.labelBlurred = createEvent(this, "labelBlurred", 7);
    this.focused = false;
    this.textareaRows = 1;
    this.computeContentSize = () => {
      readDOM(() => {
        if (this.contentEl != null) {
          this.contentElBounds = this.contentEl.getBoundingClientRect();
          const computedStyles = getComputedStyle(this.contentEl);
          this.textareaRows = Math.max(1, Math.ceil((parseFloat(computedStyles.height) -
            parseFloat(computedStyles.borderWidth) * 2) /
            parseFloat(computedStyles.lineHeight)));
        }
      });
    };
    this.handleInputKeyDown = (event) => {
      event.stopPropagation();
      if (event.key === 'Enter' &&
        (event.ctrlKey || event.metaKey || event.shiftKey)) {
        this.textareaRows += 1;
        this.value += '\n';
      }
      else if (event.key === 'Enter') {
        event.preventDefault();
        this.submit();
      }
    };
    this.handlePointerDown = (event) => {
      var _a;
      if (!this.focused) {
        if (this.elementBounds != null) {
          if (event.buttons !== 2) {
            event.preventDefault();
            event.stopPropagation();
          }
          this.relativePointerDownPosition = translatePointToRelative(getMouseClientPosition(event, this.elementBounds), this.elementBounds);
          this.pinPointerDownPosition = (_a = this.pin) === null || _a === void 0 ? void 0 : _a.label.point;
          window.addEventListener('pointermove', this.handlePointerMove);
          window.addEventListener('pointerup', this.handlePointerUp);
        }
      }
    };
    this.handlePointerMove = (event) => {
      var _a;
      if (this.elementBounds != null &&
        this.relativePointerDownPosition &&
        this.pinPointerDownPosition != null) {
        // Prevent selection of any text while interacting with the label
        event.preventDefault();
        event.stopPropagation();
        const point$1 = getMouseClientPosition(event, this.elementBounds);
        const relative = translatePointToRelative(point$1, this.elementBounds);
        const relativeDelta = point.subtract(relative, this.relativePointerDownPosition);
        const myUpdatedPin = this.pin != null
          ? Object.assign(Object.assign({}, this.pin), { label: Object.assign(Object.assign({}, this.pin.label), { point: point.add(this.pinPointerDownPosition, relativeDelta) }) }) : undefined;
        if (myUpdatedPin) {
          (_a = this.pinController) === null || _a === void 0 ? void 0 : _a.updatePin(myUpdatedPin);
          this.computeScreenPosition();
        }
      }
    };
    this.handlePointerUp = (event) => {
      var _a, _b;
      if (((_a = this.pinController) === null || _a === void 0 ? void 0 : _a.getToolMode()) === 'edit' &&
        this.relativePointerDownPosition != null &&
        this.elementBounds != null) {
        const pointerDownScreen = translatePointToScreen(this.relativePointerDownPosition, this.elementBounds);
        const distanceBetweenStartAndEndPoint = point.distance(pointerDownScreen, getMouseClientPosition(event, this.elementBounds));
        if (distanceBetweenStartAndEndPoint <= 2) {
          this.focused = true;
          this.labelFocused.emit((_b = this.pin) === null || _b === void 0 ? void 0 : _b.id);
        }
      }
      this.relativePointerDownPosition = undefined;
      this.pinPointerDownPosition = undefined;
      window.removeEventListener('pointermove', this.handlePointerMove);
      window.removeEventListener('pointerup', this.handlePointerUp);
    };
    this.handleTextBlur = () => {
      this.submit();
    };
    this.handleTextInput = (event) => {
      const input = event.target;
      this.value = input.value;
    };
    this.value = this.getPinText();
  }
  /**
   * Gives focus to the the component's internal text input.
   */
  async setFocus() {
    var _a, _b, _c;
    // HTMLInputElement.focus() doesn't exist in tests.
    if (typeof ((_a = this.inputEl) === null || _a === void 0 ? void 0 : _a.focus) === 'function') {
      this.focused = true;
      this.labelFocused.emit((_b = this.pin) === null || _b === void 0 ? void 0 : _b.id);
      (_c = this.inputEl) === null || _c === void 0 ? void 0 : _c.focus();
    }
  }
  /**
   * @internal
   * submits the current text, unfocuses the input and emits the blur event to consumers
   */
  async submit() {
    var _a, _b;
    this.focused = false;
    this.labelBlurred.emit((_a = this.pin) === null || _a === void 0 ? void 0 : _a.id);
    if (this.pin != null) {
      (_b = this.pinController) === null || _b === void 0 ? void 0 : _b.updatePin(Object.assign(Object.assign({}, this.pin), { label: {
          point: this.pin.label.point,
          text: this.value,
        } }));
    }
  }
  watchFocusChange() {
    var _a, _b;
    this.labelChanged.emit();
    if (this.focused) {
      (_a = this.pinController) === null || _a === void 0 ? void 0 : _a.setSelectedPinId((_b = this.pin) === null || _b === void 0 ? void 0 : _b.id);
    }
  }
  watchPinChange() {
    this.value = this.getPinText();
    this.computeScreenPosition();
  }
  watchElementBoundsChange() {
    this.computeScreenPosition();
  }
  componentWillLoad() {
    this.computeScreenPosition();
  }
  componentDidLoad() {
    this.resizeObserver = new ResizeObserver(() => {
      this.labelChanged.emit();
    });
    this.contentResizeObserver = new ResizeObserver(this.computeContentSize);
    if (this.hostEl != null) {
      this.resizeObserver.observe(this.hostEl);
    }
    if (this.contentEl != null) {
      this.contentResizeObserver.observe(this.contentEl);
    }
    if (this.pinController == null) {
      this.pinController = new PinController(new PinModel());
    }
  }
  disconnectedCallback() {
    var _a, _b;
    (_a = this.resizeObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
    (_b = this.contentResizeObserver) === null || _b === void 0 ? void 0 : _b.disconnect();
  }
  componentDidRender() {
    if (this.focused && this.inputEl != null) {
      this.setFocus();
    }
  }
  render() {
    var _a, _b, _c;
    const { primaryColor, accentColor } = getPinColors(this.pin);
    return (h(Host, null, h("div", { class: classnames('pin-label-input-wrapper', {
        focused: this.focused,
      }), onPointerDown: this.handlePointerDown, style: {
        top: `${((_a = this.computedScreenPosition) === null || _a === void 0 ? void 0 : _a.y.toString()) || 0}px`,
        left: `${((_b = this.computedScreenPosition) === null || _b === void 0 ? void 0 : _b.x.toString()) || 0}px`,
        minWidth: this.computeMinWidth(),
        maxWidth: this.computeMaxWidth(),
        maxHeight: this.computeMaxHeight(),
        borderColor: primaryColor,
        background: accentColor,
      } }, h("textarea", { id: `pin-label-input-${(_c = this.pin) === null || _c === void 0 ? void 0 : _c.id}`, class: classnames('pin-label-input', 'pin-label-text', {
        ['readonly']: !this.focused,
      }), disabled: !this.focused, ref: (ref) => (this.inputEl = ref), value: this.value, rows: this.textareaRows, onKeyDown: this.handleInputKeyDown, onInput: this.handleTextInput, onBlur: this.handleTextBlur }), !this.focused && (h("div", { class: "pin-input-drag-target", onPointerDown: this.handlePointerDown }))), h("div", { ref: (el) => (this.contentEl = el), class: classnames('pin-label-text', 'pin-label-hidden'), style: {
        maxWidth: this.computeMaxWidth(),
        maxHeight: this.computeMaxHeight(),
      } }, this.hiddenContent())));
  }
  hiddenContent() {
    // This corrects some inconsistencies in how a newline character
    // is represented in a div. Leveraging `<br>`s results in a more
    // consistent representation of the newlines in a textarea.
    return this.value.includes('\n')
      ? this.value
        .split('\n')
        .reduce((res, str) => [...res, str, h("br", null)], [])
      : [this.value];
  }
  computeMinWidth() {
    if (this.contentElBounds != null) {
      const expected = `${this.contentElBounds.width + 8}px`;
      return `min(${expected}, ${this.computeMaxWidth()})`;
    }
    else {
      return `var(--viewer-annotations-pin-label-min-width)`;
    }
  }
  computeMaxWidth() {
    const expected = `var(--viewer-annotations-pin-label-max-width)`;
    return `min(${expected}, ${this.computeRemainingWidth()})`;
  }
  computeMaxHeight() {
    const expected = `var(--viewer-annotations-pin-label-max-height)`;
    return `min(${expected}, ${this.computeRemainingHeight()})`;
  }
  computeRemainingWidth() {
    var _a, _b;
    return `calc(${((_a = this.elementBounds) === null || _a === void 0 ? void 0 : _a.width.toString()) || 0}px - ${((_b = this.computedScreenPosition) === null || _b === void 0 ? void 0 : _b.x.toString()) || 0}px)`;
  }
  computeRemainingHeight() {
    var _a, _b;
    return `calc(${((_a = this.elementBounds) === null || _a === void 0 ? void 0 : _a.height.toString()) || 0}px - ${((_b = this.computedScreenPosition) === null || _b === void 0 ? void 0 : _b.y.toString()) || 0}px)`;
  }
  getPinText() {
    var _a;
    if (((_a = this.pin) === null || _a === void 0 ? void 0 : _a.label.text) != null) {
      this.value = this.pin.label.text;
    }
    else {
      this.value = '';
    }
    return this.value;
  }
  computeScreenPosition() {
    this.computedScreenPosition =
      isTextPin(this.pin) &&
        this.elementBounds != null &&
        this.pin.label.point != null
        ? translatePointToScreen(this.pin.label.point, this.elementBounds)
        : undefined;
  }
  get hostEl() { return this; }
  static get watchers() { return {
    "focused": ["watchFocusChange"],
    "pin": ["watchPinChange"],
    "elementBounds": ["watchElementBoundsChange"]
  }; }
  static get style() { return viewerPinLabelCss; }
}, [0, "vertex-viewer-pin-label", {
    "pin": [16],
    "elementBounds": [16],
    "value": [1025],
    "pinController": [16],
    "focused": [32],
    "computedScreenPosition": [32],
    "textareaRows": [32],
    "contentElBounds": [32],
    "setFocus": [64],
    "submit": [64]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-viewer-pin-label"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-viewer-pin-label":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, VertexPinLabel);
      }
      break;
  } });
}
defineCustomElement();

export { PinController as P, VertexPinLabel as V, translatePointToRelative as a, defineCustomElement as d, translatePointToScreen as t };

//# sourceMappingURL=viewer-pin-label.js.map