/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { p as point, v as vector3, b as boundingBox, f as plane, r as ray } from './bundle.esm.js';
import { g as getMouseClientPosition } from './dom.js';
import { E as ElementRectObserver } from './elementRectObserver.js';
import { E as EventDispatcher } from './browser.esm.js';

class TeleportInteractionHandler {
  constructor(model, animationConfiguration) {
    this.model = model;
    this.animationConfiguration = animationConfiguration;
    this.rectObserver = new ElementRectObserver();
    this.handlePointerDown = this.handlePointerDown.bind(this);
    this.handlePointerUp = this.handlePointerUp.bind(this);
    this.handleEnabledChange = this.handleEnabledChange.bind(this);
    this.handleTeleportModeChange = this.handleTeleportModeChange.bind(this);
    this.enabledChangeDisposable = this.model.onEnabledChange(this.handleEnabledChange);
  }
  dispose() {
    var _a;
    this.disable();
    (_a = this.enabledChangeDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    this.element = undefined;
    this.api = undefined;
    this.rectObserver.disconnect();
  }
  initialize(element, api) {
    this.element = element;
    this.api = api;
    this.rectObserver.observe(element);
    this.handleEnabledChange(this.model.getEnabled());
  }
  setAnimations(config) {
    this.animationConfiguration = config;
  }
  enable() {
    var _a;
    this.disable();
    this.teleportModeChangeDisposable = this.model.onTeleportModeChange(this.handleTeleportModeChange);
    (_a = this.element) === null || _a === void 0 ? void 0 : _a.addEventListener('pointerdown', this.handlePointerDown);
    this.handleTeleportModeChange(this.model.getTeleportMode());
  }
  disable() {
    var _a, _b, _c;
    (_a = this.teleportModeChangeDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    (_b = this.cursorDisposable) === null || _b === void 0 ? void 0 : _b.dispose();
    (_c = this.element) === null || _c === void 0 ? void 0 : _c.removeEventListener('pointerdown', this.handlePointerDown);
  }
  async handlePointerDown(event) {
    this.downPosition = getMouseClientPosition(event, this.rectObserver.rect);
    this.downButtons = event.buttons;
    window.addEventListener('pointerup', this.handlePointerUp);
  }
  async handlePointerUp(event) {
    var _a, _b;
    const mode = this.model.getTeleportMode();
    const threshold = (_b = (_a = this.api) === null || _a === void 0 ? void 0 : _a.pixelThreshold()) !== null && _b !== void 0 ? _b : 2;
    const point$1 = getMouseClientPosition(event, this.rectObserver.rect);
    const isRightClick = this.downButtons === 2;
    const hasModifier = event.shiftKey || event.altKey || event.metaKey || event.ctrlKey;
    if (mode != null &&
      this.downPosition != null &&
      point.distance(this.downPosition, point$1) <= threshold &&
      !isRightClick &&
      !hasModifier) {
      switch (mode) {
        case 'teleport':
          return this.teleportToHit(point$1);
        case 'teleport-toward':
          return this.teleportTowardHit(point$1);
        case 'teleport-and-align':
          return this.teleportAndAlign(point$1);
      }
    }
    this.downPosition = undefined;
    window.removeEventListener('pointerup', this.handlePointerUp);
  }
  async beginInteraction() {
    var _a, _b;
    if (!((_a = this.api) === null || _a === void 0 ? void 0 : _a.isInteracting())) {
      await ((_b = this.api) === null || _b === void 0 ? void 0 : _b.beginInteraction());
    }
  }
  async endInteraction() {
    var _a;
    await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.endInteraction());
  }
  async teleportTowardHit(point) {
    var _a, _b;
    const worldPoint = await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.getWorldPointFromViewport(point));
    const mouseRay = (_b = this.api) === null || _b === void 0 ? void 0 : _b.getRayFromPoint(point);
    if (mouseRay != null) {
      const { teleportDistancePercentage, teleportCollisionDistance } = this.model.getConfiguration();
      const teleportDistanceScalar = teleportDistancePercentage / 100;
      return this.performInteraction(({ camera, boundingBox: boundingBox$1 }) => {
        const stepDistance = Math.max(...vector3.toArray(boundingBox.lengths(boundingBox$1))) *
          teleportDistanceScalar;
        const positionPlane = plane.fromNormalAndCoplanarPoint(camera.up, camera.position);
        const projectedWorldPoint = plane.projectPoint(positionPlane, worldPoint !== null && worldPoint !== void 0 ? worldPoint : camera.position);
        const distanceToWorldPoint = vector3.distance(camera.position, projectedWorldPoint);
        if (distanceToWorldPoint < stepDistance &&
          distanceToWorldPoint > teleportCollisionDistance * 1.1) {
          const mouseRayAtWorldPoint = ray.create(Object.assign(Object.assign({}, mouseRay), { origin: projectedWorldPoint }));
          const nextPosition = plane.projectPoint(positionPlane, ray.at(mouseRayAtWorldPoint, -teleportCollisionDistance));
          return camera.update({
            position: nextPosition,
            lookAt: vector3.add(camera.lookAt, vector3.subtract(nextPosition, camera.position)),
          });
        }
        else {
          const projectedNextStep = plane.projectPoint(positionPlane, ray.at(mouseRay, stepDistance));
          return camera.update({
            position: projectedNextStep,
            lookAt: vector3.add(camera.lookAt, vector3.subtract(projectedNextStep, camera.position)),
          });
        }
      });
    }
  }
  async teleportToHit(point) {
    const worldPoint = await this.getWorldPointWithFallback(point);
    if (worldPoint != null) {
      return this.performInteraction(({ camera, boundingBox }) => {
        const shortestBoundingBoxLength = this.shortestLength(boundingBox);
        const heightScalar = this.model.getTeleportHeightPercentage() / 100;
        const cameraPlane = plane.fromNormalAndCoplanarPoint(camera.up, camera.position);
        const projectedHitPosition = plane.projectPoint(cameraPlane, worldPoint);
        const rayToHitPosition = ray.create({
          origin: camera.position,
          direction: vector3.normalize(vector3.subtract(projectedHitPosition, camera.position)),
        });
        const distanceToHitPosition = vector3.distance(camera.position, projectedHitPosition);
        const distanceToLookAt = vector3.distance(camera.position, camera.lookAt);
        const newPosition = ray.at(rayToHitPosition, distanceToHitPosition - shortestBoundingBoxLength * heightScalar);
        const newPositionViewVectorRay = ray.create({
          origin: newPosition,
          direction: vector3.normalize(camera.viewVector),
        });
        return camera.update({
          position: newPosition,
          lookAt: ray.at(newPositionViewVectorRay, distanceToLookAt),
        });
      });
    }
  }
  async teleportAndAlign(point) {
    var _a;
    const hits = await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.hitItems(point));
    const hit = hits != null ? hits[0] : undefined;
    if ((hit === null || hit === void 0 ? void 0 : hit.hitNormal) != null && (hit === null || hit === void 0 ? void 0 : hit.hitPoint)) {
      await this.performInteraction(({ camera, boundingBox }) => {
        const shortestBoundingBoxLength = this.shortestLength(boundingBox);
        const heightScalar = this.model.getTeleportHeightPercentage() / 100;
        const upRay = ray.create({
          origin: hit.hitPoint,
          direction: hit.hitNormal,
        });
        return camera.alignTo(ray.at(upRay, shortestBoundingBoxLength * heightScalar), hit.hitNormal);
      });
    }
  }
  async performInteraction(fn) {
    var _a;
    await this.beginInteraction();
    await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.transformCamera(fn, this.renderConfiguration()));
    await this.endInteraction();
  }
  async getWorldPointWithFallback(point) {
    var _a, _b;
    const worldPoint = await ((_a = this.api) === null || _a === void 0 ? void 0 : _a.getWorldPointFromViewport(point));
    if (worldPoint == null) {
      const hits = await ((_b = this.api) === null || _b === void 0 ? void 0 : _b.hitItems(point));
      const hit = hits != null ? hits[0] : undefined;
      return (hit === null || hit === void 0 ? void 0 : hit.hitPoint) != null
        ? hit.hitPoint
        : worldPoint;
    }
    return worldPoint;
  }
  handleEnabledChange(enabled) {
    if (enabled) {
      this.enable();
    }
    else {
      this.disable();
    }
  }
  handleTeleportModeChange(mode) {
    var _a, _b;
    (_a = this.cursorDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    if (mode != null) {
      this.cursorDisposable = (_b = this.api) === null || _b === void 0 ? void 0 : _b.addCursor('crosshair');
    }
  }
  shortestLength(boundingBox$1) {
    return Math.min(...vector3.toArray(boundingBox.lengths(boundingBox$1)));
  }
  renderConfiguration() {
    return this.animationConfiguration != null
      ? { animation: { milliseconds: this.animationConfiguration.durationMs } }
      : undefined;
  }
}

class KeyBinding {
  constructor(...bindings) {
    this.bindings = bindings.map((binding) => binding.toLowerCase());
  }
  matches(keys) {
    return (this.bindings.length > 0 &&
      this.bindings.every((binding) => this.bindingMatches(keys, binding)));
  }
  bindingMatches(keys, binding) {
    const negated = binding.includes('!');
    const key = binding.replace('!', '');
    return negated ? !keys[key] : keys[key];
  }
}

class WalkModeController {
  constructor(model) {
    this.model = model;
    this.excludeTagNames = [
      'VERTEX-SCENE-TREE',
      'VERTEX-SCENE-TREE-SEARCH',
      'VERTEX-VIEWER-PIN-TOOL',
      'INPUT',
      'TEXTAREA',
    ];
    this.excludePredicates = [];
    this.updateModelExclusions();
  }
  /**
   * @internal
   */
  updateModel(model) {
    this.model = model;
  }
  /**
   * Sets whether downstream walk mode interaction handlers are enabled.
   * Setting this value to `false` will remove all event listeners for
   * the interactions, and setting this value to `true` will add or
   * re-add the event listeners.
   */
  setEnabled(enabled) {
    this.model.setEnabled(enabled);
  }
  /**
   * Sets the `ViewerTeleportMode` to be used with a `<vertex-viewer-teleport-tool>`.
   */
  setTeleportMode(mode) {
    this.model.setTeleportMode(mode);
  }
  /**
   * Updates the configuration for downstream walk mode interaction handlers.
   *
   * `teleportHeightPercentage` - percentage used for fine-tuning the distance to offset
   * the camera from a surface when performing a `teleport-and-align`. This percentage is
   * used alongside the shortest side of the visible bounding box to determine how far to
   * place the camera from the surface that has been hit, with a larger percentage placing
   * the camera further from the surface and vice-versa. Defaults to 11.75%.
   *
   * `teleportDistancePercentage` - percentage used for fine-tuning the distance moved per
   * click with the `teleport-toward` mode. This percentage is used alongside the longest
   * side of the visible bounding box to determine how far to move the camera's position
   * with each click, with a larger percentage moving the camera further and vice-versa.
   * Defaults to 2%.
   *
   * `teleportCollisionDistance` - distance used alongside the `teleportCollisionPercentage`
   * to determine whether a click with the `teleport-toward` mode would cause a "collision",
   * or would pass through the point clicked. If such a collision would occur, the camera is
   * instead placed this distance back from the point clicked once, then allowed to pass
   * through the point on subsequent clicks. Note that this will not apply if clicks occur
   * in quick succession without setting the `depthBuffers` property on the `<vertex-viewer>`
   * to `all` frames. Defaults to `1000`.
   *
   * `keyboardWalkSpeed` - speed to move the camera when performing keyboard-based
   * walk interactions. A larger number here will result in a faster walk speed through
   * the model and vice-versa. Defaults to `5`.
   *
   * `keyboardPivotDegrees` - number of degrees to move the camera when performing
   * keyboard-based pivot interactions. Defaults to `1`.
   *
   * `keyboardRepeatIntervalMs` - number of milliseconds to repeat keyboard-based interactions.
   * this value is multiplicative with the `keyboardWalkSpeed` and `keyboardPivotDegrees`, and
   * lower numbers will result in faster movement and vice-versa. Defaults to `25`.
   *
   */
  updateConfiguration(configuration) {
    const existing = this.model.getConfiguration();
    this.model.setConfiguration(Object.assign(Object.assign({}, existing), configuration));
  }
  /**
   * Adds a custom keybinding for a specific walk operation. This will append to the
   * existing set of keybindings to allow for multiple keybindings for a specific
   * operation. To replace the defaults, see `replaceKeyBinding`.
   */
  addKeyBinding(operation, ...keys) {
    this.model.addKeyBinding(operation, new KeyBinding(...keys));
  }
  /**
   * Adds a custom keybinding for a specific walk operation. This will replace any
   * existing keybindings to allow for overriding the default behavior.
   *
   * @example
   * ```
   * const walkModeTool = document.querySelector('vertex-viewer-walk-mode-tool');
   *
   * // Remove keybinding for the `PIVOT_UP` operation
   * walkModeTool.controller.replaceKeyBinding('PIVOT_UP');
   *
   * // Replace keybinding for `WALK_FORWARD` with `ArrowUp` instead of `w`
   * walkModeTool.controller.replaceKeyBinding('WALK_FORWARD', 'ArrowUp');
   * ```
   */
  replaceKeyBinding(operation, ...keys) {
    this.model.replaceKeyBinding(operation, new KeyBinding(...keys));
  }
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  excludeElement(...args) {
    if (typeof args[0] === 'string') {
      this.excludeTagNames = [...this.excludeTagNames, args[0]];
      this.updateModelExclusions();
      return {
        dispose: () => {
          this.excludeTagNames = this.excludeTagNames.filter((tn) => tn !== args[0]);
        },
      };
    }
    else {
      this.excludePredicates = [...this.excludePredicates, args[0]];
      this.updateModelExclusions();
      return {
        dispose: () => {
          this.excludePredicates = this.excludePredicates.filter((tn) => tn !== args[0]);
        },
      };
    }
  }
  updateModelExclusions() {
    this.model.setExcludedPredicate((el) => this.excludeTagNames.some((tn) => tn === el.tagName) ||
      this.excludePredicates.some((p) => p(el)));
  }
}

var ViewerWalkModeOperation;
(function (ViewerWalkModeOperation) {
  ViewerWalkModeOperation["MOVE_DOWN"] = "MOVE_DOWN";
  ViewerWalkModeOperation["MOVE_UP"] = "MOVE_UP";
  ViewerWalkModeOperation["PIVOT_DOWN"] = "PIVOT_DOWN";
  ViewerWalkModeOperation["PIVOT_LEFT"] = "PIVOT_LEFT";
  ViewerWalkModeOperation["PIVOT_RIGHT"] = "PIVOT_RIGHT";
  ViewerWalkModeOperation["PIVOT_UP"] = "PIVOT_UP";
  ViewerWalkModeOperation["WALK_BACKWARD"] = "WALK_BACKWARD";
  ViewerWalkModeOperation["WALK_FORWARD"] = "WALK_FORWARD";
  ViewerWalkModeOperation["WALK_LEFT"] = "WALK_LEFT";
  ViewerWalkModeOperation["WALK_RIGHT"] = "WALK_RIGHT";
})(ViewerWalkModeOperation || (ViewerWalkModeOperation = {}));
class WalkModeModel {
  constructor() {
    this.keyBindings = {
      [ViewerWalkModeOperation.MOVE_DOWN]: [
        new KeyBinding('PageDown', '!Shift', '!Alt'),
      ],
      [ViewerWalkModeOperation.MOVE_UP]: [
        new KeyBinding('PageUp', '!Shift', '!Alt'),
      ],
      [ViewerWalkModeOperation.PIVOT_DOWN]: [
        new KeyBinding('ArrowDown', '!Shift', '!Alt'),
      ],
      [ViewerWalkModeOperation.PIVOT_LEFT]: [
        new KeyBinding('ArrowLeft', '!Shift', '!Alt'),
      ],
      [ViewerWalkModeOperation.PIVOT_RIGHT]: [
        new KeyBinding('ArrowRight', '!Shift', '!Alt'),
      ],
      [ViewerWalkModeOperation.PIVOT_UP]: [
        new KeyBinding('ArrowUp', '!Shift', '!Alt'),
      ],
      [ViewerWalkModeOperation.WALK_BACKWARD]: [new KeyBinding('s')],
      [ViewerWalkModeOperation.WALK_FORWARD]: [new KeyBinding('w')],
      [ViewerWalkModeOperation.WALK_LEFT]: [new KeyBinding('a')],
      [ViewerWalkModeOperation.WALK_RIGHT]: [new KeyBinding('d')],
    };
    this.configuration = {
      teleportHeightPercentage: 11.75,
      teleportDistancePercentage: 2,
      teleportCollisionDistance: 1000,
      keyboardWalkSpeed: 5,
      keyboardPivotDegrees: 1,
      keyboardRepeatIntervalMs: 25,
    };
    this.enabled = true;
    this.enabledChanged = new EventDispatcher();
    this.teleportModeChanged = new EventDispatcher();
    this.configurationChanged = new EventDispatcher();
    this.keyBindingsChanged = new EventDispatcher();
  }
  setEnabled(enabled) {
    this.enabled = enabled;
    this.enabledChanged.emit(enabled);
  }
  setTeleportMode(mode) {
    this.teleportMode = mode;
    this.teleportModeChanged.emit(mode);
  }
  setConfiguration(configuration) {
    this.configuration = configuration;
    this.configurationChanged.emit(configuration);
  }
  addKeyBinding(operation, keyBinding) {
    this.keyBindings[operation] = [...this.keyBindings[operation], keyBinding];
    this.keyBindingsChanged.emit(this.keyBindings);
  }
  replaceKeyBinding(operation, keyBinding) {
    this.keyBindings[operation] = [keyBinding];
    this.keyBindingsChanged.emit(this.keyBindings);
  }
  setExcludedPredicate(predicate) {
    this.excludedPredicate = predicate;
  }
  getEnabled() {
    return this.enabled;
  }
  getTeleportMode() {
    return this.teleportMode;
  }
  getTeleportHeightPercentage() {
    return this.configuration.teleportHeightPercentage;
  }
  getKeyboardWalkSpeed() {
    return this.configuration.keyboardWalkSpeed;
  }
  getKeyboardPivotDegrees() {
    return this.configuration.keyboardPivotDegrees;
  }
  getKeyboardRepeatInterval() {
    return this.configuration.keyboardRepeatIntervalMs;
  }
  getConfiguration() {
    return this.configuration;
  }
  getKeyBindings() {
    return this.keyBindings;
  }
  operationMatches(operation, state) {
    return this.keyBindings[operation].some((binding) => binding.matches(state));
  }
  isElementExcluded(el) {
    var _a;
    return !!((_a = this.excludedPredicate) === null || _a === void 0 ? void 0 : _a.call(this, el));
  }
  onEnabledChange(listener) {
    return this.enabledChanged.on(listener);
  }
  onTeleportModeChange(listener) {
    return this.teleportModeChanged.on(listener);
  }
  onConfigurationChange(listener) {
    return this.configurationChanged.on(listener);
  }
  onKeyBindingsChange(listener) {
    return this.keyBindingsChanged.on(listener);
  }
}

const viewerTeleportToolCss = ":host{position:absolute;top:0;bottom:0;left:0;right:0;pointer-events:none}";

const ViewerTeleportTool = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.controllerChanged = createEvent(this, "controllerChanged", 7);
    /**
     * Indicates whether animations will be used when performing camera
     * operations. Defaults to `false`.
     */
    this.animationsDisabled = false;
    /**
     * The duration of animations, in milliseconds. Defaults to `500`.
     */
    this.animationMs = 500;
    this.model = new WalkModeModel();
    this.stateMap = {};
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.setupController();
    this.setupInteractionHandler();
    this.setDepthBuffers();
  }
  /**
   * @ignore
   */
  connectedCallback() {
    this.setupInteractionHandler();
    this.setDepthBuffers();
  }
  /**
   * @ignore
   */
  disconnectedCallback() {
    this.clearInteractionHandler();
    this.resetDepthBuffers();
  }
  /**
   * @ignore
   */
  handleModeChange() {
    var _a;
    (_a = this.controller) === null || _a === void 0 ? void 0 : _a.setTeleportMode(this.mode);
    if (this.mode != null) {
      this.setDepthBuffers();
    }
    else {
      this.resetDepthBuffers();
    }
  }
  /**
   * @ignore
   */
  handleViewerChanged() {
    this.clearInteractionHandler();
    this.setupInteractionHandler();
    this.setDepthBuffers();
  }
  /**
   * @ignore
   */
  handleAnimationMsChanged() {
    var _a;
    if (this.animationMs != null) {
      (_a = this.interactionHandler) === null || _a === void 0 ? void 0 : _a.setAnimations({ durationMs: this.animationMs });
    }
  }
  /**
   * @ignore
   */
  handleAnimationsDisabledChanged() {
    var _a, _b;
    if (this.animationsDisabled) {
      (_a = this.interactionHandler) === null || _a === void 0 ? void 0 : _a.setAnimations(undefined);
    }
    else if (this.animationMs != null) {
      (_b = this.interactionHandler) === null || _b === void 0 ? void 0 : _b.setAnimations({ durationMs: this.animationMs });
    }
  }
  /**
   * @ignore
   */
  handleControllerChanged() {
    this.setupInteractionHandler();
    this.setupController();
    this.controllerChanged.emit(this.controller);
  }
  handleModelChanged() {
    this.setupController();
    this.clearInteractionHandler();
    this.setupInteractionHandler();
  }
  render() {
    return h(Host, null);
  }
  setupController() {
    if (this.controller == null) {
      this.controller = new WalkModeController(this.model);
      this.controller.setTeleportMode(this.mode);
      this.controllerChanged.emit(this.controller);
    }
    else {
      this.controller.setTeleportMode(this.mode);
      this.controller.updateModel(this.model);
    }
  }
  setDepthBuffers() {
    if (this.mode != null &&
      this.viewer != null &&
      this.viewer.depthBuffers == null) {
      this.stateMap.shouldClearDepthBuffers = true;
      this.viewer.depthBuffers = 'final';
    }
  }
  resetDepthBuffers() {
    if (this.stateMap.shouldClearDepthBuffers && this.viewer != null) {
      this.viewer.depthBuffers = undefined;
      this.stateMap.shouldClearDepthBuffers = undefined;
    }
  }
  clearInteractionHandler() {
    var _a, _b;
    (_a = this.interactionHandlerDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    this.interactionHandlerDisposable = undefined;
    (_b = this.interactionHandler) === null || _b === void 0 ? void 0 : _b.dispose();
    this.interactionHandler = undefined;
  }
  async setupInteractionHandler() {
    var _a;
    if (this.interactionHandler == null) {
      this.interactionHandler = new TeleportInteractionHandler(this.model, !this.animationsDisabled && this.animationMs != null
        ? { durationMs: this.animationMs }
        : undefined);
      this.interactionHandlerDisposable =
        await ((_a = this.viewer) === null || _a === void 0 ? void 0 : _a.registerInteractionHandler(this.interactionHandler));
    }
  }
  static get watchers() { return {
    "mode": ["handleModeChange"],
    "viewer": ["handleViewerChanged"],
    "animationMs": ["handleAnimationMsChanged"],
    "animationsDisabled": ["handleAnimationsDisabledChanged"],
    "controller": ["handleControllerChanged"],
    "model": ["handleModelChanged"]
  }; }
  static get style() { return viewerTeleportToolCss; }
}, [1, "vertex-viewer-teleport-tool", {
    "viewer": [16],
    "mode": [1025],
    "animationsDisabled": [4, "animations-disabled"],
    "animationMs": [2, "animation-ms"],
    "controller": [1040],
    "model": [1040]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["vertex-viewer-teleport-tool"];
  components.forEach(tagName => { switch (tagName) {
    case "vertex-viewer-teleport-tool":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, ViewerTeleportTool);
      }
      break;
  } });
}
defineCustomElement();

export { ViewerTeleportTool as V, WalkModeModel as W, ViewerWalkModeOperation as a, WalkModeController as b, defineCustomElement as d };

//# sourceMappingURL=viewer-teleport-tool.js.map