/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
export { C as Camera, c as ColorMaterial, a as CrossSectioner, I as ItemOperationBuilder, l as LoadableResource, M as ModelViewController, P as PmiAnnotationOperationsBuilder, b as PmiAnnotationsQueryExecutor, k as PmiController, R as Raycaster, d as RootQuery, S as Scene, e as SceneElementQueryExecutor, f as SceneElementsOperationExecutor, g as SceneItemOperationsBuilder, h as SceneItemQueryExecutor, i as SceneItemsOperationExecutor, s as SceneViewStateIdentifier, j as SynchronizedClock } from './queries-57b3b092.js';
export { S as SceneTreeController } from './controller-e5d088c8.js';
export { f as fromNodeProto, i as isLoadedRow } from './row-4db87b64.js';
export { S as SCENE_ITEM_NAME_METADATA_KEY } from './types-218ef28f.js';
export { C as CursorManager, b as boxQueryCursor, l as labelPinCursor, m as measurementCursor, a as measurementWithArrowCursor, p as pinCursor } from './cursors-6019bcb8.js';
export { M as MeasurementController, a as MeasurementEntity } from './entities-accf8fe7.js';
export { M as MeasurementModel } from './model-a810a7fb.js';
export { M as MeasurementOverlayManager } from './overlays-bbc3c7cf.js';
export { m as makeMinimumDistanceResult } from './results-2685e21c.js';
import { v as vector3, q as quaternion, m as matrix4 } from './bundle.esm-0508e4f1.js';
export { T as TransformController } from './controller-63835173.js';
export { A as AngleUnits, g as AreaUnits, c as CrossSectioning, D as DistanceUnits, F as FrameCameraBase, O as Orientation, a as ReceivedFrame, b as ReceivedFrameImage, d as ReceivedFrameScene, e as ReceivedOrthographicCamera, f as ReceivedPerspectiveCamera, S as StandardView } from './streamAttributes-a4b7f379.js';
export { E as EntityType } from './entities-f59f520e.js';
export { A as ArrowMarkup, C as CircleMarkup, F as FreeformMarkup } from './markup-66ddf24a.js';
export { V as Viewport } from './viewport-10c07b6f.js';
export { V as VolumeIntersectionQueryController, a as VolumeIntersectionQueryModel } from './model-77e8cd08.js';
import './browser.esm-956a68fc.js';
import './errors-3444cd2f.js';
import './_commonjsHelpers-6e425ac3.js';
import './wrappers_pb-8ca57f4b.js';
import './scene_view_api_pb-acffb910.js';
import './grpc-40f0c608.js';
import './mapper-ae398c2d.js';
import './bundle.esm-4b7e63bd.js';
import './mapper-1db0f247.js';

const ALMOST_ONE = 0.9999;
/**
 * For any single normal vector, there are an infinite number of potential orthogonal vectors. This function will
 * calculate a single vector evaluating the length of two candidates, and picking the one with the higher squared magnitude
 * @param normal
 * @returns
 */
function calculateOrthogonalCoordinate(norm) {
  const orthogonalCandidate0 = vector3.create(0, norm.z, -norm.y);
  const orthogonalCandidate1 = vector3.create(-norm.z, 0, norm.x);
  const theChosenOne = vector3.magnitudeSquared(orthogonalCandidate0) >
    vector3.magnitudeSquared(orthogonalCandidate1)
    ? orthogonalCandidate0
    : orthogonalCandidate1;
  return vector3.normalize(vector3.cross(norm, theChosenOne));
}
/**
 * Computes the rotation matrix for two normals. If both normals are neither parallel or anti-parallel,
 * this will compute the rotation matrix delta based on the angle from both normals.
 * If the normals are anti-parallel, the identity matrix will be returned, as no rotation is necessary in this case.
 * If the normals are parallel, an axis direction based on a chosen orthogonal vector will be used
 * to compute the rotation matrix to rotate the plane 180 degrees.
 * @param normal1
 * @param normal2
 * @returns an anti-parallel rotation Matrix4 betwen the given normals
 */
function computeRotationMatrix(normal1, normal2) {
  const dot = vector3.dot(normal1, normal2);
  // the angle is almost 0 in this case.
  if (dot > ALMOST_ONE) {
    const axisDirection = calculateOrthogonalCoordinate(normal1);
    const quaternion$1 = quaternion.fromAxisAngle(axisDirection, Math.PI);
    return matrix4.makeRotation(quaternion$1);
  }
  // the angle is almost 180 in this case.
  else if (dot <= -ALMOST_ONE) {
    return matrix4.makeIdentity();
  }
  // the angle is between 0 & 180
  else {
    const angle = vector3.angleTo(normal2, normal1);
    const axisDirection = vector3.normalize(vector3.cross(normal1, normal2));
    return matrix4.makeRotation(quaternion.fromAxisAngle(axisDirection, angle + Math.PI));
  }
}
/**
 * Computes the translation & rotation matrix delta between two world positions and two normals.
 * such that the computed translation matrix will be the delta between position 1 and position 2,
 * and the rotation will be rotated to be anti-parallel.
 *
 * @param normal1
 * @param position1
 * @param normal2
 * @param position2
 *
 * @returns Matrix4 translation matrix delta from position1 to
 * position2 & an anti-parallel rotation delta.
 */
function computeTransformationDelta(normal1, position1, normal2, position2) {
  const rotationMatrix = computeRotationMatrix(normal1, normal2);
  const translationDeltaMatrix = matrix4.makeTranslation(position2);
  return matrix4.multiply(matrix4.multiply(translationDeltaMatrix, rotationMatrix), matrix4.makeTranslation(vector3.negate(position1)));
}

const transformationDelta = /*#__PURE__*/Object.freeze({
  __proto__: null,
  ALMOST_ONE: ALMOST_ONE,
  calculateOrthogonalCoordinate: calculateOrthogonalCoordinate,
  computeRotationMatrix: computeRotationMatrix,
  computeTransformationDelta: computeTransformationDelta
});

export { transformationDelta as TransformationDelta };

//# sourceMappingURL=index.js.map