/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { c as color, u as uri, a as uuid, b as async } from './browser.esm-956a68fc.js';
import { I as InvalidResourceUrnError, a as InvalidArgumentError, b as InvalidCameraError } from './errors-3444cd2f.js';
import { s as scene_view_api_pb, u as uuid_pb, p as paging_pb, r as representation_pb } from './scene_view_api_pb-acffb910.js';
import { w as wrappers_pb } from './wrappers_pb-8ca57f4b.js';
import { r as requestUnary, c as createMetadata } from './grpc-40f0c608.js';
import { m as mapListItemModelViewsResponseOrThrow, a as mapItemModelViewOrThrow, b as mapListPmiAnnotationsResponseOrThrow } from './mapper-ae398c2d.js';
import { b as boundingBox, v as vector3, m as matrix4, q as quaternion, r as ray, a as boundingSphere, p as point, d as dimensions } from './bundle.esm-0508e4f1.js';
import { t as toProtobuf, v as vertexvis, L as Long, w as withPositionAndViewVector, h as fromBoundingBoxAndLookAtCamera, f as FramePerspectiveCamera, i as constrainViewVector, u as updateLookAtRelativeToBoundingBoxCenter, e as FrameOrthographicCamera } from './streamAttributes-a4b7f379.js';
import './entities-f59f520e.js';
import { t as toProtoDuration } from './bundle.esm-4b7e63bd.js';

const defaultColor = {
  opacity: 255,
  glossiness: 4,
  diffuse: {
    r: 0,
    g: 0,
    b: 0,
    a: 0,
  },
  ambient: {
    r: 0,
    g: 0,
    b: 0,
    a: 0,
  },
  specular: {
    r: 0,
    g: 0,
    b: 0,
    a: 0,
  },
  emissive: {
    r: 0,
    g: 0,
    b: 0,
    a: 0,
  },
};
/**
 * This helper creates an rgb value color
 * @param hex
 * @param opacity
 */
const create = (r, g, b, opacity) => {
  return Object.assign(Object.assign({}, defaultColor), { opacity: opacity !== null && opacity !== void 0 ? opacity : defaultColor.opacity, diffuse: {
      r,
      g,
      b,
      a: 0,
    } });
};
/**
 * This helper converts a hex string to a ColorMaterial object
 * @param hex
 * @param opacity
 */
const fromHex = (hex, opacity) => {
  const color$1 = color.fromHexString(hex);
  return Object.assign(Object.assign({}, defaultColor), { opacity: opacity !== null && opacity !== void 0 ? opacity : defaultColor.opacity, diffuse: color$1 != null ? Object.assign({}, color$1) : Object.assign({}, defaultColor.diffuse) });
};

const colorMaterial = /*#__PURE__*/Object.freeze({
  __proto__: null,
  defaultColor: defaultColor,
  create: create,
  fromHex: fromHex
});

function fromUrn(urn) {
  var _a;
  const uri$1 = uri.parse(urn);
  if (uri$1.scheme !== 'urn' || uri$1.path == null) {
    throw new Error('Invalid URN. Expected URN scheme.');
  }
  const [nid, resourceType, resourceId, ...subResourcePath] = uri$1.path.split(/[:/]/);
  if (nid !== 'vertexvis' && nid !== 'vertex') {
    throw new Error('Invalid URN. Expected URN to be vertex namespace');
  }
  if (nid === 'vertexvis') {
    console.warn("vertexvis namespace is deprecated. Use 'vertex' for the namespace urn instead");
  }
  switch (resourceType) {
    case 'stream-key':
      const queries = fromQuery(uri$1.query);
      const subResource = (_a = fromSubResourcePath(subResourcePath.join('/'))) !== null && _a !== void 0 ? _a : queries.find((q) => q.type === 'scene-view-state');
      return {
        resource: { type: 'stream-key', id: resourceId },
        subResource,
        queries: queries.filter((q) => q.type !== 'scene-view-state'),
      };
    default:
      throw new InvalidResourceUrnError(`Invalid URN. Unknown resource type ${resourceType}`);
  }
}
function fromSubResourcePath(path) {
  if (path != null) {
    const [subResourceType, subResourceId] = path.split('/');
    switch (subResourceType) {
      case 'scene-view-states':
        return {
          type: 'scene-view-state',
          id: subResourceId,
        };
      default:
        return undefined;
    }
  }
}
function fromQuery(query) {
  if (query != null) {
    return query.split('&').map((queryFragment) => {
      const [resourceType, resourceId] = queryFragment.split('=');
      switch (resourceType) {
        case 'supplied-id':
          return { type: 'supplied-id', id: resourceId };
        case 'scene-view-state':
          return { type: 'scene-view-state', id: resourceId };
        default:
          throw new Error('Invalid URN. Unknown query value type');
      }
    });
  }
  else {
    return [];
  }
}

const loadableResource = /*#__PURE__*/Object.freeze({
  __proto__: null,
  fromUrn: fromUrn
});

function isSceneViewStateId(identifier) {
  return identifier.id != null;
}
function isSceneViewStateSuppliedId(identifier) {
  return identifier.suppliedId != null;
}

const sceneViewStateIdentifier = /*#__PURE__*/Object.freeze({
  __proto__: null,
  isSceneViewStateId: isSceneViewStateId,
  isSceneViewStateSuppliedId: isSceneViewStateSuppliedId
});

/**
 * A `SynchronizedClock` allows for approximating the time on a remote machine.
 *
 * During initialization, a known time of the remote host must be supplied. Once
 * the remote time is known, this class can approximate the time on the remote
 * machine from a date or duration.
 *
 * @example
 * ```
 * const knownRemoteTime = await getRemoteTime();
 * const clock = new SynchronizedClock(knownRemoteTime);
 *
 * const localDate = new Date();
 * const serverTime = clock.remoteTime(localDate);
 *
 * const remoteDate = await getRemoteTime();
 * const localTime = clock.localTime(remoteDate);
 *
 * const currentRemoteTime = clock.remoteNow();
 * ```
 */
class SynchronizedClock {
  constructor(knownRemoteTime, knownLocalTime = new Date()) {
    this.knownRemoteTime = knownRemoteTime;
    this.knownLocalTime = knownLocalTime;
  }
  /**
   * Computes an offset duration from when this clock was instantiated.
   */
  localOffset() {
    return this.duration(this.knownLocalTime, this.localNow());
  }
  /**
   * Approximates the local time from a remote timestamp or duration.
   *
   * @param remoteTimeOrDurationInMs A remote timestamp or duration.
   */
  localTime(remoteTimeOrDurationInMs) {
    const remoteTime = typeof remoteTimeOrDurationInMs === 'number'
      ? new Date(this.knownLocalTime.getTime() + remoteTimeOrDurationInMs)
      : remoteTimeOrDurationInMs;
    const duration = this.duration(this.knownRemoteTime, remoteTime);
    return this.addDuration(this.knownLocalTime, duration);
  }
  /**
   * Approximates an offset duration on the remote machine.
   *
   * @param localTime A local timestamp. Defaults to the current local time.
   */
  remoteOffset(localTime = new Date()) {
    return this.duration(this.remoteTime(this.knownLocalTime), this.remoteTime(localTime));
  }
  /**
   * Approximates the current time on the remote machine.
   */
  remoteNow() {
    return this.remoteTime(this.localNow());
  }
  /**
   * Approximates the remote time from either a local timestamp or duration.
   *
   * @param localTimeOrDurationInMs A local timestamp or duration.
   */
  remoteTime(localTimeOrDurationInMs) {
    const localTime = typeof localTimeOrDurationInMs === 'number'
      ? new Date(this.knownLocalTime.getTime() + localTimeOrDurationInMs)
      : localTimeOrDurationInMs;
    const duration = this.duration(this.knownLocalTime, localTime);
    return this.addDuration(this.knownRemoteTime, duration);
  }
  addDuration(start, duration) {
    return new Date(start.getTime() + duration);
  }
  duration(start, end) {
    return end.getTime() - start.getTime();
  }
  localNow() {
    return new Date(Date.now());
  }
}

/**
 * The controller for managing the model views of a scene view.
 */
class ModelViewController {
  constructor(client, stream, jwtProvider, deviceIdProvider) {
    this.client = client;
    this.stream = stream;
    this.jwtProvider = jwtProvider;
    this.deviceIdProvider = deviceIdProvider;
  }
  /**
   * Fetches a page of model views for a given item. The result includes a
   * cursor that can be used to fetch the next page of results.
   *
   * @param itemId The ID of the scene item to retrieve model views for.
   * @param options Options to configure fetching of items.
   * @returns A page of model views.
   */
  async listByItem(itemId, { hasAnnotations, cursor, size = 50 } = {}) {
    const res = await requestUnary(async (handler) => {
      const deviceId = this.deviceIdProvider();
      const meta = await createMetadata(this.jwtProvider, deviceId);
      const req = new scene_view_api_pb.ListItemModelViewsRequest();
      const { msb, lsb } = uuid.toMsbLsb(itemId);
      const id = new uuid_pb.Uuid2l();
      id.setMsb(msb);
      id.setLsb(lsb);
      req.setItemId(id);
      if (hasAnnotations != null) {
        const hasAnnotationsVal = new wrappers_pb.BoolValue();
        hasAnnotationsVal.setValue(hasAnnotations);
        req.setHasAnnotations(hasAnnotationsVal);
      }
      const page = new paging_pb.Pager();
      page.setLimit(size);
      if (cursor != null) {
        page.setCursor(cursor);
      }
      req.setPage(page);
      this.client.listItemModelViews(req, meta, handler);
    });
    return mapListItemModelViewsResponseOrThrow(res.toObject());
  }
  /**
   * Loads the provided model view within the current scene view.
   *
   * @param sceneItemId The ID of the scene item this model view is being loaded for.
   * @param modelViewId The ID of the model view to load.
   */
  async load(sceneItemId, modelViewId) {
    const itemModelView = mapItemModelViewOrThrow({ modelViewId, sceneItemId });
    this.stream.updateModelView({ itemModelView }, true);
  }
  /**
   * Unloads any previously loaded model view within the current scene view,
   * then performs a `reset` on the scene to return to the initial state for
   * the scene view.
   */
  async unload() {
    this.stream.updateModelView({}, true);
  }
}

class PmiController {
  constructor(client, jwtProvider, deviceIdProvider) {
    this.client = client;
    this.jwtProvider = jwtProvider;
    this.deviceIdProvider = deviceIdProvider;
  }
  async listAnnotations({ modelViewId, cursor, size = 50, } = {}) {
    const res = await requestUnary(async (handler) => {
      const deviceId = this.deviceIdProvider();
      const meta = await createMetadata(this.jwtProvider, deviceId);
      const req = new scene_view_api_pb.ListPmiAnnotationsRequest();
      if (modelViewId != null) {
        const { msb, lsb } = uuid.toMsbLsb(modelViewId);
        const id = new uuid_pb.Uuid2l();
        id.setMsb(msb);
        id.setLsb(lsb);
        req.setModelViewId(id);
      }
      const page = new paging_pb.Pager();
      page.setLimit(size);
      if (cursor != null) {
        page.setCursor(cursor);
      }
      req.setPage(page);
      this.client.listPmiAnnotations(req, meta, handler);
    });
    return mapListPmiAnnotationsResponseOrThrow(res.toObject());
  }
}

const DEFAULT_TIMEOUT_IN_MS = 1000 * 10;
class StreamApiEventDispatcher {
  constructor(stream, predicate, transform, timeout = DEFAULT_TIMEOUT_IN_MS) {
    this.stream = stream;
    this.predicate = predicate;
    this.transform = transform;
    this.timeout = timeout;
    this.listeners = [];
    this.handleMessage = this.handleMessage.bind(this);
  }
  on(listener) {
    this.listeners = [...this.listeners, listener];
    if (this.eventDisposable == null || this.requestDisposable == null) {
      this.addListeners();
    }
  }
  off(listener) {
    this.listeners = this.listeners.filter((l) => l !== listener);
    if (this.listeners.length === 0) {
      this.removeListeners();
    }
  }
  once() {
    let handler;
    return async.timeout(this.timeout, new Promise((resolve) => {
      handler = (data) => {
        resolve(data);
        this.off(handler);
      };
      this.on(handler);
    })).finally(() => {
      this.off(handler);
    });
  }
  handleMessage(msg) {
    if (this.predicate(msg)) {
      const transformed = this.transform(msg);
      if (transformed != null) {
        this.listeners.forEach((l) => l(transformed));
      }
    }
  }
  addListeners() {
    this.eventDisposable = this.stream.onEvent(this.handleMessage);
    this.requestDisposable = this.stream.onRequest(this.handleMessage);
  }
  removeListeners() {
    var _a, _b;
    (_a = this.eventDisposable) === null || _a === void 0 ? void 0 : _a.dispose();
    (_b = this.requestDisposable) === null || _b === void 0 ? void 0 : _b.dispose();
  }
}

class CameraRenderResult {
  constructor(stream, decodeFrame, { animationId, correlationId }, timeout) {
    this.data = undefined;
    this.onAnimationCompleted = new StreamApiEventDispatcher(stream, (msg) => { var _a, _b, _c; return ((_c = (_b = (_a = msg.event) === null || _a === void 0 ? void 0 : _a.animationCompleted) === null || _b === void 0 ? void 0 : _b.animationId) === null || _c === void 0 ? void 0 : _c.hex) === animationId; }, (msg) => { var _a, _b, _c; return ((_c = (_b = (_a = msg.event) === null || _a === void 0 ? void 0 : _a.animationCompleted) === null || _b === void 0 ? void 0 : _b.animationId) === null || _c === void 0 ? void 0 : _c.hex) || undefined; }, timeout);
    this.onFrameReceived = new StreamApiEventDispatcher(stream, (msg) => {
      var _a, _b, _c;
      return !!((_c = (_b = (_a = msg.request) === null || _a === void 0 ? void 0 : _a.drawFrame) === null || _b === void 0 ? void 0 : _b.frameCorrelationIds) === null || _c === void 0 ? void 0 : _c.some((id) => id === correlationId));
    }, (msg) => {
      var _a;
      return ((_a = msg.request) === null || _a === void 0 ? void 0 : _a.drawFrame) != null
        ? decodeFrame(msg.request.drawFrame)
        : undefined;
    }, timeout);
  }
}

function buildSceneViewStateIdentifier(identifier) {
  if (typeof identifier === 'string') {
    return { sceneViewStateId: { hex: identifier } };
  }
  else if (isSceneViewStateId(identifier)) {
    return { sceneViewStateId: { hex: identifier.id } };
  }
  else if (isSceneViewStateSuppliedId(identifier)) {
    return { sceneViewStateSuppliedId: { value: identifier.suppliedId } };
  }
  else {
    throw new Error('Unable to build scene view state identifier, input must be a string or `SceneViewStateIdentifier`.');
  }
}
function buildQueryExpression(query, context) {
  switch (query.type) {
    case 'and':
    case 'or':
      return {
        operand: {
          itemCollection: {
            queries: query.expressions.map((exp) => ({
              sceneItemQuery: buildSceneItemQuery(exp),
            })),
          },
        },
      };
    case 'not':
      return {
        not: {
          expression: buildQueryExpression(query.query, context),
        },
      };
    case 'item-id':
    case 'supplied-id':
      return {
        operand: {
          item: {
            sceneItemQuery: buildSceneItemQuery(query),
          },
        },
      };
    case 'all':
      return {
        operand: {
          root: {},
        },
      };
    case 'scene-tree-range':
      return {
        operand: {
          sceneTreeRange: {
            start: query.range.start,
            end: query.range.end,
          },
        },
      };
    case 'metadata':
      if (query.removeHiddenItems) {
        return {
          and: {
            first: {
              operand: {
                metadata: {
                  valueFilter: query.filter,
                  keys: query.keys,
                  exactMatch: query.exactMatch,
                },
              },
            },
            second: {
              operand: {
                override: {
                  visibility: {
                    visibilityState: true,
                  },
                },
              },
            },
          },
        };
      }
      else {
        return {
          operand: {
            metadata: {
              valueFilter: query.filter,
              keys: query.keys,
              exactMatch: query.exactMatch,
            },
          },
        };
      }
    case 'all-selected':
      return {
        operand: {
          override: {
            selection: {},
          },
        },
      };
    case 'all-visible':
      return {
        operand: {
          override: {
            visibility: {
              visibilityState: true,
            },
          },
        },
      };
    case 'point':
      return {
        operand: {
          point: {
            point: query.point,
            viewport: context.dimensions,
          },
        },
      };
    case 'volume-intersection':
      return {
        operand: {
          volume: {
            frustumByRectangle: {
              rectangle: query.rectangle,
            },
            exclusive: query.exclusive,
            viewport: context.dimensions,
          },
        },
      };
    default:
      return {};
  }
}
function buildAnnotationQueryExpression(query, context) {
  switch (query.type) {
    case 'and':
      const numberOfExpressionsAnd = query.expressions.length;
      // Exactly one or two expressions should have been provided in the query.
      // If only one expression is given, query only on that expression.
      if (numberOfExpressionsAnd === 1) {
        return buildAnnotationQueryExpression({
          type: 'annotation-id',
          value: query.expressions[0].value,
        });
      }
      else if (numberOfExpressionsAnd !== 2) {
        throw new Error('Incorrect number of query expressions provided.');
      }
      return {
        and: {
          first: buildAnnotationQueryExpression(query.expressions[0]),
          second: buildAnnotationQueryExpression(query.expressions[1]),
        },
      };
    case 'or':
      const numberOfExpressionsOr = query.expressions.length;
      // Exactly one or two expressions should have been provided in the query.
      // If only one expression is given, query only on that expression.
      if (numberOfExpressionsOr === 1) {
        return buildAnnotationQueryExpression({
          type: 'annotation-id',
          value: query.expressions[0].value,
        });
      }
      else if (numberOfExpressionsOr !== 2) {
        throw new Error('Incorrect number of query expressions provided.');
      }
      return {
        or: {
          first: buildAnnotationQueryExpression(query.expressions[0]),
          second: buildAnnotationQueryExpression(query.expressions[1]),
        },
      };
    case 'not':
      return {
        not: {
          expression: buildAnnotationQueryExpression(query.query),
        },
      };
    case 'annotation-id':
      const { msb, lsb } = uuid.toMsbLsb(query.value);
      return {
        operand: {
          annotation: {
            id: new vertexvis.protobuf.core.Uuid2l({
              msb: Long.fromString(msb),
              lsb: Long.fromString(lsb),
            }),
          },
        },
      };
    case 'all':
      return {
        operand: {
          all: {},
        },
      };
    default:
      return {};
  }
}
function buildSceneElementOperationOnItem(query, operations, context) {
  const operationTypes = buildOperationTypes(operations);
  const queryExpression = buildQueryExpression(query, context);
  return { sceneItemOperation: { queryExpression, operationTypes } };
}
function buildSceneElementOperationOnAnnotation(query, operations, context) {
  const operationTypes = buildAnnotationOperationTypes(operations);
  const queryExpression = buildAnnotationQueryExpression(query);
  return { pmiAnnotationOperation: { queryExpression, operationTypes } };
}
function buildSceneItemQuery(item) {
  switch (item.type) {
    case 'item-id':
      return {
        id: new vertexvis.protobuf.core.Uuid({
          hex: item.value,
        }),
      };
    case 'supplied-id':
      return {
        suppliedId: item.value,
      };
    default:
      return {};
  }
}
function buildFlyToOperation(frameCorrelationId, options, animation, baseCamera) {
  const payload = {
    frameCorrelationId: {
      value: frameCorrelationId,
    },
    animation: animation
      ? {
        duration: toProtoDuration(animation.milliseconds),
      }
      : undefined,
    baseCamera: baseCamera != null ? toProtobuf(baseCamera) : undefined,
  };
  switch (options.flyTo.type) {
    case 'supplied': {
      return Object.assign(Object.assign({}, payload), { itemSuppliedId: options.flyTo.data });
    }
    case 'internal': {
      return Object.assign(Object.assign({}, payload), { itemId: new vertexvis.protobuf.core.Uuid({
          hex: options.flyTo.data,
        }) });
    }
    case 'camera': {
      return Object.assign(Object.assign({}, payload), { camera: toProtobuf(options.flyTo.data) });
    }
    case 'bounding-box': {
      return Object.assign(Object.assign({}, payload), { boundingBox: {
          xmin: options.flyTo.data.min.x,
          xmax: options.flyTo.data.max.x,
          ymin: options.flyTo.data.min.y,
          ymax: options.flyTo.data.max.y,
          zmin: options.flyTo.data.min.z,
          zmax: options.flyTo.data.max.z,
        } });
    }
    default:
      return {};
  }
}
function buildOperationTypes(operations) {
  return operations.map((op) => {
    var _a, _b;
    switch (op.type) {
      case 'change-material':
        return {
          changeMaterial: {
            materialOverride: {
              colorMaterial: {
                d: op.material.opacity,
                ns: op.material.glossiness,
                ka: op.material.ambient,
                kd: op.material.diffuse,
                ks: op.material.specular,
                ke: op.material.emissive,
              },
            },
          },
        };
      case 'clear-override':
        return { clearMaterial: {} };
      case 'change-transform':
        return { changeTransform: { transform: Object.assign({}, op.transform) } };
      case 'clear-transform':
        return { clearTransform: { cascade: op.cascade } };
      case 'hide':
        return { changeVisibility: { visible: false } };
      case 'show':
        return { changeVisibility: { visible: true } };
      case 'select':
        return { changeSelection: { selected: true } };
      case 'deselect':
        return { changeSelection: { selected: false } };
      case 'change-phantom':
        return { changePhantom: { phantom: (_a = op.phantomState) !== null && _a !== void 0 ? _a : true } };
      case 'clear-phantom':
        return { changePhantom: { phantom: false } };
      case 'change-end-item':
        return { changeEndItem: { endItem: (_b = op.endItemState) !== null && _b !== void 0 ? _b : true } };
      case 'clear-end-item':
        return { changeEndItem: { endItem: false } };
      case 'view-rendition-by-id':
        return {
          viewRendition: {
            id: new vertexvis.protobuf.core.Uuid({ hex: op.id }),
          },
        };
      case 'view-rendition-by-supplied-id':
        return { viewRendition: { suppliedId: op.suppliedId } };
      case 'view-default-rendition':
        return { viewDefaultRendition: {} };
      case 'clear-rendition':
        return { clearRendition: {} };
      case 'view-representation':
        if (op.id === 'empty') {
          return {
            viewRepresentation: {
              predefinedId: representation_pb.RepresentationPredefinedId.REPRESENTATION_PREDEFINED_ID_EMPTY,
            },
          };
        }
        else if (op.id === 'entire-part') {
          return {
            viewRepresentation: {
              predefinedId: representation_pb.RepresentationPredefinedId.REPRESENTATION_PREDEFINED_ID_ENTIRE_PART,
            },
          };
        }
        else {
          return {
            viewRepresentation: {
              id: new vertexvis.protobuf.core.Uuid({ hex: op.id }),
            },
          };
        }
      case 'clear-representation':
        return { clearRepresentation: {} };
      default:
        return {};
    }
  });
}
function buildAnnotationOperationTypes(operations) {
  return operations.map((op) => {
    switch (op.type) {
      case 'show':
        return { changeVisibility: { visible: true } };
      case 'hide':
        return { changeVisibility: { visible: false } };
      case 'select':
        return { changeSelection: { selected: true } };
      case 'deselect':
        return { changeSelection: { selected: false } };
      default:
        return {};
    }
  });
}
function toPbSceneViewStateFeatures(features) {
  return features.map((feature) => {
    switch (feature) {
      case 'camera':
        return vertexvis.protobuf.stream.SceneViewStateFeature
          .SCENE_VIEW_STATE_FEATURE_CAMERA;
      case 'material_overrides':
        return vertexvis.protobuf.stream.SceneViewStateFeature
          .SCENE_VIEW_STATE_FEATURE_MATERIAL_OVERRIDE;
      case 'selection':
        return vertexvis.protobuf.stream.SceneViewStateFeature
          .SCENE_VIEW_STATE_FEATURE_SELECTION;
      case 'visibility':
        return vertexvis.protobuf.stream.SceneViewStateFeature
          .SCENE_VIEW_STATE_FEATURE_VISIBILITY;
      case 'transforms':
        return vertexvis.protobuf.stream.SceneViewStateFeature
          .SCENE_VIEW_STATE_FEATURE_TRANSFORM;
      case 'cross_section':
        return vertexvis.protobuf.stream.SceneViewStateFeature
          .SCENE_VIEW_STATE_FEATURE_CROSS_SECTION;
      case 'phantom':
        return vertexvis.protobuf.stream.SceneViewStateFeature
          .SCENE_VIEW_STATE_FEATURE_PHANTOM;
      case 'shading':
        return vertexvis.protobuf.stream.SceneViewStateFeature
          .SCENE_VIEW_STATE_FEATURE_SHADING;
      default:
        return vertexvis.protobuf.stream.SceneViewStateFeature
          .SCENE_VIEW_STATE_FEATURE_INVALID;
    }
  });
}

class TerminalFlyToExecutor {
  constructor(flyToOptions) {
    this.flyToOptions = flyToOptions;
  }
  build() {
    return this.flyToOptions;
  }
}
class FlyToExecutor {
  withItemId(id) {
    return new TerminalFlyToExecutor({
      flyTo: {
        type: 'internal',
        data: id,
      },
    });
  }
  withSuppliedId(id) {
    return new TerminalFlyToExecutor({
      flyTo: {
        type: 'supplied',
        data: id,
      },
    });
  }
  withCamera(camera) {
    return new TerminalFlyToExecutor({
      flyTo: {
        type: 'camera',
        data: camera,
      },
    });
  }
  withBoundingBox(boundingBox) {
    return new TerminalFlyToExecutor({
      flyTo: {
        type: 'bounding-box',
        data: boundingBox,
      },
    });
  }
  build() {
    return this.flyToOptions;
  }
}
/**
 * The `Camera` class contains properties that reflect a world space position, a
 * view direction (lookAt), and normalized vector representing the up direction.
 *
 * It also provides utility methods to update orientation of the camera and
 * rerender the scene.
 *
 * This class in intended to be treated as an immutable type. Any mutations return
 * a new instance of the class with the updated properties.
 */
class Camera {
  constructor(stream, aspect, data, boundingBox, decodeFrame, flyToOptions) {
    this.stream = stream;
    this.aspect = aspect;
    this.data = data;
    this.boundingBox = boundingBox;
    this.decodeFrame = decodeFrame;
    this.flyToOptions = flyToOptions;
  }
  /**
   * Updates the position of the camera such that the given bounding box will
   * be contained within the camera's view.
   *
   * @param boundingBox The bounding box to position to.
   */
  fitToBoundingBox(boundingBox) {
    return this.flyTo({
      boundingBox,
    });
  }
  /**
   * Returns the distance from the camera's position to the center
   * of the provided bounding box (or the scene's visible bounding box if not provided).
   *
   * @param boundingBox - The bounding box to determine distance from.
   */
  signedDistanceToBoundingBoxCenter(boundingBox$1) {
    const { position, viewVector } = withPositionAndViewVector(this.data);
    const boundingBoxCenter = boundingBox.center(boundingBox$1 !== null && boundingBox$1 !== void 0 ? boundingBox$1 : this.boundingBox);
    const cameraToCenter = vector3.subtract(boundingBoxCenter, position);
    return (vector3.dot(viewVector, cameraToCenter) / vector3.magnitude(viewVector));
  }
  /**
   * Specifies that the next render of the camera will be repositioned to one of
   * the options specified in `options`.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector("vertex-viewer");
   * const scene = await viewer.scene();
   * const camera = scene.camera();
   *
   * // Fly to and fit to a specific item by ID with an animation of 1 second
   * await camera
   *   .flyTo({ itemId: "item-id" })
   *   .render({ animation: { milliseconds: 1000 } });
   *
   * // Fly to and fit to a specific item by supplied ID with an animation of 1 second
   * await camera
   *   .flyTo({ itemSuppliedId: "item-supplied-id" })
   *   .render({ animation: { milliseconds: 1000 } });
   *
   * // Fly to and fit to the bounding box of the current set of selected items
   * // with an animation of 1 second
   * await camera
   *   .flyTo({
   *     boundingBox:
   *       viewer.frame.scene.sceneViewSummary.selectedVisibleSummary.boundingBox,
   *   })
   *   .render({ animation: { milliseconds: 1000 } });
   *
   * // Fly to a specific camera
   * await camera
   *   .flyTo({
   *     camera: {
   *       position: {
   *         x: 1,
   *         y: 2,
   *         z: 3,
   *       },
   *       lookAt: {
   *         x: 0,
   *         y: 0,
   *         z: 0,
   *       },
   *       up: {
   *         x: 0,
   *         y: 1,
   *         z: 0,
   *       },
   *     },
   *   })
   *   .render({ animation: { milliseconds: 1000 } });
   * ```
   *
   * @param paramsOrQuery An object or query describing how the camera should
   * be positioned.
   */
  flyTo(paramsOrQuery) {
    if (typeof paramsOrQuery !== 'function') {
      return this.updateFlyToOptions({
        flyTo: this.buildFlyToType(paramsOrQuery),
      });
    }
    else {
      return this.updateFlyToOptions(paramsOrQuery(new FlyToExecutor()).build());
    }
  }
  /**
   * Performs a `flyTo` operation with the current visible bounding box of
   * the scene.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   * const camera = scene.camera();
   *
   * // Fit to the visible bounding box of the scene with an animation of 1 second
   * await camera.viewAll().render({ animation: { milliseconds: 1000 } });
   * ```
   */
  viewAll() {
    return this.fitToBoundingBox(this.boundingBox);
  }
  /**
   * Queues the rendering for a new frame using this camera. The returned
   * promise will resolve when a frame is received that contains this camera.
   */
  async render(renderOptions) {
    var _a, _b, _c;
    if (this.flyToOptions == null && renderOptions != null) {
      this.flyToOptions = {
        flyTo: {
          type: 'camera',
          data: this.data,
        },
      };
    }
    try {
      const corrId = uuid.create();
      if (this.flyToOptions != null) {
        const payload = buildFlyToOperation(corrId, this.flyToOptions, renderOptions === null || renderOptions === void 0 ? void 0 : renderOptions.animation, this.toFrameCamera());
        const flyToResponse = await this.stream.flyTo(payload, true);
        return new CameraRenderResult(this.stream, this.decodeFrame, {
          correlationId: corrId,
          animationId: ((_b = (_a = flyToResponse.flyTo) === null || _a === void 0 ? void 0 : _a.animationId) === null || _b === void 0 ? void 0 : _b.hex) || undefined,
        }, ((_c = renderOptions === null || renderOptions === void 0 ? void 0 : renderOptions.animation) === null || _c === void 0 ? void 0 : _c.milliseconds) != null
          ? renderOptions.animation.milliseconds + DEFAULT_TIMEOUT_IN_MS
          : undefined);
      }
      else {
        this.stream.replaceCamera({
          camera: toProtobuf(this.data),
          frameCorrelationId: { value: corrId },
        });
        return new CameraRenderResult(this.stream, this.decodeFrame, {
          correlationId: corrId,
        });
      }
    }
    catch (e) {
      console.warn('Error when performing render: ', e);
      throw e;
    }
  }
  /**
   * Repositions the camera by rotating its current position around an axis placed
   * at the `lookAt` point. This method internally will call {@link Camera.rotateAroundAxisAtPoint}
   * with the `point` parameter set to the current `lookAt` point.
   *
   * @see {@link Camera.rotateAroundAxisAtPoint} for more information.
   *
   * @param angleInRadians The angle, in radians, to rotate.
   * @param axis A normalized vector to rotate around.
   */
  rotateAroundAxis(angleInRadians, axis) {
    return this.rotateAroundAxisAtPoint(angleInRadians, this.data.lookAt, axis);
  }
  /**
   * Aligns the camera to the plane defined by the provided position and normal.
   * This will place the camera at the provided position, set the up vector to
   * the provided normal, and place the lookAt on the defined plane. The point
   * chosen for the lookAt will be determined using the current view vector.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector("vertex-viewer");
   *
   * viewer.addEventListener("tap", async (event) => {
   *   const scene = await viewer.scene();
   *   const raycaster = scene.raycaster();
   *
   *   const [hit] = await raycaster.hitItems(event.detail.position);
   *
   *   if (hit != null) {
   *     const camera = scene.camera();
   *
   *     // Align to the plane represented by the hit result with an animation of 1 second
   *     await camera
   *       .alignTo(hit.hitPoint, hit.hitNormal)
   *       .render({ animation: { milliseconds: 1000 } });
   *   }
   * });
   * ```
   *
   * @param position The position to place the camera at.
   * @param normal The normal of the plane to align to.
   */
  alignTo(position, normal) {
    const worldX = vector3.normalize(vector3.cross(this.up, vector3.normalize(this.viewVector)));
    const positiveWorldY = vector3.normalize(vector3.cross(vector3.normalize(this.viewVector), worldX));
    // Invert the world y axis if the provided normal is more than 90 degrees from it
    // to compute a proper angle to rotate the view vector by
    const worldY = vector3.angleTo(normal, positiveWorldY) > Math.PI / 2
      ? vector3.negate(positiveWorldY)
      : positiveWorldY;
    const localX = vector3.isEqual(worldY, normal)
      ? worldX
      : vector3.normalize(vector3.cross(worldY, normal));
    const transformedViewVector = vector3.transformMatrix(this.viewVector, matrix4.makeRotation(quaternion.fromAxisAngle(localX, vector3.angleTo(normal, worldY))));
    const lookAtRay = ray.create({
      origin: position,
      direction: vector3.normalize(transformedViewVector),
    });
    const lookAt = ray.at(lookAtRay, vector3.magnitude(this.viewVector));
    return this.update({
      position,
      lookAt,
      up: normal,
    });
  }
  /**
   * Updates the `position` and `up` vectors of the camera to the given standard
   * view, and sets the `lookAt` point to the origin.
   *
   * @see {@link StandardView} for the available standard views.
   *
   * @param standardView The standard view to apply.
   * @returns A new camera.
   */
  standardView(standardView) {
    return this.update({
      position: standardView.position,
      viewVector: vector3.subtract(vector3.origin(), standardView.position),
      lookAt: vector3.origin(),
      up: standardView.up,
    });
  }
  /**
   * Updates the `position` and `up` vectors of the camera to the given standard
   * view, maintaining the existing `lookAt` point.
   *
   * @see {@link StandardView} for the available standard views.
   *
   * @param standardView The standard view to apply.
   * @returns A new camera.
   */
  standardViewFixedLookAt(standardView) {
    const standardViewRay = ray.create({
      origin: this.lookAt,
      direction: vector3.normalize(standardView.position),
    });
    const updatedPosition = ray.at(standardViewRay, vector3.magnitude(this.viewVector));
    return this.update({
      up: standardView.up,
      position: updatedPosition,
      viewVector: vector3.subtract(this.lookAt, updatedPosition),
    });
  }
  buildFlyToType(options) {
    if (options.boundingBox != null) {
      return { type: 'bounding-box', data: options.boundingBox };
    }
    else if (options.camera != null) {
      return { type: 'camera', data: options.camera };
    }
    else if (options.itemId != null) {
      return { type: 'internal', data: options.itemId };
    }
    else if (options.itemSuppliedId != null) {
      return { type: 'supplied', data: options.itemSuppliedId };
    }
    else {
      throw new Error('Fly to must specify at least one option.');
    }
  }
  computeClippingPlanes(camera) {
    return fromBoundingBoxAndLookAtCamera(this.boundingBox, camera);
  }
}
class PerspectiveCamera extends Camera {
  constructor(stream, aspect, perspectiveData, boundingBox, decodeFrame, flyToOptions) {
    super(stream, aspect, perspectiveData, boundingBox, decodeFrame, flyToOptions);
    this.perspectiveData = perspectiveData;
  }
  /**
   * Shifts the position of the camera by the given delta.
   *
   * @param delta The number of units to shift the camera on the X, Y, and Z
   * axis.
   */
  moveBy(delta) {
    return this.update({
      position: vector3.add(this.position, delta),
      lookAt: vector3.add(this.lookAt, delta),
    });
  }
  rotateAroundAxisAtPoint(angleInRadians, point, axis) {
    return this.update({
      position: vector3.rotateAboutAxis(angleInRadians, this.position, axis, point),
      lookAt: vector3.rotateAboutAxis(angleInRadians, this.lookAt, axis, point),
      up: vector3.rotateAboutAxis(angleInRadians, this.up, axis, vector3.origin()),
    });
  }
  update(camera) {
    var _a;
    const fovY = (_a = camera.fovY) !== null && _a !== void 0 ? _a : this.fovY;
    return new PerspectiveCamera(this.stream, this.aspect, Object.assign(Object.assign(Object.assign({}, this.perspectiveData), camera), { fovY }), this.boundingBox, this.decodeFrame, this.flyToOptions);
  }
  toFrameCamera() {
    var _a;
    return new FramePerspectiveCamera(this.position, this.lookAt, this.up, this.near, this.far, this.aspectRatio, (_a = this.fovY) !== null && _a !== void 0 ? _a : 45);
  }
  get viewVector() {
    return vector3.subtract(this.lookAt, this.position);
  }
  /**
   * The position vector for the camera, in world space coordinates.
   */
  get position() {
    return Object.assign({}, this.perspectiveData.position);
  }
  /**
   * A normalized vector representing the up direction.
   */
  get up() {
    return Object.assign({}, this.perspectiveData.up);
  }
  /**
   * A vector, in world space coordinates, of where the camera is pointed at.
   */
  get lookAt() {
    return Object.assign({}, this.perspectiveData.lookAt);
  }
  /**
   * The camera's field of view.
   */
  get fovY() {
    return this.perspectiveData.fovY;
  }
  /**
   * The aspect ratio of the camera.
   */
  get aspectRatio() {
    return this.aspect;
  }
  /**
   * The camera's near clipping plane.
   */
  get near() {
    const { near } = this.computeClippingPlanes(this.perspectiveData);
    return near;
  }
  /**
   * The camera's far clipping plane.
   */
  get far() {
    const { far } = this.computeClippingPlanes(this.perspectiveData);
    return far;
  }
  updateFlyToOptions(flyToOptions) {
    return new PerspectiveCamera(this.stream, this.aspect, this.perspectiveData, this.boundingBox, this.decodeFrame, flyToOptions);
  }
}
class OrthographicCamera extends Camera {
  constructor(stream, aspect, orthographicData, boundingBox, decodeFrame, flyToOptions) {
    super(stream, aspect, orthographicData, boundingBox, decodeFrame, flyToOptions);
    this.orthographicData = orthographicData;
  }
  /**
   * Shifts the position of the camera by the given delta.
   *
   * @param delta The number of units to shift the camera on the X, Y, and Z
   * axis.
   */
  moveBy(delta) {
    // The rotation point should match the lookAt point when panning
    const updatedPoint = vector3.add(this.lookAt, delta);
    return this.update({
      lookAt: updatedPoint,
      rotationPoint: updatedPoint,
    });
  }
  rotateAroundAxisAtPoint(angleInRadians, point, axis) {
    const updatedLookAt = vector3.rotateAboutAxis(angleInRadians, this.lookAt, axis, point);
    const updatedPosition = vector3.rotateAboutAxis(angleInRadians, this.position, axis, point);
    const viewVector = constrainViewVector(vector3.subtract(updatedLookAt, updatedPosition), boundingSphere.create(this.boundingBox));
    return this.update({
      viewVector: viewVector,
      lookAt: updatedLookAt,
      up: vector3.rotateAboutAxis(angleInRadians, this.up, axis, vector3.origin()),
    });
  }
  update(camera) {
    return new OrthographicCamera(this.stream, this.aspect, Object.assign(Object.assign({}, this.orthographicData), camera), this.boundingBox, this.decodeFrame, this.flyToOptions);
  }
  toFrameCamera() {
    const boundingSphere$1 = boundingSphere.create(this.boundingBox);
    const viewVector = constrainViewVector(this.viewVector, boundingSphere$1);
    const lookAt = updateLookAtRelativeToBoundingBoxCenter(this.lookAt, viewVector, boundingSphere$1.center);
    return new FrameOrthographicCamera(viewVector, lookAt, this.up, this.near, this.far, this.aspectRatio, this.fovHeight, this.rotationPoint);
  }
  get viewVector() {
    return Object.assign({}, this.orthographicData.viewVector);
  }
  get position() {
    return vector3.add(this.lookAt, vector3.negate(this.viewVector));
  }
  /**
   * A vector, in world space coordinates, of where the camera should be rotated around.
   * Most interactions (like pan and zoom) will update lookAt and rotationPoint to be the same point.
   * However, rotation interactions will only update lookAt and not rotationPoint.
   */
  get rotationPoint() {
    const rotationPoint = this.orthographicData.rotationPoint;
    if (rotationPoint != null) {
      return rotationPoint;
    }
    else {
      return undefined;
    }
  }
  /**
   * A normalized vector representing the up direction.
   */
  get up() {
    return Object.assign({}, this.orthographicData.up);
  }
  /**
   * A vector, in world space coordinates, of where the camera is pointed at.
   */
  get lookAt() {
    return Object.assign({}, this.orthographicData.lookAt);
  }
  /**
   * The camera's field of view.
   */
  get fovHeight() {
    return this.orthographicData.fovHeight;
  }
  /**
   * The aspect ratio of the camera.
   */
  get aspectRatio() {
    return this.aspect;
  }
  /**
   * The camera's near clipping plane.
   */
  get near() {
    const { near } = this.computeClippingPlanes(this.orthographicData);
    return near;
  }
  /**
   * The camera's far clipping plane.
   */
  get far() {
    const { far } = this.computeClippingPlanes(this.orthographicData);
    return far;
  }
  updateFlyToOptions(flyToOptions) {
    return new OrthographicCamera(this.stream, this.aspect, this.orthographicData, this.boundingBox, this.decodeFrame, flyToOptions);
  }
}

/**
 * The `CrossSectioner` class is here.
 */
class CrossSectioner {
  constructor(stream, data) {
    this.stream = stream;
    this.data = data;
  }
  /**
   * Performs request on the stream to update the cross sectioning config.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector("vertex-viewer");
   * const scene = await viewer.scene();
   * const crossSectioner = scene.crossSectioning();
   *
   * await scene.crossSectioning().update({
   *   sectionPlanes: [
   *     {
   *       normal: {
   *         x: 1,
   *         y: 0,
   *         z: 0,
   *       },
   *       offset: 0,
   *     },
   *   ],
   *   highlightColor: Color.fromHexString("#ff0000"),
   *   lineWidth: 0.5,
   * });
   * ```
   *
   * @see {@link SectionPlane} for more information on the section planes.
   *
   * @see {@link CrossSectioning} for more information on the highlight color and
   * line width.
   *
   * @param crossSectioning The new cross sectioning.
   */
  async update(crossSectioning) {
    const res = await this.stream.updateCrossSectioning({
      crossSectioning: {
        sectionPlanes: crossSectioning.sectionPlanes,
        highlightColor: crossSectioning.highlightColor,
        lineWidth: (crossSectioning === null || crossSectioning === void 0 ? void 0 : crossSectioning.lineWidth) != null
          ? { value: crossSectioning.lineWidth }
          : null,
      },
    }, true);
    return res.updateCrossSectioning || undefined;
  }
  current() {
    return this.data;
  }
}

/**
 * A class to handle the building of operations on scene items for a scene.
 */
class ItemOperationBuilder {
  constructor(operations = []) {
    this.operations = operations;
  }
  /**
   * Constructs the scene operations and returns a definition describing each
   * operation.
   */
  build() {
    return this.operations.concat();
  }
  materialOverride(material) {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'change-material', material }]));
  }
  show() {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'show' }]));
  }
  hide() {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'hide' }]));
  }
  select() {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'select' }]));
  }
  deselect() {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'deselect' }]));
  }
  clearMaterialOverrides() {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'clear-override' }]));
  }
  transform(matrix) {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'change-transform', transform: matrix }]));
  }
  clearTransforms(cascade = true) {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'clear-transform', cascade }]));
  }
  setPhantom(phantomState) {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'change-phantom', phantomState }]));
  }
  clearPhantom() {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'clear-phantom' }]));
  }
  setEndItem(endItemState) {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'change-end-item', endItemState }]));
  }
  clearEndItem() {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'clear-end-item' }]));
  }
  viewRenditionById(id) {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'view-rendition-by-id', id }]));
  }
  viewRenditionBySuppliedId(suppliedId) {
    return new ItemOperationBuilder(this.operations.concat([
      { type: 'view-rendition-by-supplied-id', suppliedId },
    ]));
  }
  viewDefaultRendition() {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'view-default-rendition' }]));
  }
  clearRendition() {
    return new ItemOperationBuilder(this.operations.concat([{ type: 'clear-rendition' }]));
  }
  viewRepresentation(id) {
    return new ItemOperationBuilder(this.operations.concat({ type: 'view-representation', id }));
  }
  clearRepresentation() {
    return new ItemOperationBuilder(this.operations.concat({ type: 'clear-representation' }));
  }
}
/**
 * A class to handle the building of operations on PMI annotations for a scene.
 */
class PmiAnnotationOperationBuilder {
  constructor(operations = []) {
    this.operations = operations;
  }
  /**
   * Constructs the scene PMI annotation operations and returns a definition describing each
   * operation.
   */
  build() {
    return this.operations.concat();
  }
  show() {
    return new PmiAnnotationOperationBuilder(this.operations.concat([{ type: 'show' }]));
  }
  hide() {
    return new PmiAnnotationOperationBuilder(this.operations.concat([{ type: 'hide' }]));
  }
  select() {
    return new PmiAnnotationOperationBuilder(this.operations.concat([{ type: 'select' }]));
  }
  deselect() {
    return new PmiAnnotationOperationBuilder(this.operations.concat([{ type: 'deselect' }]));
  }
}

/**
 * The `Raycaster` class is here.
 */
class Raycaster {
  constructor(stream, imageScaleProvider) {
    this.stream = stream;
    this.imageScaleProvider = imageScaleProvider;
  }
  /**
   * Performs request on the stream to find items that intersect
   * the given point.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector("vertex-viewer");
   *
   * viewer.addEventListener("tap", async (event) => {
   *   const scene = await viewer.scene();
   *   const raycaster = scene.raycaster();
   *
   *   // Query the scene for the item at the position of the `tap` event
   *   const [hit] = await raycaster.hitItems(event.detail.position);
   *
   *   if (hit != null) {
   *     // If there was an item present at the position, select it
   *     await scene.elements((op) =>
   *       op.items.where((q) => q.withItemId(hit.itemId.hex)).select()
   *     );
   *   }
   * });
   * ```
   *
   * @see {@link Scene.items} for more information on the operations that
   * can be performed on a hit result.
   *
   * @param point The point to cast from looking for intersections.
   * @param options Optional set of options for the request @see {@link RaycasterOptions}
   * for available options.
   */
  async hitItems(point$1, options) {
    const scale = this.imageScaleProvider();
    const res = await this.stream.hitItems({
      point: point.scale(point$1, (scale === null || scale === void 0 ? void 0 : scale.x) || 1, (scale === null || scale === void 0 ? void 0 : scale.y) || 1),
      includeMetadata: options === null || options === void 0 ? void 0 : options.includeMetadata,
    }, true);
    return res.hitItems || undefined;
  }
}

const SCENE_ALTERATION_DEFAULT_TIMEOUT_MS = 1000 * 30;
class SceneOperationExecuteResult {
  constructor(stream, decodeFrame, correlationId, timeout) {
    this.onFrameDrawn = new StreamApiEventDispatcher(stream, (msg) => {
      var _a, _b, _c;
      return !!((_c = (_b = (_a = msg.request) === null || _a === void 0 ? void 0 : _a.drawFrame) === null || _b === void 0 ? void 0 : _b.frameCorrelationIds) === null || _c === void 0 ? void 0 : _c.some((id) => id === correlationId));
    }, (msg) => {
      var _a;
      return ((_a = msg.request) === null || _a === void 0 ? void 0 : _a.drawFrame) != null
        ? decodeFrame(msg.request.drawFrame)
        : undefined;
    }, timeout !== null && timeout !== void 0 ? timeout : SCENE_ALTERATION_DEFAULT_TIMEOUT_MS);
  }
}

class SceneViewStateLoader {
  constructor(stream, decodeFrame, encodeCameraType, sceneId, sceneViewId) {
    this.stream = stream;
    this.decodeFrame = decodeFrame;
    this.encodeCameraType = encodeCameraType;
    this.sceneId = sceneId;
    this.sceneViewId = sceneViewId;
  }
  async applySceneViewState(sceneViewStateId, opts = {}) {
    const pbIdField = buildSceneViewStateIdentifier(sceneViewStateId);
    await this.animateToSceneViewState(pbIdField, opts);
    return await this.stream.loadSceneViewState(Object.assign(Object.assign({}, pbIdField), { frameCorrelationId: opts.suppliedCorrelationId
        ? { value: opts.suppliedCorrelationId }
        : undefined, cameraType: opts.cameraTypeOverride
        ? this.encodeCameraType(opts.cameraTypeOverride)
        : undefined }), true);
  }
  async applyPartialSceneViewState(sceneViewStateId, featuresToApply, opts = {}) {
    const pbIdField = buildSceneViewStateIdentifier(sceneViewStateId);
    const pbFeatures = toPbSceneViewStateFeatures(featuresToApply);
    if (featuresToApply.includes('camera')) {
      await this.animateToSceneViewState(pbIdField, opts);
    }
    return await this.stream.loadSceneViewState(Object.assign(Object.assign({}, pbIdField), { frameCorrelationId: opts.suppliedCorrelationId
        ? { value: opts.suppliedCorrelationId }
        : undefined, sceneViewStateFeatureSubset: pbFeatures, cameraType: opts.cameraTypeOverride
        ? this.encodeCameraType(opts.cameraTypeOverride)
        : undefined }), true);
  }
  async animateToSceneViewState(identifier, opts) {
    var _a, _b, _c;
    if (opts.animation != null) {
      const corrId = uuid.create();
      const flyToResult = await this.stream.flyTo({
        sceneViewStateIdentifier: identifier,
        animation: {
          duration: toProtoDuration(opts.animation.milliseconds),
        },
        frameCorrelationId: {
          value: corrId,
        },
      }, true);
      if (opts.waitForAnimation == null || opts.waitForAnimation) {
        const renderResult = new CameraRenderResult(this.stream, this.decodeFrame, {
          animationId: (_c = (_b = (_a = flyToResult.flyTo) === null || _a === void 0 ? void 0 : _a.animationId) === null || _b === void 0 ? void 0 : _b.hex) !== null && _c !== void 0 ? _c : undefined,
          correlationId: corrId,
        }, opts.animation.milliseconds + DEFAULT_TIMEOUT_IN_MS);
        await renderResult.onAnimationCompleted.once();
      }
    }
  }
}

/**
 * A class that is responsible for building operations on scene items for a specific scene.
 * This executor requires a query, and expects `execute()` to be invoked in
 * order for the changes to take effect.
 */
class SceneItemOperationsBuilder {
  constructor(query, givenBuilder) {
    this.query = query;
    this.builder = givenBuilder !== null && givenBuilder !== void 0 ? givenBuilder : new ItemOperationBuilder();
  }
  isItemBuilder() {
    return true;
  }
  isAnnotationBuilder() {
    return false;
  }
  /**
   * Specifies that the scene items matching the query should have their default
   * material overridden to match the specified material.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Override the material for the item with the `item-uuid` ID to
   * // be red with an opacity of 0.5.
   * await scene.elements((op) => [
   *   op
   *     .items.where((q) => q.withItemId('item-uuid'))
   *     .materialOverride(ColorMaterial.create(255, 0, 0, 0.5)),
   * ]);
   *
   * // Override the material for the item with the `item-uuid` ID to
   * // be red with an opacity of 1.
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).materialOverride('#ff0000'),
   * ]).execute();
   * ```
   */
  materialOverride(color) {
    if (typeof color === 'string') {
      return new SceneItemOperationsBuilder(this.query, this.builder.materialOverride(fromHex(color)));
    }
    else {
      return new SceneItemOperationsBuilder(this.query, this.builder.materialOverride(color));
    }
  }
  /**
   * Specifies that the scene items matching the query should be hidden.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Hide the item with the `item-uuid` ID
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).hide(),
   * ]).execute();
   * ```
   */
  hide() {
    return new SceneItemOperationsBuilder(this.query, this.builder.hide());
  }
  /**
   * Specifies that the scene items matching the query should be shown.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Show the item with the `item-uuid` ID
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).show(),
   * ]).execute();
   * ```
   */
  show() {
    return new SceneItemOperationsBuilder(this.query, this.builder.show());
  }
  /**
   * Specifies that the scene items matching the query should be selected.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Select the item with the `item-uuid` ID
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).select(),
   * ]).execute();
   * ```
   */
  select() {
    return new SceneItemOperationsBuilder(this.query, this.builder.select());
  }
  /**
   * Specifies that the scene items matching the query should be deselected.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Deselect the item with the `item-uuid` ID
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).deselect(),
   * ]).execute();
   * ```
   */
  deselect() {
    return new SceneItemOperationsBuilder(this.query, this.builder.deselect());
  }
  /**
   * Specifies that the scene items matching the query should have any overridden
   * material removed.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Clear the overridden material on the item with the `item-uuid` ID
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).clearMaterialOverrides(),
   * ]);
   * ```
   */
  clearMaterialOverrides() {
    return new SceneItemOperationsBuilder(this.query, this.builder.clearMaterialOverrides());
  }
  /**
   * Specifies that the scene items matching the query should have their
   * transformation matrix overridden to match the specified transformation
   * matrix.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Override the transformation matrix for the item with the `item-uuid` ID to
   * // move the element along the x-axis
   * await scene.elements((op) => [
   *   op
   *     .items.where((q) => q.withItemId('item-uuid'))
   *     .transform(Matrix4.makeTranslation(Vector3.create(100, 0, 0))),
   * ]);
   * ```
   */
  transform(matrix) {
    if (Array.isArray(matrix)) {
      if (matrix.length !== 16) {
        throw new InvalidArgumentError('Matrix provided must contain exactly 16 values (4x4).');
      }
      return new SceneItemOperationsBuilder(this.query, this.builder.transform({
        r0: {
          x: matrix[0],
          y: matrix[1],
          z: matrix[2],
          w: matrix[3],
        },
        r1: {
          x: matrix[4],
          y: matrix[5],
          z: matrix[6],
          w: matrix[7],
        },
        r2: {
          x: matrix[8],
          y: matrix[9],
          z: matrix[10],
          w: matrix[11],
        },
        r3: {
          x: matrix[12],
          y: matrix[13],
          z: matrix[14],
          w: matrix[15],
        },
      }));
    }
    else {
      return new SceneItemOperationsBuilder(this.query, this.builder.transform(matrix));
    }
  }
  /**
   * Specifies that the scene items matching the query should have their overridden
   * transformation matrix removed. The `cascade` flag determines whether
   * children of the scene items matching the query should also have their overridden
   * transformation matrix removed, and defaults to `true`.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Clear the overridden the transformation matrix for the item with the `item-uuid` ID
   * // and do not cascade to preserve transformations on children
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).clearTransforms(false),
   * ]);
   *
   * // Clear the overridden the transformation matrix for the item with the `item-uuid` ID
   * // and cascade to clear overridden transformations on children
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).clearTransforms(true),
   * ]);
   * ```
   */
  clearTransforms(cascade = true) {
    return new SceneItemOperationsBuilder(this.query, this.builder.clearTransforms(cascade));
  }
  /**
   * Specifies that the scene items matching the query should have their phantom state
   * overridden to match the specified `phantomState` flag. If the
   * `phantomState` flag is not provided, it will default to `true`.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Mark the item with the `item-uuid` ID as phantom
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).setPhantom(true),
   * ]);
   *
   * // Unmark the item with the `item-uuid` ID as phantom
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).setPhantom(false),
   * ]);
   * ```
   */
  setPhantom(phantomState) {
    return new SceneItemOperationsBuilder(this.query, this.builder.setPhantom(phantomState));
  }
  /**
   * Specifies that the scene items matching the query should have their overridden
   * phantom state removed.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Clear the overridden phantom state of the item with the `item-uuid` ID
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).clearPhantom(),
   * ]);
   * ```
   */
  clearPhantom() {
    return new SceneItemOperationsBuilder(this.query, this.builder.clearPhantom());
  }
  /**
   * Specifies that the scene items matching the query should have their end item
   * state overridden to match the specified `endItemState` flag. If the
   * `endItemState` flag is not provided, it will default to `true`.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Mark the item with the `item-uuid` ID as an end item
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).setEndItem(true),
   * ]);
   *
   * // Unmark the item with the `item-uuid` ID as an end item
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).setEndItem(false),
   * ]);
   * ```
   *
   * @remarks
   * End item states do not propagate to children similar to other states like
   * other operations. I.e. calling setEndItem(false) on an item will cause it
   * to be unmarked as an end item, but any children where setEndItem(true) was
   * called previously will remain as end items.
   */
  setEndItem(endItemState) {
    return new SceneItemOperationsBuilder(this.query, this.builder.setEndItem(endItemState));
  }
  /**
   * Specifies that the scene items matching the query should have their overridden
   * end item state removed.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Clear the overridden end item state of the item with the `item-uuid` ID
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).clearEndItem(),
   * ]);
   * ```
   */
  clearEndItem() {
    return new SceneItemOperationsBuilder(this.query, this.builder.clearEndItem());
  }
  /**
   * Changes the rendition of a scene item matching the query. This operation only
   * applies to scene items that reference a revision that contains the given
   * rendition.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Switch the rendition of the matching item.
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).viewRenditionById('rendition-uuid'),
   * ]);
   * ```
   */
  viewRenditionById(id) {
    return new SceneItemOperationsBuilder(this.query, this.builder.viewRenditionById(id));
  }
  /**
   * Changes the rendition of any scene item matching the query that contains a
   * rendition with the given supplied ID. This operation only applies to scene items
   * that reference a revision that contain a rendition with a matching supplied
   * ID.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Switch the rendition of the given item.
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).viewRenditionBySuppliedId('rendition-supplied-id'),
   * ]);
   * ```
   */
  viewRenditionBySuppliedId(suppliedId) {
    return new SceneItemOperationsBuilder(this.query, this.builder.viewRenditionBySuppliedId(suppliedId));
  }
  /**
   * Changes the rendition of scene items matching the query back to their revision's
   * default rendition. This operation only applies to scene items that reference a
   * revision.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Switch the rendition of the given item.
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).viewDefaultRendition(),
   * ]);
   * ```
   */
  viewDefaultRendition() {
    return new SceneItemOperationsBuilder(this.query, this.builder.viewDefaultRendition());
  }
  /**
   * Clears the rendition of scene items matching the query, which will revert the
   * scene item back to the rendition used when creating the item.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Switch the rendition of the given item.
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).clearRendition(),
   * ]);
   * ```
   */
  clearRendition() {
    return new SceneItemOperationsBuilder(this.query, this.builder.clearRendition());
  }
  /**
   * Changes the representation of scene items matching a query. This operation only
   * applies to scene items that reference a rendition with the given representation
   * ID.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Switch the rendition of the given item.
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).viewRepresentation('rep-id'),
   * ]);
   * ```
   */
  viewRepresentation(id) {
    return new SceneItemOperationsBuilder(this.query, this.builder.viewRepresentation(id));
  }
  /**
   * Clears the representation for scene items matching the query.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Switch the rendition of the given item.
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).clearRepresentation(),
   * ]);
   * ```
   */
  clearRepresentation() {
    return new SceneItemOperationsBuilder(this.query, this.builder.clearRepresentation());
  }
  /**
   * @internal
   */
  build() {
    return {
      query: this.query,
      operations: this.builder.build(),
    };
  }
}
/**
 * @ignore
 * @beta PMI Annotation operations are being actively developed. These APIs
 * should be considered unstable, and will potentially change with following
 * releases.
 *
 * A class that is responsible for building operations on pmi annotations for a specific scene.
 * This executor requires a query, and expects `execute()` to be invoked in
 * order for the changes to take effect.
 */
class PmiAnnotationOperationsBuilder {
  constructor(query, givenBuilder) {
    this.query = query;
    this.builder = givenBuilder !== null && givenBuilder !== void 0 ? givenBuilder : new PmiAnnotationOperationBuilder();
  }
  isItemBuilder() {
    return false;
  }
  isAnnotationBuilder() {
    return true;
  }
  /**
   * @ignore
   * @beta PMI Annotation operations are being actively developed. These APIs
   * should be considered unstable, and will potentially change with following
   * releases.
   *
   * Specifies that the PMI annotations matching the query should be hidden.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Hide the annotation with the `annotation-uuid` ID
   * await scene.elements((op) => [
   *   op.annotations.where((q) => q.withAnnotationId('annotation-uuid')).hide(),
   * ]).execute();
   * ```
   */
  hide() {
    return new PmiAnnotationOperationsBuilder(this.query, this.builder.hide());
  }
  /**
   * @ignore
   * @beta PMI Annotation operations are being actively developed. These APIs
   * should be considered unstable, and will potentially change with following
   * releases.
   *
   * Specifies that the PMI annotations matching the query should be shown.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Show the annotation with the `annotation-uuid` ID
   * await scene.elements((op) => [
   *   op.annotations.where((q) => q.withAnnotationId('annotation-uuid')).show(),
   * ]).execute();
   * ```
   */
  show() {
    return new PmiAnnotationOperationsBuilder(this.query, this.builder.show());
  }
  /**
   * @ignore
   * @beta PMI Annotation operations are being actively developed. These APIs
   * should be considered unstable, and will potentially change with following
   * releases.
   *
   * Specifies that the PMI annotations matching the query should be selected.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Select the annotation with the `annotation-uuid` ID
   * await scene.elements((op) => [
   *   op.annotations.where((q) => q.withAnnotationId('annotation-uuid')).select(),
   * ]).execute();
   * ```
   */
  select() {
    return new PmiAnnotationOperationsBuilder(this.query, this.builder.select());
  }
  /**
   * @ignore
   * @beta PMI Annotation operations are being actively developed. These APIs
   * should be considered unstable, and will potentially change with following
   * releases.
   *
   * Specifies that the PMI annotations matching the query should be deselected.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Deselect the annotation with the `annotation-uuid` ID
   * await scene.elements((op) => [
   *   op.annotations.where((q) => q.withAnnotationId('annotation-uuid')).deselect(),
   * ]).execute();
   * ```
   */
  deselect() {
    return new PmiAnnotationOperationsBuilder(this.query, this.builder.deselect());
  }
  /**
   * @internal
   */
  build() {
    return {
      query: this.query,
      operations: this.builder.build(),
    };
  }
}
class OperationExecutor {
  constructor(sceneViewId, stream, decodeFrame, dimensions, sceneItemQueryOperations, pmiAnnotationQueryOperations) {
    this.sceneViewId = sceneViewId;
    this.stream = stream;
    this.decodeFrame = decodeFrame;
    this.dimensions = dimensions;
    this.sceneItemQueryOperations = sceneItemQueryOperations;
    this.pmiAnnotationQueryOperations = pmiAnnotationQueryOperations;
  }
  async execute(executionOptions) {
    var _a;
    const pbItemOperations = this.sceneItemQueryOperations.map((op) => buildSceneElementOperationOnItem(op.query, op.operations, {
      dimensions: this.dimensions,
    }));
    const pbPmiAnnotationOperations = this.pmiAnnotationQueryOperations.map((op) => buildSceneElementOperationOnAnnotation(op.query, op.operations));
    const requestCorrelationId = (_a = executionOptions === null || executionOptions === void 0 ? void 0 : executionOptions.suppliedCorrelationId) !== null && _a !== void 0 ? _a : uuid.create();
    const request = {
      sceneViewId: {
        hex: this.sceneViewId,
      },
      elementOperations: [...pbItemOperations, ...pbPmiAnnotationOperations],
      suppliedCorrelationId: {
        value: requestCorrelationId,
      },
    };
    await this.stream.createSceneAlteration(request);
  }
}
class SceneItemsOperationExecutor extends OperationExecutor {
  async execute(executionOptions) {
    var _a;
    if (executionOptions === null || executionOptions === void 0 ? void 0 : executionOptions.awaitCorrelatedFrame) {
      const correlationId = (_a = executionOptions.suppliedCorrelationId) !== null && _a !== void 0 ? _a : uuid.create();
      const executeResult = new SceneOperationExecuteResult(this.stream, this.decodeFrame, correlationId);
      await super.execute(Object.assign(Object.assign({}, executionOptions), { suppliedCorrelationId: correlationId }));
      await executeResult.onFrameDrawn.once();
    }
    else {
      await super.execute(executionOptions);
    }
  }
}
class SceneElementsOperationExecutor extends OperationExecutor {
  async execute(executionOptions) {
    var _a;
    if (executionOptions === null || executionOptions === void 0 ? void 0 : executionOptions.skipAwaitCorrelatedFrame) {
      await super.execute(executionOptions);
    }
    else {
      const correlationId = (_a = executionOptions === null || executionOptions === void 0 ? void 0 : executionOptions.suppliedCorrelationId) !== null && _a !== void 0 ? _a : uuid.create();
      const executeResult = new SceneOperationExecuteResult(this.stream, this.decodeFrame, correlationId);
      await super.execute(Object.assign(Object.assign({}, executionOptions), { suppliedCorrelationId: correlationId }));
      await executeResult.onFrameDrawn.once();
    }
  }
}
/**
 * A class that represents the `Scene` that has been loaded into the viewer. On
 * it, you can retrieve attributes of the scene, such as the camera. It also
 * contains methods for updating the scene and performing requests to rerender
 * the scene.
 */
class Scene {
  constructor(stream, frame, decodeFrame, encodeCameraType, imageScaleProvider, dimensions, sceneId, sceneViewId) {
    this.stream = stream;
    this.frame = frame;
    this.decodeFrame = decodeFrame;
    this.encodeCameraType = encodeCameraType;
    this.imageScaleProvider = imageScaleProvider;
    this.dimensions = dimensions;
    this.sceneId = sceneId;
    this.sceneViewId = sceneViewId;
    this.sceneViewStateLoader = new SceneViewStateLoader(stream, decodeFrame, encodeCameraType, sceneId, sceneViewId);
  }
  /**
   * Applies the provided scene view state to the scene.
   */
  async applySceneViewState(sceneViewStateId, opts = {}) {
    return await this.sceneViewStateLoader.applySceneViewState(sceneViewStateId, opts);
  }
  /**
   * Applies the specified features of the provided scene view state to the scene.
   */
  async applyPartialSceneViewState(sceneViewStateId, featuresToApply, opts = {}) {
    return await this.sceneViewStateLoader.applyPartialSceneViewState(sceneViewStateId, featuresToApply, opts);
  }
  /**
   * Resets the view to its default state, with the ability to reset the camera to that of the base scene.
   */
  async reset(opts = {}) {
    return await this.stream.resetSceneView({
      includeCamera: opts.includeCamera,
      frameCorrelationId: opts.suppliedCorrelationId
        ? { value: opts.suppliedCorrelationId }
        : undefined,
      cameraType: opts.cameraTypeOverride
        ? this.encodeCameraType(opts.cameraTypeOverride)
        : undefined,
    }, true);
  }
  /**
   * Returns an executor that accepts a function as a parameter that contains one or more operations to apply
   * to the scene view. The operations will be applied transactionally.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Deselect everything, then select a specific item by ID
   * await scene.items(op => [
   *   op.items.where(q => q.all()).deselect(),
   *   op.items.where(q => q.withItemId('item-id')).select(),
   * ]).execute();
   * ```
   *
   * @see {@link RootQuery} for more information on available queries.
   *
   * @see {@link SceneItemOperationsBuilder} for more information on available operations.
   *
   * @param operations
   */
  items(operations) {
    const sceneOperations = operations(new SceneItemQueryExecutor());
    const ops = Array.isArray(sceneOperations) ? sceneOperations : [sceneOperations];
    const itemOps = ops.filter((op) => op.isItemBuilder());
    const operationList = itemOps.reduce((acc, builder) => acc.concat(builder.build()), []);
    return new SceneItemsOperationExecutor(this.sceneViewId, this.stream, this.decodeFrame, this.dimensions, operationList, []);
  }
  /**
   * Returns an executor that accepts a function as a parameter that contains one or more operations to apply
   * to items or annotations in the scene view. The operations will be applied transactionally.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Deselect everything, then select a specific scene item by ID
   * await scene.elements(op => [
   *   op.items.where(q => q.all()).deselect(),
   *   op.annotations.where(q => q.all()).deselect(),
   *   op.items.where(q => q.withItemId('item-id')).select(),
   * ]).execute();
   * ```
   *
   * @see {@link RootQuery} for more information on available queries on scene items.
   *
   * @see {@link SceneItemOperationsBuilder} for more information on available operations to the scene items.
   *
   * @see {@link PmiAnnotationRootQuery} for more information on available queries on PMI annotations.
   *
   * @see {@link PmiAnnotationOperationsBuilder} for more information on available operations to the PMI annotations.
   *
   * @param operations
   */
  elements(operations) {
    const ops = operations(new SceneElementQueryExecutor());
    const opsAsArray = Array.isArray(ops) ? ops : [ops];
    // Operations on scene items
    const sceneItemOps = opsAsArray.filter((op) => op.isItemBuilder());
    const sceneItemsOperationList = sceneItemOps.reduce((acc, builder) => acc.concat(builder.build()), []);
    // Operations on PMI annotations
    const pmiAnnotationOps = opsAsArray.filter((op) => op.isAnnotationBuilder());
    const pmiAnnotationOperationList = pmiAnnotationOps.reduce((acc, builder) => acc.concat(builder.build()), []);
    return new SceneElementsOperationExecutor(this.sceneViewId, this.stream, this.decodeFrame, this.dimensions, sceneItemsOperationList, pmiAnnotationOperationList);
  }
  /**
   * An instance of the current camera of the scene. The camera provides a number of
   * methods that can be used in combination with the `render` method to make programmatic
   * updates to the scene's camera.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   * const camera = scene.camera();
   *
   * // Fit the camera to the visible bounding box of the scene with a 1 second animation
   * await camera.viewAll().render({ animation: { milliseconds: 1000 } });
   * ```
   *
   * @see {@link Camera} for more information on available camera operations.
   */
  camera() {
    const { scene } = this.frame;
    if (scene.camera.isOrthographic()) {
      return new OrthographicCamera(this.stream, dimensions.aspectRatio(this.viewport()), {
        viewVector: scene.camera.viewVector,
        lookAt: scene.camera.lookAt,
        up: scene.camera.up,
        fovHeight: scene.camera.fovHeight,
        rotationPoint: scene.camera.lookAt,
      }, this.frame.scene.boundingBox, this.decodeFrame);
    }
    else if (scene.camera.isPerspective()) {
      return new PerspectiveCamera(this.stream, dimensions.aspectRatio(this.viewport()), {
        position: scene.camera.position,
        lookAt: scene.camera.lookAt,
        up: scene.camera.up,
        fovY: scene.camera.fovY,
      }, this.frame.scene.boundingBox, this.decodeFrame);
    }
    else {
      throw new InvalidCameraError('Cannot retrieve camera. Scene has an unknown or invalid camera type.');
    }
  }
  isOrthographic() {
    const { scene } = this.frame;
    return scene.camera.isOrthographic();
  }
  /**
   * Returns the current visible BoundingBox for the scene.
   */
  boundingBox() {
    return this.frame.scene.boundingBox;
  }
  /**
   * CrossSectioner to update cross sectioning planes and get current configuration.
   */
  crossSectioning() {
    return new CrossSectioner(this.stream, this.frame.scene.crossSection);
  }
  /**
   * Raycaster to request items that intersect a point.
   */
  raycaster() {
    return new Raycaster(this.stream, this.imageScaleProvider);
  }
  /**
   * The current viewport of the scene, in pixels.
   */
  viewport() {
    return this.frame.dimensions;
  }
  /**
   * The current x and y scale of the rendered image.
   */
  scale() {
    return this.imageScaleProvider() || point.create(1, 1);
  }
}

/**
 * An interface that represents a query is "complete" and can be turned into an
 * expression.
 */
class TerminalQuery {
  constructor(inverted) {
    this.inverted = inverted;
  }
  build() {
    if (this.inverted) {
      return {
        type: 'not',
        query: this.queryExpressionBuilder(),
      };
    }
    else {
      return this.queryExpressionBuilder();
    }
  }
}
class RootQuery {
  constructor(inverted = false) {
    this.inverted = inverted;
  }
  /**
   * Specifies that the operation should be performed on all items in the scene.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Deselect all items in the scene
   * await scene.elements((op) => [op.items.where((q) => q.all()).deselect()]).execute();
   * ```
   */
  all() {
    return new AllQuery();
  }
  /**
   * Specifies that the operation should be performed on all items that do not match any following queries.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Hide all items that are not selected
   * await scene.elements((op) => [op.items.where((q) => q.not().withSelected()).hide()]).execute();
   * ```
   */
  not() {
    return new NotSceneItemQuery(!this.inverted);
  }
  /**
   * Specifies that the operation should be performed on any item matching any one of the provided IDs.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Hide the item with the `item-uuid-1` ID and the `item-uuid-2` ID
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemIds(['item-uuid-1', 'item-uuid-2'])).hide(),
   * ]).execute();
   * ```
   */
  withItemIds(ids) {
    return new BulkQuery(ids, 'item-id', this.inverted);
  }
  /**
   * Specifies that the operation should be performed on any item matching any one of the provided custom supplied IDs.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Hide the item with the `item-supplied-id-1` supplied ID
   * // and the `item-supplied-id-2` supplied ID
   * await scene.elements((op) => [
   *   op
   *     .items.where((q) => q.withItemIds(['item-supplied-id-1', 'item-supplied-id-2']))
   *     .hide(),
   * ]).execute();
   * ```
   */
  withSuppliedIds(ids) {
    return new BulkQuery(ids, 'supplied-id', this.inverted);
  }
  /**
   * Specifies that the operation should be performed on any item matching the provided ID.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Hide the item with the `item-uuid` ID
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withItemId('item-uuid')).hide(),
   * ]).execute();
   * ```
   */
  withItemId(id) {
    return new SingleSceneItemQuery({ type: 'item-id', value: id }, this.inverted);
  }
  /**
   * Specifies that the operation should be performed on any item matching the provided custom supplied ID.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Hide the item with the `item-supplied-id` supplied ID
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withSuppliedId('item-supplied-id')).hide(),
   * ]).execute();
   * ```
   */
  withSuppliedId(id) {
    return new SingleSceneItemQuery({ type: 'supplied-id', value: id }, this.inverted);
  }
  /**
   * Specifies that the operation should be performed on a range within the `<vertex-scene-tree>` component.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Hide all items from the 2nd row to the 5th row of the scene-tree
   * await scene.elements((op) => [
   *   op
   *     .items.where((q) =>
   *       q.withSceneTreeRange({
   *         start: 2,
   *         end: 5,
   *       })
   *     )
   *     .hide(),
   * ]).execute();
   * ```
   */
  withSceneTreeRange(range) {
    return new SceneTreeRangeQuery(range, this.inverted);
  }
  /**
   * Specifies that the operation should be performed on any item that has a metadata value matching the
   * filter provided for any of the keys specified. Can optionally be set to perform an exactMatch,
   * which will require that the filter matches the value exactly.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Hide all items where the `PART_NAME_KEY` includes a value of `PartName`
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withMetadata('PartName', ['PART_NAME_KEY'])).hide(),
   * ]).execute();
   *
   * // Hide all items where the `PART_NAME_KEY` has exactly a value of `PartName`
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withMetadata('PartName', ['PART_NAME_KEY'], true)).hide(),
   * ]).execute();
   * ```
   */
  withMetadata(filter, keys, exactMatch, removeHiddenItems) {
    return new MetadataQuery(filter, keys, exactMatch, this.inverted, removeHiddenItems);
  }
  /**
   * Specifies that the operation should be performed on any item that has been selected.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Hide all items that are selected
   * await scene.elements((op) => [op.items.where((q) => q.withSelected()).hide()]).execute();
   * ```
   */
  withSelected() {
    return new AllSelectedQuery(this.inverted);
  }
  /**
   * Specifies that the operation should be performed on any item that is visible.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Select all items that are visible
   * await scene.elements((op) => [op.items.where((q) => q.withVisible()).select()]).execute();
   * ```
   */
  withVisible() {
    return new AllVisibleQuery(this.inverted);
  }
  /**
   * Specifies that the operation should be performed on any item present at the provided `point` in the image.
   * This query operates on the item found at that `point` similar to using `withItemId` in combination with
   * `raycaster.hitItems`, which can be useful if the additional metadata from the `raycaster.hitItems`
   * method is not needed to eliminate a network request.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Select the item present at the [100, 100] coordinate of the image
   * await scene.elements((op) => [
   *   op.items.where((q) => q.withPoint(Point.create(100, 100))).select(),
   * ]).execute();
   * ```
   */
  withPoint(point) {
    return new PointQuery(point, this.inverted);
  }
  /**
   * Specifies that the operation should be performed on items within the specified `rectangle` boundary
   * within the Viewer. The `exclusive` flag here determines whether items that intersect with the `rectangle`,
   * but are not contained should be included in the result.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();

   * // Select all items within the specified 100x100 region of the image
   * // excluding any elements that are not fully contained by the region
   * await scene.elements((op) => [
   *   op
   *     .items.where((q) =>
   *       q.withVolumeIntersection(
   *         Rectangle.create(100, 100, 100, 100),
   *         true
   *       )
   *     )
   *     .hide(),
   * ]).execute();

   * // Select all items within the specified 100x100 region of the image
   * // including any elements that intersect with the region
   * await scene.elements((op) => [
   *   op
   *     .items.where((q) =>
   *       q.withVolumeIntersection(
   *         Rectangle.create(100, 100, 100, 100),
   *         false
   *       )
   *     )
   *     .hide(),
   * ]).execute();
   * ```
   */
  withVolumeIntersection(rectangle, exclusive) {
    return new VolumeIntersectionQuery(rectangle, this.inverted, exclusive);
  }
}
class NotSceneItemQuery extends RootQuery {
  constructor(inverted) {
    super(inverted);
  }
}
class PmiAnnotationRootQuery {
  constructor(inverted = false) {
    this.inverted = inverted;
  }
  /**
   * Specifies that the operation should be performed on all PMI annotations in the scene.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Deselect all PMI annotations in the scene
   * await scene.elements((op) => [op.annotations.where((q) => q.all()).deselect()]).execute();
   * ```
   */
  all() {
    return new AllQuery();
  }
  /**
   * Specifies that the operation should be performed on all annotations that do not match any following queries.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Hide all PMI annotations that are not selected
   * await scene.elements((op) => [op.annotations.where((q) => q.not().withSelected()).hide()]).execute();
   * ```
   */
  not() {
    return new NotAnnotationQuery(!this.inverted);
  }
  /**
   * Specifies that the operation should be performed on any annotation matching any one of the provided IDs.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Hide the annotation with the `item-uuid-1` ID and the `item-uuid-2` ID
   * await scene.elements((op) => [
   *   op.annotations.where((q) => q.withAnnotationIds(['item-uuid-1', 'item-uuid-2'])).hide(),
   * ]).execute();
   * ```
   */
  withAnnotationIds(ids) {
    return new BulkQuery(ids, 'annotation-id', this.inverted);
  }
  /**
   * Specifies that the operation should be performed on any annotation matching the provided ID.
   *
   * @example
   * ```typescript
   * const viewer = document.querySelector('vertex-viewer');
   * const scene = await viewer.scene();
   *
   * // Hide the item with the `item-uuid` ID
   * await scene.elements((op) => [
   *   op.annotations.where((q) => q.withAnnotationId('item-uuid')).hide(),
   * ]).execute();
   * ```
   */
  withAnnotationId(id) {
    return new SingleAnnotationQuery({ type: 'annotation-id', value: id }, this.inverted);
  }
}
class NotAnnotationQuery extends PmiAnnotationRootQuery {
  constructor(inverted) {
    super(inverted);
  }
}
class AllQuery extends TerminalQuery {
  constructor(inverted = false) {
    super(inverted);
  }
  queryExpressionBuilder() {
    return { type: 'all' };
  }
}
class SceneTreeRangeQuery extends TerminalQuery {
  constructor(range, inverted) {
    super(inverted);
    this.range = range;
  }
  queryExpressionBuilder() {
    return {
      type: 'scene-tree-range',
      range: this.range,
    };
  }
}
class MetadataQuery extends TerminalQuery {
  constructor(filter, keys, exactMatch, inverted, removeHiddenItems) {
    super(inverted);
    this.filter = filter;
    this.keys = keys;
    this.exactMatch = exactMatch;
    this.removeHiddenItems = removeHiddenItems;
  }
  queryExpressionBuilder() {
    return {
      type: 'metadata',
      filter: this.filter,
      keys: this.keys,
      exactMatch: this.exactMatch,
      removeHiddenItems: this.removeHiddenItems,
    };
  }
}
class AllSelectedQuery extends TerminalQuery {
  constructor(inverted) {
    super(inverted);
  }
  queryExpressionBuilder() {
    return {
      type: 'all-selected',
    };
  }
}
class AllVisibleQuery extends TerminalQuery {
  constructor(inverted) {
    super(inverted);
  }
  queryExpressionBuilder() {
    return {
      type: 'all-visible',
    };
  }
}
class PointQuery extends TerminalQuery {
  constructor(point, inverted) {
    super(inverted);
    this.point = point;
  }
  queryExpressionBuilder() {
    return {
      type: 'point',
      point: this.point,
    };
  }
}
class VolumeIntersectionQuery extends TerminalQuery {
  constructor(rectangle, inverted, exclusive) {
    super(inverted);
    this.rectangle = rectangle;
    this.exclusive = exclusive;
  }
  queryExpressionBuilder() {
    return {
      type: 'volume-intersection',
      rectangle: this.rectangle,
      exclusive: !!this.exclusive,
    };
  }
}
class BulkQuery extends TerminalQuery {
  constructor(ids, type, inverted) {
    super(inverted);
    this.ids = ids;
    this.type = type;
  }
  queryExpressionBuilder() {
    return {
      type: 'or',
      expressions: this.ids.map((id) => {
        return {
          type: this.type,
          value: id,
        };
      }),
    };
  }
}
class SingleSceneItemQuery extends TerminalQuery {
  constructor(query, inverted) {
    super(inverted);
    this.query = query;
  }
  queryExpressionBuilder() {
    return Object.assign({}, this.query);
  }
  and() {
    return new AndSceneItemQuery([this.query], this.inverted);
  }
  or() {
    return new OrSceneItemQuery([this.query], this.inverted);
  }
}
class OrSceneItemQuery extends TerminalQuery {
  constructor(expressions, inverted) {
    super(inverted);
    this.expressions = expressions;
  }
  queryExpressionBuilder() {
    return { type: 'or', expressions: [...this.expressions] };
  }
  withItemId(id) {
    return new OrSceneItemQuery([...this.expressions, { type: 'item-id', value: id }], this.inverted);
  }
  withSuppliedId(id) {
    return new OrSceneItemQuery([...this.expressions, { type: 'supplied-id', value: id }], this.inverted);
  }
  or() {
    return this;
  }
}
class AndSceneItemQuery extends TerminalQuery {
  constructor(expressions, inverted) {
    super(inverted);
    this.expressions = expressions;
  }
  queryExpressionBuilder() {
    return { type: 'and', expressions: [...this.expressions] };
  }
  withItemId(id) {
    return new AndSceneItemQuery([...this.expressions, { type: 'item-id', value: id }], this.inverted);
  }
  withSuppliedId(id) {
    return new AndSceneItemQuery([...this.expressions, { type: 'supplied-id', value: id }], this.inverted);
  }
  and() {
    return this;
  }
}
class SingleAnnotationQuery extends TerminalQuery {
  constructor(query, inverted) {
    super(inverted);
    this.query = query;
  }
  queryExpressionBuilder() {
    return Object.assign({}, this.query);
  }
  and() {
    return new AndAnnotationQuery([this.query], this.inverted);
  }
  or() {
    return new OrAnnotationQuery([this.query], this.inverted);
  }
}
class OrAnnotationQuery extends TerminalQuery {
  constructor(expressions, inverted) {
    super(inverted);
    this.expressions = expressions;
  }
  queryExpressionBuilder() {
    return { type: 'or', expressions: [...this.expressions] };
  }
  withAnnotationId(id) {
    return new OrAnnotationQuery([...this.expressions, { type: 'annotation-id', value: id }], this.inverted);
  }
  or() {
    return this;
  }
}
class AndAnnotationQuery extends TerminalQuery {
  constructor(expressions, inverted) {
    super(inverted);
    this.expressions = expressions;
  }
  queryExpressionBuilder() {
    return { type: 'and', expressions: [...this.expressions] };
  }
  withAnnotationId(id) {
    return new AndAnnotationQuery([...this.expressions, { type: 'annotation-id', value: id }], this.inverted);
  }
  and() {
    return this;
  }
}
class SceneElementQueryExecutor {
  get items() {
    return new SceneItemQueryExecutor();
  }
  get annotations() {
    return new PmiAnnotationsQueryExecutor();
  }
}
class PmiAnnotationsQueryExecutor {
  where(query) {
    const expression = query(new PmiAnnotationRootQuery()).build();
    return new PmiAnnotationOperationsBuilder(expression);
  }
}
class SceneItemQueryExecutor {
  where(query) {
    const expression = query(new RootQuery()).build();
    return new SceneItemOperationsBuilder(expression);
  }
}

export { Camera as C, ItemOperationBuilder as I, ModelViewController as M, PmiAnnotationOperationsBuilder as P, Raycaster as R, Scene as S, CrossSectioner as a, PmiAnnotationsQueryExecutor as b, colorMaterial as c, RootQuery as d, SceneElementQueryExecutor as e, SceneElementsOperationExecutor as f, SceneItemOperationsBuilder as g, SceneItemQueryExecutor as h, SceneItemsOperationExecutor as i, SynchronizedClock as j, PmiController as k, loadableResource as l, fromUrn as m, sceneViewStateIdentifier as s };

//# sourceMappingURL=queries-57b3b092.js.map