/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { r as registerInstance, c as createEvent, h, H as Host, g as getElement } from './index-fef00dee.js';
import { s as stampTemplateWithId } from './templates-f0400734.js';
import { F as FreeformMarkup, C as CircleMarkup, A as ArrowMarkup } from './markup-66ddf24a.js';
import { b as isVertexViewerFreeformMarkup, a as isVertexViewerCircleMarkup, i as isVertexViewerArrowMarkup } from './utils-4ff4fe60.js';
import './bundle.esm-0508e4f1.js';
import './browser.esm-956a68fc.js';

const viewerMarkupToolCss = ":host{position:absolute;top:0;bottom:0;left:0;right:0;pointer-events:none}:host([elevated=\"\"]){z-index:100}";

const ViewerMarkupTool = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.markupBegin = createEvent(this, "markupBegin", 7);
    this.markupEnd = createEvent(this, "markupEnd", 7);
    /**
     * The type of markup.
     *
     * This property will automatically be set when a child of a
     * `<vertex-viewer-markup>` element.
     */
    this.tool = 'arrow';
    /**
     * Disables markups.
     *
     * This property will automatically be set when a child of a
     * `<vertex-viewer-markup>` element.
     */
    this.disabled = false;
    /**
     * The style of the starting anchor. This defaults to none.
     */
    this.startLineAnchorStyle = 'none';
    /**
     * The style of the ending anchor. This defaults to 'arrow-triangle.'
     */
    this.endLineAnchorStyle = 'arrow-triangle';
    /**
     * Captures internal state that you want to preserve across dev refreshes, but
     * don't want to trigger a render when the state changes.
     */
    this.stateMap = {};
    this.handleMarkupInteractionBegin = () => {
      this.markupBegin.emit();
    };
    this.handleMarkupInteractionEnd = () => {
      const { markupElement } = this.stateMap;
      if (isVertexViewerFreeformMarkup(markupElement)) {
        const { points, bounds } = markupElement;
        if (points != null && points.length > 0 && bounds != null) {
          this.markupEnd.emit(new FreeformMarkup({ points, bounds }));
        }
      }
      else if (isVertexViewerCircleMarkup(markupElement)) {
        const { bounds } = markupElement;
        if (bounds != null) {
          this.markupEnd.emit(new CircleMarkup({ bounds }));
        }
      }
      else if (isVertexViewerArrowMarkup(markupElement)) {
        const { start, end } = markupElement;
        if (start != null && end != null) {
          this.markupEnd.emit(new ArrowMarkup({ start, end }));
        }
      }
    };
  }
  async handleViewerChanged() {
    if (this.stateMap.markupElement != null) {
      this.stateMap.markupElement.viewer = this.viewer;
    }
  }
  /**
   * @ignore
   */
  handleToolChanged() {
    this.updateMarkupElement();
  }
  /**
   * @ignore
   */
  handleArrowTemplateIdChanged() {
    this.updateMarkupElement();
  }
  /**
   * @ignore
   */
  handleCircleTemplateIdChanged() {
    this.updateMarkupElement();
  }
  /**
   * @ignore
   */
  handleFreeformTemplateIdChanged() {
    this.updateMarkupElement();
  }
  /**
   * @ignore
   */
  handleDisabledChanged() {
    this.updateMarkupElement();
  }
  /**
   * @ignore
   */
  handleStartLineAnchorStyleChanged() {
    this.updateMarkupElement();
  }
  /**
   * @ignore
   */
  handleEndLineAnchorStyleChanged() {
    this.updateMarkupElement();
  }
  /**
   * @ignore
   */
  componentDidLoad() {
    this.updateMarkupElement();
  }
  /**
   * Resets the state of the internally managed markup element
   * to allow for creating a new markup. This state is automatically
   * managed when this element is placed as a child of a
   * `<vertex-viewer-markup>` element.
   */
  async reset() {
    const { markupElement } = this.stateMap;
    if (isVertexViewerFreeformMarkup(markupElement)) {
      markupElement.points = undefined;
      markupElement.bounds = undefined;
      markupElement.mode = 'create';
    }
    else if (isVertexViewerCircleMarkup(markupElement)) {
      markupElement.bounds = undefined;
      markupElement.mode = 'create';
    }
    else if (isVertexViewerArrowMarkup(markupElement)) {
      markupElement.start = undefined;
      markupElement.end = undefined;
      markupElement.mode = 'create';
    }
  }
  /**
   * @ignore
   */
  render() {
    if (!this.disabled) {
      if (this.tool === 'arrow' ||
        this.tool === 'circle' ||
        this.tool === 'freeform') {
        return (h(Host, null, h("slot", null)));
      }
      else {
        return h(Host, null, `Unsupported tool type '${this.tool}'.`);
      }
    }
    else {
      return h(Host, null);
    }
  }
  createArrowMarkupElement() {
    if (this.arrowTemplateId != null) {
      const element = stampTemplateWithId(window.document.body, this.arrowTemplateId, isVertexViewerArrowMarkup, () => console.warn(`Arrow template with ID ${this.arrowTemplateId} not found. Using default arrow element.`), () => console.warn(`Arrow template does not contain a vertex-viewer-markup-arrow. Using default arrow element.`));
      if (element != null) {
        return element;
      }
    }
    return document.createElement('vertex-viewer-markup-arrow');
  }
  createCircleMarkupElement() {
    if (this.circleTemplateId != null) {
      const element = stampTemplateWithId(window.document.body, this.circleTemplateId, isVertexViewerCircleMarkup, () => console.warn(`Circle template with ID ${this.circleTemplateId} not found. Using default circle element.`), () => console.warn(`Circle template does not contain a vertex-viewer-markup-circle. Using default circle element.`));
      if (element != null) {
        return element;
      }
    }
    return document.createElement('vertex-viewer-markup-circle');
  }
  createFreeformMarkupElement() {
    if (this.freeformTemplateId != null) {
      const element = stampTemplateWithId(window.document.body, this.freeformTemplateId, isVertexViewerFreeformMarkup, () => console.warn(`Freeform template with ID ${this.freeformTemplateId} not found. Using default freeform element.`), () => console.warn(`Freeform template does not contain a vertex-viewer-markup-freeform. Using default freeform element.`));
      if (element != null) {
        return element;
      }
    }
    return document.createElement('vertex-viewer-markup-freeform');
  }
  createNewMarkupElement() {
    switch (this.tool) {
      case 'arrow':
        return this.createArrowMarkupElement();
      case 'circle':
        return this.createCircleMarkupElement();
      case 'freeform':
        return this.createFreeformMarkupElement();
    }
  }
  updateMarkupElement() {
    const { markupElement } = this.stateMap;
    if (markupElement != null) {
      markupElement.remove();
      markupElement.dispose();
      markupElement.viewer = undefined;
      markupElement.removeEventListener('interactionBegin', this.handleMarkupInteractionBegin);
      markupElement.removeEventListener('interactionEnd', this.handleMarkupInteractionEnd);
    }
    if (!this.disabled) {
      const newMarkupElement = this.createNewMarkupElement();
      if (this.tool === 'arrow') {
        newMarkupElement.startLineAnchorStyle = this.startLineAnchorStyle;
        newMarkupElement.endLineAnchorStyle = this.endLineAnchorStyle;
      }
      newMarkupElement.mode = 'create';
      newMarkupElement.viewer = this.viewer;
      newMarkupElement.addEventListener('interactionBegin', this.handleMarkupInteractionBegin);
      newMarkupElement.addEventListener('interactionEnd', this.handleMarkupInteractionEnd);
      this.stateMap.markupElement = newMarkupElement;
      this.hostEl.append(newMarkupElement);
    }
  }
  get hostEl() { return getElement(this); }
  static get watchers() { return {
    "viewer": ["handleViewerChanged"],
    "tool": ["handleToolChanged"],
    "arrowTemplateId": ["handleArrowTemplateIdChanged"],
    "circleTemplateId": ["handleCircleTemplateIdChanged"],
    "freeformTemplateId": ["handleFreeformTemplateIdChanged"],
    "disabled": ["handleDisabledChanged"],
    "startLineAnchorStyle": ["handleStartLineAnchorStyleChanged"],
    "endLineAnchorStyle": ["handleEndLineAnchorStyleChanged"]
  }; }
};
ViewerMarkupTool.style = viewerMarkupToolCss;

export { ViewerMarkupTool as vertex_viewer_markup_tool };

//# sourceMappingURL=vertex-viewer-markup-tool.entry.js.map