/*!
 * Copyright (c) 2025 Vertex Software LLC. All rights reserved.
 */
import { r as registerInstance, h, F as Fragment, H as Host } from './index-fef00dee.js';
import { p as point, c as angle } from './bundle.esm-0508e4f1.js';
import './scene_view_api_pb-acffb910.js';
import './wrappers_pb-8ca57f4b.js';
import './mapper-1db0f247.js';
import './streamAttributes-a4b7f379.js';
import { M as MeasurementOverlayManager } from './overlays-bbc3c7cf.js';
import { t as translateWorldLineToViewport } from './utils-4556bfcf.js';
import './_commonjsHelpers-6e425ac3.js';
import './browser.esm-956a68fc.js';
import './bundle.esm-4b7e63bd.js';
import './entities-f59f520e.js';

const viewerMeasurementLineCss = ":host{display:block;--viewer-measurement-line-stroke:var(--white);--viewer-measurement-line-stroke-opacity:0.75;--viewer-measurement-line-stroke-width:1.25px;--viewer-measurement-line-fill:var(--neutral-900);--viewer-measurement-line-fill-width:2px;--viewer-measurement-line-start-cap-visibility:initial;--viewer-measurement-line-end-cap-visibility:initial}svg{width:100%;height:100%}.line-stroke{stroke-width:calc(\n    var(--viewer-measurement-line-fill-width) +\n      var(--viewer-measurement-line-stroke-width)\n  );stroke:var(--viewer-measurement-line-stroke);opacity:var(--viewer-measurement-line-stroke-opacity)}.line-fill{stroke-width:var(--viewer-measurement-line-fill-width);stroke:var(--viewer-measurement-line-fill)}.start-cap{visibility:var(--viewer-measurement-line-start-cap-visibility)}.end-cap{visibility:var(--viewer-measurement-line-end-cap-visibility)}";

const ViewerMeasurementLine = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    /**
     * A point that specifies the starting point of the line.
     */
    this.start = point.create();
    /**
     * A point that specifies the ending point of the line.
     */
    this.end = point.create();
    /**
     * A length of the line cap. The line cap is a line at each end of a line.
     */
    this.capLength = 0;
    /**
     * The type of [SVG pointer
     * events](https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/pointer-events)
     * that the line should respond to.
     */
    this.pointerEvents = 'none';
  }
  /**
   * @ignore
   */
  render() {
    const angle$1 = angle.fromPoints(this.start, this.end);
    const lineFillEndCaps = getEndCapPoints(this.start, this.end, angle$1, this.capLength);
    const lineStrokeEndCaps = getEndCapPoints(this.start, this.end, angle$1, this.capLength + 1);
    return (h("svg", null, h("g", { class: "line-stroke", "pointer-events": this.pointerEvents }, h("line", { class: "line", x1: this.start.x, y1: this.start.y, x2: this.end.x, y2: this.end.y }), h("line", { class: "start-cap", x1: lineStrokeEndCaps.startEndCap.start.x, y1: lineStrokeEndCaps.startEndCap.start.y, x2: lineStrokeEndCaps.startEndCap.end.x, y2: lineStrokeEndCaps.startEndCap.end.y }), h("line", { class: "end-cap", x1: lineStrokeEndCaps.endEndCap.start.x, y1: lineStrokeEndCaps.endEndCap.start.y, x2: lineStrokeEndCaps.endEndCap.end.x, y2: lineStrokeEndCaps.endEndCap.end.y })), h("g", { class: "line-fill", "pointer-events": this.pointerEvents }, h("line", { class: "line", x1: this.start.x, y1: this.start.y, x2: this.end.x, y2: this.end.y }), h("line", { class: "start-cap", x1: lineFillEndCaps.startEndCap.start.x, y1: lineFillEndCaps.startEndCap.start.y, x2: lineFillEndCaps.startEndCap.end.x, y2: lineFillEndCaps.startEndCap.end.y }), h("line", { class: "end-cap", x1: lineFillEndCaps.endEndCap.start.x, y1: lineFillEndCaps.endEndCap.start.y, x2: lineFillEndCaps.endEndCap.end.x, y2: lineFillEndCaps.endEndCap.end.y }))));
  }
};
function getEndCapPoints(start, end, angle, length) {
  return {
    startEndCap: getPerpendicularLine(start, angle, length),
    endEndCap: getPerpendicularLine(end, angle, length),
  };
}
function getPerpendicularLine(pt, angle, length) {
  const perpAngle = angle + Math.PI / 2;
  const start = point.add(pt, point.polar(length / 2, perpAngle));
  const end = point.add(pt, point.polar(length / 2, perpAngle + Math.PI));
  return { start, end };
}
ViewerMeasurementLine.style = viewerMeasurementLineCss;

// eslint-disable-next-line @typescript-eslint/no-unused-vars
const MeasurementOverlayView = ({ overlay, viewport, camera }) => {
  if (overlay.type === 'line') {
    return (h(LineOverlayView, { overlay: overlay, viewport: viewport, camera: camera }));
  }
  else {
    return (h(DistanceVectorOverlayView, { overlay: overlay, viewport: viewport, camera: camera }));
  }
};
const LineOverlayView = ({ overlay, camera, viewport }) => {
  const { start, end } = translateWorldLineToViewport(overlay, {
    camera,
    viewport,
  });
  return (h("vertex-viewer-measurement-line", { class: "measurement-line", start: start, end: end }));
};
const DistanceVectorOverlayView = ({ overlay: { x, y, z }, camera, viewport }) => {
  const params = { camera, viewport };
  const { start: xs, end: xe } = translateWorldLineToViewport(x, params);
  const { start: ys, end: ye } = translateWorldLineToViewport(y, params);
  const { start: zs, end: ze } = translateWorldLineToViewport(z, params);
  return (h(Fragment, null,
    h("vertex-viewer-measurement-line", { class: "measurement-line distance-vector-x", start: xs, end: xe }),
    h("vertex-viewer-measurement-line", { class: "measurement-line distance-vector-y", start: ys, end: ye }),
    h("vertex-viewer-measurement-line", { class: "measurement-line distance-vector-z", start: zs, end: ze })));
};

const viewerMeasurementOverlaysCss = ":host{display:block;position:absolute;top:0;bottom:0;left:0;right:0;pointer-events:none;--viewer-measurement-overlays-x-axis-color:var(--x-axis-color);--viewer-measurement-overlays-y-axis-color:var(--y-axis-color);--viewer-measurement-overlays-z-axis-color:var(--z-axis-color)}.measurement-line{position:absolute;width:100%;height:100%}.measurement-line.distance-vector-x{--viewer-measurement-line-fill:var(\n    --viewer-measurement-overlays-x-axis-color\n  )}.measurement-line.distance-vector-y{--viewer-measurement-line-fill:var(\n    --viewer-measurement-overlays-y-axis-color\n  )}.measurement-line.distance-vector-z{--viewer-measurement-line-fill:var(\n    --viewer-measurement-overlays-z-axis-color\n  )}";

const ViewerMeasurementOverlays = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    /**
     * The model that contains the overlays to present.
     */
    this.measurementOverlays = new MeasurementOverlayManager();
    this.overlays = [];
    this.updateCamera = () => {
      var _a, _b;
      this.camera = (_b = (_a = this.viewer) === null || _a === void 0 ? void 0 : _a.frame) === null || _b === void 0 ? void 0 : _b.scene.camera;
    };
  }
  /**
   * @ignore
   */
  handleMeasurementOverlaysChanged() {
    this.removeModelListeners();
    this.addModelListeners();
    this.updateOverlays();
  }
  /**
   * @ignore
   */
  handleViewerChanged(newViewer, oldViewer) {
    this.removeViewerListeners(oldViewer);
    this.addViewerListeners();
    this.updateCamera();
  }
  /**
   * @ignore
   */
  componentWillLoad() {
    this.addModelListeners();
    this.addViewerListeners();
    this.updateOverlays();
    this.updateCamera();
  }
  addModelListeners() {
    this.onOverlaysChangedListener = this.measurementOverlays.onOverlaysChanged(() => this.updateOverlays());
  }
  removeModelListeners() {
    var _a;
    (_a = this.onOverlaysChangedListener) === null || _a === void 0 ? void 0 : _a.dispose();
    this.onOverlaysChangedListener = undefined;
  }
  addViewerListeners() {
    var _a;
    (_a = this.viewer) === null || _a === void 0 ? void 0 : _a.addEventListener('frameReceived', this.updateCamera);
  }
  removeViewerListeners(viewer) {
    viewer === null || viewer === void 0 ? void 0 : viewer.removeEventListener('frameReceived', this.updateCamera);
  }
  updateOverlays() {
    this.overlays = this.measurementOverlays.getOverlays();
  }
  /**
   * @ignore
   */
  render() {
    return (h(Host, null, this.overlays.map((overlay) => {
      if (this.camera != null && this.viewer != null) {
        return (h(MeasurementOverlayView, { overlay: overlay, viewport: this.viewer.viewport, camera: this.camera }));
      }
    })));
  }
  static get watchers() { return {
    "measurementOverlays": ["handleMeasurementOverlaysChanged"],
    "viewer": ["handleViewerChanged"]
  }; }
};
ViewerMeasurementOverlays.style = viewerMeasurementOverlaysCss;

export { ViewerMeasurementLine as vertex_viewer_measurement_line, ViewerMeasurementOverlays as vertex_viewer_measurement_overlays };

//# sourceMappingURL=vertex-viewer-measurement-line_2.entry.js.map