import { EventEmitter, h } from '../../stencil-public-runtime';
import { Point, Rectangle } from '@vertexvis/geometry';
import { MarkupInteraction } from '../../lib/types/markup';
/**
 * The supported markup modes.
 *
 * @see {@link ViewerMarkupFreeformMode.mode} - For more details about modes.
 */
export declare type ViewerMarkupFreeformMode = 'edit' | 'create' | '';
export declare class ViewerMarkupFreeform {
  /**
   * The positions of the various points of this freeform markup. Can either be an array of
   * `Point`s or a JSON string representation in the format of `[[x1, y1], [x2, y2]]` or
   * `[{"x": 0, "y": 0}, {"x": 0, "y": 0}]`.
   *
   * Points are expected to be relative coordinates from `[-0.5, 0.5]`,
   * e.g. `[0, 0]` corresponds to a point in the center of the viewport.
   */
  points?: Point.Point[];
  /**
   * The positions of the various points of this freeform markup. Can either be an array of
   * `Point`s or a JSON string representation in the format of `[[x1, y1], [x2, y2]]` or
   * `[{"x": 0, "y": 0}, {"x": 0, "y": 0}]`.
   *
   * Points are expected to be relative coordinates from `[-0.5, 0.5]`,
   * e.g. `[0, 0]` corresponds to a point in the center of the viewport.
   */
  pointsJson?: string;
  /**
   * The bounds of the freeform. Can either be an instance of a `Rectangle` or
   * a JSON string representation in the format of `[x, y, width, height]` or
   * `{"x": 0, "y": 0, "width": 10, "height": 10}`.
   *
   * Bounds are expected to have relative coordinates, with `[x, y]` from `[-0.5, 0.5]`
   * and `[width, height]` from `[0, 1]`, e.g. `[0, 0, 0.25, 0.25]`corresponds to a freeform
   * with a diameter of one fourth the viewport's smallest size in the center of the viewport.
   */
  bounds?: Rectangle.Rectangle;
  /**
   * The bounds of the freeform. Can either be an instance of a `Rectangle` or
   * a JSON string representation in the format of `[x, y, width, height]` or
   * `{"x": 0, "y": 0, "width": 0.1, "height": 0.1}`.
   *
   * Bounds are expected to have relative coordinates, with `[x, y]` from `[-0.5, 0.5]`
   * and `[width, height]` from `[0, 1]`, e.g. `[0, 0, 0.25, 0.25]`corresponds to a freeform
   * with a diameter of one fourth the viewport's smallest size in the center of the viewport.
   */
  boundsJson?: string;
  /**
   * A mode that specifies how the markup component should behave. When
   * unset, the component will not respond to interactions with the handles.
   * When `edit`, the markup anchors are interactive and the user is able
   * to reposition them. When `create`, anytime the user clicks on the canvas,
   * a new markup will be performed.
   */
  mode: ViewerMarkupFreeformMode;
  /**
   * The viewer to connect to markups.
   *
   * This property will automatically be set when a child of a
   * `<vertex-viewer-markup>` or `<vertex-viewer>` element.
   */
  viewer?: HTMLVertexViewerElement;
  /**
   * An event that is dispatched anytime the user begins interacting with the
   * markup.
   */
  interactionBegin: EventEmitter<void>;
  /**
   * An event that is dispatched when the user has finished interacting with the
   * markup.
   */
  interactionEnd: EventEmitter<MarkupInteraction>;
  /**
   * An event that is dispatched when this markup element is in view
   * mode (`this.mode === ""`), and it completes a rerender.
   */
  viewRendered: EventEmitter<void>;
  private hostEl;
  private elementBounds?;
  private screenPoints;
  private interactionHandler;
  private registeredInteraction?;
  /**
   * @ignore
   */
  protected componentWillLoad(): void;
  protected componentDidLoad(): void;
  protected componentDidRender(): void;
  protected disconnectedCallback(): void;
  dispose(): Promise<void>;
  /**
   * @ignore
   */
  protected handleViewerChanged(newViewer?: HTMLVertexViewerElement): Promise<void>;
  protected handlePointsChange(): void;
  protected handleBoundsChange(): void;
  protected handleModeChange(): void;
  private updateViewport;
  private updatePointsFromProps;
  render(): h.JSX.IntrinsicElements;
  private handleWindowPointerDown;
  private handleTouchStart;
  private convertPointsToScreen;
}
