import { EventEmitter, h } from '../../stencil-public-runtime';
import { Markup, MarkupInteraction } from '../../lib/types/markup';
import { LineAnchorStyle } from '../viewer-markup-arrow/utils';
import { ViewerMarkupToolType } from '../viewer-markup-tool/viewer-markup-tool';
export declare class ViewerMarkup {
  /**
   * An HTML template that describes the HTML to use for new arrow
   * markup. It's expected that the template contains a
   * `<vertex-viewer-markup-arrow>`.
   */
  arrowTemplateId?: string;
  /**
   * An HTML template that describes the HTML to use for new circle
   * markup. It's expected that the template contains a
   * `<vertex-viewer-markup-circle>`.
   */
  circleTemplateId?: string;
  /**
   * An HTML template that describes the HTML to use for new freeform
   * markup. It's expected that the template contains a
   * `<vertex-viewer-markup-freeform>`.
   */
  freeformTemplateId?: string;
  /**
   * The type of markup to perform.
   */
  tool: ViewerMarkupToolType;
  /**
   * If `true`, disables adding or editing of markup through user
   * interaction.
   */
  disabled: boolean;
  /**
   * The viewer to connect to markup. If nested within a <vertex-viewer>,
   * this property will be populated automatically.
   */
  viewer?: HTMLVertexViewerElement;
  /**
   * The ID of the markup that is selected.
   */
  selectedMarkupId?: string;
  /**
   * Indicates if new markup should be automatically selected.
   */
  selectNew: boolean;
  /**
   * The style of the starting anchor. This defaults to none.
   */
  startLineAnchorStyle: LineAnchorStyle;
  /**
   * The style of the ending anchor. This defaults to 'arrow-triangle.'
   */
  endLineAnchorStyle: LineAnchorStyle;
  /**
   * Dispatched when a new markup is added, either through user interaction
   * or programmatically.
   */
  markupAdded: EventEmitter<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement>;
  /**
   * Dispatched when an existing piece of markup is changed, either through user interaction
   * or programmatically.
   */
  markupChanged: EventEmitter<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement>;
  /**
   * Dispatched when a markup is removed, either through user
   * interaction or programmatically.
   */
  markupRemoved: EventEmitter<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement>;
  /**
   * Dispatched when markup selection changes. Will either be the
   * selected element or `undefined` indicating that selection
   * was cleared.
   */
  markupSelectionChanged: EventEmitter<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement | undefined>;
  private hostEl;
  private toSelectMarkupId?;
  private pointerDownPosition?;
  protected disconnectedCallback(): void;
  /**
   * Adds a new markup as a child to this component. A new markup
   * component will be created from the template specified by
   * `arrow-template-id`, `circle-template-id`, or if undefined
   * a default element will be created.
   *
   * @param markup The markup to add.
   * @returns The markup element that was created.
   * @see {@link ViewerMarkups.arrowTemplateId}
   * @see {@link ViewerMarkups.circleTemplateId}
   */
  addMarkup(markup: Markup): Promise<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement>;
  /**
   * Removes a markup with the given ID, and returns the HTML element
   * associated to the markup. Returns `undefined` if no markup is
   * found.
   *
   * @param id The ID of the markup to remove.
   * @returns The markup element, or undefined.
   */
  removeMarkup(id: string): Promise<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement | undefined>;
  /**
   * Returns a list of markup elements that are children of this component.
   *
   * @returns A list of all markups.
   * @see {@link ViewerMarkup.getMarkupElement}
   */
  getMarkupElements(): Promise<Array<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement>>;
  /**
   * Returns the markup element associated to the given ID.
   *
   * @param id The ID of the markup element to return.
   * @returns A markup element, or `undefined`.
   * @see {@link ViewerMarkup.getMarkupElements}
   */
  getMarkupElement(id: string): Promise<HTMLVertexViewerMarkupArrowElement | HTMLVertexViewerMarkupCircleElement | HTMLVertexViewerMarkupFreeformElement | undefined>;
  /**
   * @ignore
   */
  protected handleSelectedMarkupIdChanged(): Promise<void>;
  /**
   * @ignore
   */
  protected handleToolChanged(): void;
  /**
   * @ignore
   */
  protected handleArrowTemplateIdChanged(): void;
  /**
   * @ignore
   */
  protected handleCircleTemplateIdChanged(): void;
  /**
   * @ignore
   */
  protected handleFreeformTemplateIdChanged(): void;
  /**
   * @ignore
   */
  protected handleViewerChanged(newViewer: HTMLVertexViewerElement | undefined): Promise<void>;
  /**
   * @ignore
   */
  protected handleDisabledChanged(): void;
  /**
   * @ignore
   */
  protected handleStartLineAnchorStyleChanged(): Promise<void>;
  /**
   * @ignore
   */
  protected handleEndLineAnchorStyleChanged(): Promise<void>;
  /**
   * @ignore
   */
  protected handleMarkupEnd(event: CustomEvent<Markup>): Promise<void>;
  /**
   * @ignore
   */
  protected handleMarkupUpdated(event: CustomEvent<MarkupInteraction>): Promise<void>;
  /**
   * @ignore
   */
  protected handleMarkupPointerDown(event: PointerEvent): Promise<void>;
  /**
   * @ignore
   */
  protected componentDidLoad(): void;
  /**
   * @ignore
   */
  protected render(): h.JSX.IntrinsicElements;
  private handlePointerMove;
  private handlePointerUp;
  private appendMarkupElement;
  private createArrowMarkupElement;
  private createCircleMarkupElement;
  private createFreeformMarkupElement;
  private updatePropsOnMarkups;
  private updatePropsOnMarkup;
  private updatePropsOnMarkupTool;
  private getMarkupTool;
  private getSelectedArrowMarkup;
}
