import { EventEmitter, h } from '../../stencil-public-runtime';
import { Euler, Matrix4, Vector3 } from '@vertexvis/geometry';
import { TransformController } from '../../lib/transforms/controller';
import { Drawable } from '../../lib/transforms/drawable';
import { AngleUnitType, DistanceUnitType } from '../../lib/types';
import { PointAndPlacement } from './util';
export declare class ViewerTransformWidget {
  /**
   * An event that is emitted when the position of the widget changes.
   */
  positionChanged: EventEmitter<Vector3.Vector3 | undefined>;
  /**
   * An event that is emitted when the rotation of the widget changes.
   */
  rotationChanged: EventEmitter<Euler.Euler | undefined>;
  /**
   * An event that is emitted when the interaction has ended
   */
  interactionEnded: EventEmitter<Matrix4.Matrix4 | undefined>;
  /**
   * An event that is emitted an interaction with the widget has started
   */
  interactionStarted: EventEmitter<void>;
  /**
   * The viewer to connect to transforms. If nested within a <vertex-viewer>,
   * this property will be populated automatically.
   */
  viewer?: HTMLVertexViewerElement;
  /**
   * The starting position of this transform widget. This position will be updated
   * as transforms occur. Setting this value to `undefined` will remove the widget.
   */
  position?: Vector3.Vector3;
  /**
   * The starting angle for the transform widget. This rotation will be updated
   * as the rotations occur.
   */
  rotation?: Euler.Euler;
  /**
   * The controller that is responsible for performing transforms.
   */
  controller?: TransformController;
  /**
   * Determines whether or not the x-rotation is disabled on the widget
   */
  xRotationDisabled: boolean;
  /**
   * Determines whether or not the y-rotation is disabled on the widget
   */
  yRotationDisabled: boolean;
  /**
   * Determines whether or not the z-rotation is disabled on the widget
   */
  zRotationDisabled: boolean;
  /**
   * Determines whether or not the x-translation is disabled on the widget
   */
  xTranslationDisabled: boolean;
  /**
   * Determines whether or not the y-translation is disabled on the widget
   */
  yTranslationDisabled: boolean;
  /**
   * Determines whether or not the z-translation is disabled on the widget
   */
  zTranslationDisabled: boolean;
  /**
   * Whether to show inputs beside the widget handles when they are interacted with.
   * Defaults to `true`.
   */
  showInputs: boolean;
  /**
   * The unit to show for translation inputs. Defaults to `millimeters`.
   *
   * @see DistanceUnitType
   */
  distanceUnit: DistanceUnitType;
  /**
   * The unit to show for rotation inputs. Defaults to `degrees`.
   *
   * @see AngleUnitType
   */
  angleUnit: AngleUnitType;
  /**
   * The number of decimal places to show in the input. Defaults to `1`.
   */
  decimalPlaces: number;
  /**
   * **EXPERIMENTAL.**
   *
   * Enables Command+Z and Control+Z keybindings to perform an undo of
   * the previous delta transformation.
   *
   * @see `EXPERIMENTAL_undo`
   */
  EXPERIMENTAL_undoKeybindings: boolean;
  /**
   * @internal
   * @ignore
   *
   * Visible for testing.
   */
  hovered?: Drawable;
  protected inputPosition?: PointAndPlacement;
  protected inputValue: number;
  protected isEndingTransform: boolean;
  protected inputShouldFocus: boolean;
  private hostEl;
  private startingTransform?;
  private currentTransform?;
  private dragStartTransform?;
  private xArrowColor;
  private yArrowColor;
  private zArrowColor;
  private hoveredColor;
  private widget?;
  private lastDragged?;
  private dragging?;
  private lastAngle;
  private lastWorldPosition?;
  private lastInputValue?;
  private canvasBounds?;
  private inputBounds?;
  private canvasResizeObserver?;
  private inputResizeObserver?;
  private hostStyleObserver?;
  private canvasRef?;
  private inputWrapperRef?;
  private inputRef?;
  private hoveredChangeDisposable?;
  protected componentDidLoad(): void;
  protected disconnectedCallback(): void;
  protected componentDidRender(): void;
  /**
   * **EXPERIMENTAL.**
   *
   * Performs an undo of the most recent set of transform manipulations,
   * categorized by the last pause in interactivity (pointerup on a handle,
   * Enter press on an input, keyup on an input).
   *
   * Note that this *does not* work repeatedly. I.e. only one undo can be
   * performed, and once another set of transform manipulations is performed,
   * the history is lost.
   */
  EXPERIMENTAL_undo(): Promise<void>;
  /**
   * @ignore
   */
  protected handleViewerChanged(newViewer?: HTMLVertexViewerElement, oldViewer?: HTMLVertexViewerElement): void;
  /**
   * @ignore
   */
  protected handleDisabledPropertyChanged(): void;
  /**
   * @ignore
   */
  protected handleRotationChanged(newRotation?: Euler.Euler, oldRotation?: Euler.Euler): void;
  /**
   * @ignore
   */
  protected handlePositionChanged(newPosition?: Vector3.Vector3, oldPosition?: Vector3.Vector3): void;
  /**
   * @ignore
   */
  protected handleInputFormattingChanged(): void;
  render(): h.JSX.IntrinsicElements;
  private handleHoveredDrawableChanged;
  private handleViewerFrameDrawn;
  private handleViewerDimensionsChange;
  private handleResize;
  private handleInputResize;
  private handleStyleChange;
  private handlePointerMove;
  private handleBeginDrag;
  private handleDrag;
  private handleEndTransform;
  private handleInputStep;
  private handleInputChange;
  private updateTransformFromInput;
  private updatePropsFromViewer;
  private transform;
  private handleSettingDisabledAxis;
  private setupTransformWidget;
  private updateCanvasBounds;
  private updateInputValue;
  private transformCurrent;
  private updateInputPosition;
  private beginEndTransform;
  private completeEndTransform;
  private isModifyingAngleUnits;
  private getDisplayedUnits;
  private getDisplayedAngle;
  private getDisplayedDistance;
  private getTransformForNewPosition;
  private getTransformForNewRotation;
  private getCanvasBounds;
  private getTransformWidget;
}
